@@ -3,28 +3,41 @@
  */
 
 /*
- * Copyright (C) 2011-2015 Oracle Corporation
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-#ifndef ___iprt_cpp_list_h
-#define ___iprt_cpp_list_h
+#ifndef IPRT_INCLUDED_cpp_list_h
+#define IPRT_INCLUDED_cpp_list_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/cpp/meta.h>
 #include <iprt/mem.h>
@@ -122,7 +135,11 @@ public:
     inline void leaveWrite() {}
 
     /* Define our own new and delete. */
+#ifdef RT_NEED_NEW_AND_DELETE
+    RTMEM_IMPLEMENT_NEW_AND_DELETE();
+#else
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
+#endif
 };
 
 /**
@@ -134,6 +151,7 @@ class RTCListHelper
 public:
     static inline void      set(T2 *p, size_t i, const T1 &v) { p[i] = v; }
     static inline T1 &      at(T2 *p, size_t i) { return p[i]; }
+    static inline const T1 &atConst(T2 const *p, size_t i) { return p[i]; }
     static inline size_t    find(T2 *p, const T1 &v, size_t cElements)
     {
         size_t i = cElements;
@@ -160,6 +178,7 @@ class RTCListHelper<T1, T1*>
 public:
     static inline void      set(T1 **p, size_t i, const T1 &v) { p[i] = new T1(v); }
     static inline T1 &      at(T1 **p, size_t i) { return *p[i]; }
+    static inline const T1 &atConst(T1 * const *p, size_t i) { return *p[i]; }
     static inline size_t    find(T1 **p, const T1 &v, size_t cElements)
     {
         size_t i = cElements;
@@ -207,7 +226,7 @@ public:
      *
      * This preallocates @a cCapacity elements within the list.
      *
-     * @param   cCapacitiy   The initial capacity the list has.
+     * @param   cCapacity    The initial capacity the list has.
      * @throws  std::bad_alloc
      */
     RTCListBase(size_t cCapacity = kDefaultCapacity)
@@ -405,6 +424,25 @@ public:
         return insert(0, other);
     }
 
+    /**
+     * Append a default item to the list.
+     *
+     * @return  a mutable reference to the item
+     * @throws  std::bad_alloc
+     */
+    GET_RTYPE append()
+    {
+        m_guard.enterWrite();
+        if (m_cElements == m_cCapacity)
+            growArray(m_cCapacity + kDefaultCapacity);
+        RTCListHelper<T, ITYPE>::set(m_pArray, m_cElements, T());
+        GET_RTYPE rRet = RTCListHelper<T, ITYPE>::at(m_pArray, m_cElements);
+        ++m_cElements;
+        m_guard.leaveWrite();
+
+        return rRet;
+    }
+
     /**
      * Append an item to the list.
      *
@@ -459,7 +497,6 @@ public:
         /* Prevent self assignment */
         if (RT_LIKELY(this != &other))
         {
-
             other.m_guard.enterRead();
             m_guard.enterWrite();
 
@@ -480,6 +517,53 @@ public:
         return *this;
     }
 
+    /**
+     * Compares if this list's items match the other list.
+     *
+     * @returns \c true if both lists contain the same items, \c false if not.
+     * @param   other   The list to compare this list with.
+     */
+    bool operator==(const RTCListBase<T, ITYPE, MT>& other)
+    {
+        /* Prevent self comparrison */
+        if (RT_LIKELY(this == &other))
+            return true;
+
+        other.m_guard.enterRead();
+        m_guard.enterRead();
+
+        bool fEqual = true;
+        if (other.m_cElements == m_cElements)
+        {
+            for (size_t i = 0; i < m_cElements; i++)
+            {
+                if (RTCListHelper<T, ITYPE>::at(m_pArray, i) != RTCListHelper<T, ITYPE>::at(other.m_pArray, i))
+                {
+                    fEqual = false;
+                    break;
+                }
+            }
+        }
+        else
+            fEqual = false;
+
+        m_guard.leaveRead();
+        other.m_guard.leaveRead();
+
+        return fEqual;
+    }
+
+    /**
+     * Compares if this list's items do not match the other list.
+     *
+     * @returns \c true if the lists do not match, \c false if otherwise.
+     * @param   other   The list to compare this list with.
+     */
+    bool operator!=(const RTCListBase<T, ITYPE, MT>& other)
+    {
+        return !(*this == other);
+    }
+
     /**
      * Replace an item in the list.
      *
@@ -504,6 +588,33 @@ public:
         return *this;
     }
 
+    /**
+     * Applies a filter of type T to this list.
+     *
+     * @param   other           The list which contains the elements to filter out from this list.
+     * @return  a reference to this list.
+     */
+    RTCListBase<T, ITYPE, MT> &filter(const RTCListBase<T, ITYPE, MT> &other)
+    {
+        AssertReturn(this != &other, *this);
+
+        other.m_guard.enterRead();
+        m_guard.enterWrite();
+
+        for (size_t i = 0; i < m_cElements; i++)
+        {
+            for (size_t f = 0; f < other.m_cElements; f++)
+            {
+                if (RTCListHelper<T, ITYPE>::at(m_pArray, i) == RTCListHelper<T, ITYPE>::at(other.m_pArray, f))
+                    removeAtLocked(i);
+            }
+        }
+
+        m_guard.leaveWrite();
+        other.m_guard.leaveRead();
+        return *this;
+    }
+
     /**
      * Return the first item as constant object.
      *
@@ -627,10 +738,28 @@ public:
     }
 
     /**
-     * Return the item at position @a i or default value if out of range.
+     * Return the item at position @a i as immutable reference.
+     *
+     * @param   i     The position of the item to return.  This better not be out of
+     *                bounds, however should it be the last element of the array
+     *                will be return and strict builds will raise an assertion.
+     *                Should the array be empty, a crash is very likely.
+     * @return   The item at position @a i.
+     */
+    const T &operator[](size_t i) const
+    {
+        m_guard.enterRead();
+        AssertMsgStmt(i < m_cElements, ("i=%zu m_cElements=%zu\n", i, m_cElements), i = m_cElements - 1);
+        const T &rRet = RTCListHelper<T, ITYPE>::atConst(m_pArray, i);
+        m_guard.leaveRead();
+        return rRet;
+    }
+
+    /**
+     * Return a copy of the item at position @a i or default value if out of range.
      *
      * @param   i              The position of the item to return.
-     * @return  The item at position @a i or default value.
+     * @return  Copy of the item at position @a i or default value.
      */
     T value(size_t i) const
     {
@@ -646,11 +775,11 @@ public:
     }
 
     /**
-     * Return the item at position @a i, or @a defaultVal if out of range.
+     * Return a copy of the item at position @a i, or @a defaultVal if out of range.
      *
      * @param   i              The position of the item to return.
      * @param   defaultVal     The value to return in case @a i is invalid.
-     * @return  The item at position @a i or @a defaultVal.
+     * @return  Copy of the item at position @a i or @a defaultVal.
      */
     T value(size_t i, const T &defaultVal) const
     {
@@ -784,7 +913,11 @@ public:
     }
 
     /* Define our own new and delete. */
+#ifdef RT_NEED_NEW_AND_DELETE
+    RTMEM_IMPLEMENT_NEW_AND_DELETE();
+#else
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
+#endif
 
     /**
      * The default capacity of the list. This is also used as grow factor.
@@ -923,7 +1056,7 @@ public:
      *
      * This preallocates @a cCapacity elements within the list.
      *
-     * @param   cCapacitiy   The initial capacity the list has.
+     * @param   cCapacity    The initial capacity the list has.
      * @throws  std::bad_alloc
      */
     RTCList(size_t cCapacity = BASE::kDefaultCapacity)
@@ -933,7 +1066,11 @@ public:
         : BASE(other) {}
 
     /* Define our own new and delete. */
+#ifdef RT_NEED_NEW_AND_DELETE
+    RTMEM_IMPLEMENT_NEW_AND_DELETE();
+#else
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
+#endif
 };
 
 /**
@@ -954,14 +1091,18 @@ public:
      *
      * This preallocates @a cCapacity elements within the list.
      *
-     * @param   cCapacitiy   The initial capacity the list has.
+     * @param   cCapacity    The initial capacity the list has.
      * @throws  std::bad_alloc
      */
     RTCList(size_t cCapacity = BASE::kDefaultCapacity)
         : BASE(cCapacity) {}
 
     /* Define our own new and delete. */
+#ifdef RT_NEED_NEW_AND_DELETE
+    RTMEM_IMPLEMENT_NEW_AND_DELETE();
+#else
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
+#endif
 };
 
 /**
@@ -982,17 +1123,21 @@ public:
      *
      * This preallocates @a cCapacity elements within the list.
      *
-     * @param   cCapacitiy   The initial capacity the list has.
+     * @param   cCapacity    The initial capacity the list has.
      * @throws  std::bad_alloc
      */
     RTCList(size_t cCapacity = BASE::kDefaultCapacity)
         : BASE(cCapacity) {}
 
     /* Define our own new and delete. */
+#ifdef RT_NEED_NEW_AND_DELETE
+    RTMEM_IMPLEMENT_NEW_AND_DELETE();
+#else
     RTMEMEF_NEW_AND_DELETE_OPERATORS();
+#endif
 };
 
 /** @} */
 
-#endif /* !___iprt_cpp_list_h */
+#endif /* !IPRT_INCLUDED_cpp_list_h */
 
