@@ -1,5 +1,5 @@
 /**********************************************************
- * Copyright 2008-2023 VMware, Inc.  All rights reserved.
+ * Copyright 2008-2009 VMware, Inc.  All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -28,7 +28,7 @@
 
 #include "pipe/p_state.h"
 #include "pipe/p_defines.h"
-#include "util/u_thread.h"
+#include "os/os_thread.h"
 #include "util/format/u_format.h"
 #include "util/u_inlines.h"
 #include "util/u_math.h"
@@ -43,7 +43,6 @@
 #include "svga_resource_texture.h"
 #include "svga_resource_buffer.h"
 #include "svga_sampler_view.h"
-#include "svga_surface.h"
 #include "svga_winsys.h"
 #include "svga_debug.h"
 
@@ -105,7 +104,7 @@ svga_transfer_dma(struct svga_context *svga,
    assert(!st->use_direct_map);
 
    if (transfer == SVGA3D_READ_HOST_VRAM) {
-      SVGA_DBG(DEBUG_PERF, "%s: readback transfer\n", __func__);
+      SVGA_DBG(DEBUG_PERF, "%s: readback transfer\n", __FUNCTION__);
    }
 
    /* Ensure any pending operations on host surfaces are queued on the command
@@ -123,7 +122,7 @@ svga_transfer_dma(struct svga_context *svga,
 
       if (transfer == SVGA3D_READ_HOST_VRAM) {
          svga_context_flush(svga, &fence);
-         sws->fence_finish(sws, fence, OS_TIMEOUT_INFINITE, 0);
+         sws->fence_finish(sws, fence, PIPE_TIMEOUT_INFINITE, 0);
          sws->fence_reference(sws, &fence, NULL);
       }
    }
@@ -178,11 +177,11 @@ svga_transfer_dma(struct svga_context *svga,
           * Prevent the texture contents to be discarded on the next band
           * upload.
           */
-         flags.discard = false;
+         flags.discard = FALSE;
 
          if (transfer == SVGA3D_READ_HOST_VRAM) {
             svga_context_flush(svga, &fence);
-            sws->fence_finish(sws, fence, OS_TIMEOUT_INFINITE, 0);
+            sws->fence_finish(sws, fence, PIPE_TIMEOUT_INFINITE, 0);
 
             hw = sws->buffer_map(sws, st->hwbuf, PIPE_MAP_READ);
             assert(hw);
@@ -210,9 +209,7 @@ svga_resource_get_handle(struct pipe_screen *screen,
    if (texture->target == PIPE_BUFFER)
       return false;
 
-   SVGA_DBG(DEBUG_DMA, "%s: texture=%p cachable=%d\n", __FUNCTION__,
-            texture, svga_texture(texture)->key.cachable);
-
+   assert(svga_texture(texture)->key.cachable == 0);
    svga_texture(texture)->key.cachable = 0;
 
    stride = util_format_get_nblocksx(texture->format, texture->width0) *
@@ -226,35 +223,47 @@ svga_resource_get_handle(struct pipe_screen *screen,
 /**
  * Determine if we need to read back a texture image before mapping it.
  */
-static inline bool
+static inline boolean
 need_tex_readback(struct svga_transfer *st)
 {
    if (st->base.usage & PIPE_MAP_READ)
-      return true;
+      return TRUE;
 
    if ((st->base.usage & PIPE_MAP_WRITE) &&
        ((st->base.usage & PIPE_MAP_DISCARD_WHOLE_RESOURCE) == 0)) {
-      return svga_was_texture_rendered_to(svga_texture(st->base.resource));
+      return svga_was_texture_rendered_to(svga_texture(st->base.resource),
+                                          st->slice, st->base.level);
    }
 
-   return false;
+   return FALSE;
 }
 
 
 static void
-readback_texture_surface(struct svga_context *svga,
-                         struct svga_texture *tex,
-                         struct svga_winsys_surface *surf)
+readback_image_vgpu9(struct svga_context *svga,
+                   struct svga_winsys_surface *surf,
+                   unsigned slice,
+                   unsigned level)
 {
-   SVGA_RETRY(svga, SVGA3D_ReadbackGBSurface(svga->swc, surf));
-
-   /* Mark the texture surface as UPDATED */
-   tex->surface_state = SVGA_SURFACE_STATE_UPDATED;
-
-   svga->hud.num_readbacks++;
-   SVGA_STATS_COUNT_INC(svga_sws(svga), SVGA_STATS_COUNT_TEXREADBACK);
+   SVGA_RETRY(svga, SVGA3D_ReadbackGBImage(svga->swc, surf, slice, level));
 }
 
+
+static void
+readback_image_vgpu10(struct svga_context *svga,
+                    struct svga_winsys_surface *surf,
+                    unsigned slice,
+                    unsigned level,
+                    unsigned numMipLevels)
+{
+   unsigned subResource;
+
+   subResource = slice * numMipLevels + level;
+   SVGA_RETRY(svga, SVGA3D_vgpu10_ReadbackSubResource(svga->swc, surf,
+                                                      subResource));
+}
+
+
 /**
  * Use DMA for the transfer request
  */
@@ -296,7 +305,7 @@ svga_texture_transfer_map_dma(struct svga_context *svga,
       if (0) {
          debug_printf("%s: failed to allocate %u KB of DMA, "
                       "splitting into %u x %u KB DMA transfers\n",
-                      __func__,
+                      __FUNCTION__,
                       (nblocksy * st->base.stride + 1023) / 1024,
                       (nblocksy + st->hw_nblocksy - 1) / st->hw_nblocksy,
                       (st->hw_nblocksy * st->base.stride + 1023) / 1024);
@@ -337,28 +346,36 @@ svga_texture_transfer_map_direct(struct svga_context *svga,
    struct svga_texture *tex = svga_texture(texture);
    struct svga_winsys_surface *surf = tex->handle;
    unsigned level = st->base.level;
-   unsigned w, h, nblocksx, nblocksy;
+   unsigned w, h, nblocksx, nblocksy, i;
    unsigned usage = st->base.usage;
 
    if (need_tex_readback(st)) {
       svga_surfaces_flush(svga);
 
       if (!svga->swc->force_coherent || tex->imported) {
-         /* Readback the whole surface */
-         readback_texture_surface(svga, tex, surf);
+         for (i = 0; i < st->box.d; i++) {
+            if (svga_have_vgpu10(svga)) {
+               readback_image_vgpu10(svga, surf, st->slice + i, level,
+                                     tex->b.last_level + 1);
+            } else {
+               readback_image_vgpu9(svga, surf, st->slice + i, level);
+            }
+         }
+         svga->hud.num_readbacks++;
+         SVGA_STATS_COUNT_INC(sws, SVGA_STATS_COUNT_TEXREADBACK);
 
-         svga_context_finish(svga);
+         svga_context_flush(svga, NULL);
       }
       /*
        * Note: if PIPE_MAP_DISCARD_WHOLE_RESOURCE were specified
        * we could potentially clear the flag for all faces/layers/mips.
        */
-      svga_clear_texture_rendered_to(tex);
+      svga_clear_texture_rendered_to(tex, st->slice, level);
    }
    else {
       assert(usage & PIPE_MAP_WRITE);
       if ((usage & PIPE_MAP_UNSYNCHRONIZED) == 0) {
-         if (svga_is_texture_level_dirty(tex, st->slice, level)) {
+         if (svga_is_texture_dirty(tex, st->slice, level)) {
             /*
              * do a surface flush if the subresource has been modified
              * in this command buffer.
@@ -388,7 +405,7 @@ svga_texture_transfer_map_direct(struct svga_context *svga,
    {
       SVGA3dSize baseLevelSize;
       uint8_t *map;
-      bool retry, rebind;
+      boolean retry, rebind;
       unsigned offset, mip_width, mip_height;
       struct svga_winsys_context *swc = svga->swc;
 
@@ -410,6 +427,7 @@ svga_texture_transfer_map_direct(struct svga_context *svga,
          map = svga->swc->surface_map(svga->swc, surf, usage, &retry, &rebind);
          svga_retry_exit(svga);
       }
+
       if (map && rebind) {
          enum pipe_error ret;
 
@@ -481,8 +499,12 @@ svga_texture_transfer_map(struct pipe_context *pipe,
    struct svga_texture *tex = svga_texture(texture);
    struct svga_transfer *st;
    struct svga_winsys_surface *surf = tex->handle;
-   bool use_direct_map = svga_have_gb_objects(svga) &&
-                         (!svga_have_gb_dma(svga) || (usage & PIPE_MAP_WRITE));
+#ifndef VBOX_WITH_MESA3D_HACKS
+   boolean use_direct_map = svga_have_gb_objects(svga) &&
+       (!svga_have_gb_dma(svga) || (usage & PIPE_MAP_WRITE));
+#else
+   boolean use_direct_map = false;
+#endif
    void *map = NULL;
    int64_t begin = svga_get_time(svga);
 
@@ -494,7 +516,7 @@ svga_texture_transfer_map(struct pipe_context *pipe,
    /* We can't map texture storage directly unless we have GB objects */
    if (usage & PIPE_MAP_DIRECTLY) {
       if (svga_have_gb_objects(svga))
-         use_direct_map = true;
+         use_direct_map = TRUE;
       else
          goto done;
    }
@@ -538,12 +560,11 @@ svga_texture_transfer_map(struct pipe_context *pipe,
       break;
    }
 
-   /* We never want to use DMA transfers on systems with GBObjects because
-    * it causes serialization issues and in SVGAv3 vram is gone which
-    * makes it impossible to support both at the same time.
-    */
-   if (svga_have_gb_objects(svga)) {
-      use_direct_map = true;
+   /* Force direct map for multisample surface */
+   if (texture->nr_samples > 1) {
+      assert(svga_have_gb_objects(svga));
+      assert(sws->have_sm4_1);
+      use_direct_map = TRUE;
    }
 
    st->use_direct_map = use_direct_map;
@@ -564,19 +585,19 @@ svga_texture_transfer_map(struct pipe_context *pipe,
       map = svga_texture_transfer_map_dma(svga, st);
    }
    else {
-      bool can_use_upload = tex->can_use_upload &&
-                            !(st->base.usage & PIPE_MAP_READ);
-      bool was_rendered_to =
-         svga_was_texture_rendered_to(svga_texture(texture));
-      bool is_dirty = svga_is_texture_dirty(svga_texture(texture));
+      boolean can_use_upload = tex->can_use_upload &&
+                               !(st->base.usage & PIPE_MAP_READ);
+      boolean was_rendered_to =
+         svga_was_texture_rendered_to(svga_texture(texture),
+                                      st->slice, st->base.level);
 
-      /* If the texture was already rendered to or has pending changes and
-       * upload buffer is supported, then we will use upload buffer to
+      /* If the texture was already rendered to and upload buffer
+       * is supported, then we will use upload buffer to
        * avoid the need to read back the texture content; otherwise,
        * we'll first try to map directly to the GB surface, if it is blocked,
        * then we'll try the upload buffer.
        */
-      if ((was_rendered_to || is_dirty) && can_use_upload) {
+      if (was_rendered_to && can_use_upload) {
          map = svga_texture_transfer_map_upload(svga, st);
       }
       else {
@@ -637,7 +658,7 @@ svga_texture_surface_unmap(struct svga_context *svga,
 {
    struct svga_winsys_surface *surf = svga_texture(transfer->resource)->handle;
    struct svga_winsys_context *swc = svga->swc;
-   bool rebind;
+   boolean rebind;
 
    assert(surf);
 
@@ -697,14 +718,14 @@ svga_texture_transfer_unmap_dma(struct svga_context *svga,
 
       memset(&flags, 0, sizeof flags);
       if (st->base.usage & PIPE_MAP_DISCARD_WHOLE_RESOURCE) {
-         flags.discard = true;
+         flags.discard = TRUE;
       }
       if (st->base.usage & PIPE_MAP_UNSYNCHRONIZED) {
-         flags.unsynchronized = true;
+         flags.unsynchronized = TRUE;
       }
 
       svga_transfer_dma(svga, st, SVGA3D_WRITE_HOST_VRAM, flags);
-      svga_set_texture_rendered_to(tex);
+      svga_set_texture_rendered_to(tex, st->slice, st->base.level);
    }
 
    FREE(st->swbuf);
@@ -749,7 +770,7 @@ svga_texture_transfer_unmap_direct(struct svga_context *svga,
 
       if (0)
          debug_printf("%s %d, %d, %d  %d x %d x %d\n",
-                      __func__,
+                      __FUNCTION__,
                       box.x, box.y, box.z,
                       box.w, box.h, box.d);
 
@@ -768,9 +789,6 @@ svga_texture_transfer_unmap_direct(struct svga_context *svga,
                                transfer->level);
          }
       }
-
-      /* Mark the texture surface state as UPDATED */
-      tex->surface_state = SVGA_SURFACE_STATE_UPDATED;
    }
 }
 
@@ -819,7 +837,7 @@ svga_texture_transfer_unmap(struct pipe_context *pipe,
 /**
  * Does format store depth values?
  */
-static inline bool
+static inline boolean
 format_has_depth(enum pipe_format format)
 {
    const struct util_format_description *desc = util_format_description(format);
@@ -887,6 +905,12 @@ svga_texture_create(struct pipe_screen *screen,
       goto fail_notex;
    }
 
+   tex->rendered_to = CALLOC(template->depth0 * template->array_size,
+                             sizeof(tex->rendered_to[0]));
+   if (!tex->rendered_to) {
+      goto fail;
+   }
+
    tex->dirty = CALLOC(template->depth0 * template->array_size,
                              sizeof(tex->dirty[0]));
    if (!tex->dirty) {
@@ -1046,22 +1070,14 @@ svga_texture_create(struct pipe_screen *screen,
       goto fail;
    }
 
-   bool use_typeless = false;
-   if (svgascreen->sws->have_gl43) {
-      /* Do not use typeless for SHARED, SCANOUT or DISPLAY_TARGET surfaces. */
-      use_typeless = !(bindings & (PIPE_BIND_SHARED | PIPE_BIND_SCANOUT |
-                                   PIPE_BIND_DISPLAY_TARGET));
-   } else if (svgascreen->sws->have_vgpu10) {
-      /* For VGPU10 device, use typeless formats only for sRGB and depth resources
-       * if they do not have SHARED, SCANOUT or DISPLAY_TARGET bind flags
-       */
-      use_typeless = (util_format_is_srgb(template->format) ||
-                      format_has_depth(template->format)) &&
-                     !(bindings & (PIPE_BIND_SHARED | PIPE_BIND_SCANOUT |
-                                   PIPE_BIND_DISPLAY_TARGET));
-   }
-
-   if (use_typeless) {
+   /* Use typeless formats for sRGB and depth resources.  Typeless
+    * formats can be reinterpreted as other formats.  For example,
+    * SVGA3D_R8G8B8A8_UNORM_TYPELESS can be interpreted as
+    * SVGA3D_R8G8B8A8_UNORM_SRGB or SVGA3D_R8G8B8A8_UNORM.
+    */
+   if (svgascreen->sws->have_vgpu10 &&
+       (util_format_is_srgb(template->format) ||
+        format_has_depth(template->format))) {
       SVGA3dSurfaceFormat typeless = svga_typeless_format(tex->key.format);
       if (0) {
          debug_printf("Convert resource type %s -> %s (bind 0x%x)\n",
@@ -1082,35 +1098,13 @@ svga_texture_create(struct pipe_screen *screen,
       tex->key.format = typeless;
    }
 
-   if (svgascreen->sws->have_sm5 &&
-       bindings & (PIPE_BIND_SAMPLER_VIEW | PIPE_BIND_RENDER_TARGET)) {
-       if (template->nr_samples < 2 &&
-           screen->is_format_supported(screen, template->format,
-                                       template->target,
-                                       template->nr_samples,
-                                       template->nr_storage_samples,
-                                       PIPE_BIND_SHADER_IMAGE)) {
-          /* Any non multi-samples texture that can be used as a render target
-           * or sampler view can be bound to an image unit.
-           * So make sure to set the UAV flag here.
-           */
-          tex->key.flags |= SVGA3D_SURFACE_BIND_UAVIEW;
-       }
-   }
-
    SVGA_DBG(DEBUG_DMA, "surface_create for texture\n");
-   bool invalidated;
    tex->handle = svga_screen_surface_create(svgascreen, bindings,
                                             tex->b.usage,
-                                            &invalidated, &tex->key);
+                                            &tex->validated, &tex->key);
    if (!tex->handle) {
       goto fail;
    }
-   if (invalidated) {
-      tex->surface_state = SVGA_SURFACE_STATE_INVALIDATED;
-   } else {
-      tex->surface_state = SVGA_SURFACE_STATE_CREATED;
-   }
 
    SVGA_DBG(DEBUG_DMA, "  --> got sid %p (texture)\n", tex->handle);
 
@@ -1136,6 +1130,8 @@ svga_texture_create(struct pipe_screen *screen,
 fail:
    if (tex->dirty)
       FREE(tex->dirty);
+   if (tex->rendered_to)
+      FREE(tex->rendered_to);
    if (tex->defined)
       FREE(tex->defined);
    FREE(tex);
@@ -1193,36 +1189,23 @@ svga_texture_from_handle(struct pipe_screen *screen,
    tex->key.format = format;
    tex->handle = srf;
 
-
-   /* set bind flags for the imported texture handle according to the bind
-    * flags in the template
-    */
-   if (template->bind & PIPE_BIND_RENDER_TARGET){
-      tex->key.flags |= SVGA3D_SURFACE_HINT_RENDERTARGET;
-      tex->key.flags |= SVGA3D_SURFACE_BIND_RENDER_TARGET;
-   }
-
-   if (template->bind & PIPE_BIND_DEPTH_STENCIL) {
-      tex->key.flags |= SVGA3D_SURFACE_HINT_DEPTHSTENCIL;
-      tex->key.flags |= SVGA3D_SURFACE_BIND_DEPTH_STENCIL;
-   }
-
-   if (template->bind & PIPE_BIND_SAMPLER_VIEW) {
-      tex->key.flags |= SVGA3D_SURFACE_HINT_TEXTURE;
-      tex->key.flags |= SVGA3D_SURFACE_BIND_SHADER_RESOURCE;
-   }
+   tex->rendered_to = CALLOC(1, sizeof(tex->rendered_to[0]));
+   if (!tex->rendered_to)
+      goto out_no_rendered_to;
 
    tex->dirty = CALLOC(1, sizeof(tex->dirty[0]));
    if (!tex->dirty)
       goto out_no_dirty;
 
-   tex->imported = true;
+   tex->imported = TRUE;
 
    ss->hud.num_resources++;
 
    return &tex->b;
 
 out_no_dirty:
+   FREE(tex->rendered_to);
+out_no_rendered_to:
    FREE(tex->defined);
 out_no_defined:
    FREE(tex);
@@ -1247,6 +1230,10 @@ svga_texture_generate_mipmap(struct pipe_context *pipe,
 
    assert(svga_have_vgpu10(svga));
 
+   /* Only support 2D texture for now */
+   if (pt->target != PIPE_TEXTURE_2D)
+      return false;
+
    /* Fallback to the mipmap generation utility for those formats that
     * do not support hw generate mipmap
     */
@@ -1260,21 +1247,11 @@ svga_texture_generate_mipmap(struct pipe_context *pipe,
       return false;
 
    templ.format = format;
-   templ.target = pt->target;
    templ.u.tex.first_layer = first_layer;
    templ.u.tex.last_layer = last_layer;
    templ.u.tex.first_level = base_level;
    templ.u.tex.last_level = last_level;
 
-   if (pt->target == PIPE_TEXTURE_CUBE) {
-      /**
-       * state tracker generates mipmap one face at a time.
-       * But SVGA generates mipmap for the entire cubemap.
-       */
-      templ.u.tex.first_layer = 0;
-      templ.u.tex.last_layer = 5;
-   }
-
    psv = pipe->create_sampler_view(pipe, pt, &templ);
    if (psv == NULL)
       return false;
@@ -1285,9 +1262,6 @@ svga_texture_generate_mipmap(struct pipe_context *pipe,
    SVGA_RETRY(svga, SVGA3D_vgpu10_GenMips(svga->swc, sv->id, tex->handle));
    pipe_sampler_view_reference(&psv, NULL);
 
-   /* Mark the texture surface as RENDERED */
-   svga_set_texture_rendered_to(tex);
-
    svga->hud.num_generate_mipmap++;
 
    return true;
@@ -1300,7 +1274,7 @@ svga_texture_generate_mipmap(struct pipe_context *pipe,
 /**
  * Create a texture upload buffer
  */
-bool
+boolean
 svga_texture_transfer_map_upload_create(struct svga_context *svga)
 {
    svga->tex_upload = u_upload_create(&svga->pipe, TEX_UPLOAD_DEFAULT_SIZE,
@@ -1325,74 +1299,29 @@ svga_texture_transfer_map_upload_destroy(struct svga_context *svga)
 /**
  * Returns true if this transfer map request can use the upload buffer.
  */
-bool
+boolean
 svga_texture_transfer_map_can_upload(const struct svga_screen *svgascreen,
                                      const struct pipe_resource *texture)
 {
-   if (svgascreen->sws->have_transfer_from_buffer_cmd == false)
-      return false;
+   if (svgascreen->sws->have_transfer_from_buffer_cmd == FALSE)
+      return FALSE;
 
    /* TransferFromBuffer command is not well supported with multi-samples surface */
    if (texture->nr_samples > 1)
-      return false;
+      return FALSE;
 
    if (util_format_is_compressed(texture->format)) {
       /* XXX Need to take a closer look to see why texture upload
        * with 3D texture with compressed format fails
        */ 
       if (texture->target == PIPE_TEXTURE_3D)
-          return false;
+          return FALSE;
    }
    else if (texture->format == PIPE_FORMAT_R9G9B9E5_FLOAT) {
-      return false;
+      return FALSE;
    }
 
-   return true;
-}
-
-
-/**
- *  Return TRUE if the same texture is bound to the specified
- *  surface view and a backing resource is created for the surface view.
- */
-static bool
-need_update_texture_resource(struct pipe_surface *surf,
-		             struct svga_texture *tex)
-{
-   struct svga_texture *stex = svga_texture(surf->texture);
-   struct svga_surface *s = svga_surface(surf);
-
-   return (stex == tex && s->handle != tex->handle);
-}
-
-
-/**
- *  Make sure the texture resource is up-to-date. If the texture is
- *  currently bound to a render target view and a backing resource is
- *  created, we will need to update the original resource with the
- *  changes in the backing resource.
- */
-static void
-svga_validate_texture_resource(struct svga_context *svga,
-		               struct svga_texture *tex)
-{
-   if (svga_was_texture_rendered_to(tex) == false)
-      return;
-
-   if ((svga->state.hw_draw.has_backed_views == false) ||
-       (tex->backed_handle == NULL))
-      return;
-
-   struct pipe_surface *s;
-   for (unsigned i = 0; i < svga->state.hw_clear.num_rendertargets; i++) {
-      s = svga->state.hw_clear.rtv[i];
-      if (s && need_update_texture_resource(s, tex))
-         svga_propagate_surface(svga, s, true);
-   }
-
-   s = svga->state.hw_clear.dsv;
-   if (s && need_update_texture_resource(s, tex))
-      svga_propagate_surface(svga, s, true);
+   return TRUE;
 }
 
 
@@ -1405,7 +1334,6 @@ svga_texture_transfer_map_upload(struct svga_context *svga,
 {
    struct pipe_resource *texture = st->base.resource;
    struct pipe_resource *tex_buffer = NULL;
-   struct svga_texture *tex = svga_texture(texture);
    void *tex_map;
    unsigned nblocksx, nblocksy;
    unsigned offset;
@@ -1413,14 +1341,6 @@ svga_texture_transfer_map_upload(struct svga_context *svga,
 
    assert(svga->tex_upload);
 
-   /* Validate the texture resource in case there is any changes
-    * in the backing resource that needs to be updated to the original
-    * texture resource first before the transfer upload occurs, otherwise,
-    * the later update from backing resource to original will overwrite the
-    * changes in this transfer map update.
-    */
-   svga_validate_texture_resource(svga, tex);
-
    st->upload.box.x = st->base.box.x;
    st->upload.box.y = st->base.box.y;
    st->upload.box.z = st->base.box.z;
@@ -1466,6 +1386,7 @@ svga_texture_transfer_map_upload(struct svga_context *svga,
 
 #ifdef DEBUG
    if (util_format_is_compressed(texture->format)) {
+      struct svga_texture *tex = svga_texture(texture);
       unsigned blockw, blockh, bytesPerBlock;
 
       svga_format_size(tex->key.format, &blockw, &blockh, &bytesPerBlock);
@@ -1536,10 +1457,10 @@ svga_texture_transfer_unmap_upload(struct svga_context *svga,
                                                         dstsurf, subResource,
                                                         &st->upload.box));
       offset += st->base.layer_stride;
-   }
 
-   /* Mark the texture surface state as RENDERED */
-   svga_set_texture_rendered_to(tex);
+      /* Set rendered-to flag */
+      svga_set_texture_rendered_to(tex, layer, st->base.level);
+   }
 
    pipe_resource_reference(&st->upload.buf, NULL);
 }
@@ -1557,18 +1478,14 @@ svga_texture_transfer_unmap_upload(struct svga_context *svga,
  * or not. However, for textures backed by imported svga surfaces that is
  * not always true, and we have to look at the SVGA3D utilities.
  */
-bool
+boolean
 svga_texture_device_format_has_alpha(struct pipe_resource *texture)
 {
    /* the svga_texture() call below is invalid for PIPE_BUFFER resources */
    assert(texture->target != PIPE_BUFFER);
 
-   const struct svga3d_surface_desc *surf_desc =
-      svga3dsurface_get_desc(svga_texture(texture)->key.format);
+   enum svga3d_block_desc block_desc =
+      svga3dsurface_get_desc(svga_texture(texture)->key.format)->block_desc;
 
-   enum svga3d_block_desc block_desc = surf_desc->block_desc;
-
-   return !!((block_desc & SVGA3DBLOCKDESC_ALPHA) ||
-             ((block_desc == SVGA3DBLOCKDESC_TYPELESS) &&
-              (surf_desc->bitDepth.alpha > 0)));
+   return !!(block_desc & SVGA3DBLOCKDESC_ALPHA);
 }
