@@ -28,9 +28,9 @@
 #include "pipe/p_state.h"
 #include "pipe/p_defines.h"
 #include "util/u_inlines.h"
-#include "os/os_thread.h"
+#include "util/u_thread.h"
 #include "util/u_bitmask.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "util/u_math.h"
 #include "util/u_memory.h"
 
@@ -44,6 +44,36 @@
 
 static void svga_mark_surface_dirty(struct pipe_surface *surf);
 
+void
+svga_texture_copy_region(struct svga_context *svga,
+                         struct svga_winsys_surface *src_handle,
+                         unsigned srcSubResource,
+                         unsigned src_x, unsigned src_y, unsigned src_z,
+                         struct svga_winsys_surface *dst_handle,
+                         unsigned dstSubResource,
+                         unsigned dst_x, unsigned dst_y, unsigned dst_z,
+                         unsigned width, unsigned height, unsigned depth)
+{
+   SVGA3dCopyBox box;
+
+   assert(svga_have_vgpu10(svga));
+
+   box.x = dst_x;
+   box.y = dst_y;
+   box.z = dst_z;
+   box.w = width;
+   box.h = height;
+   box.d = depth;
+   box.srcx = src_x;
+   box.srcy = src_y;
+   box.srcz = src_z;
+
+   SVGA_RETRY(svga, SVGA3D_vgpu10_PredCopyRegion
+              (svga->swc, dst_handle, dstSubResource,
+               src_handle, srcSubResource, &box));
+}
+
+
 void
 svga_texture_copy_handle(struct svga_context *svga,
                          struct svga_winsys_surface *src_handle,
@@ -55,7 +85,6 @@ svga_texture_copy_handle(struct svga_context *svga,
                          unsigned width, unsigned height, unsigned depth)
 {
    struct svga_surface dst, src;
-   enum pipe_error ret;
    SVGA3dCopyBox box, *boxes;
 
    assert(svga);
@@ -86,18 +115,11 @@ svga_texture_copy_handle(struct svga_context *svga,
             dst_handle, dst_level, dst_x, dst_y, dst_z);
 */
 
-   ret = SVGA3D_BeginSurfaceCopy(svga->swc,
-                                 &src.base,
-                                 &dst.base,
-                                 &boxes, 1);
-   if (ret != PIPE_OK) {
-      svga_context_flush(svga, NULL);
-      ret = SVGA3D_BeginSurfaceCopy(svga->swc,
-                                    &src.base,
-                                    &dst.base,
-                                    &boxes, 1);
-      assert(ret == PIPE_OK);
-   }
+   SVGA_RETRY(svga, SVGA3D_BeginSurfaceCopy(svga->swc,
+                                            &src.base,
+                                            &dst.base,
+                                            &boxes, 1));
+
    *boxes = box;
    SVGA_FIFOCommitAll(svga->swc);
 }
@@ -130,17 +152,26 @@ svga_texture_copy_handle_resource(struct svga_context *svga,
       for (j = 0; j < numLayers; j++) {
          if (svga_is_texture_level_defined(src_tex, j+layeroffset, miplevel)) {
             unsigned depth = (zslice_pick < 0 ?
-                              u_minify(src_tex->b.b.depth0, miplevel) : 1);
+                              u_minify(src_tex->b.depth0, miplevel) : 1);
 
-            svga_texture_copy_handle(svga,
-                                     src_tex->handle,
-                                     0, 0, zoffset,
-                                     miplevel,
-                                     j + layeroffset,
-                                     dst, 0, 0, 0, i, j,
-                                     u_minify(src_tex->b.b.width0, miplevel),
-                                     u_minify(src_tex->b.b.height0, miplevel),
-                                     depth);
+            if (src_tex->b.nr_samples > 1) {
+               unsigned subResource = j * numMipLevels + i;
+               svga_texture_copy_region(svga, src_tex->handle,
+                                        subResource, 0, 0, zoffset,
+                                        dst, subResource, 0, 0, 0,
+                                        src_tex->b.width0, src_tex->b.height0, depth);
+            }
+            else {
+               svga_texture_copy_handle(svga,
+                                        src_tex->handle,
+                                        0, 0, zoffset,
+                                        miplevel,
+                                        j + layeroffset,
+                                        dst, 0, 0, 0, i, j,
+                                        u_minify(src_tex->b.width0, miplevel),
+                                        u_minify(src_tex->b.height0, miplevel),
+                                        depth);
+            }
          }
       }
    }
@@ -151,20 +182,20 @@ struct svga_winsys_surface *
 svga_texture_view_surface(struct svga_context *svga,
                           struct svga_texture *tex,
                           unsigned bind_flags,
-                          SVGA3dSurfaceFlags flags,
+                          SVGA3dSurfaceAllFlags flags,
                           SVGA3dSurfaceFormat format,
                           unsigned start_mip,
                           unsigned num_mip,
                           int layer_pick,
                           unsigned num_layers,
                           int zslice_pick,
-                          boolean cacheable,
+                          bool cacheable,
                           struct svga_host_surface_cache_key *key) /* OUT */
 {
    struct svga_screen *ss = svga_screen(svga->pipe.screen);
    struct svga_winsys_surface *handle = NULL;
-   boolean validated;
-   boolean needCopyResource;
+   bool invalidated;
+   bool needCopyResource;
 
    SVGA_DBG(DEBUG_PERF,
             "svga: Create surface view: layer %d zslice %d mips %d..%d\n",
@@ -175,25 +206,30 @@ svga_texture_view_surface(struct svga_context *svga,
    key->flags = flags;
    key->format = format;
    key->numMipLevels = num_mip;
-   key->size.width = u_minify(tex->b.b.width0, start_mip);
-   key->size.height = u_minify(tex->b.b.height0, start_mip);
-   key->size.depth = zslice_pick < 0 ? u_minify(tex->b.b.depth0, start_mip) : 1;
+   key->size.width = u_minify(tex->b.width0, start_mip);
+   key->size.height = u_minify(tex->b.height0, start_mip);
+   key->size.depth = zslice_pick < 0 ? u_minify(tex->b.depth0, start_mip) : 1;
+#ifndef VBOX_WITH_MESA3D_NINE_SVGA
    key->cachable = 1;
+#else
+   key->cachable = 0;
+#endif
    key->arraySize = 1;
    key->numFaces = 1;
 
    /* single sample surface can be treated as non-multisamples surface */
-   key->sampleCount = tex->b.b.nr_samples > 1 ? tex->b.b.nr_samples : 0;
+   key->sampleCount = tex->b.nr_samples > 1 ? tex->b.nr_samples : 0;
 
    if (key->sampleCount > 1) {
-      key->flags |= SVGA3D_SURFACE_MASKABLE_ANTIALIAS;
+      assert(ss->sws->have_sm4_1);
+      key->flags |= SVGA3D_SURFACE_MULTISAMPLE;
    }
 
-   if (tex->b.b.target == PIPE_TEXTURE_CUBE && layer_pick < 0) {
+   if (tex->b.target == PIPE_TEXTURE_CUBE && layer_pick < 0) {
       key->flags |= SVGA3D_SURFACE_CUBEMAP;
       key->numFaces = 6;
-   } else if (tex->b.b.target == PIPE_TEXTURE_1D_ARRAY ||
-              tex->b.b.target == PIPE_TEXTURE_2D_ARRAY) {
+   } else if (tex->b.target == PIPE_TEXTURE_1D_ARRAY ||
+              tex->b.target == PIPE_TEXTURE_2D_ARRAY) {
       key->arraySize = num_layers;
    }
 
@@ -209,8 +245,8 @@ svga_texture_view_surface(struct svga_context *svga,
    } else {
       SVGA_DBG(DEBUG_DMA, "surface_create for texture view\n");
       handle = svga_screen_surface_create(ss, bind_flags, PIPE_USAGE_DEFAULT,
-                                          &validated, key);
-      needCopyResource = TRUE;
+                                          &invalidated, key);
+      needCopyResource = true;
 
       if (cacheable && !tex->backed_handle) {
          tex->backed_handle = handle;
@@ -245,14 +281,14 @@ done:
 
 /**
  * A helper function to create a surface view.
- * The view boolean flag specifies whether svga_texture_view_surface()
- * will be called to create a cloned surface and resource for the view.
+ * The clone_resource boolean flag specifies whether to clone the resource
+ * for the surface view.
  */
 static struct pipe_surface *
 svga_create_surface_view(struct pipe_context *pipe,
                          struct pipe_resource *pt,
                          const struct pipe_surface *surf_tmpl,
-                         boolean view)
+                         bool clone_resource)
 {
    struct svga_context *svga = svga_context(pipe);
    struct svga_texture *tex = svga_texture(pt);
@@ -261,7 +297,7 @@ svga_create_surface_view(struct pipe_context *pipe,
    struct svga_surface *s;
    unsigned layer, zslice, bind;
    unsigned nlayers = 1;
-   SVGA3dSurfaceFlags flags = 0;
+   SVGA3dSurfaceAllFlags flags = 0;
    SVGA3dSurfaceFormat format;
    struct pipe_surface *retVal = NULL;
 
@@ -276,7 +312,8 @@ svga_create_surface_view(struct pipe_context *pipe,
       zslice = 0;
    }
    else if (pt->target == PIPE_TEXTURE_1D_ARRAY ||
-            pt->target == PIPE_TEXTURE_2D_ARRAY) {
+            pt->target == PIPE_TEXTURE_2D_ARRAY ||
+            pt->target == PIPE_TEXTURE_CUBE_ARRAY) {
       layer = surf_tmpl->u.tex.first_layer;
       zslice = 0;
       nlayers = surf_tmpl->u.tex.last_layer - surf_tmpl->u.tex.first_layer + 1;
@@ -324,7 +361,7 @@ svga_create_surface_view(struct pipe_context *pipe,
 
    assert(format != SVGA3D_FORMAT_INVALID);
 
-   if (view) {
+   if (clone_resource) {
       SVGA_DBG(DEBUG_VIEWS,
                "New backed surface view: resource %p, level %u layer %u z %u, %p\n",
                pt, surf_tmpl->u.tex.level, layer, zslice, s);
@@ -347,6 +384,10 @@ svga_create_surface_view(struct pipe_context *pipe,
             if (nlayers == 6)
                flags |= SVGA3D_SURFACE_CUBEMAP;
             break;
+         case PIPE_TEXTURE_CUBE_ARRAY:
+            if (nlayers % 6 == 0)
+               flags |= SVGA3D_SURFACE_CUBEMAP | SVGA3D_SURFACE_ARRAY;
+            break;   
          default:
             break;
          }
@@ -359,7 +400,7 @@ svga_create_surface_view(struct pipe_context *pipe,
                                             tex->key.format,
                                             surf_tmpl->u.tex.level, 1,
                                             layer, nlayers, zslice,
-                                            TRUE, &s->key);
+                                            true, &s->key);
       if (!s->handle) {
          FREE(s);
          goto done;
@@ -399,22 +440,22 @@ svga_create_surface(struct pipe_context *pipe,
    struct svga_context *svga = svga_context(pipe);
    struct pipe_screen *screen = pipe->screen;
    struct pipe_surface *surf = NULL;
-   boolean view = FALSE;
+   bool view = false;
 
    SVGA_STATS_TIME_PUSH(svga_sws(svga), SVGA_STATS_TIME_CREATESURFACE);
 
    if (svga_screen(screen)->debug.force_surface_view)
-      view = TRUE;
+      view = true;
 
    if (surf_tmpl->u.tex.level != 0 &&
        svga_screen(screen)->debug.force_level_surface_view)
-      view = TRUE;
+      view = true;
 
    if (pt->target == PIPE_TEXTURE_3D)
-      view = TRUE;
+      view = true;
 
    if (svga_have_vgpu10(svga) || svga_screen(screen)->debug.no_surface_view)
-      view = FALSE;
+      view = false;
 
    surf = svga_create_surface_view(pipe, pt, surf_tmpl, view);
 
@@ -425,10 +466,11 @@ svga_create_surface(struct pipe_context *pipe,
 
 
 /**
- * Clone the surface view and its associated resource.
+ * Create an alternate surface view and clone the resource if specified
  */
 static struct svga_surface *
-create_backed_surface_view(struct svga_context *svga, struct svga_surface *s)
+create_backed_surface_view(struct svga_context *svga, struct svga_surface *s,
+                           bool clone_resource)
 {
    struct svga_texture *tex = svga_texture(s->base.texture);
 
@@ -439,9 +481,9 @@ create_backed_surface_view(struct svga_context *svga, struct svga_surface *s)
                            SVGA_STATS_TIME_CREATEBACKEDSURFACEVIEW);
 
       backed_view = svga_create_surface_view(&svga->pipe,
-                                             &tex->b.b,
+                                             &tex->b,
                                              &s->base,
-                                             TRUE);
+                                             clone_resource);
       if (!backed_view)
          goto done;
 
@@ -449,7 +491,8 @@ create_backed_surface_view(struct svga_context *svga, struct svga_surface *s)
 
       SVGA_STATS_TIME_POP(svga_sws(svga));
    }
-   else if (s->backed->age < tex->age) {
+   else if (s->backed->handle != tex->handle &&
+            s->backed->age < tex->age) {
       /*
        * There is already an existing backing surface, but we still need to
        * sync the backing resource if the original resource has been modified
@@ -460,8 +503,9 @@ create_backed_surface_view(struct svga_context *svga, struct svga_surface *s)
 
       assert(bs->handle);
 
-      switch (tex->b.b.target) {
+      switch (tex->b.target) {
       case PIPE_TEXTURE_CUBE:
+      case PIPE_TEXTURE_CUBE_ARRAY:
       case PIPE_TEXTURE_1D_ARRAY:
       case PIPE_TEXTURE_2D_ARRAY:
          layer = s->base.u.tex.first_layer;
@@ -481,6 +525,8 @@ create_backed_surface_view(struct svga_context *svga, struct svga_surface *s)
    svga_mark_surface_dirty(&s->backed->base);
    s->backed->age = tex->age;
 
+   assert(s->backed->base.context == &svga->pipe);
+
 done:
    return s->backed;
 }
@@ -503,33 +549,44 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
 
    /**
     * DX spec explicitly specifies that no resource can be bound to a render
-    * target view and a shader resource view simultanously.
+    * target view and a shader resource view simultaneously.
     * So first check if the resource bound to this surface view collides with
     * a sampler view. If so, then we will clone this surface view and its
     * associated resource. We will then use the cloned surface view for
     * render target.
     */
-   for (shader = PIPE_SHADER_VERTEX; shader <= PIPE_SHADER_GEOMETRY; shader++) {
+   for (shader = PIPE_SHADER_VERTEX; shader <= PIPE_SHADER_COMPUTE; shader++) {
       if (svga_check_sampler_view_resource_collision(svga, s->handle, shader)) {
          SVGA_DBG(DEBUG_VIEWS,
                   "same resource used in shaderResource and renderTarget 0x%x\n",
                   s->handle);
-         s = create_backed_surface_view(svga, s);
+         s = create_backed_surface_view(svga, s, true);
 
          if (s)
-            svga->state.hw_draw.has_backed_views = TRUE;
+            svga->state.hw_draw.has_backed_views = true;
 
          /* s may be null here if the function failed */
          break;
       }
    }
 
+   /**
+    * Create an alternate surface view for the specified context if the
+    * view was created for another context.
+    */
+   if (s && s->base.context != &svga->pipe) {
+      s = create_backed_surface_view(svga, s, false);
+
+      if (s)
+         svga->state.hw_draw.has_backed_views = true;
+   }
+
    if (s && s->view_id == SVGA3D_INVALID_ID) {
       SVGA3dResourceType resType;
       SVGA3dRenderTargetViewDesc desc;
       struct svga_texture *stex = svga_texture(s->base.texture);
 
-      if (stex->validated == FALSE) {
+      if (stex->surface_state < SVGA_SURFACE_STATE_INVALIDATED) {
          assert(stex->handle);
 
          /* We are about to render into a surface that has not been validated.
@@ -537,12 +594,8 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
           * need to update the host-side copy with the invalid
           * content when the associated mob is first bound to the surface.
           */
-         if (svga->swc->surface_invalidate(svga->swc, stex->handle) != PIPE_OK) {
-            svga_context_flush(svga, NULL);
-            ret = svga->swc->surface_invalidate(svga->swc, stex->handle);
-            assert(ret == PIPE_OK);
-         }
-         stex->validated = TRUE;
+         SVGA_RETRY(svga, SVGA3D_InvalidateGBSurface(svga->swc, stex->handle));
+         stex->surface_state = SVGA_SURFACE_STATE_INVALIDATED;
       }
 
       desc.tex.mipSlice = s->real_level;
@@ -550,38 +603,48 @@ svga_validate_surface_view(struct svga_context *svga, struct svga_surface *s)
       desc.tex.arraySize =
          s->base.u.tex.last_layer - s->base.u.tex.first_layer + 1;
 
-      s->view_id = util_bitmask_add(svga->surface_view_id_bm);
-
       resType = svga_resource_type(s->base.texture->target);
 
       if (util_format_is_depth_or_stencil(s->base.format)) {
-         ret = SVGA3D_vgpu10_DefineDepthStencilView(svga->swc,
-                                                    s->view_id,
-                                                    s->handle,
-                                                    s->key.format,
-                                                    resType,
-                                                    &desc);
+
+         /* Create depth stencil view only if the resource is created
+          * with depth stencil bind flag.
+          */
+         if (stex->key.flags & SVGA3D_SURFACE_BIND_DEPTH_STENCIL) {
+            s->view_id = util_bitmask_add(svga->surface_view_id_bm);
+            ret = SVGA3D_vgpu10_DefineDepthStencilView(svga->swc,
+                                                       s->view_id,
+                                                       s->handle,
+                                                       s->key.format,
+                                                       resType,
+                                                       &desc);
+         }
       }
       else {
-         SVGA3dSurfaceFormat view_format = s->key.format;
-         const struct svga_texture *stex = svga_texture(s->base.texture);
-
-         /* Can't create RGBA render target view of a RGBX surface so adjust
-          * the view format.  We do something similar for texture samplers in
-          * svga_validate_pipe_sampler_view().
+         /* Create render target view only if the resource is created
+          * with render target bind flag.
           */
-         if (view_format == SVGA3D_B8G8R8A8_UNORM &&
-             (stex->key.format == SVGA3D_B8G8R8X8_UNORM ||
-              stex->key.format == SVGA3D_B8G8R8X8_TYPELESS)) {
-            view_format = SVGA3D_B8G8R8X8_UNORM;
-         }
+         if (stex->key.flags & SVGA3D_SURFACE_BIND_RENDER_TARGET) {
+            SVGA3dSurfaceFormat view_format = s->key.format;
 
-         ret = SVGA3D_vgpu10_DefineRenderTargetView(svga->swc,
-                                                    s->view_id,
-                                                    s->handle,
-                                                    view_format,
-                                                    resType,
-                                                    &desc);
+            /* Can't create RGBA render target view of a RGBX surface so adjust
+             * the view format.  We do something similar for texture samplers in
+             * svga_validate_pipe_sampler_view().
+             */
+            if (view_format == SVGA3D_B8G8R8A8_UNORM &&
+                (stex->key.format == SVGA3D_B8G8R8X8_UNORM ||
+                 stex->key.format == SVGA3D_B8G8R8X8_TYPELESS)) {
+               view_format = SVGA3D_B8G8R8X8_UNORM;
+            }
+
+            s->view_id = util_bitmask_add(svga->surface_view_id_bm);
+            ret = SVGA3D_vgpu10_DefineRenderTargetView(svga->swc,
+                                                       s->view_id,
+                                                       s->handle,
+                                                       view_format,
+                                                       resType,
+                                                       &desc);
+         }
       }
 
       if (ret != PIPE_OK) {
@@ -606,7 +669,6 @@ svga_surface_destroy(struct pipe_context *pipe,
    struct svga_surface *s = svga_surface(surf);
    struct svga_texture *t = svga_texture(surf->texture);
    struct svga_screen *ss = svga_screen(surf->texture->screen);
-   enum pipe_error ret = PIPE_OK;
 
    SVGA_STATS_TIME_PUSH(ss->sws, SVGA_STATS_TIME_DESTROYSURFACE);
 
@@ -621,12 +683,12 @@ svga_surface_destroy(struct pipe_context *pipe,
     */
    if (s->handle != t->handle && s->handle != t->backed_handle) {
       SVGA_DBG(DEBUG_DMA, "unref sid %p (tex surface)\n", s->handle);
-      svga_screen_surface_destroy(ss, &s->key, &s->handle);
+      svga_screen_surface_destroy(ss, &s->key,
+                                  svga_was_texture_rendered_to(t),
+                                  &s->handle);
    }
 
    if (s->view_id != SVGA3D_INVALID_ID) {
-      unsigned try;
-
       /* The SVGA3D device will generate a device error if the
        * render target view or depth stencil view is destroyed from
        * a context other than the one it was created with.
@@ -638,18 +700,14 @@ svga_surface_destroy(struct pipe_context *pipe,
       }
       else {
          assert(svga_have_vgpu10(svga));
-         for (try = 0; try < 2; try++) {
-            if (util_format_is_depth_or_stencil(s->base.format)) {
-               ret = SVGA3D_vgpu10_DestroyDepthStencilView(svga->swc, s->view_id);
-            }
-            else {
-               ret = SVGA3D_vgpu10_DestroyRenderTargetView(svga->swc, s->view_id);
-            }
-            if (ret == PIPE_OK)
-               break;
-            svga_context_flush(svga, NULL);
+         if (util_format_is_depth_or_stencil(s->base.format)) {
+            SVGA_RETRY(svga, SVGA3D_vgpu10_DestroyDepthStencilView(svga->swc,
+                                                                   s->view_id));
+         }
+         else {
+            SVGA_RETRY(svga, SVGA3D_vgpu10_DestroyRenderTargetView(svga->swc,
+                                                                   s->view_id));
          }
-         assert(ret == PIPE_OK);
          util_bitmask_clear(svga->surface_view_id_bm, s->view_id);
       }
    }
@@ -669,7 +727,7 @@ svga_mark_surface_dirty(struct pipe_surface *surf)
    struct svga_texture *tex = svga_texture(surf->texture);
 
    if (!s->dirty) {
-      s->dirty = TRUE;
+      s->dirty = true;
 
       if (s->handle == tex->handle) {
          /* hmm so 3d textures always have all their slices marked ? */
@@ -725,7 +783,7 @@ svga_mark_surfaces_dirty(struct svga_context *svga)
  */
 void
 svga_propagate_surface(struct svga_context *svga, struct pipe_surface *surf,
-                       boolean reset)
+                       bool reset)
 {
    struct svga_surface *s = svga_surface(surf);
    struct svga_texture *tex = svga_texture(surf->texture);
@@ -751,13 +809,19 @@ svga_propagate_surface(struct svga_context *svga, struct pipe_surface *surf,
       unsigned zslice, layer;
       unsigned nlayers = 1;
       unsigned i;
+      unsigned numMipLevels = tex->b.last_level + 1;
+      unsigned srcLevel = s->real_level;
+      unsigned dstLevel = surf->u.tex.level;
+      unsigned width = u_minify(tex->b.width0, dstLevel);
+      unsigned height = u_minify(tex->b.height0, dstLevel);
 
       if (surf->texture->target == PIPE_TEXTURE_CUBE) {
          zslice = 0;
          layer = surf->u.tex.first_layer;
       }
       else if (surf->texture->target == PIPE_TEXTURE_1D_ARRAY ||
-               surf->texture->target == PIPE_TEXTURE_2D_ARRAY) {
+               surf->texture->target == PIPE_TEXTURE_2D_ARRAY ||
+               surf->texture->target == PIPE_TEXTURE_CUBE_ARRAY) {
          zslice = 0;
          layer = surf->u.tex.first_layer;
          nlayers = surf->u.tex.last_layer - surf->u.tex.first_layer + 1;
@@ -770,16 +834,32 @@ svga_propagate_surface(struct svga_context *svga, struct pipe_surface *surf,
       SVGA_DBG(DEBUG_VIEWS,
                "Propagate surface %p to resource %p, level %u\n",
                surf, tex, surf->u.tex.level);
-      for (i = 0; i < nlayers; i++) {
-         svga_texture_copy_handle(svga,
-                                  s->handle, 0, 0, 0, s->real_level,
-                                  s->real_layer + i,
-                                  tex->handle, 0, 0, zslice, surf->u.tex.level,
-                                  layer + i,
-                                  u_minify(tex->b.b.width0, surf->u.tex.level),
-                                  u_minify(tex->b.b.height0, surf->u.tex.level),
-                                  1);
-         svga_define_texture_level(tex, layer + i, surf->u.tex.level);
+
+      if (svga_have_vgpu10(svga)) {
+         unsigned srcSubResource, dstSubResource;
+
+         for (i = 0; i < nlayers; i++) {
+            srcSubResource = (s->real_layer + i) * numMipLevels + srcLevel;
+            dstSubResource = (layer + i) * numMipLevels + dstLevel;
+
+            svga_texture_copy_region(svga,
+                                     s->handle, srcSubResource, 0, 0, 0,
+                                     tex->handle, dstSubResource, 0, 0, zslice,
+                                     width, height, 1);
+            svga_define_texture_level(tex, layer + i, dstLevel);
+         }
+      }
+      else {
+         for (i = 0; i < nlayers; i++) {
+            svga_texture_copy_handle(svga,
+                                     s->handle, 0, 0, 0, srcLevel,
+                                     s->real_layer + i,
+                                     tex->handle, 0, 0, zslice, dstLevel,
+                                     layer + i,
+                                     width, height, 1);
+        
+            svga_define_texture_level(tex, layer + i, dstLevel);
+         }
       }
 
       /* Sync the surface view age with the texture age */
@@ -817,12 +897,12 @@ svga_propagate_rendertargets(struct svga_context *svga)
    for (i = 0; i < svga->state.hw_clear.num_rendertargets; i++) {
       struct pipe_surface *s = svga->state.hw_clear.rtv[i];
       if (s) {
-         svga_propagate_surface(svga, s, FALSE);
+         svga_propagate_surface(svga, s, false);
       }
    }
 
    if (svga->state.hw_clear.dsv) {
-      svga_propagate_surface(svga, svga->state.hw_clear.dsv, FALSE);
+      svga_propagate_surface(svga, svga->state.hw_clear.dsv, false);
    }
 }
 
@@ -830,7 +910,7 @@ svga_propagate_rendertargets(struct svga_context *svga)
 /**
  * Check if we should call svga_propagate_surface on the surface.
  */
-boolean
+bool
 svga_surface_needs_propagation(const struct pipe_surface *surf)
 {
    const struct svga_surface *s = svga_surface_const(surf);
@@ -851,6 +931,10 @@ svga_get_sample_position(struct pipe_context *context,
    static const float pos1[1][2] = {
       { 0.5, 0.5 }
    };
+   static const float pos2[2][2] = {
+      { 0.75, 0.75 },
+      { 0.25, 0.25 }
+   };
    static const float pos4[4][2] = {
       { 0.375000, 0.125000 },
       { 0.875000, 0.375000 },
@@ -888,6 +972,9 @@ svga_get_sample_position(struct pipe_context *context,
    const float (*positions)[2];
 
    switch (sample_count) {
+   case 2:
+      positions = pos2;
+      break;
    case 4:
       positions = pos4;
       break;
