@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2011-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2011-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -215,6 +215,9 @@ typedef struct VBOXDX_RESOURCE
     RTLISTANCHOR                   listRTV;                 /* Render target views. */
     RTLISTANCHOR                   listDSV;                 /* Depth stencil views. */
     RTLISTANCHOR                   listUAV;                 /* Unordered access views. */
+    RTLISTANCHOR                   listVDOV;                /* Video decoder output views. */
+    RTLISTANCHOR                   listVPIV;                /* Video processor input views. */
+    RTLISTANCHOR                   listVPOV;                /* Video processor output views. */
 
     RT_FLEXIBLE_ARRAY_EXTENSION
     D3D10DDI_MIPINFO               aMipInfoList[RT_FLEXIBLE_ARRAY]; /** @todo Remove this array if it will not be needed. */
@@ -324,6 +327,170 @@ typedef struct VBOXDXUNORDEREDACCESSVIEW
 } VBOXDXUNORDEREDACCESSVIEW, *PVBOXDXUNORDEREDACCESSVIEW;
 
 
+typedef struct VBOXDXVIDEOPROCESSORINPUTVIEW
+{
+    RTLISTNODE                     nodeView;                /* listVPIV of the resource. */
+    bool                           fDefined : 1;            /* Whether the view has been defined on the host. */
+    D3D11_1DDI_HRTVIDEOPROCESSORINPUTVIEW hRTView;
+    uint32_t                       uVideoProcessorInputViewId;
+    PVBOXDX_RESOURCE               pResource;
+
+    struct
+    {
+        VBSVGA3dVideoProcessorDesc ContentDesc;
+        VBSVGA3dVPIVDesc           VPIVDesc;
+    } svga;
+} VBOXDXVIDEOPROCESSORINPUTVIEW, *PVBOXDXVIDEOPROCESSORINPUTVIEW;
+
+
+typedef struct VBOXDXVIDEOPROCESSOROUTPUTVIEW
+{
+    RTLISTNODE                     nodeView;                /* listVPOV of the resource. */
+    bool                           fDefined : 1;            /* Whether the view has been defined on the host. */
+    D3D11_1DDI_HRTVIDEOPROCESSOROUTPUTVIEW hRTView;
+    uint32_t                       uVideoProcessorOutputViewId;
+    PVBOXDX_RESOURCE               pResource;
+
+    struct
+    {
+        VBSVGA3dVideoProcessorDesc ContentDesc;
+        VBSVGA3dVPOVDesc           VPOVDesc;
+    } svga;
+} VBOXDXVIDEOPROCESSOROUTPUTVIEW, *PVBOXDXVIDEOPROCESSOROUTPUTVIEW;
+
+
+typedef struct VBOXDXVIDEOPROCESSORFILTER
+{
+    BOOL Enable;
+    int Level;
+} VBOXDXVIDEOPROCESSORFILTER;
+
+
+typedef struct VBOXDXVIDEOPROCESSORSTREAM
+{
+    D3D11_1DDI_VIDEO_FRAME_FORMAT FrameFormat;
+    D3D11_1DDI_VIDEO_PROCESSOR_COLOR_SPACE ColorSpace;
+    struct {
+        D3D11_1DDI_VIDEO_PROCESSOR_OUTPUT_RATE OutputRate;
+        BOOL RepeatFrame;
+        DXGI_RATIONAL CustomRate;
+    } OutputRate;
+    struct {
+        BOOL Enable;
+        RECT SourceRect;
+    } SourceRect;
+    struct {
+        BOOL Enable;
+        RECT DestRect;
+    } DestRect;
+    struct {
+        BOOL Enable;
+        FLOAT Lower;
+        FLOAT Upper;
+    } LumaKey;
+    struct {
+        BOOL Enable;
+    } AutoProcessingMode;
+    struct {
+        BOOL Enable;
+        D3D11_1DDI_VIDEO_PROCESSOR_STEREO_FORMAT Format;
+    } StereoFormat;
+    struct {
+        BOOL Enable;
+        D3D11_1DDI_VIDEO_PROCESSOR_ROTATION Rotation;
+    } Rotation;
+    VBOXDXVIDEOPROCESSORFILTER aFilters[D3D11_1DDI_VIDEO_PROCESSOR_FILTER_STEREO_ADJUSTMENT + 1];
+} VBOXDXVIDEOPROCESSORSTREAM;
+
+
+typedef struct VBOXDXVIDEOPROCESSOR
+{
+    D3D11_1DDI_HRTVIDEOPROCESSOR   hRTVideoProcessor;
+    uint32_t                       uVideoProcessorId;
+    struct {
+        VBSVGA3dVideoProcessorDesc desc;
+    } svga;
+    struct {
+        BOOL Enable;
+        RECT Rect;
+    } OutputRect;
+    struct {
+        BOOL YCbCr;
+        D3D11_1DDI_VIDEO_COLOR Color;
+    } OutputBackgroundColor;
+    D3D11_1DDI_VIDEO_PROCESSOR_COLOR_SPACE Colorspace;
+    struct {
+        BOOL Enabled;
+        SIZE ConstrictonSize;
+    } OutputConstriction;
+
+    struct {
+        D3D11_1DDI_VIDEO_PROCESSOR_ALPHA_FILL_MODE FillMode;
+        UINT StreamIndex;
+    } AlphaFillMode;
+
+    struct {
+        BOOL Enable;
+    } StereoMode;
+
+    VBOXDXVIDEOPROCESSORSTREAM aStreams[VBSVGA3D_MAX_VIDEO_STREAMS];
+} VBOXDXVIDEOPROCESSOR, *PVBOXDXVIDEOPROCESSOR;
+
+
+typedef struct VBOXDXVIDEOPROCESSORENUM
+{
+    D3D11_1DDI_HRTVIDEOPROCESSORENUM hRTVideoProcessorEnum;
+
+    struct {
+        VBSVGA3dVideoProcessorDesc desc;
+    } svga;
+} VBOXDXVIDEOPROCESSORENUM, *PVBOXDXVIDEOPROCESSORENUM;
+
+
+typedef struct VBOXDXVIDEODECODEROUTPUTVIEW
+{
+    RTLISTNODE                     nodeView;                /* listVDOV of the resource. */
+    bool                           fDefined : 1;            /* Whether the view has been defined on the host. */
+    D3D11_1DDI_HRTVIDEODECODEROUTPUTVIEW hRTView;
+    uint32_t                       uVideoDecoderOutputViewId;
+    PVBOXDX_RESOURCE               pResource;
+
+    struct {
+        VBSVGA3dVDOVDesc           desc;
+    } svga;
+} VBOXDXVIDEODECODEROUTPUTVIEW, *PVBOXDXVIDEODECODEROUTPUTVIEW;
+
+
+typedef struct VBOXDXVIDEODECODER
+{
+    D3D11_1DDI_HRTDECODE           hRTVideoDecoder;
+    uint32_t                       uVideoDecoderId;
+
+    struct {
+        VBSVGA3dVideoDecoderDesc   Desc;
+        VBSVGA3dVideoDecoderConfig Config;
+    } svga;
+
+    struct {
+        PVBOXDXVIDEODECODEROUTPUTVIEW pOutputView;
+    } Frame;
+} VBOXDXVIDEODECODER, *PVBOXDXVIDEODECODER;
+
+
+typedef struct VBOXDXVIDEOCRYPTOSESSION
+{
+    D3D11_1DDI_HRTCRYPTOSESSION hRTCryptoSession;
+    uint32_t au32Dummy[3];
+} VBOXDXVIDEOCRYPTOSESSION, *PVBOXDXVIDEOCRYPTOSESSION;
+
+
+typedef struct VBOXDXVIDEOAUTHCHANNEL
+{
+    D3D11_1DDI_HRTAUTHCHANNEL hRTAuthChannel;
+    uint32_t au32Dummy[4];
+} VBOXDXVIDEOAUTHCHANNEL, *PVBOXDXVIDEOAUTHCHANNEL;
+
+
 typedef struct VBOXDXCONSTANTBUFFERSSTATE
 {
     UINT                           StartSlot;
@@ -351,6 +518,14 @@ typedef struct VBOXDXINDEXBUFFERSTATE
     UINT                           Offset;
 } VBOXDXINDEXBUFFERSTATE, *PVBOXDXINDEXBUFFERSTATE;
 
+
+typedef struct VBOXDXSRVSTATE
+{
+    uint32_t cShaderResourceView;
+    PVBOXDXSHADERRESOURCEVIEW apShaderResourceView[D3D11_COMMONSHADER_INPUT_RESOURCE_SLOT_COUNT];
+} VBOXDXSRVSTATE, *PVBOXDXSRVSTATE;
+
+
 typedef struct VBOXDX_DEVICE
 {
     /* DX runtime data. */
@@ -393,6 +568,11 @@ typedef struct VBOXDX_DEVICE
     RTHANDLETABLE hHTQuery;
     RTHANDLETABLE hHTUnorderedAccessView;
     RTHANDLETABLE hHTStreamOutput;
+    RTHANDLETABLE hHTVideoProcessor;
+    RTHANDLETABLE hHTVideoDecoderOutputView;
+    RTHANDLETABLE hHTVideoDecoder;
+    RTHANDLETABLE hHTVideoProcessorInputView;
+    RTHANDLETABLE hHTVideoProcessorOutputView;
 
     /* Resources */
     RTLISTANCHOR                listResources;              /* All resources of this device, for cleanup. */
@@ -409,7 +589,7 @@ typedef struct VBOXDX_DEVICE
     /* Queries */
     RTLISTANCHOR                listQueries;                /* All queries of this device, to be able to repack them in the allocation. */
     RTLISTANCHOR                listCOAQuery;               /* List of VBOXDXCOALLOCATION for all query types. */
-    uint64_t                    u64MobFenceValue;
+    uint64_t volatile           u64MobFenceValue;
 
     /* Stream output declarations */
     RTLISTANCHOR                listCOAStreamOutput;        /* List of VBOXDXCOALLOCATION for stream output declarations. */
@@ -427,9 +607,35 @@ typedef struct VBOXDX_DEVICE
         VBOXDXCONSTANTBUFFERSSTATE aConstantBuffers[SVGA3D_SHADERTYPE_MAX - SVGA3D_SHADERTYPE_MIN]; /* For each shader type. */
         VBOXDXVERTEXBUFFERSSTATE   VertexBuffers;
         VBOXDXINDEXBUFFERSTATE     IndexBuffer;
+        VBOXDXSRVSTATE             aSRVs[SVGA3D_SHADERTYPE_MAX - SVGA3D_SHADERTYPE_MIN]; /* For each shader type. */
     } pipeline;
+
+    /* Video decoding and processing. */
+    struct
+    {
+        D3DKMT_HANDLE           hAllocation;                /* For querying capabilities of the host device. */
+        uint32_t                cbAllocation;
+        uint64_t volatile       u64MobFenceValue;           /* Fence value for host queries. */
+
+        bool                    fDecodeProfilesQueried;
+        UINT                    cDecodeProfile;             /* Number of elements in paDecodeProfile array. */
+        VBSVGA3dDecodeProfileInfo *paDecodeProfile;
+
+        struct
+        {
+            uint32_t cConfig;                               /* Number of elements in pConfigInfo->aConfig array. */
+            VBSVGA3dDecodeConfigInfo *pConfigInfo;
+        } config;
+
+        struct
+        {
+            VBSVGA3dVideoProcessorDesc desc;                /* Info has been queried for this desc; */
+            VBSVGA3dVideoProcessorEnumInfo info;            /* Last queried info. */
+        } videoProcessorEnum;
+    } VideoDevice;
 } VBOXDX_DEVICE, *PVBOXDX_DEVICE;
 
+HRESULT ddi11_1RetrieveVideoFunctions(PVBOXDX_DEVICE pDevice, D3D11_1DDI_VIDEO_INPUT *pVideoInput);
 
 HRESULT vboxDXDeviceInit(PVBOXDX_DEVICE pDevice);
 void vboxDXDestroyDevice(PVBOXDX_DEVICE pDevice);
@@ -519,13 +725,15 @@ void vboxDXDestroyDepthStencilView(PVBOXDX_DEVICE pDevice, PVBOXDXDEPTHSTENCILVI
 void vboxDXSetRenderTargets(PVBOXDX_DEVICE pDevice, PVBOXDXDEPTHSTENCILVIEW pDepthStencilView,
                             uint32_t NumRTVs, UINT ClearSlots, PVBOXDXRENDERTARGETVIEW *papRenderTargetViews);
 void vboxDXSetShaderResourceViews(PVBOXDX_DEVICE pDevice, SVGA3dShaderType enmShaderType, uint32_t StartSlot,
-                                  uint32_t NumViews, uint32_t *paViewIds);
+                                  uint32_t NumViews, PVBOXDXSHADERRESOURCEVIEW const *papViews);
 void vboxDXSetConstantBuffers(PVBOXDX_DEVICE pDevice, SVGA3dShaderType enmShaderType, UINT StartSlot, UINT NumBuffers,
                               PVBOXDX_RESOURCE *papBuffers, const UINT *pFirstConstant, const UINT *pNumConstants);
 void vboxDXResourceCopyRegion(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pDstResource, UINT DstSubresource,
                               UINT DstX, UINT DstY, UINT DstZ, PVBOXDX_RESOURCE pSrcResource, UINT SrcSubresource,
                               const D3D10_DDI_BOX *pSrcBox, UINT CopyFlags);
 void vboxDXResourceCopy(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pDstResource, PVBOXDX_RESOURCE pSrcResource);
+void vboxDXResourceResolveSubresource(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pDstResource, UINT DstSubresource,
+                                      PVBOXDX_RESOURCE pSrcResource, UINT SrcSubresource, DXGI_FORMAT ResolveFormat);
 void vboxDXCreateUnorderedAccessView(PVBOXDX_DEVICE pDevice, PVBOXDXUNORDEREDACCESSVIEW pUnorderedAccessView);
 void vboxDXDestroyUnorderedAccessView(PVBOXDX_DEVICE pDevice, PVBOXDXUNORDEREDACCESSVIEW pUnorderedAccessView);
 void vboxDXClearUnorderedAccessViewUint(PVBOXDX_DEVICE pDevice, PVBOXDXUNORDEREDACCESSVIEW pUnorderedAccessView, const UINT Values[4]);
@@ -537,12 +745,87 @@ void vboxDXDispatchIndirect(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pResource,
 void vboxDXDrawIndexedInstancedIndirect(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pResource, UINT AlignedByteOffsetForArgs);
 void vboxDXDrawInstancedIndirect(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pResource, UINT AlignedByteOffsetForArgs);
 void vboxDXCopyStructureCount(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pDstBuffer, UINT DstAlignedByteOffset, PVBOXDXUNORDEREDACCESSVIEW pSrcView);
+void vboxDXClearView(PVBOXDX_DEVICE pDevice, D3D11DDI_HANDLETYPE ViewType, uint32_t ViewId, FLOAT const Color[4], D3D10_DDI_RECT const *pRect, UINT NumRects);
 HRESULT vboxDXBlt(PVBOXDX_DEVICE pDevice, PVBOXDX_RESOURCE pDstResource, UINT DstSubresource,
                   PVBOXDX_RESOURCE pSrcResource, UINT SrcSubresource,
                   UINT DstLeft, UINT DstTop, UINT DstRight, UINT DstBottom,
                   DXGI_DDI_ARG_BLT_FLAGS Flags, DXGI_DDI_MODE_ROTATION Rotate);
 HRESULT vboxDXFlush(PVBOXDX_DEVICE pDevice, bool fForce);
 
+void vboxDXGetVideoDecoderProfileCount(PVBOXDX_DEVICE pDevice, UINT *pDecodeProfileCount);
+void vboxDXGetVideoDecoderProfile(PVBOXDX_DEVICE pDevice, UINT Index, GUID *pGuid);
+void vboxDXCheckVideoDecoderFormat(PVBOXDX_DEVICE pDevice, GUID const *pDecodeProfile, DXGI_FORMAT Format, BOOL *pSupported);
+void vboxDXGetVideoDecoderConfigCount(PVBOXDX_DEVICE pDevice, D3D11_1DDI_VIDEO_DECODER_DESC const *pDecodeDesc, UINT *pConfigCount);
+void vboxDXGetVideoDecoderConfig(PVBOXDX_DEVICE pDevice, D3D11_1DDI_VIDEO_DECODER_DESC const *pDecodeDesc, UINT Index,
+                                 D3D11_1DDI_VIDEO_DECODER_CONFIG *pConfig);
+HRESULT vboxDXCreateVideoProcessorEnum(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                       D3D11_1DDI_VIDEO_PROCESSOR_CONTENT_DESC const *Desc);
+void vboxDXCheckVideoProcessorFormat(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                     DXGI_FORMAT Format, UINT *pSupported);
+void vboxDXGetVideoProcessorCaps(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                 D3D11_1DDI_VIDEO_PROCESSOR_CAPS *pCaps);
+void vboxDXGetVideoProcessorRateConversionCaps(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                               D3D11_1DDI_VIDEO_PROCESSOR_RATE_CONVERSION_CAPS *pCaps);
+void vboxDXGetVideoProcessorCustomRate(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                       UINT CustomRateIndex, D3D11_1DDI_VIDEO_PROCESSOR_CUSTOM_RATE *pRate);
+void vboxDXGetVideoProcessorFilterRange(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                        D3D11_1DDI_VIDEO_PROCESSOR_FILTER Filter, D3D11_1DDI_VIDEO_PROCESSOR_FILTER_RANGE *pFilterRange);
+HRESULT vboxDXCreateVideoProcessor(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor,
+                                   PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum, UINT RateConversionCapsIndex);
+HRESULT vboxDXCreateVideoDecoderOutputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODEROUTPUTVIEW pVideoDecoderOutputView, PVBOXDX_RESOURCE pResource,
+                                           GUID const &DecodeProfile, UINT MipSlice, UINT FirstArraySlice, UINT ArraySize);
+HRESULT vboxDXCreateVideoDecoder(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODER pVideoDecoder,
+                                 D3D11_1DDI_VIDEO_DECODER_DESC const &Desc, D3D11_1DDI_VIDEO_DECODER_CONFIG const &Config);
+HRESULT vboxDXVideoDecoderBeginFrame(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODER pVideoDecoder,
+                                     PVBOXDXVIDEODECODEROUTPUTVIEW pVideoDecoderOutputView,
+                                     void const *pContentKey, UINT ContentKeySize);
+HRESULT vboxDXVideoDecoderSubmitBuffers(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODER pVideoDecoder,
+                                        UINT BufferCount, D3D11_1DDI_VIDEO_DECODER_BUFFER_DESC const *pBufferDesc);
+HRESULT vboxDXVideoDecoderEndFrame(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODER pVideoDecoder);
+HRESULT vboxDXCreateVideoProcessorInputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORINPUTVIEW pVideoProcessorInputView,
+                                            PVBOXDX_RESOURCE pResource, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                            UINT FourCC, UINT MipSlice, UINT FirstArraySlice, UINT ArraySize);
+HRESULT vboxDXCreateVideoProcessorOutputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOROUTPUTVIEW pVideoProcessorOutputView,
+                                             PVBOXDX_RESOURCE pResource, PVBOXDXVIDEOPROCESSORENUM pVideoProcessorEnum,
+                                             UINT MipSlice, UINT FirstArraySlice, UINT ArraySize);
+HRESULT vboxDXVideoProcessorBlt(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, PVBOXDXVIDEOPROCESSOROUTPUTVIEW pVideoProcessorOutputView,
+                                UINT OutputFrame, UINT StreamCount, D3D11_1DDI_VIDEO_PROCESSOR_STREAM const *paStream);
+void vboxDXDestroyVideoDecoder(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODER pVideoDecoder);
+void vboxDXDestroyVideoDecoderOutputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEODECODEROUTPUTVIEW pVideoDecoderOutputView);
+void vboxDXDestroyVideoProcessor(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor);
+void vboxDXDestroyVideoProcessorInputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSORINPUTVIEW pVideoProcessorInputView);
+void vboxDXDestroyVideoProcessorOutputView(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOROUTPUTVIEW pVideoProcessorOutputView);
+void vboxDXVideoProcessorSetOutputTargetRect(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, BOOL Enable, RECT const *pOutputRect);
+void vboxDXVideoProcessorSetOutputBackgroundColor(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, BOOL YCbCr, D3D11_1DDI_VIDEO_COLOR const *pColor);
+void vboxDXVideoProcessorSetOutputColorSpace(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, D3D11_1DDI_VIDEO_PROCESSOR_COLOR_SPACE const *pColorSpace);
+void vboxDXVideoProcessorSetOutputAlphaFillMode(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, D3D11_1DDI_VIDEO_PROCESSOR_ALPHA_FILL_MODE FillMode, UINT StreamIndex);
+void vboxDXVideoProcessorSetOutputConstriction(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, BOOL Enabled, SIZE ConstrictonSize);
+void vboxDXVideoProcessorSetOutputStereoMode(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, BOOL Enable);
+void vboxDXVideoProcessorSetStreamFrameFormat(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, D3D11_1DDI_VIDEO_FRAME_FORMAT Format);
+void vboxDXVideoProcessorSetStreamColorSpace(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, D3D11_1DDI_VIDEO_PROCESSOR_COLOR_SPACE const *pColorSpace);
+void vboxDXVideoProcessorSetStreamOutputRate(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                             D3D11_1DDI_VIDEO_PROCESSOR_OUTPUT_RATE OutputRate, BOOL RepeatFrame, DXGI_RATIONAL const *pCustomRate);
+void vboxDXVideoProcessorSetStreamSourceRect(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                             BOOL Enable, RECT const *pSourceRect);
+void vboxDXVideoProcessorSetStreamDestRect(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                           BOOL Enable, RECT const *pDestRect);
+void vboxDXVideoProcessorSetStreamAlpha(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                        BOOL Enable, FLOAT Alpha);
+void vboxDXVideoProcessorSetStreamPalette(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                          UINT Count, UINT const *pEntries);
+void vboxDXVideoProcessorSetStreamPixelAspectRatio(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                                   BOOL Enable, DXGI_RATIONAL const *pSourceRatio, DXGI_RATIONAL const *pDestRatio);
+void vboxDXVideoProcessorSetStreamLumaKey(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex,
+                                          BOOL Enable, FLOAT Lower, FLOAT Upper);
+void vboxDXVideoProcessorSetStreamStereoFormat(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, BOOL Enable,
+                                               D3D11_1DDI_VIDEO_PROCESSOR_STEREO_FORMAT StereoFormat, BOOL LeftViewFrame0, BOOL BaseViewFrame0,
+                                               D3D11_1DDI_VIDEO_PROCESSOR_STEREO_FLIP_MODE FlipMode, int MonoOffset);
+void vboxDXVideoProcessorSetStreamAutoProcessingMode(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, BOOL Enable);
+void vboxDXVideoProcessorSetStreamFilter(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, BOOL Enable,
+                                         D3D11_1DDI_VIDEO_PROCESSOR_FILTER Filter, int Level);
+void vboxDXVideoProcessorSetStreamRotation(PVBOXDX_DEVICE pDevice, PVBOXDXVIDEOPROCESSOR pVideoProcessor, UINT StreamIndex, BOOL Enable,
+                                           D3D11_1DDI_VIDEO_PROCESSOR_ROTATION Rotation);
+
 DECLINLINE(void) vboxDXDeviceSetError(PVBOXDX_DEVICE pDevice, HRESULT hr)
 {
     Assert(hr == DXGI_DDI_ERR_WASSTILLDRAWING);
