@@ -722,6 +722,33 @@ void SvgaCmdBufCommit(PVBOXWDDM_EXT_VMSVGA pSvga, uint32_t cbActual)
 }
 
 
+/** Commit space for the current command in the current miniport command buffer
+ *  and attach a completion callback to the command buffer.
+ *
+ * @param pSvga            The device instance.
+ * @param cbActual         Actual size of the command data. Must be not greater than the reserved size.
+ * @param pfn              Callback to invoke when the command buffer has been processed by the host.
+ * @param pv               Callback parameters.
+ * @param cb               Size of callback parameters.
+ */
+void SvgaCmdBufCommitWithCompletionCallback(PVBOXWDDM_EXT_VMSVGA pSvga, uint32_t cbActual,
+    PFNCBCOMPLETION pfn, void const *pv, uint32_t cb)
+{
+    PVMSVGACBSTATE pCBState = pSvga->pCBState;
+
+    VMSVGACBCOMPLETION *p = (VMSVGACBCOMPLETION *)RTMemAlloc(sizeof(VMSVGACBCOMPLETION) + cb);
+    AssertReturnVoidStmt(p, ExReleaseFastMutex(&pCBState->CBCurrentMutex));
+
+    p->pfn = pfn;
+    p->cb = cb;
+    memcpy(&p[1], pv, cb);
+
+    RTListAppend(&pCBState->pCBCurrent->listCompletion, &p->nodeCompletion);
+
+    SvgaCmdBufCommit(pSvga, cbActual);
+}
+
+
 /** Submit the current miniport command buffer to the host.
  * If the buffer contains no command data, then this function does nothing.
  *
@@ -798,7 +825,9 @@ void SvgaCmdBufProcess(PVBOXWDDM_EXT_VMSVGA pSvga)
     for (unsigned i = 0; i < RT_ELEMENTS(pCBState->aCBContexts); ++i)
     {
         PVMSVGACBCONTEXT pCBCtx = &pCBState->aCBContexts[i];
+
         PVMSVGACB pIter, pNext;
+        pCBCtx->cCompleted = 0;
         RTListForEachSafe(&pCBCtx->QueueSubmitted, pIter, pNext, VMSVGACB, nodeQueue)
         {
             /* Buffers are processed sequentially, so if this one has not been processed,
@@ -810,7 +839,7 @@ void SvgaCmdBufProcess(PVBOXWDDM_EXT_VMSVGA pSvga)
             /* Remove the command buffer from the submitted queue and add to the local queue. */
             RTListNodeRemove(&pIter->nodeQueue);
             RTListAppend(&listCompleted, &pIter->nodeQueue);
-            --pCBCtx->cSubmitted;
+            ++pCBCtx->cCompleted;
         }
 
         /* Try to submit pending buffers. */
@@ -866,6 +895,21 @@ void SvgaCmdBufProcess(PVBOXWDDM_EXT_VMSVGA pSvga)
                 break;
         }
     }
+
+    /* Decrement 'cSubmitted' after freeing completed command buffers
+     * in order to make sure that SvgaCmdBufIsIdle returns true only
+     * if there are no buffers being processed.
+     */
+    KeAcquireSpinLock(&pCBState->SpinLock, &OldIrql);
+    for (unsigned i = 0; i < RT_ELEMENTS(pCBState->aCBContexts); ++i)
+    {
+        PVMSVGACBCONTEXT pCBCtx = &pCBState->aCBContexts[i];
+        if (pCBCtx->cCompleted <= pCBCtx->cSubmitted)
+            pCBCtx->cSubmitted -= pCBCtx->cCompleted;
+        else
+            AssertFailedStmt(pCBCtx->cSubmitted = 0);
+    }
+    KeReleaseSpinLock(&pCBState->SpinLock, OldIrql);
 }
 
 
@@ -892,22 +936,6 @@ bool SvgaCmdBufIsIdle(PVBOXWDDM_EXT_VMSVGA pSvga)
 }
 
 
-void SvgaCmdBufSetCompletionCallback(PVBOXWDDM_EXT_VMSVGA pSvga, PFNCBCOMPLETION pfn, void const *pv, uint32_t cb)
-{
-    VMSVGACBCOMPLETION *p = (VMSVGACBCOMPLETION *)RTMemAlloc(sizeof(VMSVGACBCOMPLETION) + cb);
-    AssertReturnVoid(p);
-
-    p->pfn = pfn;
-    p->cb = cb;
-    memcpy(&p[1], pv, cb);
-
-    PVMSVGACBSTATE pCBState = pSvga->pCBState;
-    ExAcquireFastMutex(&pCBState->CBCurrentMutex);
-    RTListAppend(&pCBState->pCBCurrent->listCompletion, &p->nodeCompletion);
-    ExReleaseFastMutex(&pCBState->CBCurrentMutex);
-}
-
-
 NTSTATUS SvgaCmdBufDestroy(PVBOXWDDM_EXT_VMSVGA pSvga)
 {
     /** PVMSVGACBSTATE pCBState as parameter. */
