@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2012-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2012-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -36,7 +36,7 @@
 /*********************************************************************************************************************************
 *   Defined Constants And Macros                                                                                                 *
 *********************************************************************************************************************************/
-static UNICODE_STRING g_UnicodeBackslash = { 2, 4, L"\\" };
+static UNICODE_STRING g_UnicodeBackslash = { 2, 4, (PWSTR)L"\\" };
 
 
 /**
