@@ -209,7 +209,11 @@ static DECLCALLBACK(int) vbsvcAutomounterInit(void)
     {
         /* If the service was not found, we disable this service without
            causing VBoxService to fail. */
-        if (rc == VERR_HGCM_SERVICE_NOT_FOUND) /* Host service is not available. */
+        if (   rc == VERR_HGCM_SERVICE_NOT_FOUND                         /* Host service is not available. */
+#ifdef RT_OS_WINDOWS
+            || RTSystemGetNtVersion() <= RTSYSTEM_MAKE_NT_VERSION(4,0,0) /* On <= NT4 guests no Shared Folders are available. */
+#endif
+           )
         {
             VGSvcVerbose(0, "vbsvcAutomounterInit: Shared Folders service is not available\n");
             rc = VERR_SERVICE_DISABLED;
@@ -1406,8 +1410,13 @@ static int vbsvcAutomounterMountIt(PVBSVCAUTOMOUNTERENTRY pEntry)
                      pEntry->pszName, pEntry->pszActualMountPoint);
         return VINF_SUCCESS;
     }
-    VGSvcError("vbsvcAutomounterMountIt: Failed to attach '%s' to '%s': %Rrc (%u)\n",
-               pEntry->pszName, pEntry->pszActualMountPoint, RTErrConvertFromWin32(dwErr), dwErr);
+    else if (dwErr == ERROR_BAD_PROVIDER)
+        VGSvcError("vbsvcAutomounterMountIt: Failed to attach '%s' to '%s': VirtualBox Shared Folders provider not or " \
+                   "incorrectly installed, can't continue\n",
+                   pEntry->pszName, pEntry->pszActualMountPoint);
+    else
+        VGSvcError("vbsvcAutomounterMountIt: Failed to attach '%s' to '%s': %Rrc (%u)\n",
+                   pEntry->pszName, pEntry->pszActualMountPoint, RTErrConvertFromWin32(dwErr), dwErr);
     return VERR_OPEN_FAILED;
 
 #elif defined(RT_OS_OS2)
