@@ -660,12 +660,13 @@ static int vbox_get_modes(struct drm_connector *connector)
     return num_modes;
 }
 
-#if RTLNX_VER_MAX(3,14,0) && !RTLNX_RHEL_MAJ_PREREQ(7,1)
-static int vbox_mode_valid(struct drm_connector *connector,
+#if RTLNX_VER_MIN(6,15,0)
+static enum drm_mode_status vbox_mode_valid(struct drm_connector *connector, const struct drm_display_mode *mode)
+#elif RTLNX_VER_MIN(3,14,0) || RTLNX_RHEL_MAJ_PREREQ(7,1)
+static enum drm_mode_status vbox_mode_valid(struct drm_connector *connector, struct drm_display_mode *mode)
 #else
-static enum drm_mode_status vbox_mode_valid(struct drm_connector *connector,
+static int vbox_mode_valid(struct drm_connector *connector, struct drm_display_mode *mode)
 #endif
-               struct drm_display_mode *mode)
 {
     return MODE_OK;
 }
