@@ -4,27 +4,38 @@
  */
 
 /*
- * Copyright (C) 2008-2019 Oracle Corporation
+ * Copyright (C) 2008-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
 /*********************************************************************************************************************************
 *   Header files                                                                                                                 *
 *********************************************************************************************************************************/
-
 #include <iprt/errcore.h>
 #include <iprt/assert.h>
 #include <iprt/vector.h>
+#include <iprt/thread.h>
 #include <VBox/log.h>
 
+#include "seamless.h"
 #include "seamless-x11.h"
 #include "VBoxClient.h"
 
@@ -33,19 +44,24 @@
 
 #include <limits.h>
 
+
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 #ifdef TESTCASE
-#undef DefaultRootWindow
-#define DefaultRootWindow XDefaultRootWindow
+# undef DefaultRootWindow
+# define DefaultRootWindow XDefaultRootWindow
 #endif
 
-/*****************************************************************************
-* Static functions                                                           *
-*****************************************************************************/
 
-static unsigned char *XXGetProperty (Display *aDpy, Window aWnd, Atom aPropType,
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
+
+static unsigned char *XXGetProperty(Display *aDpy, Window aWnd, Atom aPropType,
                                     const char *aPropName, unsigned long *nItems)
 {
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     Atom propNameAtom = XInternAtom (aDpy, aPropName,
                                      True /* only_if_exists */);
     if (propNameAtom == None)
@@ -64,38 +80,62 @@ static unsigned char *XXGetProperty (Display *aDpy, Window aWnd, Atom aPropType,
     if (rc != Success)
         return NULL;
 
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
     return propVal;
 }
 
-/**
-  * Initialise the guest and ensure that it is capable of handling seamless mode
-  *
-  * @param  pHostCallback   host callback.
-  * @returns true if it can handle seamless, false otherwise
-  */
-int SeamlessX11::init(PFNSENDREGIONUPDATE pHostCallback)
+int VBClX11SeamlessMonitor::init(PFNSENDREGIONUPDATE pHostCallback)
 {
     int rc = VINF_SUCCESS;
 
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     if (mHostCallback != NULL)  /* Assertion */
     {
-        LogRel(("VBoxClient: ERROR: attempt to initialise seamless guest object twice!\n"));
+        VBClLogError("Attempting to initialise seamless guest object twice!\n");
         return VERR_INTERNAL_ERROR;
     }
     if (!(mDisplay = XOpenDisplay(NULL)))
     {
-        LogRel(("VBoxClient: seamless guest object failed to acquire a connection to the display.\n"));
+        VBClLogError("Seamless guest object failed to acquire a connection to the display\n");
         return VERR_ACCESS_DENIED;
     }
     mHostCallback = pHostCallback;
     mEnabled = false;
     unmonitorClientList();
-    LogRelFlowFunc(("returning %Rrc\n", rc));
+    LogRelFlowFuncLeaveRC(rc);
     return rc;
 }
 
+/**
+ * Shutdown seamless event monitoring.
+ */
+void VBClX11SeamlessMonitor::uninit(void)
+{
+    if (mHostCallback)
+        stop();
+    mHostCallback = NULL;
+
+    /* Before closing a Display, make sure X11 is still running. The indicator
+     * that is when XOpenDisplay() returns non NULL. If it is not a
+     * case, XCloseDisplay() will hang on internal X11 mutex forever. */
+    Display *pDisplay = XOpenDisplay(NULL);
+    if (pDisplay)
+    {
+        XCloseDisplay(pDisplay);
+        if (mDisplay)
+        {
+            XCloseDisplay(mDisplay);
+            mDisplay = NULL;
+        }
+    }
+
+    if (mpRects)
+    {
+        RTMemFree(mpRects);
+        mpRects = NULL;
+    }
+}
+
 /**
  * Read information about currently visible windows in the guest and subscribe to X11
  * events about changes to this information.
@@ -104,45 +144,45 @@ int SeamlessX11::init(PFNSENDREGIONUPDATE pHostCallback)
  *       call nextConfigurationEvent() for as long as events are wished.
  * @todo This function should switch the guest to fullscreen mode.
  */
-int SeamlessX11::start(void)
+int VBClX11SeamlessMonitor::start(void)
 {
     int rc = VINF_SUCCESS;
     /** Dummy values for XShapeQueryExtension */
     int error, event;
 
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     if (mEnabled)
         return VINF_SUCCESS;
     mSupportsShape = XShapeQueryExtension(mDisplay, &event, &error);
     mEnabled = true;
     monitorClientList();
     rebuildWindowTree();
-    LogRelFlowFunc(("returning %Rrc\n", rc));
+    LogRelFlowFuncLeaveRC(rc);
     return rc;
 }
 
 /** Stop reporting seamless events to the host.  Free information about guest windows
     and stop requesting updates. */
-void SeamlessX11::stop(void)
+void VBClX11SeamlessMonitor::stop(void)
 {
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     if (!mEnabled)
         return;
     mEnabled = false;
     unmonitorClientList();
     freeWindowTree();
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
 }
 
-void SeamlessX11::monitorClientList(void)
+void VBClX11SeamlessMonitor::monitorClientList(void)
 {
-    LogRelFlowFunc(("called\n"));
+    LogRelFlowFuncEnter();
     XSelectInput(mDisplay, DefaultRootWindow(mDisplay), PropertyChangeMask | SubstructureNotifyMask);
 }
 
-void SeamlessX11::unmonitorClientList(void)
+void VBClX11SeamlessMonitor::unmonitorClientList(void)
 {
-    LogRelFlowFunc(("called\n"));
+    LogRelFlowFuncEnter();
     XSelectInput(mDisplay, DefaultRootWindow(mDisplay), PropertyChangeMask);
 }
 
@@ -150,9 +190,9 @@ void SeamlessX11::unmonitorClientList(void)
  * Recreate the table of toplevel windows of clients on the default root window of the
  * X server.
  */
-void SeamlessX11::rebuildWindowTree(void)
+void VBClX11SeamlessMonitor::rebuildWindowTree(void)
 {
-    LogRelFlowFunc(("called\n"));
+    LogRelFlowFuncEnter();
     freeWindowTree();
     addClients(DefaultRootWindow(mDisplay));
     mChanged = true;
@@ -165,7 +205,7 @@ void SeamlessX11::rebuildWindowTree(void)
  *
  * @param hRoot the virtual root window to be examined
  */
-void SeamlessX11::addClients(const Window hRoot)
+void VBClX11SeamlessMonitor::addClients(const Window hRoot)
 {
     /** Unused out parameters of XQueryTree */
     Window hRealRoot, hParent;
@@ -176,20 +216,20 @@ void SeamlessX11::addClients(const Window hRoot)
     /** The number of children of the root supplied */
     unsigned cChildren;
 
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     if (!XQueryTree(mDisplay, hRoot, &hRealRoot, &hParent, &phChildrenRaw, &cChildren))
         return;
     phChildren = phChildrenRaw;
     for (unsigned i = 0; i < cChildren; ++i)
         addClientWindow(phChildren[i]);
     XFree(phChildrenRaw);
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
 }
 
 
-void SeamlessX11::addClientWindow(const Window hWin)
+void VBClX11SeamlessMonitor::addClientWindow(const Window hWin)
 {
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     XWindowAttributes winAttrib;
     bool fAddWin = true;
     Window hClient = XmuClientWindow(mDisplay, hWin);
@@ -198,7 +238,7 @@ void SeamlessX11::addClientWindow(const Window hWin)
         fAddWin = false;
     if (fAddWin && !XGetWindowAttributes(mDisplay, hWin, &winAttrib))
     {
-        LogRelFunc(("VBoxClient: Failed to get the window attributes for window %d\n", hWin));
+        VBClLogError("Failed to get the window attributes for window %d\n", hWin);
         fAddWin = false;
     }
     if (fAddWin && (winAttrib.map_state == IsUnmapped))
@@ -210,8 +250,7 @@ void SeamlessX11::addClientWindow(const Window hWin)
     if (fAddWin && (!XGetWMNormalHints(mDisplay, hClient, &dummyHints,
                                        &dummyLong)))
     {
-        LogRelFlowFunc(("window %lu, client window %lu has no size hints\n",
-                     hWin, hClient));
+        LogRelFlowFunc(("window %lu, client window %lu has no size hints\n", hWin, hClient));
         fAddWin = false;
     }
     if (fAddWin)
@@ -243,7 +282,7 @@ void SeamlessX11::addClientWindow(const Window hWin)
                                 winAttrib.width, winAttrib.height, cRects,
                                 pRects);
     }
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
 }
 
 
@@ -252,14 +291,14 @@ void SeamlessX11::addClientWindow(const Window hWin)
  * @returns true if it is, false otherwise
  * @param hWin the window to be examined
  */
-bool SeamlessX11::isVirtualRoot(Window hWin)
+bool VBClX11SeamlessMonitor::isVirtualRoot(Window hWin)
 {
     unsigned char *windowTypeRaw = NULL;
     Atom *windowType;
     unsigned long ulCount;
     bool rc = false;
 
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     windowTypeRaw = XXGetProperty(mDisplay, hWin, XA_ATOM, WM_TYPE_PROP, &ulCount);
     if (windowTypeRaw != NULL)
     {
@@ -286,12 +325,12 @@ DECLCALLBACK(int) VBoxGuestWinFree(VBoxGuestWinInfo *pInfo, void *pvParam)
 /**
  * Free all information in the tree of visible windows
  */
-void SeamlessX11::freeWindowTree(void)
+void VBClX11SeamlessMonitor::freeWindowTree(void)
 {
     /* We use post-increment in the operation to prevent the iterator from being invalidated. */
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     mGuestWindows.detachAll(VBoxGuestWinFree, mDisplay);
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
 }
 
 
@@ -300,11 +339,11 @@ void SeamlessX11::freeWindowTree(void)
  *
  * @note Called from the guest event thread.
  */
-void SeamlessX11::nextConfigurationEvent(void)
+void VBClX11SeamlessMonitor::nextConfigurationEvent(void)
 {
     XEvent event;
 
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     /* Start by sending information about the current window setup to the host.  We do this
        here because we want to send all such information from a single thread. */
     if (mChanged && mEnabled)
@@ -313,9 +352,20 @@ void SeamlessX11::nextConfigurationEvent(void)
         mHostCallback(mpRects, mcRects);
     }
     mChanged = false;
-    /* We execute this even when seamless is disabled, as it also waits for
-     * enable and disable notification. */
-    XNextEvent(mDisplay, &event);
+
+    if (XPending(mDisplay) > 0)
+    {
+        /* We execute this even when seamless is disabled, as it also waits for
+         * enable and disable notification. */
+        XNextEvent(mDisplay, &event);
+    } else
+    {
+        /* This function is called in a loop by upper layer. In order to
+         * prevent CPU spinning, sleep a bit before returning. */
+        RTThreadSleep(300 /* ms */);
+        return;
+    }
+
     if (!mEnabled)
         return;
     switch (event.type)
@@ -340,7 +390,7 @@ void SeamlessX11::nextConfigurationEvent(void)
         if (   event.xproperty.atom != XInternAtom(mDisplay, "_NET_CLIENT_LIST", True /* only_if_exists */)
             || event.xproperty.window != DefaultRootWindow(mDisplay))
             break;
-        LogRelFlowFunc(("_NET_CLIENT_LIST property event on root window.\n"));
+        LogRelFlowFunc(("_NET_CLIENT_LIST property event on root window\n"));
         rebuildWindowTree();
         break;
     case VBoxShapeNotify:  /* This is defined wrong in my X11 header files! */
@@ -367,7 +417,7 @@ void SeamlessX11::nextConfigurationEvent(void)
  *
  * @param hWin the window to be examined
  */
-void SeamlessX11::doConfigureEvent(Window hWin)
+void VBClX11SeamlessMonitor::doConfigureEvent(Window hWin)
 {
     VBoxGuestWinInfo *pInfo = mGuestWindows.find(hWin);
     if (pInfo)
@@ -389,9 +439,9 @@ void SeamlessX11::doConfigureEvent(Window hWin)
  *
  * @param hWin the window to be examined
  */
-void SeamlessX11::doShapeEvent(Window hWin)
+void VBClX11SeamlessMonitor::doShapeEvent(Window hWin)
 {
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     VBoxGuestWinInfo *pInfo = mGuestWindows.find(hWin);
     if (pInfo)
     {
@@ -409,13 +459,13 @@ void SeamlessX11::doShapeEvent(Window hWin)
         pInfo->mpRects = pRects;
         mChanged = true;
     }
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
 }
 
 /**
  * Gets the list of visible rectangles
  */
-RTRECT *SeamlessX11::getRects(void)
+RTRECT *VBClX11SeamlessMonitor::getRects(void)
 {
     return mpRects;
 }
@@ -423,15 +473,14 @@ RTRECT *SeamlessX11::getRects(void)
 /**
  * Gets the number of rectangles in the visible rectangle list
  */
-size_t SeamlessX11::getRectCount(void)
+size_t VBClX11SeamlessMonitor::getRectCount(void)
 {
     return mcRects;
 }
 
 RTVEC_DECL(RectList, RTRECT)
 
-DECLCALLBACK(int) getRectsCallback(VBoxGuestWinInfo *pInfo,
-                                   struct RectList *pRects)
+static DECLCALLBACK(int) getRectsCallback(VBoxGuestWinInfo *pInfo, struct RectList *pRects)
 {
     if (pInfo->mhasShape)
     {
@@ -474,9 +523,9 @@ DECLCALLBACK(int) getRectsCallback(VBoxGuestWinInfo *pInfo,
 /**
  * Updates the list of seamless rectangles
  */
-int SeamlessX11::updateRects(void)
+int VBClX11SeamlessMonitor::updateRects(void)
 {
-    LogRelFlowFunc(("\n"));
+    LogRelFlowFuncEnter();
     struct RectList rects = RTVEC_INITIALIZER;
 
     if (mcRects != 0)
@@ -485,13 +534,12 @@ int SeamlessX11::updateRects(void)
         if (RT_FAILURE(rc))
             return rc;
     }
-    mGuestWindows.doWithAll((PVBOXGUESTWINCALLBACK)getRectsCallback,
-                            &rects);
+    mGuestWindows.doWithAll((PFNVBOXGUESTWINCALLBACK)getRectsCallback, &rects);
     if (mpRects)
         RTMemFree(mpRects);
     mcRects = RectListSize(&rects);
     mpRects = RectListDetach(&rects);
-    LogRelFlowFunc(("returning\n"));
+    LogRelFlowFuncLeave();
     return VINF_SUCCESS;
 }
 
@@ -500,21 +548,277 @@ int SeamlessX11::updateRects(void)
  *
  * @note This function should only be called from the host event thread.
  */
-bool SeamlessX11::interruptEventWait(void)
+bool VBClX11SeamlessMonitor::interruptEventWait(void)
 {
-    bool rc = false;
+    LogRelFlowFuncEnter();
+
     Display *pDisplay = XOpenDisplay(NULL);
-
-    LogRelFlowFunc(("\n"));
     if (pDisplay == NULL)
-        VBClFatalError(("Failed to open X11 display.\n"));
-    /* Message contents set to zero. */
-    XClientMessageEvent clientMessage = { ClientMessage, 0, 0, 0, 0, 0, 8 };
+    {
+        VBClLogError("Failed to open X11 display\n");
+        return false;
+    }
 
+    /* Message contents set to zero. */
+    XClientMessageEvent clientMessage =
+    {
+        /* .type         = */ ClientMessage,
+        /* .serial       = */ 0,
+        /* .send_event   = */ 0,
+        /* .display      = */ 0,
+        /* .window       = */ 0,
+        /* .message_type = */ XInternAtom(pDisplay, "VBOX_CLIENT_SEAMLESS_HEARTBEAT", false),
+        /* .format       = */ 8,
+        /* .data ... */
+    };
+
+    bool rc = false;
     if (XSendEvent(pDisplay, DefaultRootWindow(mDisplay), false,
                    PropertyChangeMask, (XEvent *)&clientMessage))
         rc = true;
+
     XCloseDisplay(pDisplay);
     LogRelFlowFunc(("returning %RTbool\n", rc));
     return rc;
 }
+
+
+/*********************************************************************************************************************************
+ * VBClX11SeamlessSvc implementation                                                                                             *
+ ********************************************************************************************************************************/
+
+VBClX11SeamlessSvc::VBClX11SeamlessSvc(void)
+{
+    mX11MonitorThread         = NIL_RTTHREAD;
+    mX11MonitorThreadStopping = false;
+
+    mMode    = VMMDev_Seamless_Disabled;
+    mfPaused = true;
+}
+
+VBClX11SeamlessSvc::~VBClX11SeamlessSvc()
+{
+    /* Stopping will be done via main.cpp. */
+}
+
+/** @copydoc VBCLSERVICE::pfnInit */
+int VBClX11SeamlessSvc::init(void)
+{
+    int rc;
+    const char *pcszStage;
+
+    do
+    {
+        pcszStage = "Connecting to the X server";
+        rc = mX11Monitor.init(VBClSeamlessSendRegionUpdate);
+        if (RT_FAILURE(rc))
+            break;
+        pcszStage = "Setting guest IRQ filter mask";
+        rc = VbglR3CtlFilterMask(VMMDEV_EVENT_SEAMLESS_MODE_CHANGE_REQUEST, 0);
+        if (RT_FAILURE(rc))
+            break;
+        pcszStage = "Reporting support for seamless capability";
+        rc = VbglR3SeamlessSetCap(true);
+        if (RT_FAILURE(rc))
+            break;
+        rc = startX11MonitorThread();
+        if (RT_FAILURE(rc))
+            break;
+
+    } while(0);
+
+    if (RT_FAILURE(rc))
+        VBClLogError("Failed to start in stage '%s' -- error %Rrc\n", pcszStage, rc);
+
+    return rc;
+}
+
+/** @copydoc VBCLSERVICE::pfnWorker */
+int VBClX11SeamlessSvc::worker(bool volatile *pfShutdown)
+{
+    int rc = VINF_SUCCESS;
+
+    /* Let the main thread know that it can continue spawning services. */
+    RTThreadUserSignal(RTThreadSelf());
+
+    /* This will only exit if something goes wrong. */
+    for (;;)
+    {
+        if (ASMAtomicReadBool(pfShutdown))
+            break;
+
+        rc = nextStateChangeEvent();
+
+        if (rc == VERR_TRY_AGAIN)
+            rc = VINF_SUCCESS;
+
+        if (RT_FAILURE(rc))
+            break;
+
+        if (ASMAtomicReadBool(pfShutdown))
+            break;
+
+        /* If we are not stopping, sleep for a bit to avoid using up too
+           much CPU while retrying. */
+        RTThreadYield();
+    }
+
+    return rc;
+}
+
+/** @copydoc VBCLSERVICE::pfnStop */
+void VBClX11SeamlessSvc::stop(void)
+{
+    VbglR3SeamlessSetCap(false);
+    VbglR3CtlFilterMask(0, VMMDEV_EVENT_SEAMLESS_MODE_CHANGE_REQUEST);
+    stopX11MonitorThread();
+}
+
+/** @copydoc VBCLSERVICE::pfnTerm */
+int VBClX11SeamlessSvc::term(void)
+{
+    mX11Monitor.uninit();
+    return VINF_SUCCESS;
+}
+
+/**
+ * Waits for a seamless state change events from the host and dispatch it.
+ *
+ * @returns VBox return code, or
+ *          VERR_TRY_AGAIN if no new status is available and we have to try it again
+ *          at some later point in time.
+ */
+int VBClX11SeamlessSvc::nextStateChangeEvent(void)
+{
+    VMMDevSeamlessMode newMode = VMMDev_Seamless_Disabled;
+
+    int rc = VbglR3SeamlessWaitEvent(&newMode);
+    if (RT_SUCCESS(rc))
+    {
+        mMode = newMode;
+        switch (newMode)
+        {
+            case VMMDev_Seamless_Visible_Region:
+                /* A simplified seamless mode, obtained by making the host VM window
+                 * borderless and making the guest desktop transparent. */
+                VBClLogVerbose(2, "\"Visible region\" mode requested\n");
+                break;
+            case VMMDev_Seamless_Disabled:
+                VBClLogVerbose(2, "\"Disabled\" mode requested\n");
+                break;
+            case VMMDev_Seamless_Host_Window:
+                /* One host window represents one guest window.  Not yet implemented. */
+                VBClLogVerbose(2, "Unsupported \"host window\" mode requested\n");
+                return VERR_NOT_SUPPORTED;
+            default:
+                VBClLogError("Unsupported mode %d requested\n", newMode);
+                return VERR_NOT_SUPPORTED;
+        }
+    }
+    if (   RT_SUCCESS(rc)
+        || rc == VERR_TRY_AGAIN)
+    {
+        if (mMode == VMMDev_Seamless_Visible_Region)
+            mfPaused = false;
+        else
+            mfPaused = true;
+        mX11Monitor.interruptEventWait();
+    }
+    else
+        VBClLogError("VbglR3SeamlessWaitEvent returned %Rrc\n", rc);
+
+    return rc;
+}
+
+/**
+ * The actual X11 window configuration change monitor thread function.
+ */
+int VBClX11SeamlessSvc::x11MonitorThread(RTTHREAD hThreadSelf, void *pvUser)
+{
+    RT_NOREF(hThreadSelf);
+
+    VBClX11SeamlessSvc *pThis = (VBClX11SeamlessSvc *)pvUser;
+    AssertPtrReturn(pThis, VERR_INVALID_POINTER);
+
+    int rc = VINF_SUCCESS;
+
+    RTThreadUserSignal(hThreadSelf);
+
+    VBClLogVerbose(2, "X11 monitor thread started\n");
+
+    while (!pThis->mX11MonitorThreadStopping)
+    {
+        if (!pThis->mfPaused)
+        {
+            rc = pThis->mX11Monitor.start();
+            if (RT_FAILURE(rc))
+                VBClLogFatalError("Failed to change the X11 seamless service state, mfPaused=%RTbool, rc=%Rrc\n",
+                                  pThis->mfPaused, rc);
+        }
+
+        pThis->mX11Monitor.nextConfigurationEvent();
+
+        if (   pThis->mfPaused
+            || pThis->mX11MonitorThreadStopping)
+        {
+            pThis->mX11Monitor.stop();
+        }
+    }
+
+    VBClLogVerbose(2, "X11 monitor thread ended\n");
+
+    return rc;
+}
+
+/**
+ * Start the X11 window configuration change monitor thread.
+ */
+int VBClX11SeamlessSvc::startX11MonitorThread(void)
+{
+    mX11MonitorThreadStopping = false;
+
+    if (isX11MonitorThreadRunning())
+        return VINF_SUCCESS;
+
+    int rc = RTThreadCreate(&mX11MonitorThread, x11MonitorThread, this, 0,
+                            RTTHREADTYPE_MSG_PUMP, RTTHREADFLAGS_WAITABLE,
+                            "seamless x11");
+    if (RT_SUCCESS(rc))
+        rc = RTThreadUserWait(mX11MonitorThread, RT_MS_30SEC);
+
+    if (RT_FAILURE(rc))
+        VBClLogError("Failed to start X11 monitor thread, rc=%Rrc\n", rc);
+
+    return rc;
+}
+
+/**
+ * Stops the monitor thread.
+ */
+int VBClX11SeamlessSvc::stopX11MonitorThread(void)
+{
+    if (!isX11MonitorThreadRunning())
+        return VINF_SUCCESS;
+
+    mX11MonitorThreadStopping = true;
+    if (!mX11Monitor.interruptEventWait())
+    {
+        VBClLogError("Unable to notify X11 monitor thread\n");
+        return VERR_INVALID_STATE;
+    }
+
+    int rcThread;
+    int rc = RTThreadWait(mX11MonitorThread, RT_MS_30SEC, &rcThread);
+    if (RT_SUCCESS(rc))
+        rc = rcThread;
+
+    if (RT_SUCCESS(rc))
+    {
+        mX11MonitorThread = NIL_RTTHREAD;
+    }
+    else
+        VBClLogError("Waiting for X11 monitor thread to stop failed, rc=%Rrc\n", rc);
+
+    return rc;
+}
+
