@@ -1,3 +1,4 @@
+/* $XFree86$ */
 /*
  * Copyright 2001-2003 Red Hat Inc., Durham, North Carolina.
  *
@@ -50,10 +51,6 @@
 #ifndef DMX_H
 #define DMX_H
 
-#if HAVE_DMX_CONFIG_H
-#include <dmx-config.h>
-#endif
-
 #include "gcstruct.h"
 
 /* Handle client-side include files in one place. */
@@ -62,7 +59,9 @@
 #include "globals.h"
 #include "scrnintstr.h"
 
+#ifdef RENDER
 #include "picturestr.h"
+#endif
 
 #ifdef GLXEXT
 #include <GL/glx.h>
@@ -207,6 +206,8 @@ typedef struct _DMXScreenInfo {
     UnrealizeWindowProcPtr         UnrealizeWindow;
     RestackWindowProcPtr           RestackWindow;
     WindowExposuresProcPtr         WindowExposures;
+    PaintWindowBackgroundProcPtr   PaintWindowBackground;
+    PaintWindowBorderProcPtr       PaintWindowBorder;
     CopyWindowProcPtr              CopyWindow;
 
     ResizeWindowProcPtr            ResizeWindow;
@@ -229,8 +230,11 @@ typedef struct _DMXScreenInfo {
     InstallColormapProcPtr         InstallColormap;
     StoreColorsProcPtr             StoreColors;
 
+#ifdef SHAPE
     SetShapeProcPtr                SetShape;
+#endif
 
+#ifdef RENDER
     CreatePictureProcPtr           CreatePicture;
     DestroyPictureProcPtr          DestroyPicture;
     ChangePictureClipProcPtr       ChangePictureClip;
@@ -249,6 +253,9 @@ typedef struct _DMXScreenInfo {
 
     TrapezoidsProcPtr              Trapezoids;
     TrianglesProcPtr               Triangles;
+    TriStripProcPtr                TriStrip;
+    TriFanProcPtr                  TriFan;
+#endif
 } DMXScreenInfo;
 
 /* Global variables available to all Xserver/hw/dmx routines. */
@@ -335,21 +342,23 @@ do {									\
 #define _MAXSCREENSALLOCF(o,size,fatal)                                 \
     do {                                                                \
         if (!o) {                                                       \
-            o = calloc((size), sizeof(*(o)));                          \
+            o = xalloc((size) * sizeof(*(o)));                          \
+            if (o) memset(o, 0, (size) * sizeof(*(o)));                 \
             if (!o && fatal) FatalError(MAXSCREEN_FAILED_TXT #o);       \
         }                                                               \
     } while (0)
 #define _MAXSCREENSALLOCR(o,size,retval)                                \
     do {                                                                \
         if (!o) {                                                       \
-            o = calloc((size), sizeof(*(o)));                          \
+            o = xalloc((size) * sizeof(*(o)));                          \
+            if (o) memset(o, 0, (size) * sizeof(*(o)));                 \
             if (!o) return retval;                                      \
         }                                                               \
     } while (0)
         
 #define MAXSCREENSFREE(o)                                               \
     do {                                                                \
-        free(o);                                                \
+        if (o) xfree(o);                                                \
         o = NULL;                                                       \
     } while (0)
 
@@ -362,27 +371,4 @@ do {									\
 #define MAXSCREENSCALLOC_FATAL(o,m)     _MAXSCREENSALLOCF(o,MAXSCREENS*(m),1)
 #endif
 
-extern DevPrivateKeyRec dmxGCPrivateKeyRec;
-#define dmxGCPrivateKey (&dmxGCPrivateKeyRec) /**< Private index for GCs       */
-
-extern DevPrivateKeyRec dmxWinPrivateKeyRec;
-#define dmxWinPrivateKey (&dmxWinPrivateKeyRec) /**< Private index for Windows   */
-
-extern DevPrivateKeyRec dmxPixPrivateKeyRec;
-#define dmxPixPrivateKey (&dmxPixPrivateKeyRec) /**< Private index for Pixmaps   */
-
-extern int dmxFontPrivateIndex;        /**< Private index for Fonts     */
-
-extern DevPrivateKeyRec dmxScreenPrivateKeyRec;
-#define dmxScreenPrivateKey (&dmxScreenPrivateKeyRec) /**< Private index for Screens   */
-
-extern DevPrivateKeyRec dmxColormapPrivateKeyRec;
-#define dmxColormapPrivateKey (&dmxColormapPrivateKeyRec) /**< Private index for Colormaps */
-
-extern DevPrivateKeyRec dmxPictPrivateKeyRec;
-#define dmxPictPrivateKey (&dmxPictPrivateKeyRec) /**< Private index for Picts     */
-
-extern DevPrivateKeyRec dmxGlyphSetPrivateKeyRec;
-#define dmxGlyphSetPrivateKey (&dmxGlyphSetPrivateKeyRec) /**< Private index for GlyphSets */
-
 #endif /* DMX_H */
