@@ -48,9 +48,14 @@ SOFTWARE.
 #ifndef DIX_H
 #define DIX_H
 
+#include "callback.h"
 #include "gc.h"
 #include "window.h"
 #include "input.h"
+#include "cursor.h"
+#include "geext.h"
+#include "events.h"
+#include <X11/extensions/XI.h>
 
 #define EARLIER -1
 #define SAMETIME 0
@@ -58,7 +63,7 @@ SOFTWARE.
 
 #define NullClient ((ClientPtr) 0)
 #define REQUEST(type) \
-	register type *stuff = (type *)client->requestBuffer
+	type *stuff = (type *)client->requestBuffer
 
 
 #define REQUEST_SIZE_MATCH(req)\
@@ -78,32 +83,19 @@ SOFTWARE.
     if (!LegalNewID(id,client)) \
     {\
 	client->errorValue = id;\
-        return(BadIDChoice);\
+        return BadIDChoice;\
     }
 
-#define VALIDATE_DRAWABLE_AND_GC(drawID, pDraw, pGC, client)\
-    if ((stuff->gc == INVALID) || (client->lastGCID != stuff->gc) ||\
-	(client->lastDrawableID != drawID))\
+#define VALIDATE_DRAWABLE_AND_GC(drawID, pDraw, mode)\
     {\
-	int rc;\
-	rc = dixLookupDrawable(&(pDraw), drawID, client, M_ANY,\
-			       DixWriteAccess);\
+	int rc = dixLookupDrawable(&(pDraw), drawID, client, M_ANY, mode);\
 	if (rc != Success)\
 	    return rc;\
-	rc = dixLookupGC(&(pGC), stuff->gc, client, DixReadAccess);\
+	rc = dixLookupGC(&(pGC), stuff->gc, client, DixUseAccess);\
 	if (rc != Success)\
 	    return rc;\
 	if ((pGC->depth != pDraw->depth) || (pGC->pScreen != pDraw->pScreen))\
-	    return (BadMatch);\
-	client->lastDrawable = pDraw;\
-	client->lastDrawableID = drawID;\
-	client->lastGC = pGC;\
-	client->lastGCID = stuff->gc;\
-    }\
-    else\
-    {\
-        pGC = client->lastGC;\
-        pDraw = client->lastDrawable;\
+	    return BadMatch;\
     }\
     if (pGC->serialNumber != pDraw->serialNumber)\
 	ValidateGC(pDraw, pGC);
@@ -129,16 +121,15 @@ typedef struct _Client *ClientPtr; /* also in misc.h */
 
 typedef struct _WorkQueue	*WorkQueuePtr;
 
-extern ClientPtr requestingClient;
-extern ClientPtr *clients;
-extern ClientPtr serverClient;
-extern int currentMaxClients;
-extern char dispatchExceptionAtReset;
+extern _X_EXPORT ClientPtr clients[MAXCLIENTS];
+extern _X_EXPORT ClientPtr serverClient;
+extern _X_EXPORT int currentMaxClients;
+extern _X_EXPORT char dispatchExceptionAtReset;
 
 typedef int HWEventQueueType;
 typedef HWEventQueueType* HWEventQueuePtr;
 
-extern HWEventQueuePtr checkForInput[2];
+extern _X_EXPORT HWEventQueuePtr checkForInput[2];
 
 typedef struct _TimeStamp {
     CARD32 months;	/* really ~49.7 days */
@@ -147,145 +138,127 @@ typedef struct _TimeStamp {
 
 /* dispatch.c */
 
-extern void SetInputCheck(
+extern _X_EXPORT void SetInputCheck(
     HWEventQueuePtr /*c0*/,
     HWEventQueuePtr /*c1*/);
 
-extern void CloseDownClient(
+extern _X_EXPORT void CloseDownClient(
     ClientPtr /*client*/);
 
-extern void UpdateCurrentTime(void);
+extern _X_EXPORT void UpdateCurrentTime(void);
 
-extern void UpdateCurrentTimeIf(void);
+extern _X_EXPORT void UpdateCurrentTimeIf(void);
 
-extern void InitSelections(void);
-
-extern void FlushClientCaches(XID /*id*/);
-
-extern int dixDestroyPixmap(
+extern _X_EXPORT int dixDestroyPixmap(
     pointer /*value*/,
     XID /*pid*/);
 
-extern void InitClient(
+extern _X_EXPORT void InitClient(
     ClientPtr /*client*/,
     int /*i*/,
     pointer /*ospriv*/);
 
-extern ClientPtr NextAvailableClient(
+extern _X_EXPORT ClientPtr NextAvailableClient(
     pointer /*ospriv*/);
 
-extern void SendErrorToClient(
+extern _X_EXPORT void SendErrorToClient(
     ClientPtr /*client*/,
     unsigned int /*majorCode*/,
     unsigned int /*minorCode*/,
     XID /*resId*/,
     int /*errorCode*/);
 
-extern void DeleteWindowFromAnySelections(
-    WindowPtr /*pWin*/);
-
-extern void MarkClientException(
+extern _X_EXPORT void MarkClientException(
     ClientPtr /*client*/);
 
-extern int SendConnSetup(
-    ClientPtr /*client*/,
-    char* /*reason*/);
-
-#if defined(DDXBEFORERESET)
-extern void ddxBeforeReset (void);
-#endif
-
+extern _X_HIDDEN Bool CreateConnectionBlock(void);
 /* dixutils.c */
 
-extern void CopyISOLatin1Lowered(
-    unsigned char * /*dest*/,
-    unsigned char * /*source*/,
-    int /*length*/);
-
-extern int CompareISOLatin1Lowered(
+extern _X_EXPORT int CompareISOLatin1Lowered(
     unsigned char * /*a*/,
     int alen,
     unsigned char * /*b*/,
     int blen);
 
-extern int dixLookupWindow(
+extern _X_EXPORT int dixLookupWindow(
     WindowPtr *result,
     XID id,
     ClientPtr client,
     Mask access_mode);
 
-extern int dixLookupDrawable(
+extern _X_EXPORT int dixLookupDrawable(
     DrawablePtr *result,
     XID id,
     ClientPtr client,
     Mask type_mask,
     Mask access_mode);
 
-extern int dixLookupGC(
+extern _X_EXPORT int dixLookupGC(
     GCPtr *result,
     XID id,
     ClientPtr client,
     Mask access_mode);
 
-extern int dixLookupClient(
+extern _X_EXPORT int dixLookupFontable(
+    FontPtr *result,
+    XID id,
+    ClientPtr client,
+    Mask access_mode);
+
+extern _X_EXPORT int dixLookupClient(
     ClientPtr *result,
     XID id,
     ClientPtr client,
     Mask access_mode);
 
-/*
- * These are deprecated compatibility functions and will be removed soon!
- * Please use the new dixLookup*() functions above.
- */
-extern WindowPtr SecurityLookupWindow(XID, ClientPtr, Mask);
-extern WindowPtr LookupWindow(XID, ClientPtr);
-extern pointer SecurityLookupDrawable(XID, ClientPtr, Mask);
-extern pointer LookupDrawable(XID, ClientPtr);
-extern ClientPtr LookupClient(XID, ClientPtr);
-/* end deprecated functions */
+extern _X_EXPORT void NoopDDA(void);
 
-extern void NoopDDA(void);
-
-extern int AlterSaveSetForClient(
+extern _X_EXPORT int AlterSaveSetForClient(
     ClientPtr /*client*/,
     WindowPtr /*pWin*/,
     unsigned /*mode*/,
     Bool /*toRoot*/,
-    Bool /*remap*/);
+    Bool /*map*/);
   
-extern void DeleteWindowFromAnySaveSet(
+extern _X_EXPORT void DeleteWindowFromAnySaveSet(
     WindowPtr /*pWin*/);
 
-extern void BlockHandler(
+extern _X_EXPORT void BlockHandler(
     pointer /*pTimeout*/,
     pointer /*pReadmask*/);
 
-extern void WakeupHandler(
+extern _X_EXPORT void WakeupHandler(
     int /*result*/,
     pointer /*pReadmask*/);
 
+void
+EnableLimitedSchedulingLatency(void);
+
+void
+DisableLimitedSchedulingLatency(void);
+
 typedef void (* WakeupHandlerProcPtr)(
     pointer /* blockData */,
     int /* result */,
     pointer /* pReadmask */);
 
-extern Bool RegisterBlockAndWakeupHandlers(
+extern _X_EXPORT Bool RegisterBlockAndWakeupHandlers(
     BlockHandlerProcPtr /*blockHandler*/,
     WakeupHandlerProcPtr /*wakeupHandler*/,
     pointer /*blockData*/);
 
-extern void RemoveBlockAndWakeupHandlers(
+extern _X_EXPORT void RemoveBlockAndWakeupHandlers(
     BlockHandlerProcPtr /*blockHandler*/,
     WakeupHandlerProcPtr /*wakeupHandler*/,
     pointer /*blockData*/);
 
-extern void InitBlockAndWakeupHandlers(void);
+extern _X_EXPORT void InitBlockAndWakeupHandlers(void);
 
-extern void ProcessWorkQueue(void);
+extern _X_EXPORT void ProcessWorkQueue(void);
 
-extern void ProcessWorkQueueZombies(void);
+extern _X_EXPORT void ProcessWorkQueueZombies(void);
 
-extern Bool QueueWorkProc(
+extern _X_EXPORT Bool QueueWorkProc(
     Bool (* /*function*/)(
         ClientPtr /*clientUnused*/,
         pointer /*closure*/),
@@ -297,77 +270,75 @@ typedef Bool (* ClientSleepProcPtr)(
     ClientPtr /*client*/,
     pointer /*closure*/);
 
-extern Bool ClientSleep(
+extern _X_EXPORT Bool ClientSleep(
     ClientPtr /*client*/,
     ClientSleepProcPtr /* function */,
     pointer /*closure*/);
 
 #ifndef ___CLIENTSIGNAL_DEFINED___
 #define ___CLIENTSIGNAL_DEFINED___
-extern Bool ClientSignal(
+extern _X_EXPORT Bool ClientSignal(
     ClientPtr /*client*/);
 #endif /* ___CLIENTSIGNAL_DEFINED___ */
 
-extern void ClientWakeup(
+extern _X_EXPORT void ClientWakeup(
     ClientPtr /*client*/);
 
-extern Bool ClientIsAsleep(
+extern _X_EXPORT Bool ClientIsAsleep(
     ClientPtr /*client*/);
 
 /* atom.c */
 
-extern Atom MakeAtom(
-    char * /*string*/,
+extern _X_EXPORT Atom MakeAtom(
+    const char * /*string*/,
     unsigned /*len*/,
     Bool /*makeit*/);
 
-extern Bool ValidAtom(
+extern _X_EXPORT Bool ValidAtom(
     Atom /*atom*/);
 
-extern char *NameForAtom(
+extern _X_EXPORT const char *NameForAtom(
     Atom /*atom*/);
 
-extern void AtomError(void);
+extern _X_EXPORT void AtomError(void) _X_NORETURN;
 
-extern void FreeAllAtoms(void);
+extern _X_EXPORT void FreeAllAtoms(void);
 
-extern void InitAtoms(void);
+extern _X_EXPORT void InitAtoms(void);
 
 /* main.c */
 
-extern void SetVendorRelease(int release);
+extern _X_EXPORT void SetVendorRelease(int release);
 
-extern void SetVendorString(char *string);
+extern _X_EXPORT void SetVendorString(char *string);
 
 /* events.c */
 
 extern void SetMaskForEvent(
+    int /* deviceid */,
     Mask /* mask */,
     int /* event */);
 
+extern _X_EXPORT void ConfineToShape(
+    DeviceIntPtr /* pDev */, 
+    RegionPtr /* shape */, 
+    int*      /* px */,
+    int*      /* py */);
 
-extern Bool IsParent(
+extern _X_EXPORT Bool IsParent(
     WindowPtr /* maybeparent */,
     WindowPtr /* child */);
 
-extern WindowPtr GetCurrentRootWindow(void);
+extern _X_EXPORT WindowPtr GetCurrentRootWindow(DeviceIntPtr pDev);
 
-extern WindowPtr GetSpriteWindow(void);
+extern _X_EXPORT WindowPtr GetSpriteWindow(DeviceIntPtr pDev);
 
 
-extern void NoticeEventTime(xEventPtr /* xE */);
+extern _X_EXPORT void NoticeEventTime(InternalEvent *ev);
 
 extern void EnqueueEvent(
-    xEventPtr /* xE */,
-    DeviceIntPtr /* device */,
-    int	/* count */);
-
-extern void ComputeFreezes(void);
-
-extern void CheckGrabForSyncs(
-    DeviceIntPtr /* dev */,
-    Bool /* thisMode */,
-    Bool /* otherMode */);
+    InternalEvent * /* ev */,
+    DeviceIntPtr  /* device */);
 
 extern void ActivatePointerGrab(
     DeviceIntPtr /* mouse */,
@@ -387,6 +358,11 @@ extern void ActivateKeyboardGrab(
 extern void DeactivateKeyboardGrab(
     DeviceIntPtr /* keybd */);
 
+extern BOOL ActivateFocusInGrab(
+    DeviceIntPtr /* dev */,
+    WindowPtr /* old */,
+    WindowPtr /* win */);
+
 extern void AllowSome(
     ClientPtr	/* client */,
     TimeStamp /* time */,
@@ -396,56 +372,67 @@ extern void AllowSome(
 extern void ReleaseActiveGrabs(
     ClientPtr client);
 
-extern int DeliverEventsToWindow(
+extern GrabPtr CheckPassiveGrabsOnWindow(
+    WindowPtr /* pWin */,
+    DeviceIntPtr /* device */,
+    InternalEvent * /* event */,
+    BOOL /* checkCore */,
+    BOOL /* activate */);
+
+extern _X_EXPORT int DeliverEventsToWindow(
+    DeviceIntPtr /* pWin */,
     WindowPtr /* pWin */,
     xEventPtr /* pEvents */,
     int /* count */,
     Mask /* filter */,
-    GrabPtr /* grab */,
-    int /* mskidx */);
+    GrabPtr /* grab */);
+
+extern _X_EXPORT void DeliverRawEvent(
+    RawDeviceEvent* /* ev */,
+    DeviceIntPtr /* dev */
+);
 
 extern int DeliverDeviceEvents(
     WindowPtr /* pWin */,
-    xEventPtr /* xE */,
+    InternalEvent* /* event */,
     GrabPtr /* grab */,
     WindowPtr /* stopAt */,
-    DeviceIntPtr /* dev */,
-    int /* count */);
+    DeviceIntPtr /* dev */);
 
-extern void DefineInitialRootWindow(
-    WindowPtr /* win */);
+extern void InitializeSprite(
+    DeviceIntPtr /* pDev */,
+    WindowPtr    /* pWin */);
 
-extern void WindowHasNewCursor(
+extern void UpdateSpriteForScreen(
+    DeviceIntPtr /* pDev */,
+    ScreenPtr /* pScreen */);
+
+extern _X_EXPORT void WindowHasNewCursor(
     WindowPtr /* pWin */);
 
 extern Bool CheckDeviceGrabs(
     DeviceIntPtr /* device */,
-    xEventPtr /* xE */,
-    int /* checkFirst */,
-    int /* count */);
+    DeviceEvent* /* event */,
+    WindowPtr /* ancestor */);
 
 extern void DeliverFocusedEvent(
     DeviceIntPtr /* keybd */,
-    xEventPtr /* xE */,
-    WindowPtr /* window */,
-    int /* count */);
+    InternalEvent* /* event */,
+    WindowPtr /* window */);
 
-extern void DeliverGrabbedEvent(
-    xEventPtr /* xE */,
+extern int DeliverGrabbedEvent(
+    InternalEvent* /* event */,
     DeviceIntPtr /* thisDev */,
-    Bool /* deactivateGrab */,
-    int /* count */);
+    Bool /* deactivateGrab */);
 
-#ifdef XKB
 extern void FixKeyState(
-    xEvent * /* xE */,
+    DeviceEvent* /* event */,
     DeviceIntPtr /* keybd */);
-#endif /* XKB */
 
 extern void RecalculateDeliverableEvents(
     WindowPtr /* pWin */);
 
-extern int OtherClientGone(
+extern _X_EXPORT int OtherClientGone(
     pointer /* value */,
     XID /* id */);
 
@@ -471,7 +458,10 @@ extern int GrabDevice(
     Window /* grabWindow */,
     unsigned /* ownerEvents */,
     Time /* ctime */,
-    Mask /* mask */,
+    GrabMask* /* mask */,
+    int /* grabtype */,
+    Cursor /* curs */,
+    Window /* confineToWin */,
     CARD8 * /* status */);
 
 extern void InitEvents(void);
@@ -489,102 +479,64 @@ extern Mask EventMaskForClient(
 
 
 
-extern int DeliverEvents(
+extern _X_EXPORT int DeliverEvents(
     WindowPtr /*pWin*/,
     xEventPtr /*xE*/,
     int /*count*/,
     WindowPtr /*otherParent*/);
 
+extern Bool CheckMotion(
+    DeviceEvent* /* ev */,
+    DeviceIntPtr /* pDev */);
 
-extern void WriteEventsToClient(
+extern _X_EXPORT void WriteEventsToClient(
     ClientPtr /*pClient*/,
     int	     /*count*/,
     xEventPtr /*events*/);
 
-extern int TryClientEvents(
+extern _X_EXPORT int TryClientEvents(
     ClientPtr /*client*/,
+    DeviceIntPtr /* device */,
     xEventPtr /*pEvents*/,
     int /*count*/,
     Mask /*mask*/,
     Mask /*filter*/,
     GrabPtr /*grab*/);
 
-extern void WindowsRestructured(void);
+extern _X_EXPORT void WindowsRestructured(void);
+
+extern int SetClientPointer(
+        ClientPtr /* client */,
+        DeviceIntPtr /* device */);
+
+extern _X_EXPORT DeviceIntPtr PickPointer(
+    ClientPtr /* client */);
+
+extern _X_EXPORT DeviceIntPtr PickKeyboard(
+    ClientPtr /* client */);
+
+extern Bool IsInterferingGrab(
+        ClientPtr /* client */,
+        DeviceIntPtr /* dev */,
+        xEvent* /* events */);
 
 #ifdef PANORAMIX
-extern void ReinitializeRootWindow(WindowPtr win, int xoff, int yoff);
+extern _X_EXPORT void ReinitializeRootWindow(WindowPtr win, int xoff, int yoff);
 #endif
 
 #ifdef RANDR
-void
+extern _X_EXPORT void
 ScreenRestructured (ScreenPtr pScreen);
 #endif
 
-extern void ResetClientPrivates(void);
+extern _X_EXPORT int ffs(int i);
 
-extern int AllocateClientPrivateIndex(void);
-
-extern Bool AllocateClientPrivate(
-    int /*index*/,
-    unsigned /*amount*/);
-
-extern int ffs(int i);
-
-/*
- *  callback manager stuff
- */
-
-#ifndef _XTYPEDEF_CALLBACKLISTPTR
-typedef struct _CallbackList *CallbackListPtr; /* also in misc.h */
-#define _XTYPEDEF_CALLBACKLISTPTR
-#endif
-
-typedef void (*CallbackProcPtr) (
-    CallbackListPtr *, pointer, pointer);
-
-typedef Bool (*AddCallbackProcPtr) (
-    CallbackListPtr *, CallbackProcPtr, pointer);
-
-typedef Bool (*DeleteCallbackProcPtr) (
-    CallbackListPtr *, CallbackProcPtr, pointer);
-
-typedef void (*CallCallbacksProcPtr) (
-    CallbackListPtr *, pointer);
-
-typedef void (*DeleteCallbackListProcPtr) (
-    CallbackListPtr *);
-
-typedef struct _CallbackProcs {
-    AddCallbackProcPtr		AddCallback;
-    DeleteCallbackProcPtr	DeleteCallback;
-    CallCallbacksProcPtr	CallCallbacks;
-    DeleteCallbackListProcPtr	DeleteCallbackList;
-} CallbackFuncsRec, *CallbackFuncsPtr;
-
-extern Bool AddCallback(
-    CallbackListPtr * /*pcbl*/,
-    CallbackProcPtr /*callback*/,
-    pointer /*data*/);
-
-extern Bool DeleteCallback(
-    CallbackListPtr * /*pcbl*/,
-    CallbackProcPtr /*callback*/,
-    pointer /*data*/);
-
-extern void CallCallbacks(
-    CallbackListPtr * /*pcbl*/,
-    pointer /*call_data*/);
-
-extern void DeleteCallbackList(
-    CallbackListPtr * /*pcbl*/);
-
-extern void InitCallbackManager(void);
 
 /*
  *  ServerGrabCallback stuff
  */
 
-extern CallbackListPtr ServerGrabCallback;
+extern _X_EXPORT CallbackListPtr ServerGrabCallback;
 
 typedef enum {SERVER_GRABBED, SERVER_UNGRABBED,
 	      CLIENT_PERVIOUS, CLIENT_IMPERVIOUS } ServerGrabState;
@@ -598,7 +550,7 @@ typedef struct {
  *  EventCallback stuff
  */
 
-extern CallbackListPtr EventCallback;
+extern _X_EXPORT CallbackListPtr EventCallback;
 
 typedef struct {
     ClientPtr client;
@@ -610,34 +562,23 @@ typedef struct {
  *  DeviceEventCallback stuff
  */
 
-extern CallbackListPtr DeviceEventCallback;
+extern _X_EXPORT CallbackListPtr DeviceEventCallback;
 
 typedef struct {
-    xEventPtr events;
-    int count;
+    InternalEvent *event;
+    DeviceIntPtr device;
 } DeviceEventInfoRec;
 
-/*
- * SelectionCallback stuff
- */
+extern int XItoCoreType(int xi_type);
+extern Bool DevHasCursor(DeviceIntPtr pDev);
+extern Bool _X_EXPORT IsPointerDevice( DeviceIntPtr dev);
+extern Bool _X_EXPORT IsKeyboardDevice(DeviceIntPtr dev);
+extern Bool IsPointerEvent(InternalEvent *event);
+extern _X_EXPORT Bool IsMaster(DeviceIntPtr dev);
+extern _X_EXPORT Bool IsFloating(DeviceIntPtr dev);
 
-extern CallbackListPtr SelectionCallback;
-
-typedef enum {
-    SelectionSetOwner,
-    SelectionWindowDestroy,
-    SelectionClientClose
-} SelectionCallbackKind;
-
-typedef struct {
-    struct _Selection	    *selection;
-    SelectionCallbackKind   kind;
-} SelectionInfoRec;
-
-/* strcasecmp.c */
-#if NEED_STRCASECMP
-#define strcasecmp xstrcasecmp
-extern int xstrcasecmp(char *s1, char *s2);
-#endif
+extern _X_HIDDEN void CopyKeyClass(DeviceIntPtr device, DeviceIntPtr master);
+extern _X_HIDDEN int CorePointerProc(DeviceIntPtr dev, int what);
+extern _X_HIDDEN int CoreKeyboardProc(DeviceIntPtr dev, int what);
 
 #endif /* DIX_H */
