@@ -1,6 +1,4 @@
 /*
- * $Id: xfixes.h $
- *
  * Copyright © 2002 Keith Packard
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
@@ -31,24 +29,28 @@
 
 #include "resource.h"
 
-extern RESTYPE	RegionResType;
-extern int	XFixesErrorBase;
+extern RESTYPE RegionResType;
+extern int XFixesErrorBase;
 
-#define VERIFY_REGION(pRegion, rid, client, mode) { \
-    pRegion = SecurityLookupIDByType (client, rid, RegionResType, mode); \
-    if (!pRegion) { \
-	client->errorValue = rid; \
-	return XFixesErrorBase + BadRegion; \
-    } \
-}
+#define VERIFY_REGION(pRegion, rid, client, mode)			\
+    do {								\
+	int err;							\
+	err = dixLookupResourceByType((void **) &pRegion, rid,	\
+				      RegionResType, client, mode);	\
+	if (err != Success) {						\
+	    client->errorValue = rid;					\
+	    return err;							\
+	}								\
+    } while (0)
 
 #define VERIFY_REGION_OR_NONE(pRegion, rid, client, mode) { \
     pRegion = 0; \
     if (rid) VERIFY_REGION(pRegion, rid, client, mode); \
 }
 
-RegionPtr
-XFixesRegionCopy (RegionPtr pRegion);
+extern RegionPtr
+ XFixesRegionCopy(RegionPtr pRegion);
 
+#include "xibarriers.h"
 
-#endif /* _XFIXES_H_ */
+#endif                          /* _XFIXES_H_ */
