@@ -30,7 +30,6 @@
 #include "screenint.h"
 #include "regionstr.h"
 #include "miscstruct.h"
-#include "privates.h"
 
 #define GlyphFormat1	0
 #define GlyphFormat4	1
@@ -40,16 +39,13 @@
 #define GlyphFormatNum	5
 
 typedef struct _Glyph {
-    CARD32	    refcnt;
-    PrivateRec	*devPrivates;
-    unsigned char   sha1[20];
-    CARD32	    size; /* info + bitmap */
-    xGlyphInfo	    info;
-    /* per-screen pixmaps follow */
+    CARD32	refcnt;
+    DevUnion	*devPrivates;
+    CARD32	size;	/* info + bitmap */
+    xGlyphInfo	info;
+    /* bits follow */
 } GlyphRec, *GlyphPtr;
 
-#define GlyphPicture(glyph) ((PicturePtr *) ((glyph) + 1))
-
 typedef struct _GlyphRef {
     CARD32	signature;
     GlyphPtr	glyph;
@@ -75,14 +71,18 @@ typedef struct _GlyphSet {
     int		    fdepth;
     GlyphHashRec    hash;
     int             maxPrivate;
-    PrivateRec      *devPrivates;
+    pointer         *devPrivates;
 } GlyphSetRec, *GlyphSetPtr;
 
-#define GlyphSetGetPrivate(pGlyphSet,k)					\
-    dixLookupPrivate(&(pGlyphSet)->devPrivates, k)
+#define GlyphSetGetPrivate(pGlyphSet,n)					\
+	((n) > (pGlyphSet)->maxPrivate ?				\
+	 (pointer) 0 :							\
+	 (pGlyphSet)->devPrivates[n])
 
-#define GlyphSetSetPrivate(pGlyphSet,k,ptr)				\
-    dixSetPrivate(&(pGlyphSet)->devPrivates, k, ptr)
+#define GlyphSetSetPrivate(pGlyphSet,n,ptr)				\
+	((n) > (pGlyphSet)->maxPrivate ?				\
+	 _GlyphSetSetNewPrivate(pGlyphSet, n, ptr) :			\
+	 ((((pGlyphSet)->devPrivates[n] = (ptr)) != 0) || TRUE))
 
 typedef struct _GlyphList {
     INT16	    xOff;
@@ -91,58 +91,77 @@ typedef struct _GlyphList {
     PictFormatPtr   format;
 } GlyphListRec, *GlyphListPtr;
 
-extern _X_EXPORT GlyphHashSetPtr
+extern GlyphHashRec	globalGlyphs[GlyphFormatNum];
+
+GlyphHashSetPtr
 FindGlyphHashSet (CARD32 filled);
 
-extern _X_EXPORT void
+int
+AllocateGlyphSetPrivateIndex (void);
+
+void
+ResetGlyphSetPrivateIndex (void);
+
+Bool
+_GlyphSetSetNewPrivate (GlyphSetPtr glyphSet, int n, pointer ptr);
+
+void
+ResetGlyphPrivates (void);
+
+int
+AllocateGlyphPrivateIndex (void);
+
+Bool
+AllocateGlyphPrivate (ScreenPtr pScreen,
+		      int	index2,
+		      unsigned	amount);
+
+Bool
+GlyphInit (ScreenPtr pScreen);
+
+Bool
+GlyphFinishInit (ScreenPtr pScreen);
+
+void
 GlyphUninit (ScreenPtr pScreen);
 
-extern _X_EXPORT GlyphHashSetPtr
+GlyphHashSetPtr
 FindGlyphHashSet (CARD32 filled);
 
-extern _X_EXPORT GlyphRefPtr
-FindGlyphRef (GlyphHashPtr	hash,
-	      CARD32		signature,
-	      Bool		match,
-	      unsigned char	sha1[20]);
+GlyphRefPtr
+FindGlyphRef (GlyphHashPtr hash, CARD32 signature, Bool match, GlyphPtr compare);
 
-extern _X_EXPORT GlyphPtr
-FindGlyphByHash (unsigned char sha1[20], int format);
+CARD32
+HashGlyph (GlyphPtr glyph);
 
-extern _X_EXPORT int
-HashGlyph (xGlyphInfo    *gi,
-	   CARD8	 *bits,
-	   unsigned long size,
-	   unsigned char sha1[20]);
-
-extern _X_EXPORT void
+void
 FreeGlyph (GlyphPtr glyph, int format);
 
-extern _X_EXPORT void
+void
 AddGlyph (GlyphSetPtr glyphSet, GlyphPtr glyph, Glyph id);
 
-extern _X_EXPORT Bool
+Bool
 DeleteGlyph (GlyphSetPtr glyphSet, Glyph id);
 
-extern _X_EXPORT GlyphPtr
+GlyphPtr
 FindGlyph (GlyphSetPtr glyphSet, Glyph id);
 
-extern _X_EXPORT GlyphPtr
+GlyphPtr
 AllocateGlyph (xGlyphInfo *gi, int format);
 
-extern _X_EXPORT Bool
+Bool
 AllocateGlyphHash (GlyphHashPtr hash, GlyphHashSetPtr hashSet);
 
-extern _X_EXPORT Bool
+Bool
 ResizeGlyphHash (GlyphHashPtr hash, CARD32 change, Bool global);
 
-extern _X_EXPORT Bool
+Bool
 ResizeGlyphSet (GlyphSetPtr glyphSet, CARD32 change);
 
-extern _X_EXPORT GlyphSetPtr
+GlyphSetPtr
 AllocateGlyphSet (int fdepth, PictFormatPtr format);
 
-extern _X_EXPORT int
+int
 FreeGlyphSet (pointer   value,
 	      XID       gid);
 
