@@ -29,13 +29,37 @@
 /*
  * This file contains definitions of the private XFree86 data structures/types.
  * None of the data structures here should be used by video drivers.
- */
+ */ 
 
 #ifndef _XF86PRIVSTR_H
 #define _XF86PRIVSTR_H
 
+#include "xf86Pci.h"
 #include "xf86str.h"
 
+/* PCI probe flags */
+
+typedef enum {
+    PCIProbe1		= 0,
+    PCIProbe2,
+    PCIForceConfig1,
+    PCIForceConfig2,
+    PCIForceNone,
+    PCIOsConfig
+} PciProbeType;
+
+typedef enum {
+    LogNone,
+    LogFlush,
+    LogSync
+} Log;
+
+typedef enum {
+    SKNever,
+    SKWhenNeeded,
+    SKAlways
+} SpecialKeysInDDX;
+
 typedef enum {
     XF86_GlxVisualsMinimal,
     XF86_GlxVisualsTypical,
@@ -49,92 +73,119 @@ typedef enum {
  */
 
 typedef struct {
-    int consoleFd;
-    int vtno;
+    int			consoleFd;
+    int			vtno;
+    Bool		vtSysreq;
+    SpecialKeysInDDX	ddxSpecialKeys;
+
+    /* mouse part */
+    DeviceIntPtr	pMouse;
+#ifdef XINPUT
+    pointer		mouseLocal;
+#endif
 
     /* event handler part */
-    int lastEventTime;
-    Bool vtRequestsPending;
-#ifdef sun
-    int vtPendingNum;
-#endif
-    Bool dontVTSwitch;
-    Bool autoVTSwitch;
-    Bool ShareVTs;
-    Bool dontZap;
-    Bool dontZoom;
-    Bool notrapSignals;         /* don't exit cleanly - die at fault */
-    Bool caughtSignal;
+    int			lastEventTime;
+    Bool		vtRequestsPending;
+    Bool		inputPending;
+    Bool		dontVTSwitch;
+    Bool		dontZap;
+    Bool		dontZoom;
+    Bool		notrapSignals;	/* don't exit cleanly - die at fault */
+    Bool		caughtSignal;
 
     /* graphics part */
-    ScreenPtr currentScreen;
+    Bool		sharedMonitor;
+    ScreenPtr		currentScreen;
 #if defined(CSRG_BASED) || defined(__FreeBSD_kernel__)
-    int consType;               /* Which console driver? */
+    int			screenFd;	/* fd for memory mapped access to
+					 * vga card */
+    int			consType;	/* Which console driver? */
 #endif
 
     /* Other things */
-    Bool allowMouseOpenFail;
-    Bool vidModeEnabled;        /* VidMode extension enabled */
-    Bool vidModeAllowNonLocal;  /* allow non-local VidMode
-                                 * connections */
-    Bool miscModInDevEnabled;   /* Allow input devices to be
-                                 * changed */
-    Bool miscModInDevAllowNonLocal;
-    Bool useSIGIO;              /* Use SIGIO for handling
-                                   input device events */
-    Pix24Flags pixmap24;
-    MessageType pix24From;
-    Bool pmFlag;
-    Bool disableRandR;
-    MessageType randRFrom;
-    Bool aiglx;
-    MessageType aiglxFrom;
-    XF86_GlxVisuals glxVisuals;
-    MessageType glxVisualsFrom;
+    Bool		allowMouseOpenFail;
+    Bool		vidModeEnabled;		/* VidMode extension enabled */
+    Bool		vidModeAllowNonLocal;	/* allow non-local VidMode
+						 * connections */
+    Bool		miscModInDevEnabled;	/* Allow input devices to be
+						 * changed */
+    Bool		miscModInDevAllowNonLocal;
+    PciProbeType	pciFlags;
+    Pix24Flags		pixmap24;
+    MessageType		pix24From;
+#ifdef __i386__
+    Bool		pc98;
+#endif
+    Bool		pmFlag;
+    Log			log;
+    int			estimateSizesAggressively;
+    Bool		kbdCustomKeycodes;
+    Bool		disableRandR;
+    MessageType		randRFrom;
+    Bool		aiglx;
+    MessageType		aiglxFrom;
+    XF86_GlxVisuals	glxVisuals;
+    MessageType		glxVisualsFrom;
+    
+    Bool		useDefaultFontPath;
+    MessageType		useDefaultFontPathFrom;
+    Bool        ignoreABI;
+    struct {
+	Bool		disabled;		/* enable/disable deactivating
+						 * grabs or closing the
+						 * connection to the grabbing
+						 * client */
+	ClientPtr	override;		/* client that disabled
+						 * grab deactivation.
+						 */
+	Bool		allowDeactivate;
+	Bool		allowClosedown;
+	ServerGrabInfoRec server;
+    } grabInfo;
 
-    Bool useDefaultFontPath;
-    Bool ignoreABI;
+    Bool        allowEmptyInput;  /* Allow the server to start with no input
+                                   * devices. */
+    Bool        autoAddDevices; /* Whether to succeed NIDR, or ignore. */
+    Bool        autoEnableDevices; /* Whether to enable, or let the client
+                                    * control. */
 
-    Bool forceInputDevices;     /* force xorg.conf or built-in input devices */
-    Bool autoAddDevices;        /* Whether to succeed NIDR, or ignore. */
-    Bool autoEnableDevices;     /* Whether to enable, or let the client
-                                 * control. */
-
-    Bool dri2;
-    MessageType dri2From;
-
-    Bool autoAddGPU;
+    Bool		dri2;
+    MessageType		dri2From;
 } xf86InfoRec, *xf86InfoPtr;
 
 #ifdef DPMSExtension
 /* Private info for DPMS */
 typedef struct {
-    CloseScreenProcPtr CloseScreen;
-    Bool Enabled;
-    int Flags;
+    CloseScreenProcPtr	CloseScreen;
+    Bool		Enabled;
+    int			Flags;
 } DPMSRec, *DPMSPtr;
 #endif
 
 #ifdef XF86VIDMODE
 /* Private info for Video Mode Extentsion */
 typedef struct {
-    DisplayModePtr First;
-    DisplayModePtr Next;
-    int Flags;
-    CloseScreenProcPtr CloseScreen;
+    DisplayModePtr	First;
+    DisplayModePtr	Next;
+    int			Flags;
+    CloseScreenProcPtr	CloseScreen;
 } VidModeRec, *VidModePtr;
 #endif
 
 /* Information for root window properties. */
 typedef struct _RootWinProp {
-    struct _RootWinProp *next;
-    const char *name;
-    Atom type;
-    short format;
-    long size;
-    void *data;
+    struct _RootWinProp *	next;
+    char *			name;
+    Atom			type;
+    short			format;
+    long			size;
+    pointer			data;
 } RootWinProp, *RootWinPropPtr;
 
+/* private resource types */
+#define ResNoAvoid  ResBios
+
 /* ISC's cc can't handle ~ of UL constants, so explicitly type cast them. */
 #define XLED1   ((unsigned long) 0x00000001)
 #define XLED2   ((unsigned long) 0x00000002)
@@ -155,8 +206,4 @@ typedef struct _RootWinProp {
 #define WSCONS		  32
 #endif
 
-/* Root window property to tell clients whether our VT is currently active.
- * Name chosen to match the "XFree86_VT" property. */
-#define HAS_VT_ATOM_NAME "XFree86_has_VT"
-
-#endif                          /* _XF86PRIVSTR_H */
+#endif /* _XF86PRIVSTR_H */
