@@ -49,9 +49,6 @@ SOFTWARE.
 #ifndef INPUTSTRUCT_H
 #define INPUTSTRUCT_H
 
-#include <X11/extensions/XI2proto.h>
-
-#include <pixman.h>
 #include "input.h"
 #include "window.h"
 #include "dixstruct.h"
@@ -59,10 +56,9 @@ SOFTWARE.
 #include "geext.h"
 #include "privates.h"
 
-#define BitIsOn(ptr, bit) (!!(((const BYTE *) (ptr))[(bit)>>3] & (1 << ((bit) & 7))))
+#define BitIsOn(ptr, bit) (((BYTE *) (ptr))[(bit)>>3] & (1 << ((bit) & 7)))
 #define SetBit(ptr, bit)  (((BYTE *) (ptr))[(bit)>>3] |= (1 << ((bit) & 7)))
 #define ClearBit(ptr, bit) (((BYTE *)(ptr))[(bit)>>3] &= ~(1 << ((bit) & 7)))
-extern _X_EXPORT int CountBits(const uint8_t *mask, int len);
 
 #define SameClient(obj,client) \
 	(CLIENT_BITS((obj)->resource) == (client)->clientAsMask)
@@ -73,18 +69,8 @@ extern _X_EXPORT int CountBits(const uint8_t *mask, int len);
  * events to the protocol, the server will not support these events until
  * this number here is bumped.
  */
-#define XI2LASTEVENT    XI_RawTouchEnd
-#define XI2MASKSIZE     ((XI2LASTEVENT >> 3) + 1) /* no of bytes for masks */
-
-/**
- * Scroll types for ::SetScrollValuator and the scroll type in the
- * ::ScrollInfoPtr.
- */
-enum ScrollType {
-    SCROLL_TYPE_NONE = 0,           /**< Not a scrolling valuator */
-    SCROLL_TYPE_VERTICAL = 8,
-    SCROLL_TYPE_HORIZONTAL = 9,
-};
+#define XI2LASTEVENT    17 /* XI_RawMotion */
+#define XI2MASKSIZE     ((XI2LASTEVENT + 7)/8) /* no of bits for masks */
 
 /**
  * This struct stores the core event mask for each client except the client
@@ -120,7 +106,7 @@ typedef struct _InputClients {
     XID			resource; /**< id for putting into resource manager */
     Mask		mask[EMASKSIZE]; /**< Actual XI event mask, deviceid is index */
     /** XI2 event masks. One per device, each bit is a mask of (1 << type) */
-    struct _XI2Mask     *xi2mask;
+    unsigned char       xi2mask[EMASKSIZE][XI2MASKSIZE];
 } InputClients;
 
 /**
@@ -150,7 +136,7 @@ typedef struct _OtherInputMasks {
     /** The clients that selected for events */
     InputClientsPtr	inputClients;
     /* XI2 event masks. One per device, each bit is a mask of (1 << type) */
-    struct _XI2Mask     *xi2mask;
+    unsigned char       xi2mask[EMASKSIZE][XI2MASKSIZE];
 } OtherInputMasks;
 
 /*
@@ -169,10 +155,16 @@ typedef struct _DetailRec {		/* Grab details may be bit masks */
     Mask                *pMask;
 } DetailRec;
 
+typedef enum {
+    GRABTYPE_CORE,
+    GRABTYPE_XI,
+    GRABTYPE_XI2
+} GrabType;
+
 union _GrabMask {
     Mask core;
     Mask xi;
-    struct _XI2Mask *xi2mask;
+    char xi2mask[EMASKSIZE][XI2MASKSIZE];
 };
 
 /**
@@ -196,7 +188,7 @@ typedef struct _GrabRec {
     unsigned		ownerEvents:1;
     unsigned		keyboardMode:1;
     unsigned		pointerMode:1;
-    enum InputLevel	grabtype;
+    GrabType		grabtype;
     CARD8		type;		/* event type */
     DetailRec		modifiersDetail;
     DeviceIntPtr	modifierDevice;
@@ -206,50 +198,9 @@ typedef struct _GrabRec {
     Mask		eventMask;
     Mask                deviceMask;     
     /* XI2 event masks. One per device, each bit is a mask of (1 << type) */
-    struct _XI2Mask *xi2mask;
+    unsigned char       xi2mask[EMASKSIZE][XI2MASKSIZE];
 } GrabRec;
 
-/**
- * Sprite information for a device.
- */
-typedef struct _SpriteRec {
-    CursorPtr	current;
-    BoxRec	hotLimits;	/* logical constraints of hot spot */
-    Bool	confined;	/* confined to screen */
-    RegionPtr	hotShape;	/* additional logical shape constraint */
-    BoxRec	physLimits;	/* physical constraints of hot spot */
-    WindowPtr	win;		/* window of logical position */
-    HotSpot	hot;		/* logical pointer position */
-    HotSpot	hotPhys;	/* physical pointer position */
-#ifdef PANORAMIX
-    ScreenPtr	screen;		/* all others are in Screen 0 coordinates */
-    RegionRec   Reg1;	        /* Region 1 for confining motion */
-    RegionRec   Reg2;		/* Region 2 for confining virtual motion */
-    WindowPtr   windows[MAXSCREENS];
-    WindowPtr	confineWin;	/* confine window */
-#endif
-    /* The window trace information is used at dix/events.c to avoid having
-     * to compute all the windows between the root and the current pointer
-     * window each time a button or key goes down. The grabs on each of those
-     * windows must be checked.
-     * spriteTraces should only be used at dix/events.c! */
-    WindowPtr *spriteTrace;
-    int spriteTraceSize;
-    int spriteTraceGood;
-
-    /* Due to delays between event generation and event processing, it is
-     * possible that the pointer has crossed screen boundaries between the
-     * time in which it begins generating events and the time when
-     * those events are processed.
-     *
-     * pEnqueueScreen: screen the pointer was on when the event was generated
-     * pDequeueScreen: screen the pointer was on when the event is processed
-     */
-    ScreenPtr pEnqueueScreen;
-    ScreenPtr pDequeueScreen;
-
-} SpriteRec;
-
 typedef struct _KeyClassRec {
     int			sourceid;
     CARD8		down[DOWN_LENGTH];
@@ -258,12 +209,6 @@ typedef struct _KeyClassRec {
     struct _XkbSrvInfo *xkbInfo;
 } KeyClassRec, *KeyClassPtr;
 
-typedef struct _ScrollInfo {
-    enum ScrollType	type;
-    double		increment;
-    int			flags;
-} ScrollInfo, *ScrollInfoPtr;
-
 typedef struct _AxisInfo {
     int		resolution;
     int		min_resolution;
@@ -271,15 +216,12 @@ typedef struct _AxisInfo {
     int		min_value;
     int		max_value;
     Atom	label;
-    CARD8	mode;
-    ScrollInfo  scroll;
 } AxisInfo, *AxisInfoPtr;
 
 typedef struct _ValuatorAccelerationRec {
     int                         number;
     PointerAccelSchemeProc      AccelSchemeProc;
     void                       *accelData; /* at disposal of AccelScheme */
-    PointerAccelSchemeInitProc  AccelInitProc;
     DeviceCallbackProc          AccelCleanupProc;
 } ValuatorAccelerationRec, *ValuatorAccelerationPtr;
 
@@ -295,58 +237,9 @@ typedef struct _ValuatorClassRec {
     AxisInfoPtr 	  axes;
     unsigned short	  numAxes;
     double		  *axisVal; /* always absolute, but device-coord system */
+    CARD8	 	  mode;
     ValuatorAccelerationRec	accelScheme;
-    int                   h_scroll_axis; /* horiz smooth-scrolling axis */
-    int                   v_scroll_axis; /* vert smooth-scrolling axis */
-} ValuatorClassRec;
-
-typedef struct _TouchPointInfo {
-    uint32_t    client_id;          /* touch ID as seen in client events */
-    int         sourceid;           /* Source device's ID for this touchpoint */
-    Bool        active;             /* whether or not the touch is active */
-    Bool        pending_finish;     /* true if the touch is physically inactive
-                                     * but still owned by a grab */
-    SpriteRec   sprite;             /* window trace for delivery */
-    ValuatorMask *valuators;        /* last recorded axis values */
-    struct _TouchListener {
-        XID         listener;           /* grabs/event selection IDs receiving
-                                         * events for this touch */
-        enum TouchListenerType type;
-        enum TouchListenerState state;
-        enum InputLevel level;      /* matters only for emulating touches */
-        WindowPtr window;
-    } *listeners;
-    int         num_listeners;
-    int         num_grabs;          /* number of open grabs on this touch
-                                     * which have not accepted or rejected */
-    Bool        emulate_pointer;
-    DeviceEvent *history;           /* History of events on this touchpoint */
-    size_t      history_elements;   /* Number of current elements in history */
-    size_t      history_size;       /* Size of history in elements */
-} TouchPointInfoRec;
-
-typedef struct _TouchListener TouchListener;
-
-typedef struct _DDXTouchPointInfo {
-    uint32_t    client_id;          /* touch ID as seen in client events */
-    Bool        active;             /* whether or not the touch is active */
-    uint32_t    ddx_id;             /* touch ID given by the DDX */
-    Bool        emulate_pointer;
-
-    ValuatorMask* valuators;        /* last recorded axis values */
-} DDXTouchPointInfoRec;
-
-typedef struct _TouchClassRec {
-    int                sourceid;
-    TouchPointInfoPtr  touches;
-    unsigned short     num_touches;    /* number of allocated touches */
-    unsigned short     max_touches;    /* maximum number of touches, may be 0 */
-    CARD8              mode;           /* ::XIDirectTouch, XIDependentTouch */
-    /* for pointer-emulation */
-    CARD8              buttonsDown;    /* number of buttons down */
-    unsigned short     state;          /* logical button state */
-    Mask               motionMask;
-} TouchClassRec;
+} ValuatorClassRec, *ValuatorClassPtr;
 
 typedef struct _ButtonClassRec {
     int			sourceid;
@@ -377,9 +270,30 @@ typedef struct _FocusClassRec {
 
 typedef struct _ProximityClassRec {
     int		sourceid;
-    char	in_proximity;
+    char	pad;
 } ProximityClassRec, *ProximityClassPtr;
 
+typedef struct _AbsoluteClassRec {
+    int         sourceid;
+    /* Calibration. */
+    int         min_x;
+    int         max_x;
+    int         min_y;
+    int         max_y;
+    int         flip_x;
+    int         flip_y;
+    int		rotation;
+    int         button_threshold;
+
+    /* Area. */
+    int         offset_x;
+    int         offset_y;
+    int         width;
+    int         height;
+    int         screen;
+    XID		following;
+} AbsoluteClassRec, *AbsoluteClassPtr;
+
 typedef struct _KbdFeedbackClassRec *KbdFeedbackPtr;
 typedef struct _PtrFeedbackClassRec *PtrFeedbackPtr;
 typedef struct _IntegerFeedbackClassRec *IntegerFeedbackPtr;
@@ -431,10 +345,10 @@ typedef struct _LedFeedbackClassRec {
 typedef struct _ClassesRec {
     KeyClassPtr		key;
     ValuatorClassPtr	valuator;
-    TouchClassPtr	touch;
     ButtonClassPtr	button;
     FocusClassPtr	focus;
     ProximityClassPtr	proximity;
+    AbsoluteClassPtr    absolute;
     KbdFeedbackPtr	kbdfeed;
     PtrFeedbackPtr	ptrfeed;
     IntegerFeedbackPtr	intfeed;
@@ -444,6 +358,39 @@ typedef struct _ClassesRec {
 } ClassesRec;
 
 
+/**
+ * Sprite information for a device.
+ */
+typedef struct {
+    CursorPtr	current;
+    BoxRec	hotLimits;	/* logical constraints of hot spot */
+    Bool	confined;	/* confined to screen */
+    RegionPtr	hotShape;	/* additional logical shape constraint */
+    BoxRec	physLimits;	/* physical constraints of hot spot */
+    WindowPtr	win;		/* window of logical position */
+    HotSpot	hot;		/* logical pointer position */
+    HotSpot	hotPhys;	/* physical pointer position */
+#ifdef PANORAMIX
+    ScreenPtr	screen;		/* all others are in Screen 0 coordinates */
+    RegionRec   Reg1;	        /* Region 1 for confining motion */
+    RegionRec   Reg2;		/* Region 2 for confining virtual motion */
+    WindowPtr   windows[MAXSCREENS];
+    WindowPtr	confineWin;	/* confine window */ 
+#endif
+    /* The window trace information is used at dix/events.c to avoid having
+     * to compute all the windows between the root and the current pointer
+     * window each time a button or key goes down. The grabs on each of those
+     * windows must be checked.
+     * spriteTraces should only be used at dix/events.c! */
+    WindowPtr *spriteTrace;
+    int spriteTraceSize;
+    int spriteTraceGood;
+
+    ScreenPtr pEnqueueScreen; /* screen events are being delivered to */
+    ScreenPtr pDequeueScreen; /* screen events are being dispatched to */
+
+} SpriteRec, *SpritePtr;
+
 /* Device properties */
 typedef struct _XIPropertyValue
 {
@@ -496,7 +443,7 @@ typedef struct _GrabInfoRec {
     TimeStamp	    grabTime;
     Bool            fromPassiveGrab;    /* true if from passive grab */
     Bool            implicitGrab;       /* implicit from ButtonPress */
-    GrabPtr         activeGrab;
+    GrabRec         activeGrab;
     GrabPtr         grab;
     CARD8           activatingKey;
     void	    (*ActivateGrab) (
@@ -522,30 +469,18 @@ typedef struct _SpriteInfoRec {
     DeviceIntPtr        paired;      /* The paired device. Keyboard if
                                         spriteOwner is TRUE, otherwise the
                                         pointer that owns the sprite. */ 
-
-    /* keep states for animated cursor */
-    struct {
-        CursorPtr       pCursor;
-        ScreenPtr       pScreen;
-        int             elt;
-        CARD32          time;
-    } anim;
 } SpriteInfoRec, *SpriteInfoPtr;
 
 /* device types */
 #define MASTER_POINTER          1
 #define MASTER_KEYBOARD         2
 #define SLAVE                   3
-/* special types for GetMaster */
-#define MASTER_ATTACHED         4       /* Master for this device */
-#define KEYBOARD_OR_FLOAT       5       /* Keyboard master for this device or this device if floating */
-#define POINTER_OR_FLOAT        6       /* Pointer master for this device or this device if floating */
 
 typedef struct _DeviceIntRec {
     DeviceRec	public;
     DeviceIntPtr next;
     Bool	startup;		/* true if needs to be turned on at
-				          server initialization time */
+				          server intialization time */
     DeviceProc	deviceProc;		/* proc(DevicePtr, DEVICE_xx). It is
 					  used to initialize, turn on, or
 					  turn off the device */
@@ -559,10 +494,10 @@ typedef struct _DeviceIntRec {
     int			id;
     KeyClassPtr		key;
     ValuatorClassPtr	valuator;
-    TouchClassPtr	touch;
     ButtonClassPtr	button;
     FocusClassPtr	focus;
     ProximityClassPtr	proximity;
+    AbsoluteClassPtr    absolute;
     KbdFeedbackPtr	kbdfeed;
     PtrFeedbackPtr	ptrfeed;
     IntegerFeedbackPtr	intfeed;
@@ -571,28 +506,26 @@ typedef struct _DeviceIntRec {
     LedFeedbackPtr	leds;
     struct _XkbInterest *xkb_interest;
     char                *config_info; /* used by the hotplug layer */
-    ClassesPtr		unused_classes; /* for master devices */
-    int			saved_master_id;	/* for slaves while grabbed */
     PrivateRec		*devPrivates;
+    int			nPrivates;
     DeviceUnwrapProc    unwrapProc;
     SpriteInfoPtr       spriteInfo;
-    DeviceIntPtr        master;     /* master device */
-    DeviceIntPtr        lastSlave;  /* last slave device used */
+    union {
+        DeviceIntPtr        master;     /* master device */
+        DeviceIntPtr        lastSlave;  /* last slave device used */
+    } u;
 
     /* last valuator values recorded, not posted to client;
-     * for slave devices, valuators is in device coordinates, mapped to the
-     * desktop
-     * for master devices, valuators is in desktop coordinates.
+     * for slave devices, valuators is in device coordinates
+     * for master devices, valuators is in screen coordinates
      * see dix/getevents.c
      * remainder supports acceleration
      */
     struct {
-        double          valuators[MAX_VALUATORS];
+        int             valuators[MAX_VALUATORS];
+        float           remainder[MAX_VALUATORS];
         int             numValuators;
         DeviceIntPtr    slave;
-        ValuatorMask    *scroll;
-        int             num_touches; /* size of the touches array */
-        DDXTouchPointInfoPtr touches;
     } last;
 
     /* Input device property handling. */
@@ -600,12 +533,6 @@ typedef struct _DeviceIntRec {
         XIPropertyPtr   properties;
         XIPropertyHandlerPtr handlers; /* NULL-terminated */
     } properties;
-
-    /* coordinate transformation matrix for absolute input devices */
-    struct pixman_f_transform transform;
-
-    /* XTest related master device id */
-    int xtest_master_id;
 } DeviceIntRec;
 
 typedef struct {
@@ -623,7 +550,7 @@ extern _X_EXPORT InputInfo inputInfo;
 /* for keeping the events for devices grabbed synchronously */
 typedef struct _QdEvent *QdEventPtr;
 typedef struct _QdEvent {
-    struct xorg_list	next;
+    QdEventPtr		next;
     DeviceIntPtr	device;
     ScreenPtr		pScreen;	/* what screen the pointer was on */
     unsigned long	months;		/* milliseconds is in the event */
@@ -639,8 +566,8 @@ typedef struct _QdEvent {
  * replayed and processed as if they would come from the device directly.
  */
 typedef struct _EventSyncInfo {
-    struct xorg_list    pending;
-
+    QdEventPtr          pending, /**<  list of queued events */
+                        *pendtail; /**< last event in list */
     /** The device to replay events for. Only set in AllowEvents(), in which
      * case it is set to the device specified in the request. */
     DeviceIntPtr        replayDev;      /* kludgy rock to put flag for */
@@ -660,20 +587,4 @@ typedef struct _EventSyncInfo {
 
 extern EventSyncInfoRec syncEvents;
 
-/**
- * Given a sprite, returns the window at the bottom of the trace (i.e. the
- * furthest window from the root).
- */
-static inline WindowPtr DeepestSpriteWin(SpritePtr sprite)
-{
-    assert(sprite->spriteTraceGood > 0);
-    return sprite->spriteTrace[sprite->spriteTraceGood - 1];
-}
-
-struct _XI2Mask {
-    unsigned char **masks;      /* event mask in masks[deviceid][event type byte] */
-    size_t nmasks;              /* number of masks */
-    size_t mask_size;           /* size of each mask in bytes */
-};
-
 #endif /* INPUTSTRUCT_H */
