@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2022 Oracle and/or its affiliates.
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -352,18 +352,31 @@ static int dbgcProcessCommands(PDBGC pDbgc, bool fNoExecute)
         /*
          * Copy the command to the parse buffer.
          */
+        char    chQuote = 0;
         char    ch;
         char   *psz = &pDbgc->achInput[pDbgc->iRead];
         char   *pszTrg = &pDbgc->achScratch[0];
-        while ((*pszTrg = ch = *psz++) != ';' && ch != '\n' )
+        AssertCompile(sizeof(pDbgc->achScratch) > sizeof(pDbgc->achInput));
+        while ((ch = *psz++) != '\0')
         {
+            /* ';' and '\n' are termination characters, except for when they are
+               inside quotes.  So, track quoting. */
+            if (ch == '"' || ch == '\'')
+                chQuote = chQuote == ch ? 0 : chQuote == 0 ? ch : chQuote;
+            else if ((ch == ';' || ch == '\n') && chQuote == 0)
+                break;
+
+            *pszTrg = ch;
+
             if (psz == &pDbgc->achInput[sizeof(pDbgc->achInput)])
                 psz = &pDbgc->achInput[0];
 
+            /** @todo r=bird: off by one issue here? */
             if (psz == &pDbgc->achInput[pDbgc->iWrite])
             {
                 AssertMsgFailed(("The buffer contains no commands while cInputLines=%d!\n", pDbgc->cInputLines));
                 pDbgc->cInputLines = 0;
+                pDbgc->iRead = pDbgc->iWrite;
                 return 0;
             }
 
@@ -383,7 +396,7 @@ static int dbgcProcessCommands(PDBGC pDbgc, bool fNoExecute)
          */
         pDbgc->pszScratch = pszTrg + 1;
         pDbgc->iArg       = 0;
-        rc = dbgcEvalCommand(pDbgc, &pDbgc->achScratch[0], pszTrg - &pDbgc->achScratch[0] - 1, fNoExecute);
+        rc = dbgcEvalCommand(pDbgc, &pDbgc->achScratch[0], pszTrg - &pDbgc->achScratch[0], fNoExecute);
         if (   rc == VERR_DBGC_QUIT
             || rc == VWRN_DBGC_CMD_PENDING)
             break;
@@ -864,13 +877,17 @@ static int dbgcProcessEvent(PDBGC pDbgc, PCDBGFEVENT pEvent)
                     else
                         rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, "\ndbgf event/%u: %s!",
                                                      pEvent->idCpu, pEvtDesc->pszName);
-                    if (pEvent->u.Generic.cArgs <= 1)
-                        rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, " arg=%#llx\n", pEvent->u.Generic.auArgs[0]);
-                    else
+                    if (RT_SUCCESS(rc))
                     {
-                        for (uint32_t i = 0; i < pEvent->u.Generic.cArgs; i++)
-                            rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, " args[%u]=%#llx", i, pEvent->u.Generic.auArgs[i]);
-                        rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, "\n");
+                        if (pEvent->u.Generic.cArgs <= 1)
+                            rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, " arg=%#llx\n", pEvent->u.Generic.auArgs[0]);
+                        else
+                        {
+                            for (uint32_t i = 0; i < pEvent->u.Generic.cArgs; i++)
+                                rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, " args[%u]=%#llx", i, pEvent->u.Generic.auArgs[i]);
+                            if (RT_SUCCESS(rc))
+                                rc = pDbgc->CmdHlp.pfnPrintf(&pDbgc->CmdHlp, NULL, "\n");
+                        }
                     }
                 }
                 else
