@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /*
@@ -83,6 +93,7 @@ remque(PNATState pData, void *a)
 }
 
 #ifndef VBOX_NAT_TST_QUEUE
+
 /*
  * Set fd blocking and non-blocking
  */
@@ -90,8 +101,11 @@ void
 fd_nonblock(int fd)
 {
 # ifdef FIONBIO
+#  ifdef RT_OS_WINDOWS
+    u_long opt = 1;
+#  else
     int opt = 1;
-
+#  endif
     ioctlsocket(fd, FIONBIO, &opt);
 # else /* !FIONBIO */
     int opt;
@@ -102,22 +116,17 @@ fd_nonblock(int fd)
 # endif
 }
 
-# if !defined(VBOX_NAT_MEM_DEBUG)
-#  if defined (LOG_ENABLED)
-#   undef LogFlowFunc
-#   define LogFlowFunc(x)
 
-#   undef LogFlowFuncEnter
-#   define LogFlowFuncEnter()
-
-#   undef LogFlowFuncLeave
-#   define LogFlowFuncLeave()
-
-#   undef Log2
-#   define Log2(x)
-#  endif /* !LOG_ENABLED */
-# else /* VBOX_NAT_MEM_DEBUG */
-# define NAT_MEM_LOG_ENABLED
+# if defined(VBOX_NAT_MEM_DEBUG)
+#  define NATMEM_LOG_FLOW_FUNC(a)        LogFlowFunc(a)
+#  define NATMEM_LOG_FLOW_FUNC_ENTER()   LogFlowFuncEnter()
+#  define NATMEM_LOG_FLOW_FUNC_LEAVE()   LogFlowFuncLeave()
+#  define NATMEM_LOG_2(a)                Log2(a)
+# else
+#  define NATMEM_LOG_FLOW_FUNC(a)        do { } while (0)
+#  define NATMEM_LOG_FLOW_FUNC_ENTER()   do { } while (0)
+#  define NATMEM_LOG_FLOW_FUNC_LEAVE()   do { } while (0)
+#  define NATMEM_LOG_2(a)                do { } while (0)
 # endif
 
 
@@ -132,7 +141,7 @@ fd_nonblock(int fd)
  */
 DECLINLINE(void) slirp_zone_check_and_send_pending(uma_zone_t zone)
 {
-    LogFlowFunc(("ENTER: zone:%R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone]\n", zone));
     if (   zone->fDoXmitPending
         && zone->master_zone == NULL)
     {
@@ -144,7 +153,7 @@ DECLINLINE(void) slirp_zone_check_and_send_pending(uma_zone_t zone)
 
         rc2 = RTCritSectEnter(&zone->csZone); AssertRC(rc2);
     }
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 static void *slirp_uma_alloc(uma_zone_t zone,
@@ -155,12 +164,7 @@ static void *slirp_uma_alloc(uma_zone_t zone,
     void *ret = NULL;
     int rc;
 
-    LogFlowFunc(("ENTER: %R[mzone], size:%d, pflags:%p, %RTbool\n", zone, size, pflags, fWait));
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(size);
-    NOREF(pflags);
-    NOREF(fWait);
-# endif
+    NATMEM_LOG_FLOW_FUNC(("ENTER: %R[mzone], size:%d, pflags:%p, %RTbool\n", zone, size, pflags, fWait)); RT_NOREF(size, pflags, fWait);
     RTCritSectEnter(&zone->csZone);
     for (;;)
     {
@@ -170,7 +174,7 @@ static void *slirp_uma_alloc(uma_zone_t zone,
             Assert(it->magic == ITEM_MAGIC);
             rc = 0;
             if (zone->pfInit)
-                rc = zone->pfInit(zone->pData, (void *)&it[1], zone->size, M_DONTWAIT);
+                rc = zone->pfInit(zone->pData, (void *)&it[1], (int /*sigh*/)zone->size, M_DONTWAIT);
             if (rc == 0)
             {
                 zone->cur_items++;
@@ -190,7 +194,7 @@ static void *slirp_uma_alloc(uma_zone_t zone,
         if (!zone->master_zone)
         {
             /* We're on the master zone and we can't allocate more. */
-            Log2(("NAT: no room on %s zone\n", zone->name));
+            NATMEM_LOG_2(("NAT: no room on %s zone\n", zone->name));
             /* AssertMsgFailed(("NAT: OOM!")); */
             zone->fDoXmitPending = true;
             break;
@@ -199,11 +203,11 @@ static void *slirp_uma_alloc(uma_zone_t zone,
         /* we're on a sub-zone, we need get a chunk from the master zone and split
          * it into sub-zone conforming chunks.
          */
-        sub_area = slirp_uma_alloc(zone->master_zone, zone->master_zone->size, NULL, 0);
+        sub_area = slirp_uma_alloc(zone->master_zone, (int /*sigh*/)zone->master_zone->size, NULL, 0);
         if (!sub_area)
         {
             /* No room on master */
-            Log2(("NAT: no room on %s zone for %s zone\n", zone->master_zone->name, zone->name));
+            NATMEM_LOG_2(("NAT: no room on %s zone for %s zone\n", zone->master_zone->name, zone->name));
             break;
         }
         zone->max_items++;
@@ -226,7 +230,7 @@ static void *slirp_uma_alloc(uma_zone_t zone,
             LogRel(("NAT: Zone(%s) has reached it maximum\n", zone->name));
     }
     RTCritSectLeave(&zone->csZone);
-    LogFlowFunc(("LEAVE: %p\n", ret));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %p\n", ret));
     return ret;
 }
 
@@ -234,14 +238,10 @@ static void slirp_uma_free(void *item, int size, uint8_t flags)
 {
     struct item *it;
     uma_zone_t zone;
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(size);
-    NOREF(flags);
-# endif
 
     Assert(item);
     it = &((struct item *)item)[-1];
-    LogFlowFunc(("ENTER: item:%p(%R[mzoneitem]), size:%d, flags:%RX8\n", item, it, size, flags));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: item:%p(%R[mzoneitem]), size:%d, flags:%RX8\n", item, it, size, flags)); RT_NOREF(size, flags);
     Assert(it->magic == ITEM_MAGIC);
     zone = it->zone;
     /* check border magic */
@@ -252,29 +252,25 @@ static void slirp_uma_free(void *item, int size, uint8_t flags)
     LIST_REMOVE(it, list);
     if (zone->pfFini)
     {
-        zone->pfFini(zone->pData, item, zone->size);
+        zone->pfFini(zone->pData, item, (int /*sigh*/)zone->size);
     }
     if (zone->pfDtor)
     {
-        zone->pfDtor(zone->pData, item, zone->size, NULL);
+        zone->pfDtor(zone->pData, item, (int /*sigh*/)zone->size, NULL);
     }
     LIST_INSERT_HEAD(&zone->free_items, it, list);
     zone->cur_items--;
     slirp_zone_check_and_send_pending(zone); /* may exit+enter the cs! */
     RTCritSectLeave(&zone->csZone);
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 uma_zone_t uma_zcreate(PNATState pData, char *name, size_t size,
                        ctor_t ctor, dtor_t dtor, zinit_t init, zfini_t fini, int flags1, int flags2)
 {
     uma_zone_t zone = NULL;
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(flags1);
-    NOREF(flags2);
-# endif
-    LogFlowFunc(("ENTER: name:%s size:%d, ctor:%p, dtor:%p, init:%p, fini:%p, flags1:%RX32, flags2:%RX32\n",
-                name, ctor, dtor, init, fini, flags1, flags2));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: name:%s size:%d, ctor:%p, dtor:%p, init:%p, fini:%p, flags1:%RX32, flags2:%RX32\n",
+                name, ctor, dtor, init, fini, flags1, flags2));  RT_NOREF(flags1, flags2);
     zone = RTMemAllocZ(sizeof(struct uma_zone));
     Assert((pData));
     zone->magic = ZONE_MAGIC;
@@ -288,7 +284,7 @@ uma_zone_t uma_zcreate(PNATState pData, char *name, size_t size,
     zone->pfAlloc = slirp_uma_alloc;
     zone->pfFree = slirp_uma_free;
     RTCritSectInit(&zone->csZone);
-    LogFlowFunc(("LEAVE: %R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %R[mzone]\n", zone));
     return zone;
 
 }
@@ -297,12 +293,12 @@ uma_zone_t uma_zsecond_create(char *name, ctor_t ctor,
 {
     uma_zone_t zone;
     Assert(master);
-    LogFlowFunc(("ENTER: name:%s ctor:%p, dtor:%p, init:%p, fini:%p, master:%R[mzone]\n",
+    NATMEM_LOG_FLOW_FUNC(("ENTER: name:%s ctor:%p, dtor:%p, init:%p, fini:%p, master:%R[mzone]\n",
                 name, ctor, dtor, init, fini, master));
     zone = RTMemAllocZ(sizeof(struct uma_zone));
     if (zone == NULL)
     {
-        LogFlowFunc(("LEAVE: %R[mzone]\n", NULL));
+        NATMEM_LOG_FLOW_FUNC(("LEAVE: %R[mzone]\n", NULL));
         return NULL;
     }
 
@@ -319,7 +315,7 @@ uma_zone_t uma_zsecond_create(char *name, ctor_t ctor,
     zone->size = master->size;
     zone->master_zone = master;
     RTCritSectInit(&zone->csZone);
-    LogFlowFunc(("LEAVE: %R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %R[mzone]\n", zone));
     return zone;
 }
 
@@ -327,7 +323,7 @@ void uma_zone_set_max(uma_zone_t zone, int max)
 {
     int i = 0;
     struct item *it;
-    LogFlowFunc(("ENTER: zone:%R[mzone], max:%d\n", zone, max));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], max:%d\n", zone, max));
     zone->max_items = max;
     zone->area = RTMemAllocZ(max * (sizeof(struct item) + zone->size + sizeof(uint32_t)));
     for (; i < max; ++i)
@@ -338,21 +334,21 @@ void uma_zone_set_max(uma_zone_t zone, int max)
         *(uint32_t *)(((uint8_t *)&it[1]) + zone->size) = 0xabadbabe;
         LIST_INSERT_HEAD(&zone->free_items, it, list);
     }
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void uma_zone_set_allocf(uma_zone_t zone, uma_alloc_t pfAlloc)
 {
-    LogFlowFunc(("ENTER: zone:%R[mzone], pfAlloc:%Rfn\n", zone, pfAlloc));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], pfAlloc:%Rfn\n", zone, pfAlloc));
     zone->pfAlloc = pfAlloc;
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void uma_zone_set_freef(uma_zone_t zone, uma_free_t pfFree)
 {
-    LogFlowFunc(("ENTER: zone:%R[mzone], pfAlloc:%Rfn\n", zone, pfFree));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], pfAlloc:%Rfn\n", zone, pfFree));
     zone->pfFree = pfFree;
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 uint32_t *uma_find_refcnt(uma_zone_t zone, void *mem)
@@ -360,49 +356,43 @@ uint32_t *uma_find_refcnt(uma_zone_t zone, void *mem)
     /** @todo (vvl) this function supposed to work with special zone storing
     reference counters */
     struct item *it = NULL;
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(zone);
-# endif
-    LogFlowFunc(("ENTER: zone:%R[mzone], mem:%p\n", zone, mem));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], mem:%p\n", zone, mem)); RT_NOREF(zone);
     it = (struct item *)mem; /* 1st element */
     Assert(mem != NULL);
     Assert(zone->magic == ZONE_MAGIC);
     /* for returning pointer to counter we need get 0 elemnt */
     Assert(it[-1].magic == ITEM_MAGIC);
-    LogFlowFunc(("LEAVE: %p\n", &it[-1].ref_count));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %p\n", &it[-1].ref_count));
     return &it[-1].ref_count;
 }
 
 void *uma_zalloc_arg(uma_zone_t zone, void *args, int how)
 {
     void *mem;
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(how);
-# endif
     Assert(zone->magic == ZONE_MAGIC);
-    LogFlowFunc(("ENTER: zone:%R[mzone], args:%p, how:%RX32\n", zone, args, how));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], args:%p, how:%RX32\n", zone, args, how)); RT_NOREF(how);
     if (zone->pfAlloc == NULL)
     {
-        LogFlowFunc(("LEAVE: NULL\n"));
+        NATMEM_LOG_FLOW_FUNC(("LEAVE: NULL\n"));
         return NULL;
     }
     RTCritSectEnter(&zone->csZone);
-    mem = zone->pfAlloc(zone, zone->size, NULL, 0);
+    mem = zone->pfAlloc(zone, (int /*sigh*/)zone->size, NULL, 0);
     if (mem != NULL)
     {
         if (zone->pfCtor)
-            zone->pfCtor(zone->pData, mem, zone->size, args, M_DONTWAIT);
+            zone->pfCtor(zone->pData, mem, (int /*sigh*/)zone->size, args, M_DONTWAIT);
     }
     RTCritSectLeave(&zone->csZone);
-    LogFlowFunc(("LEAVE: %p\n", mem));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %p\n", mem));
     return mem;
 }
 
 void uma_zfree(uma_zone_t zone, void *item)
 {
-    LogFlowFunc(("ENTER: zone:%R[mzone], item:%p\n", zone, item));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], item:%p\n", zone, item));
     uma_zfree_arg(zone, item, NULL);
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void uma_zfree_arg(uma_zone_t zone, void *mem, void *flags)
@@ -411,29 +401,27 @@ void uma_zfree_arg(uma_zone_t zone, void *mem, void *flags)
     Assert(zone->magic == ZONE_MAGIC);
     Assert((zone->pfFree));
     Assert((mem));
-    LogFlowFunc(("ENTER: zone:%R[mzone], mem:%p, flags:%p\n", zone, mem, flags));
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(flags);
-# endif
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], mem:%p, flags:%p\n", zone, mem, flags)); RT_NOREF(flags);
 
     RTCritSectEnter(&zone->csZone);
     it = &((struct item *)mem)[-1];
     Assert((it->magic == ITEM_MAGIC));
     Assert((zone->magic == ZONE_MAGIC && zone == it->zone));
 
-    zone->pfFree(mem,  0, 0);
+    zone->pfFree(mem, 0, 0);
     RTCritSectLeave(&zone->csZone);
-    LogFlowFuncLeave();
+
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 int uma_zone_exhausted_nolock(uma_zone_t zone)
 {
     int fExhausted;
-    LogFlowFunc(("ENTER: zone:%R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone]\n", zone));
     RTCritSectEnter(&zone->csZone);
     fExhausted = (zone->cur_items == zone->max_items);
     RTCritSectLeave(&zone->csZone);
-    LogFlowFunc(("LEAVE: %RTbool\n", fExhausted));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %RTbool\n", fExhausted));
     return fExhausted;
 }
 
@@ -444,7 +432,7 @@ void zone_drain(uma_zone_t zone)
 
     /* vvl: Huh? What to do with zone which hasn't got backstore ? */
     Assert((zone->master_zone));
-    LogFlowFunc(("ENTER: zone:%R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone]\n", zone));
     master_zone = zone->master_zone;
     while (!LIST_EMPTY(&zone->free_items))
     {
@@ -464,37 +452,35 @@ void zone_drain(uma_zone_t zone)
         slirp_zone_check_and_send_pending(master_zone); /* may exit+enter the cs! */
         RTCritSectLeave(&master_zone->csZone);
     }
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void slirp_null_arg_free(void *mem, void *arg)
 {
     /** @todo (vvl) make it wiser  */
-    LogFlowFunc(("ENTER: mem:%p, arg:%p\n", mem, arg));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: mem:%p, arg:%p\n", mem, arg));
+    RT_NOREF(arg);
     Assert(mem);
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(arg);
-# endif
     RTMemFree(mem);
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void *uma_zalloc(uma_zone_t zone, int len)
 {
-# ifndef NAT_MEM_LOG_ENABLED
-    NOREF(zone);
-    NOREF(len);
-# endif
-    LogFlowFunc(("ENTER: zone:%R[mzone], len:%d\n", zone, len));
-    LogFlowFunc(("LEAVE: NULL"));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone], len:%d\n", zone, len));
+    RT_NOREF(zone, len);
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: NULL"));
     return NULL;
 }
 
 struct mbuf *slirp_ext_m_get(PNATState pData, size_t cbMin, void **ppvBuf, size_t *pcbBuf)
 {
     struct mbuf *m;
-    size_t size = MCLBYTES;
-    LogFlowFunc(("ENTER: cbMin:%d, ppvBuf:%p, pcbBuf:%p\n", cbMin, ppvBuf, pcbBuf));
+    int size = MCLBYTES;
+    NATMEM_LOG_FLOW_FUNC(("ENTER: cbMin:%d, ppvBuf:%p, pcbBuf:%p\n", cbMin, ppvBuf, pcbBuf));
+
+    *ppvBuf = NULL;
+    *pcbBuf = 0;
 
     if (cbMin < MCLBYTES)
         size = MCLBYTES;
@@ -503,49 +489,51 @@ struct mbuf *slirp_ext_m_get(PNATState pData, size_t cbMin, void **ppvBuf, size_
     else if (cbMin < MJUM16BYTES)
         size = MJUM16BYTES;
     else
-        AssertMsgFailed(("Unsupported size"));
+    {
+        AssertMsgFailed(("Unsupported size %zu", cbMin));
+        NATMEM_LOG_FLOW_FUNC(("LEAVE: NULL (bad size %zu)\n", cbMin));
+        return NULL;
+    }
 
     m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
     if (m == NULL)
     {
-        *ppvBuf = NULL;
-        *pcbBuf = 0;
-        LogFlowFunc(("LEAVE: NULL\n"));
+        NATMEM_LOG_FLOW_FUNC(("LEAVE: NULL\n"));
         return NULL;
     }
     m->m_len = size;
     *ppvBuf = mtod(m, void *);
     *pcbBuf = size;
-    LogFlowFunc(("LEAVE: %p\n", m));
+    NATMEM_LOG_FLOW_FUNC(("LEAVE: %p\n", m));
     return m;
 }
 
 void slirp_ext_m_free(PNATState pData, struct mbuf *m, uint8_t *pu8Buf)
 {
 
-    LogFlowFunc(("ENTER: m:%p, pu8Buf:%p\n", m, pu8Buf));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: m:%p, pu8Buf:%p\n", m, pu8Buf));
     if (   !pu8Buf
         && pu8Buf != mtod(m, uint8_t *))
         RTMemFree(pu8Buf); /* This buffer was allocated on heap */
     m_freem(pData, m);
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 static void zone_destroy(uma_zone_t zone)
 {
     RTCritSectEnter(&zone->csZone);
-    LogFlowFunc(("ENTER: zone:%R[mzone]\n", zone));
+    NATMEM_LOG_FLOW_FUNC(("ENTER: zone:%R[mzone]\n", zone));
     LogRel(("NAT: Zone(nm:%s, used:%d)\n", zone->name, zone->cur_items));
     RTMemFree(zone->area);
     RTCritSectLeave(&zone->csZone);
     RTCritSectDelete(&zone->csZone);
     RTMemFree(zone);
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void m_fini(PNATState pData)
 {
-    LogFlowFuncEnter();
+    NATMEM_LOG_FLOW_FUNC_ENTER();
 # define ZONE_DESTROY(zone) do { zone_destroy((zone)); (zone) = NULL;} while (0)
     ZONE_DESTROY(pData->zone_clust);
     ZONE_DESTROY(pData->zone_pack);
@@ -556,7 +544,7 @@ void m_fini(PNATState pData)
     ZONE_DESTROY(pData->zone_ext_refcnt);
 # undef ZONE_DESTROY
     /** @todo do finalize here.*/
-    LogFlowFuncLeave();
+    NATMEM_LOG_FLOW_FUNC_LEAVE();
 }
 
 void
@@ -568,4 +556,5 @@ if_init(PNATState pData)
     if_mtu = 1500;
     if_mru = 1500;
 }
+
 #endif /* VBOX_NAT_TST_QUEUE */
