@@ -7,21 +7,31 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define IN_INTNET_TESTCASE
 #define IN_INTNET_R3
 #include <VBox/cdefs.h>
@@ -47,9 +57,9 @@ typedef void *MYPSUPDRVSESSION;
 #include <iprt/test.h>
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * Security objectype.
  */
@@ -75,7 +85,7 @@ typedef enum SUPDRVOBJTYPE
  * @param   pvUser1     The first user argument.
  * @param   pvUser2     The second user argument.
  */
-typedef DECLCALLBACK(void) FNSUPDRVDESTRUCTOR(void *pvObj, void *pvUser1, void *pvUser2);
+typedef DECLCALLBACKTYPE(void, FNSUPDRVDESTRUCTOR,(void *pvObj, void *pvUser1, void *pvUser2));
 /** Pointer to a FNSUPDRVDESTRUCTOR(). */
 typedef FNSUPDRVDESTRUCTOR *PFNSUPDRVDESTRUCTOR;
 
@@ -92,15 +102,15 @@ typedef struct OBJREF
 } OBJREF, *POBJREF;
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** The test handle.*/
 static RTTEST           g_hTest      = NIL_RTTEST;
 /** The size (in bytes) of the large transfer tests. */
 static uint32_t         g_cbTransfer = _1M * 384;
 /** Fake session handle. */
-const PSUPDRVSESSION    g_pSession   = (PSUPDRVSESSION)0xdeadface;
+const PSUPDRVSESSION    g_pSession   = (PSUPDRVSESSION)(uintptr_t)0xdeadface;
 
 
 INTNETR3DECL(void *) SUPR0ObjRegister(PSUPDRVSESSION pSession, SUPDRVOBJTYPE enmType,
@@ -193,9 +203,13 @@ INTNETR3DECL(int) SUPR0MemFree(PSUPDRVSESSION pSession, RTHCUINTPTR uPtr)
 static int tstIntNetSendBuf(PINTNETRINGBUF pRingBuf, INTNETIFHANDLE hIf,
                             PSUPDRVSESSION pSession, void const *pvBuf, size_t cbBuf)
 {
-    INTNETSG Sg;
-    IntNetSgInitTemp(&Sg, (void *)pvBuf, cbBuf);
-    int rc = intnetR0RingWriteFrame(pRingBuf, &Sg, NULL);
+    union
+    {
+        uint8_t abBuf[sizeof(INTNETSG) + sizeof(INTNETSEG)];
+        INTNETSG SG;
+    } u;
+    IntNetSgInitTemp(&u.SG, (void *)pvBuf, (uint32_t)cbBuf);
+    int rc = intnetR0RingWriteFrame(pRingBuf, &u.SG, NULL);
     if (RT_SUCCESS(rc))
         rc = IntNetR0IfSend(hIf, pSession);
     return rc;
@@ -210,6 +224,8 @@ typedef struct MYARGS
     uint32_t        cbFrame;
     uint64_t        u64Start;
     uint64_t        u64End;
+    uint32_t        cbSent;
+    uint32_t        cFramesSent;
 } MYARGS, *PMYARGS;
 
 
@@ -230,7 +246,7 @@ typedef struct MYFRAMEHDR
  * Send thread.
  * This is constantly sending frames to the other interface.
  */
-DECLCALLBACK(int) SendThread(RTTHREAD hThreadSelf, void *pvArg)
+static DECLCALLBACK(int) SendThread(RTTHREAD hThreadSelf, void *pvArg)
 {
     PMYARGS pArgs = (PMYARGS)pvArg;
     int rc;
@@ -243,6 +259,7 @@ DECLCALLBACK(int) SendThread(RTTHREAD hThreadSelf, void *pvArg)
     MYFRAMEHDR     *pHdr    = (MYFRAMEHDR *)&abBuf[0];
     uint32_t        iFrame  = 0;
     uint32_t        cbSent  = 0;
+    uint32_t        cErrors = 0;
 
     pHdr->SrcMac            = pArgs->Mac;
     pHdr->DstMac            = pArgs->Mac;
@@ -256,13 +273,25 @@ DECLCALLBACK(int) SendThread(RTTHREAD hThreadSelf, void *pvArg)
                           : iFrame % 1519 + sizeof(RTMAC) * 2 + sizeof(unsigned);
         pHdr->iFrame = iFrame;
 
-        INTNETSG Sg;
-        IntNetSgInitTemp(&Sg, abBuf, cb);
-        RTTEST_CHECK_RC_OK(g_hTest, rc = intnetR0RingWriteFrame(&pArgs->pBuf->Send, &Sg, NULL));
+        union
+        {
+            uint8_t abBuf[sizeof(INTNETSG) + sizeof(INTNETSEG)];
+            INTNETSG SG;
+        } u;
+        IntNetSgInitTemp(&u.SG, abBuf, cb);
+        RTTEST_CHECK_RC_OK(g_hTest, rc = intnetR0RingWriteFrame(&pArgs->pBuf->Send, &u.SG, NULL));
         if (RT_SUCCESS(rc))
             RTTEST_CHECK_RC_OK(g_hTest, rc = IntNetR0IfSend(pArgs->hIf, g_pSession));
+        if (RT_FAILURE(rc) && ++cErrors > 64)
+        {
+            RTTestFailed(g_hTest, "Aborting xmit after >64 errors");
+            break;
+        }
+
         cbSent += cb;
     }
+    pArgs->cbSent      = cbSent;
+    pArgs->cFramesSent = iFrame;
 
     /*
      * Termination frames.
@@ -293,7 +322,7 @@ DECLCALLBACK(int) SendThread(RTTHREAD hThreadSelf, void *pvArg)
  * Receive thread.
  * This is reading stuff from the network.
  */
-DECLCALLBACK(int) ReceiveThread(RTTHREAD hThreadSelf, void *pvArg)
+static DECLCALLBACK(int) ReceiveThread(RTTHREAD hThreadSelf, void *pvArg)
 {
     uint32_t    cbReceived  = 0;
     uint32_t    cLostFrames = 0;
@@ -324,8 +353,8 @@ DECLCALLBACK(int) ReceiveThread(RTTHREAD hThreadSelf, void *pvArg)
                              "receiver thread %.6Rhxs terminating.\n"
                              "  iFrame=%u  cb=%'u  c=%'u  %'uKB/s  %'ufps  cLost=%'u \n",
                              &pArgs->Mac, iFrame, cbReceived, iFrame - cLostFrames,
-                             (unsigned)(cbReceived * 1000000000.0 / 1024 / (pArgs->u64End - pArgs->u64Start)),
-                             (unsigned)((iFrame - cLostFrames) * 1000000000.0 / (pArgs->u64End - pArgs->u64Start)),
+                             (unsigned)((double)cbReceived * 1000000000.0 / 1024 / (double)(pArgs->u64End - pArgs->u64Start)),
+                             (unsigned)((double)(iFrame - cLostFrames) * 1000000000.0 / (double)(pArgs->u64End - pArgs->u64Start)),
                              cLostFrames);
                 return VINF_SUCCESS;
             }
@@ -389,6 +418,19 @@ DECLCALLBACK(int) ReceiveThread(RTTHREAD hThreadSelf, void *pvArg)
 }
 
 
+/**
+ * Drains the interface buffer before starting a new bi-directional run.
+ *
+ * We may have termination frames from previous runs pending in the buffer.
+ */
+static void tstDrainInterfaceBuffer(PMYARGS pArgs)
+{
+    uint8_t abBuf[16384 + 1024];
+    while (IntNetRingHasMoreToRead(&pArgs->pBuf->Recv))
+        IntNetRingReadAndSkipFrame(&pArgs->pBuf->Recv, abBuf);
+}
+
+
 /**
  * Test state.
  */
@@ -412,16 +454,16 @@ typedef TSTSTATE *PTSTSTATE;
 static int tstOpenInterfaces(PTSTSTATE pThis, const char *pszNetwork, uint32_t cbSend, uint32_t cbRecv)
 {
     pThis->hIf0 = INTNET_HANDLE_INVALID;
-    RTTESTI_CHECK_RC_OK_RET(IntNetR0Open(g_pSession, pszNetwork, kIntNetTrunkType_None, "",
-                                         0/*fFlags*/, cbSend, cbRecv, &pThis->hIf0), rcCheck);
+    RTTESTI_CHECK_RC_OK_RET(IntNetR0Open(g_pSession, pszNetwork, kIntNetTrunkType_None, "", 0/*fFlags*/, cbSend, cbRecv,
+                                         NULL /*pfnRecvAvail*/, NULL /*pvUser*/, &pThis->hIf0), rcCheck);
     RTTESTI_CHECK_RET(pThis->hIf0 != INTNET_HANDLE_INVALID, VERR_INTERNAL_ERROR);
     RTTESTI_CHECK_RC_RET(IntNetR0IfGetBufferPtrs(pThis->hIf0, g_pSession, &pThis->pBuf0, NULL), VINF_SUCCESS, rcCheck);
     RTTESTI_CHECK_RET(pThis->pBuf0, VERR_INTERNAL_ERROR);
 
 
     pThis->hIf1 = INTNET_HANDLE_INVALID;
-    RTTESTI_CHECK_RC_OK_RET(IntNetR0Open(g_pSession, pszNetwork, kIntNetTrunkType_None, "",
-                                         0/*fFlags*/, cbSend, cbRecv, &pThis->hIf1), rcCheck);
+    RTTESTI_CHECK_RC_OK_RET(IntNetR0Open(g_pSession, pszNetwork, kIntNetTrunkType_None, "", 0/*fFlags*/, cbSend, cbRecv,
+                                         NULL /*pfnRecvAvail*/, NULL /*pvUser*/, &pThis->hIf1), rcCheck);
     RTTESTI_CHECK_RET(pThis->hIf1 != INTNET_HANDLE_INVALID, VERR_INTERNAL_ERROR);
     RTTESTI_CHECK_RC_RET(IntNetR0IfGetBufferPtrs(pThis->hIf1, g_pSession, &pThis->pBuf1, NULL), VINF_SUCCESS, rcCheck);
     RTTESTI_CHECK_RET(pThis->pBuf1, VERR_INTERNAL_ERROR);
@@ -460,6 +502,35 @@ static void tstCloseInterfaces(PTSTSTATE pThis)
  */
 static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
 {
+    RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS, "-------------------------------------------------------------\n");
+
+    /*
+     * Reset statistics.
+     */
+    pThis->pBuf0->cStatYieldsOk.c       = 0;
+    pThis->pBuf0->cStatYieldsNok.c      = 0;
+    pThis->pBuf0->cStatLost.c           = 0;
+    pThis->pBuf0->cStatBadFrames.c      = 0;
+    pThis->pBuf0->Recv.cStatFrames.c    = 0;
+    pThis->pBuf0->Recv.cbStatWritten.c  = 0;
+    pThis->pBuf0->Recv.cOverflows.c     = 0;
+    pThis->pBuf0->Send.cStatFrames.c    = 0;
+    pThis->pBuf0->Send.cbStatWritten.c  = 0;
+    pThis->pBuf0->Send.cOverflows.c     = 0;
+    pThis->pBuf1->cStatYieldsOk.c       = 0;
+    pThis->pBuf1->cStatYieldsNok.c      = 0;
+    pThis->pBuf1->cStatLost.c           = 0;
+    pThis->pBuf1->cStatBadFrames.c      = 0;
+    pThis->pBuf1->Recv.cStatFrames.c    = 0;
+    pThis->pBuf1->Recv.cbStatWritten.c  = 0;
+    pThis->pBuf1->Recv.cOverflows.c     = 0;
+    pThis->pBuf1->Send.cStatFrames.c    = 0;
+    pThis->pBuf1->Send.cbStatWritten.c  = 0;
+    pThis->pBuf1->Send.cOverflows.c     = 0;
+
+    /*
+     * Do the benchmarking.
+     */
     MYARGS Args0;
     RT_ZERO(Args0);
     Args0.hIf         = pThis->hIf0;
@@ -468,6 +539,8 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
     Args0.Mac.au16[1] = 0;
     Args0.Mac.au16[2] = 0;
     Args0.cbFrame     = cbFrame;
+    tstDrainInterfaceBuffer(&Args0);
+    //RTTESTI_CHECK_RC_OK(IntNetR0IfSetMacAddress(pThis->hIf0, g_pSession, &Args0.Mac));
 
     MYARGS Args1;
     RT_ZERO(Args1);
@@ -477,6 +550,8 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
     Args1.Mac.au16[1] = 0;
     Args1.Mac.au16[2] = 1;
     Args1.cbFrame     = cbFrame;
+    tstDrainInterfaceBuffer(&Args1);
+    //RTTESTI_CHECK_RC_OK(IntNetR0IfSetMacAddress(pThis->hIf1, g_pSession, &Args1.Mac));
 
     RTTHREAD ThreadRecv0 = NIL_RTTHREAD;
     RTTHREAD ThreadRecv1 = NIL_RTTHREAD;
@@ -489,12 +564,12 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
 
     int rc2 = VINF_SUCCESS;
     int rc;
-    RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadSend0, 5*60*1000, &rc2));
+    RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadSend0, RT_MS_5MIN, &rc2));
     if (RT_SUCCESS(rc))
     {
         RTTESTI_CHECK_RC_OK(rc2);
         ThreadSend0 = NIL_RTTHREAD;
-        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadSend1, 5*60*1000, RT_SUCCESS(rc2) ? &rc2 : NULL));
+        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadSend1, RT_MS_5MIN, RT_SUCCESS(rc2) ? &rc2 : NULL));
         if (RT_SUCCESS(rc))
         {
             ThreadSend1 = NIL_RTTHREAD;
@@ -512,37 +587,49 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
                &&   cYields-- > 0)
             RTThreadYield();
 
-        uint64_t u64Elapsed = RT_MAX(Args0.u64End, Args1.u64End) - RT_MIN(Args0.u64Start, Args1.u64Start);
-        uint64_t u64Speed = (uint64_t)((2 * g_cbTransfer / 1024) / (u64Elapsed / 1000000000.0));
-        RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS,
-                     "transferred %u bytes in %'RU64 ns (%'RU64 KB/s)\n",
-                     2 * g_cbTransfer, u64Elapsed, u64Speed);
-
         /*
          * Wait for the threads to finish up...
          */
-        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadRecv0, 5000, &rc2));
+        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadRecv0, RT_MS_5SEC, &rc2));
         if (RT_SUCCESS(rc))
         {
             RTTESTI_CHECK_RC_OK(rc2);
             ThreadRecv0 = NIL_RTTHREAD;
         }
 
-        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadRecv1, 5000, &rc2));
+        RTTESTI_CHECK_RC_OK(rc = RTThreadWait(ThreadRecv1, RT_MS_5MIN, &rc2));
         if (RT_SUCCESS(rc))
         {
             RTTESTI_CHECK_RC_OK(rc2);
             ThreadRecv1 = NIL_RTTHREAD;
         }
+
+        /*
+         * Report the results.
+         */
+        uint64_t cNsElapsed = RT_MAX(Args0.u64End, Args1.u64End) - RT_MIN(Args0.u64Start, Args1.u64Start);
+        uint64_t cbSent     = (uint64_t)Args0.cbSent      + Args1.cbSent;
+        uint64_t cKbps      = (uint64_t)((double)(cbSent / 1024) / ((double)cNsElapsed / 1000000000.0));
+        uint64_t cFrames    = (uint64_t)Args0.cFramesSent + Args1.cFramesSent;
+        uint64_t cFps       = (uint64_t)((double)cFrames / ((double)cNsElapsed / 1000000000.0));
+        RTTestValue(g_hTest, "frame size",  cbFrame,    RTTESTUNIT_BYTES);
+        RTTestValue(g_hTest, "xmit time",   cNsElapsed, RTTESTUNIT_NS);
+        RTTestValue(g_hTest, "bytes sent",  cbSent,     RTTESTUNIT_BYTES);
+        RTTestValue(g_hTest, "speed",       cKbps,      RTTESTUNIT_KILOBYTES_PER_SEC);
+        RTTestValue(g_hTest, "frames sent", cFrames,    RTTESTUNIT_FRAMES);
+        RTTestValue(g_hTest, "fps",         cFps,       RTTESTUNIT_FRAMES_PER_SEC);
+        RTTestValue(g_hTest, "overflows",
+                    pThis->pBuf0->Send.cOverflows.c + pThis->pBuf1->Send.cOverflows.c, RTTESTUNIT_OCCURRENCES);
+
     }
 
     /*
      * Give them a chance to complete...
      */
-    RTThreadWait(ThreadRecv0, 5000, NULL);
-    RTThreadWait(ThreadRecv1, 5000, NULL);
-    RTThreadWait(ThreadSend0, 5000, NULL);
-    RTThreadWait(ThreadSend1, 5000, NULL);
+    RTThreadWait(ThreadRecv0, RT_MS_5MIN, NULL);
+    RTThreadWait(ThreadRecv1, RT_MS_5MIN, NULL);
+    RTThreadWait(ThreadSend0, RT_MS_5MIN, NULL);
+    RTThreadWait(ThreadSend1, RT_MS_5MIN, NULL);
 
 
     /*
@@ -556,12 +643,12 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
                  pThis->pBuf0->cStatBadFrames.c);
     RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS,
                  "Buf0.Recv: Frames=%llu Bytes=%llu Overflows=%llu\n",
-                 pThis->pBuf0->Recv.cStatFrames,
+                 pThis->pBuf0->Recv.cStatFrames.c,
                  pThis->pBuf0->Recv.cbStatWritten.c,
                  pThis->pBuf0->Recv.cOverflows.c);
     RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS,
                  "Buf0.Send: Frames=%llu Bytes=%llu Overflows=%llu\n",
-                 pThis->pBuf0->Send.cStatFrames,
+                 pThis->pBuf0->Send.cStatFrames.c,
                  pThis->pBuf0->Send.cbStatWritten.c,
                  pThis->pBuf0->Send.cOverflows.c);
 
@@ -573,12 +660,12 @@ static void tstBidirectionalTransfer(PTSTSTATE pThis, uint32_t cbFrame)
                  pThis->pBuf1->cStatBadFrames.c);
     RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS,
                  "Buf1.Recv: Frames=%llu Bytes=%llu Overflows=%llu\n",
-                 pThis->pBuf1->Recv.cStatFrames,
+                 pThis->pBuf1->Recv.cStatFrames.c,
                  pThis->pBuf1->Recv.cbStatWritten.c,
                  pThis->pBuf1->Recv.cOverflows.c);
     RTTestPrintf(g_hTest, RTTESTLVL_ALWAYS,
                  "Buf1.Send: Frames=%llu Bytes=%llu Overflows=%llu\n",
-                 pThis->pBuf1->Send.cStatFrames,
+                 pThis->pBuf1->Send.cStatFrames.c,
                  pThis->pBuf1->Send.cbStatWritten.c,
                  pThis->pBuf1->Send.cOverflows.c);
 
@@ -711,13 +798,16 @@ static void doTest(PTSTSTATE pThis, uint32_t cbRecv, uint32_t cbSend)
      */
     if (!RTTestIErrorCount())
     {
-        RTTestISubF("bi-directional benchmark, cbSend=%u, cbRecv=%u, cbTransfer=%u",
+        RTTestISubF("bi-dir benchmark, xbuf=%u rbuf=%u xfer=%u",
                     pThis->pBuf0->cbSend, pThis->pBuf0->cbRecv, g_cbTransfer);
         tstBidirectionalTransfer(pThis, 256);
 
-        for (uint32_t cbFrame = 64; cbFrame < cbSend - 64; cbFrame += 8)
+        /* Only doing up to half the xmit buffer size as it is easy to get into a
+           bad frame position from a previous run and run into overflow issues. */
+        /** @todo fix the code so it skips to a more optimal buffer position? */
+        for (uint32_t cbFrame = 64; cbFrame < cbSend / 2 - 64; cbFrame += 16)
         {
-            RTTestISubF("bi-directional benchmark, cbSend=%u, cbRecv=%u, cbTransfer=%u, cbFrame=%u",
+            RTTestISubF("bi-dir benchmark, xbuf=%u rbuf=%u xmit=%u frm=%u",
                         pThis->pBuf0->cbSend, pThis->pBuf0->cbRecv, g_cbTransfer, cbFrame);
             tstBidirectionalTransfer(pThis, cbFrame);
         }
