@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -27,13 +37,15 @@
 
 #include <VBox/com/VirtualBox.h>
 
-#include <iprt/stream.h>
 #include <iprt/getopt.h>
+#include <iprt/stream.h>
 #include <iprt/time.h>
 
 #include "VBoxManage.h"
 using namespace com;
 
+DECLARE_TRANSLATION_CONTEXT(Snapshot);
+
 /**
  * Helper function used with "VBoxManage snapshot ... dump". Gets called to find the
  * snapshot in the machine's snapshot tree that uses a particular diff image child of
@@ -47,13 +59,13 @@ using namespace com;
  * @param uSnapshotLevel
  * @return
  */
-bool FindAndPrintSnapshotUsingMedium(ComPtr<IMedium> &pMedium,
-                                     ComPtr<ISnapshot> &pThisSnapshot,
-                                     ComPtr<ISnapshot> &pCurrentSnapshot,
-                                     uint32_t uMediumLevel,
-                                     uint32_t uSnapshotLevel)
+static bool FindAndPrintSnapshotUsingMedium(ComPtr<IMedium> &pMedium,
+                                            ComPtr<ISnapshot> &pThisSnapshot,
+                                            ComPtr<ISnapshot> &pCurrentSnapshot,
+                                            uint32_t uMediumLevel,
+                                            uint32_t uSnapshotLevel)
 {
-    HRESULT rc;
+    HRESULT hrc;
 
     do
     {
@@ -125,13 +137,13 @@ bool FindAndPrintSnapshotUsingMedium(ComPtr<IMedium> &pMedium,
  * @param pCurrentSnapshot constant, the machine's current snapshot (so we can mark it in the output).
  * @param uLevel variant, the recursion level for output indentation.
  */
-void DumpMediumWithChildren(ComPtr<IMedium> &pCurrentStateMedium,
-                            ComPtr<IMedium> &pMedium,
-                            ComPtr<ISnapshot> &pRootSnapshot,
-                            ComPtr<ISnapshot> &pCurrentSnapshot,
-                            uint32_t uLevel)
+static void DumpMediumWithChildren(ComPtr<IMedium> &pCurrentStateMedium,
+                                   ComPtr<IMedium> &pMedium,
+                                   ComPtr<ISnapshot> &pRootSnapshot,
+                                   ComPtr<ISnapshot> &pCurrentSnapshot,
+                                   uint32_t uLevel)
 {
-    HRESULT rc;
+    HRESULT hrc;
     do
     {
         // print this medium
@@ -185,7 +197,7 @@ static RTEXITCODE handleSnapshotList(HandlerArg *pArgs, ComPtr<IMachine> &pMachi
         {
             case 'D':   enmDetails = VMINFO_FULL; break;
             case 'M':   enmDetails = VMINFO_MACHINEREADABLE; break;
-            default:    return errorGetOpt(USAGE_SNAPSHOT, c, &ValueUnion);
+            default:    return errorGetOpt(c, &ValueUnion);
         }
     }
 
@@ -193,7 +205,7 @@ static RTEXITCODE handleSnapshotList(HandlerArg *pArgs, ComPtr<IMachine> &pMachi
     HRESULT hrc = pMachine->FindSnapshot(Bstr().raw(), pSnapshot.asOutParam());
     if (FAILED(hrc))
     {
-        RTPrintf("This machine does not have any snapshots\n");
+        RTPrintf(Snapshot::tr("This machine does not have any snapshots\n"));
         return RTEXITCODE_FAILURE;
     }
     if (pSnapshot)
@@ -214,9 +226,9 @@ static RTEXITCODE handleSnapshotList(HandlerArg *pArgs, ComPtr<IMachine> &pMachi
  * snapshots.
  * @param pMachine Machine to dump snapshots for.
  */
-void DumpSnapshot(ComPtr<IMachine> &pMachine)
+static void DumpSnapshot(ComPtr<IMachine> &pMachine)
 {
-    HRESULT rc;
+    HRESULT hrc;
 
     do
     {
@@ -249,7 +261,7 @@ void DumpSnapshot(ComPtr<IMachine> &pMachine)
                 Bstr bstrBaseMediumName;
                 CHECK_ERROR_BREAK(pBaseMedium, COMGETTER(Name)(bstrBaseMediumName.asOutParam()));
 
-                RTPrintf("[%RI32] Images and snapshots for medium \"%ls\"\n", i, bstrBaseMediumName.raw());
+                RTPrintf(Snapshot::tr("[%RI32] Images and snapshots for medium \"%ls\"\n"), i, bstrBaseMediumName.raw());
 
                 DumpMediumWithChildren(pCurrentStateMedium,
                                        pBaseMedium,
@@ -272,9 +284,9 @@ typedef enum SnapshotUniqueFlags
 
 static int parseSnapshotUniqueFlags(const char *psz, SnapshotUniqueFlags *pUnique)
 {
-    int rc = VINF_SUCCESS;
+    int vrc = VINF_SUCCESS;
     unsigned uUnique = 0;
-    while (psz && *psz && RT_SUCCESS(rc))
+    while (psz && *psz && RT_SUCCESS(vrc))
     {
         size_t len;
         const char *pszComma = strchr(psz, ',');
@@ -293,7 +305,7 @@ static int parseSnapshotUniqueFlags(const char *psz, SnapshotUniqueFlags *pUniqu
             else if (!RTStrNICmp(psz, "force", len))
                 uUnique |= SnapshotUniqueFlags_Force;
             else
-                rc = VERR_PARSE_ERROR;
+                vrc = VERR_PARSE_ERROR;
         }
         if (pszComma)
             psz += len + 1;
@@ -301,9 +313,9 @@ static int parseSnapshotUniqueFlags(const char *psz, SnapshotUniqueFlags *pUniqu
             psz += len;
     }
 
-    if (RT_SUCCESS(rc))
+    if (RT_SUCCESS(vrc))
         *pUnique = (SnapshotUniqueFlags)uUnique;
-    return rc;
+    return vrc;
 }
 
 /**
@@ -313,11 +325,18 @@ static int parseSnapshotUniqueFlags(const char *psz, SnapshotUniqueFlags *pUniqu
  */
 RTEXITCODE handleSnapshot(HandlerArg *a)
 {
-    HRESULT rc;
+    HRESULT hrc;
+
+/** @todo r=bird: sub-standard command line parsing here!
+ *
+ * 'VBoxManage snapshot empty take --help' takes a snapshot rather than display
+ * help as you would expect.
+ *
+ */
 
     /* we need at least a VM and a command */
     if (a->argc < 2)
-        return errorSyntax(USAGE_SNAPSHOT, "Not enough parameters");
+        return errorSyntax(Snapshot::tr("Not enough parameters"));
 
     /* the first argument must be the VM */
     Bstr bstrMachine(a->argv[0]);
@@ -342,11 +361,13 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
 
         if (!strcmp(a->argv[1], "take"))
         {
+            setCurrentSubcommand(HELP_SCOPE_SNAPSHOT_TAKE);
+
             /* there must be a name */
             if (a->argc < 3)
             {
-                errorSyntax(USAGE_SNAPSHOT, "Missing snapshot name");
-                rc = E_FAIL;
+                errorSyntax(Snapshot::tr("Missing snapshot name"));
+                hrc = E_FAIL;
                 break;
             }
             Bstr name(a->argv[2]);
@@ -370,7 +391,7 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
             int ch;
             RTGETOPTUNION Value;
             int vrc;
-            while (   SUCCEEDED(rc)
+            while (   SUCCEEDED(hrc)
                    && (ch = RTGetOpt(&GetOptState, &Value)))
             {
                 switch (ch)
@@ -390,24 +411,24 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
                     case 'u':
                         vrc = parseSnapshotUniqueFlags(Value.psz, &enmUnique);
                         if (RT_FAILURE(vrc))
-                            return errorArgument("Invalid unique name description '%s'", Value.psz);
+                            return errorArgument(Snapshot::tr("Invalid unique name description '%s'"), Value.psz);
                         break;
 
                     default:
-                        errorGetOpt(USAGE_SNAPSHOT, ch, &Value);
-                        rc = E_FAIL;
+                        errorGetOpt(ch, &Value);
+                        hrc = E_FAIL;
                         break;
                 }
             }
-            if (FAILED(rc))
+            if (FAILED(hrc))
                 break;
 
             if (enmUnique & (SnapshotUniqueFlags_Number | SnapshotUniqueFlags_Timestamp))
             {
                 ComPtr<ISnapshot> pSnapshot;
-                rc = sessionMachine->FindSnapshot(name.raw(),
-                                                  pSnapshot.asOutParam());
-                if (SUCCEEDED(rc) || (enmUnique & SnapshotUniqueFlags_Force))
+                hrc = sessionMachine->FindSnapshot(name.raw(),
+                                                   pSnapshot.asOutParam());
+                if (SUCCEEDED(hrc) || (enmUnique & SnapshotUniqueFlags_Force))
                 {
                     /* there is a duplicate, need to create a unique name */
                     uint32_t count = 0;
@@ -419,6 +440,7 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
                             count = 1;
                         else
                             count = 2;
+                        RTTimeSpecSetNano(&now, 0); /* Shut up MSC */
                     }
                     else
                         RTTimeNow(&now);
@@ -442,22 +464,22 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
                         else
                             tryName = BstrFmt("%ls%s", name.raw(), suffix.c_str());
                         count++;
-                        rc = sessionMachine->FindSnapshot(tryName.raw(),
-                                                          pSnapshot.asOutParam());
-                        if (FAILED(rc))
+                        hrc = sessionMachine->FindSnapshot(tryName.raw(),
+                                                           pSnapshot.asOutParam());
+                        if (FAILED(hrc))
                         {
                             name = tryName;
                             break;
                         }
                     }
-                    if (SUCCEEDED(rc))
+                    if (SUCCEEDED(hrc))
                     {
-                        errorArgument("Failed to generate a unique snapshot name");
-                        rc = E_FAIL;
+                        errorArgument(Snapshot::tr("Failed to generate a unique snapshot name"));
+                        hrc = E_FAIL;
                         break;
                     }
                 }
-                rc = S_OK;
+                hrc = S_OK;
             }
 
             ComPtr<IProgress> progress;
@@ -466,44 +488,46 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
                                                            fPause, snapId.asOutParam(),
                                                            progress.asOutParam()));
 
-            rc = showProgress(progress);
-            if (SUCCEEDED(rc))
-                RTPrintf("Snapshot taken. UUID: %ls\n", snapId.raw());
+            hrc = showProgress(progress);
+            if (SUCCEEDED(hrc))
+                RTPrintf(Snapshot::tr("Snapshot taken. UUID: %ls\n"), snapId.raw());
             else
-                CHECK_PROGRESS_ERROR(progress, ("Failed to take snapshot"));
+                CHECK_PROGRESS_ERROR(progress, (Snapshot::tr("Failed to take snapshot")));
         }
         else if (    (fDelete = !strcmp(a->argv[1], "delete"))
                   || (fRestore = !strcmp(a->argv[1], "restore"))
                   || (fRestoreCurrent = !strcmp(a->argv[1], "restorecurrent"))
                 )
         {
+            setCurrentSubcommand(fDelete    ? HELP_SCOPE_SNAPSHOT_DELETE
+                                 : fRestore ? HELP_SCOPE_SNAPSHOT_RESTORE
+                                            : HELP_SCOPE_SNAPSHOT_RESTORECURRENT);
+
             if (fRestoreCurrent)
             {
                 if (a->argc > 2)
                 {
-                    errorSyntax(USAGE_SNAPSHOT, "Too many arguments");
-                    rc = E_FAIL;
+                    errorSyntax(Snapshot::tr("Too many arguments"));
+                    hrc = E_FAIL;
                     break;
                 }
             }
             /* exactly one parameter: snapshot name */
             else if (a->argc != 3)
             {
-                errorSyntax(USAGE_SNAPSHOT, "Expecting snapshot name only");
-                rc = E_FAIL;
+                errorSyntax(Snapshot::tr("Expecting snapshot name only"));
+                hrc = E_FAIL;
                 break;
             }
 
             ComPtr<ISnapshot> pSnapshot;
-            ComPtr<IProgress> pProgress;
-            Bstr bstrSnapGuid;
 
             if (fRestoreCurrent)
             {
                 CHECK_ERROR_BREAK(sessionMachine, COMGETTER(CurrentSnapshot)(pSnapshot.asOutParam()));
                 if (pSnapshot.isNull())
                 {
-                    RTPrintf("This machine does not have any snapshots\n");
+                    RTPrintf(Snapshot::tr("This machine does not have any snapshots\n"));
                     return RTEXITCODE_FAILURE;
                 }
             }
@@ -514,8 +538,17 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
                                                                pSnapshot.asOutParam()));
             }
 
+            Bstr bstrSnapGuid;
             CHECK_ERROR_BREAK(pSnapshot, COMGETTER(Id)(bstrSnapGuid.asOutParam()));
 
+            Bstr bstrSnapName;
+            CHECK_ERROR_BREAK(pSnapshot, COMGETTER(Name)(bstrSnapName.asOutParam()));
+
+            ComPtr<IProgress> pProgress;
+
+            RTPrintf(Snapshot::tr("%s snapshot '%ls' (%ls)\n"),
+                     fDelete ? Snapshot::tr("Deleting") : Snapshot::tr("Restoring"), bstrSnapName.raw(), bstrSnapGuid.raw());
+
             if (fDelete)
             {
                 CHECK_ERROR_BREAK(sessionMachine, DeleteSnapshot(bstrSnapGuid.raw(),
@@ -524,85 +557,86 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
             else
             {
                 // restore or restore current
-                RTPrintf("Restoring snapshot %ls\n", bstrSnapGuid.raw());
                 CHECK_ERROR_BREAK(sessionMachine, RestoreSnapshot(pSnapshot, pProgress.asOutParam()));
             }
 
-            rc = showProgress(pProgress);
-            CHECK_PROGRESS_ERROR(pProgress, ("Snapshot operation failed"));
+            hrc = showProgress(pProgress);
+            CHECK_PROGRESS_ERROR(pProgress, (Snapshot::tr("Snapshot operation failed")));
         }
         else if (!strcmp(a->argv[1], "edit"))
         {
+            setCurrentSubcommand(HELP_SCOPE_SNAPSHOT_EDIT);
             if (a->argc < 3)
             {
-                errorSyntax(USAGE_SNAPSHOT, "Missing snapshot name");
-                rc = E_FAIL;
+                errorSyntax(Snapshot::tr("Missing snapshot name"));
+                hrc = E_FAIL;
                 break;
             }
 
+            /* Parse the optional arguments, allowing more freedom than the
+             * synopsis explains. Can rename multiple snapshots and so on. */
             ComPtr<ISnapshot> pSnapshot;
-
-            if (   !strcmp(a->argv[2], "--current")
-                || !strcmp(a->argv[2], "-current"))
+            static const RTGETOPTDEF s_aEditOptions[] =
             {
-                CHECK_ERROR_BREAK(sessionMachine, COMGETTER(CurrentSnapshot)(pSnapshot.asOutParam()));
-                if (pSnapshot.isNull())
+                { "--current",     'c', RTGETOPT_REQ_NOTHING },
+                { "-current",      'c', RTGETOPT_REQ_NOTHING },
+                { "--name",        'n', RTGETOPT_REQ_STRING },
+                { "-name",         'n', RTGETOPT_REQ_STRING },
+                { "-newname",      'n', RTGETOPT_REQ_STRING },
+                { "--description", 'd', RTGETOPT_REQ_STRING },
+                { "-description",  'd', RTGETOPT_REQ_STRING },
+                { "-desc",         'd', RTGETOPT_REQ_STRING }
+            };
+            RTGETOPTSTATE GetOptState;
+            RTGetOptInit(&GetOptState, a->argc, a->argv, s_aEditOptions, RT_ELEMENTS(s_aEditOptions),
+                         2, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
+            int ch;
+            RTGETOPTUNION Value;
+            while (   SUCCEEDED(hrc)
+                   && (ch = RTGetOpt(&GetOptState, &Value)))
+            {
+                switch (ch)
                 {
-                    RTPrintf("This machine does not have any snapshots\n");
-                    return RTEXITCODE_FAILURE;
-                }
-            }
-            else
-            {
-                CHECK_ERROR_BREAK(sessionMachine, FindSnapshot(Bstr(a->argv[2]).raw(),
-                                                               pSnapshot.asOutParam()));
-            }
-
-            /* parse options */
-            for (int i = 3; i < a->argc; i++)
-            {
-                if (   !strcmp(a->argv[i], "--name")
-                    || !strcmp(a->argv[i], "-name")
-                    || !strcmp(a->argv[i], "-newname"))
-                {
-                    if (a->argc <= i + 1)
-                    {
-                        errorArgument("Missing argument to '%s'", a->argv[i]);
-                        rc = E_FAIL;
+                    case 'c':
+                        CHECK_ERROR_BREAK(sessionMachine, COMGETTER(CurrentSnapshot)(pSnapshot.asOutParam()));
+                        if (pSnapshot.isNull())
+                        {
+                            RTPrintf(Snapshot::tr("This machine does not have any snapshots\n"));
+                            return RTEXITCODE_FAILURE;
+                        }
                         break;
-                    }
-                    i++;
-                    pSnapshot->COMSETTER(Name)(Bstr(a->argv[i]).raw());
-                }
-                else if (   !strcmp(a->argv[i], "--description")
-                         || !strcmp(a->argv[i], "-description")
-                         || !strcmp(a->argv[i], "-newdesc"))
-                {
-                    if (a->argc <= i + 1)
-                    {
-                        errorArgument("Missing argument to '%s'", a->argv[i]);
-                        rc = E_FAIL;
+
+                    case 'n':
+                        CHECK_ERROR_BREAK(pSnapshot, COMSETTER(Name)(Bstr(Value.psz).raw()));
+                        break;
+
+                    case 'd':
+                        CHECK_ERROR_BREAK(pSnapshot, COMSETTER(Description)(Bstr(Value.psz).raw()));
+                        break;
+
+                    case VINF_GETOPT_NOT_OPTION:
+                        CHECK_ERROR_BREAK(sessionMachine, FindSnapshot(Bstr(Value.psz).raw(), pSnapshot.asOutParam()));
+                        break;
+
+                    default:
+                        errorGetOpt(ch, &Value);
+                        hrc = E_FAIL;
                         break;
-                    }
-                    i++;
-                    pSnapshot->COMSETTER(Description)(Bstr(a->argv[i]).raw());
-                }
-                else
-                {
-                    errorSyntax(USAGE_SNAPSHOT, "Invalid parameter '%s'", Utf8Str(a->argv[i]).c_str());
-                    rc = E_FAIL;
-                    break;
                 }
             }
 
+            if (FAILED(hrc))
+                break;
         }
         else if (!strcmp(a->argv[1], "showvminfo"))
         {
+            setCurrentSubcommand(HELP_SCOPE_SNAPSHOT_SHOWVMINFO);
+
             /* exactly one parameter: snapshot name */
             if (a->argc != 3)
             {
-                errorSyntax(USAGE_SNAPSHOT, "Expecting snapshot name only");
-                rc = E_FAIL;
+                errorSyntax(Snapshot::tr("Expecting snapshot name only"));
+                hrc = E_FAIL;
                 break;
             }
 
@@ -617,18 +651,20 @@ RTEXITCODE handleSnapshot(HandlerArg *a)
             showVMInfo(a->virtualBox, pMachine2, NULL, VMINFO_NONE);
         }
         else if (!strcmp(a->argv[1], "list"))
-            rc = handleSnapshotList(a, sessionMachine) == RTEXITCODE_SUCCESS ? S_OK : E_FAIL;
+        {
+            setCurrentSubcommand(HELP_SCOPE_SNAPSHOT_LIST);
+            hrc = handleSnapshotList(a, sessionMachine) == RTEXITCODE_SUCCESS ? S_OK : E_FAIL;
+        }
         else if (!strcmp(a->argv[1], "dump"))          // undocumented parameter to debug snapshot info
             DumpSnapshot(sessionMachine);
         else
         {
-            errorSyntax(USAGE_SNAPSHOT, "Invalid parameter '%s'", Utf8Str(a->argv[1]).c_str());
-            rc = E_FAIL;
+            errorSyntax(Snapshot::tr("Invalid parameter '%s'"), a->argv[1]);
+            hrc = E_FAIL;
         }
     } while (0);
 
     a->session->UnlockMachine();
 
-    return SUCCEEDED(rc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
+    return SUCCEEDED(hrc) ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
 }
-
