@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2010-2019 Oracle Corporation
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 #ifndef FEQT_INCLUDED_SRC_extradata_UIExtraDataManager_h
@@ -24,6 +34,7 @@
 /* Qt includes: */
 #include <QMap>
 #include <QObject>
+#include <QRect>
 #include <QSize>
 #include <QUuid>
 #ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
@@ -51,10 +62,10 @@ class SHARED_LIBRARY_STUFF UIExtraDataManager : public QObject
 {
     Q_OBJECT;
 
-    /** Extra-data Manager constructor. */
+    /** Constructs Extra-data Manager. */
     UIExtraDataManager();
-    /** Extra-data Manager destructor. */
-    ~UIExtraDataManager();
+    /** Destructs Extra-data Manager. */
+    virtual ~UIExtraDataManager();
 
 signals:
 
@@ -64,6 +75,14 @@ signals:
     /** Notifies about extra-data change. */
     void sigExtraDataChange(const QUuid &uID, const QString &strKey, const QString &strValue);
 
+    /** Notifies about notification-center alignment change. */
+    void sigNotificationCenterAlignmentChange();
+    /** Notifies about notification-center order change. */
+    void sigNotificationCenterOrderChange();
+
+    /** Notifies about settings expert mode change. */
+    void sigSettingsExpertModeChange();
+
     /** Notifies about GUI language change. */
     void sigLanguageChange(QString strLanguage);
 
@@ -74,11 +93,22 @@ signals:
     /** Notifies about Runtime UI host-key combination change. */
     void sigRuntimeUIHostKeyCombinationChange();
 
+    /** Notifies about Cloud Profile Manager restriction change. */
+    void sigCloudProfileManagerRestrictionChange();
+
+    /** Notifies about Cloud Console Manager data change. */
+    void sigCloudConsoleManagerDataChange();
+    /** Notifies about Cloud Console Manager restriction change. */
+    void sigCloudConsoleManagerRestrictionChange();
+
     /** Notifies about VirtualBox Manager / Details pane categories change. */
     void sigDetailsCategoriesChange();
     /** Notifies about VirtualBox Manager / Details pane options change. */
     void sigDetailsOptionsChange(DetailsElementType enmType);
 
+    /** Notifies about visual state change. */
+    void sigVisualStateChange(const QUuid &uMachineID);
+
     /** Notifies about menu-bar configuration change. */
     void sigMenuBarConfigurationChange(const QUuid &uMachineID);
     /** Notifies about status-bar configuration change. */
@@ -93,6 +123,9 @@ signals:
     /** Notifies about the scaling optimization type change. */
     void sigScalingOptimizationTypeChange(const QUuid &uMachineID);
 
+    /** Notifies about font scale factor. */
+    void sigFontScaleFactorChanged(int iFontScaleFactor);
+
 #ifdef VBOX_WS_MAC
     /** Notifies about the HiDPI optimization type change. */
     void sigHiDPIOptimizationTypeChange(const QUuid &uMachineID);
@@ -103,15 +136,22 @@ signals:
     void sigDockIconOverlayAppearanceChange(bool fEnabled);
 #endif /* VBOX_WS_MAC */
 
+#if defined (VBOX_WS_NIX) || defined (VBOX_WS_WIN)
+    /* Is emitted when host screen saver inhibition state changes. */
+    void sigDisableHostScreenSaverStateChange(bool fDisable);
+#endif
+
 public:
 
     /** Global extra-data ID. */
     static const QUuid GlobalID;
 
+    /** Singleton object contructor. */
+    static void create();
+    /** Singleton object destructor. */
+    static void destroy();
     /** Static Extra-data Manager instance/constructor. */
     static UIExtraDataManager* instance();
-    /** Static Extra-data Manager destructor. */
-    static void destroy();
 
 #ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
     /** Static show and raise API. */
@@ -145,20 +185,46 @@ public:
 
     /** @name General
       * @{ */
-        /** Returns event handling type. */
-        EventHandlingType eventHandlingType();
+        /** Returns a list of restricted dialogs. */
+        UIExtraDataMetaDefs::DialogType restrictedDialogTypes(const QUuid &uID);
+        /** Defines a list of restricted dialogs. */
+        void setRestrictedDialogTypes(UIExtraDataMetaDefs::DialogType enmTypes, const QUuid &uID);
+
+        /** Returns color theme type. */
+        UIColorThemeType colorTheme();
+        /** Defines color theme @a enmType. */
+        void setColorTheme(const UIColorThemeType &enmType);
     /** @} */
 
     /** @name Messaging
       * @{ */
-        /** Returns the list of supressed messages for the Message/Popup center frameworks. */
+        /** Returns the list of suppressed messages for the Message/Popup center frameworks. */
         QStringList suppressedMessages(const QUuid &uID = GlobalID);
-        /** Defines the @a list of supressed messages for the Message/Popup center frameworks. */
+        /** Defines the @a list of suppressed messages for the Message/Popup center frameworks. */
         void setSuppressedMessages(const QStringList &list);
 
         /** Returns the list of messages for the Message/Popup center frameworks with inverted check-box state. */
         QStringList messagesWithInvertedOption();
 
+#ifdef VBOX_NOTIFICATION_CENTER_WITH_KEEP_BUTTON
+        /** Returns whether successfull notification-progresses should NOT close automatically. */
+        bool keepSuccessfullNotificationProgresses();
+        /** Defines whether successfull notification-progresses should NOT close (@a fKeep) automatically. */
+        void setKeepSuccessfullNotificationProgresses(bool fKeep);
+#endif /* VBOX_NOTIFICATION_CENTER_WITH_KEEP_BUTTON */
+
+        /** Returns notification-center alignment. */
+        Qt::Alignment notificationCenterAlignment();
+        /** Defines notification-progresses @a enmOrder. */
+        void setNotificationCenterAlignment(Qt::Alignment enmOrder);
+
+        /** Returns notification-center order. */
+        Qt::SortOrder notificationCenterOrder();
+        /** Defines notification-progresses @a enmOrder. */
+        void setNotificationCenterOrder(Qt::SortOrder enmOrder);
+
+        /** Returns whether BETA build label should be hidden. */
+        bool preventBetaBuildLabel();
 #if !defined(VBOX_BLEEDING_EDGE) && !defined(DEBUG)
         /** Returns version for which user wants to prevent BETA build warning. */
         QString preventBetaBuildWarningForVersion();
@@ -198,14 +264,11 @@ public:
         QList<GlobalSettingsPageType> restrictedGlobalSettingsPages();
         /** Returns restricted machine settings pages. */
         QList<MachineSettingsPageType> restrictedMachineSettingsPages(const QUuid &uID);
-    /** @} */
 
-    /** @name Settings: General
-      * @{ */
-        /** Returns whether the host screen-saver should be disabled. */
-        bool hostScreenSaverDisabled();
-        /** Defines whether the host screen-saver should be @a fDisabled. */
-        void setHostScreenSaverDisabled(bool fDisabled);
+        /** Returns whether settings are in expert mode. */
+        bool isSettingsInExpertMode();
+        /** Defines whether settings are in @a fExpertMode. */
+        void setSettingsInExpertMode(bool fExpertMode);
     /** @} */
 
     /** @name Settings: Language
@@ -219,9 +282,9 @@ public:
     /** @name Settings: Display
       * @{ */
         /** Returns maximum guest-screen resolution policy. */
-        MaxGuestResolutionPolicy maxGuestResolutionPolicy();
+        MaximumGuestScreenSizePolicy maxGuestResolutionPolicy();
         /** Defines maximum guest-screen resolution @a enmPolicy or @a resolution itself for Fixed policy. */
-        void setMaxGuestScreenResolution(MaxGuestResolutionPolicy enmPolicy, const QSize resolution = QSize());
+        void setMaxGuestScreenResolution(MaximumGuestScreenSizePolicy enmPolicy, const QSize resolution = QSize());
         /** Returns maximum guest-screen resolution for fixed policy. */
         QSize maxGuestResolutionForPolicyFixed();
         /** Defines maximum guest-screen @a resolution for fixed policy. */
@@ -231,6 +294,13 @@ public:
         bool activateHoveredMachineWindow();
         /** Defines whether hovered machine-window should be @a fActivated. */
         void setActivateHoveredMachineWindow(bool fActivate);
+        /* Return whether host screen saver is disabled when a vm is running. */
+        bool disableHostScreenSaver();
+        /* Sets whether host screen saver is disabled when a vm is running. */
+        void setDisableHostScreenSaver(bool fActivate);
+        /* Set global font scale factor as percentage. 100% is for no scaling. */
+        void setFontScaleFactor(int iFontScaleFactor);
+        int  fontScaleFactor();
     /** @} */
 
     /** @name Settings: Keyboard
@@ -268,16 +338,12 @@ public:
         QString recentFolderForOpticalDisks();
         /** Returns recent folder for floppy-disks. */
         QString recentFolderForFloppyDisks();
-        /** Returns recent folder for VISO creation content. */
-        QString recentFolderForVISOContent();
         /** Defines recent folder for hard-drives as @a strValue. */
         void setRecentFolderForHardDrives(const QString &strValue);
         /** Defines recent folder for optical-disk as @a strValue. */
         void setRecentFolderForOpticalDisks(const QString &strValue);
         /** Defines recent folder for floppy-disk as @a strValue. */
         void setRecentFolderForFloppyDisks(const QString &strValue);
-        /** Defines recent folder for VISO creation content as @a strValue. */
-        void setRecentFolderForVISOContent(const QString &strValue);
 
         /** Returns the list of recently used hard-drives. */
         QStringList recentListOfHardDrives();
@@ -293,6 +359,26 @@ public:
         void setRecentListOfFloppyDisks(const QStringList &value);
     /** @} */
 
+    /** @name Settings: Network
+      * @{ */
+        /** Returns the list of restricted network attachment types. */
+        UIExtraDataMetaDefs::DetailsElementOptionTypeNetwork restrictedNetworkAttachmentTypes();
+    /** @} */
+
+    /** @name VISO Creator
+      * @{ */
+        /** Returns recent folder for VISO creation content. */
+        QString visoCreatorRecentFolder();
+        /** Defines recent folder for VISO creation content as @a strValue. */
+        void setVISOCreatorRecentFolder(const QString &strValue);
+        /** Returns viso creator geometry using @a pWidget as the hint. */
+        QRect visoCreatorDialogGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
+        /** Set viso creator geometry. */
+        void setVisoCreatorDialogGeometry(const QRect &geometry, bool fMaximized);
+        /** Returns whether viso creator dialog should be maximized. */
+        bool visoCreatorDialogShouldBeMaximized();
+    /** @} */
+
     /** @name VirtualBox Manager
       * @{ */
         /** Returns selector-window geometry using @a pWidget as the hint. */
@@ -321,18 +407,22 @@ public:
         QList<UIToolType> toolsPaneLastItemsChosen();
         /** Defines last selected tool @a set of VirtualBox Manager. */
         void setToolsPaneLastItemsChosen(const QList<UIToolType> &set);
+        /** Returns the list of detached tools of VirtualBox Manager. */
+        QList<UIToolType> detachedTools();
+        /** Defines the list of detached @a tools of VirtualBox Manager. */
+        void setDetachedTools(const QList<UIToolType> &tools);
 
         /** Returns whether selector-window status-bar visible. */
         bool selectorWindowStatusBarVisible();
         /** Defines whether selector-window status-bar @a fVisible. */
         void setSelectorWindowStatusBarVisible(bool fVisible);
 
-        /** Clears all the existing selector-window chooser-pane' group definitions. */
-        void clearSelectorWindowGroupsDefinitions();
-        /** Returns selector-window chooser-pane' groups definitions for passed @a strGroupID. */
-        QStringList selectorWindowGroupsDefinitions(const QString &strGroupID);
-        /** Defines selector-window chooser-pane' groups @a definitions for passed @a strGroupID. */
-        void setSelectorWindowGroupsDefinitions(const QString &strGroupID, const QStringList &definitions);
+        /** Returns all the existing selector-window chooser-pane' group definition keys. */
+        QStringList knownMachineGroupDefinitionKeys();
+        /** Returns selector-window chooser-pane' group definitions for passed @a strGroupID. */
+        QStringList machineGroupDefinitions(const QString &strGroupID);
+        /** Defines selector-window chooser-pane' group @a definitions for passed @a strGroupID. */
+        void setMachineGroupDefinitions(const QString &strGroupID, const QStringList &definitions);
 
         /** Returns last-item ID of the item chosen in selector-window chooser-pane. */
         QString selectorWindowLastItemChosen();
@@ -369,6 +459,10 @@ public:
         bool virtualMediaManagerDetailsExpanded();
         /** Defines whether Virtual Media Manager details @a fExpanded. */
         void setVirtualMediaManagerDetailsExpanded(bool fExpanded);
+        /** Returns whether Virtual Media Manager search widget expanded. */
+        bool virtualMediaManagerSearchWidgetExpanded();
+        /** Defines whether Virtual Media Manager search widget @a fExpanded. */
+        void setVirtualMediaManagerSearchWidgetExpanded(bool fExpanded);
     /** @} */
 
     /** @name Host Network Manager
@@ -381,37 +475,65 @@ public:
 
     /** @name Cloud Profile Manager
       * @{ */
+        /** Returns Cloud Profile Manager restrictions. */
+        QStringList cloudProfileManagerRestrictions();
+        /** Defines Cloud Profile Manager @a restrictions. */
+        void setCloudProfileManagerRestrictions(const QStringList &restrictions);
+
         /** Returns whether Cloud Profile Manager details expanded. */
         bool cloudProfileManagerDetailsExpanded();
         /** Defines whether Cloud Profile Manager details @a fExpanded. */
         void setCloudProfileManagerDetailsExpanded(bool fExpanded);
     /** @} */
 
-    /** @name Wizards
+    /** @name Cloud Console Manager
       * @{ */
-        /** Returns mode for wizard of passed @a type. */
-        WizardMode modeForWizardType(WizardType type);
-        /** Defines @a mode for wizard of passed @a type. */
-        void setModeForWizardType(WizardType type, WizardMode mode);
+        /** Returns registered Cloud Console Manager applications. */
+        QStringList cloudConsoleManagerApplications();
+        /** Returns registered Cloud Console Manager profiles for application with @a strId. */
+        QStringList cloudConsoleManagerProfiles(const QString &strId);
+
+        /** Returns definition for Cloud Console Manager application with @a strId. */
+        QString cloudConsoleManagerApplication(const QString &strId);
+        /** Defines @a strDefinition for Cloud Console Manager application with @a strId. */
+        void setCloudConsoleManagerApplication(const QString &strId, const QString &strDefinition);
+
+        /** Returns definition for Cloud Console Manager profile with @a strProfileId for application with @a strApplicationId. */
+        QString cloudConsoleManagerProfile(const QString &strApplicationId, const QString &strProfileId);
+        /** Returns @a strDefinition for Cloud Console Manager profile with @a strProfileId for application with @a strApplicationId. */
+        void setCloudConsoleManagerProfile(const QString &strApplicationId, const QString &strProfileId, const QString &strDefinition);
+
+        /** Returns Cloud Console Manager restrictions. */
+        QStringList cloudConsoleManagerRestrictions();
+        /** Defines Cloud Console Manager @a restrictions. */
+        void setCloudConsoleManagerRestrictions(const QStringList &restrictions);
+
+        /** Returns whether Cloud Console Manager details expanded. */
+        bool cloudConsoleManagerDetailsExpanded();
+        /** Defines whether Cloud Console Manager details @a fExpanded. */
+        void setCloudConsoleManagerDetailsExpanded(bool fExpanded);
+    /** @} */
+
+    /** @name Cloud Console
+      * @{ */
+        /** Returns Cloud Console public key path. */
+        QString cloudConsolePublicKeyPath();
+        /** Defines Cloud Console public key @a strPath. */
+        void setCloudConsolePublicKeyPath(const QString &strPath);
     /** @} */
 
     /** @name Virtual Machine
       * @{ */
-        /** Returns whether machine should be shown in selector-window chooser-pane. */
-        bool showMachineInSelectorChooser(const QUuid &uID);
-        /** Returns whether machine should be shown in selector-window details-pane. */
-        bool showMachineInSelectorDetails(const QUuid &uID);
+        /** Returns whether machine should be shown in VirtualBox Manager Chooser-pane. */
+        bool showMachineInVirtualBoxManagerChooser(const QUuid &uID);
+        /** Returns whether machine should be shown in VirtualBox Manager Details-pane. */
+        bool showMachineInVirtualBoxManagerDetails(const QUuid &uID);
 
         /** Returns whether machine reconfiguration enabled. */
         bool machineReconfigurationEnabled(const QUuid &uID);
         /** Returns whether machine snapshot operations enabled. */
         bool machineSnapshotOperationsEnabled(const QUuid &uID);
 
-        /** Returns whether this machine is first time started. */
-        bool machineFirstTimeStarted(const QUuid &uID);
-        /** Returns whether this machine is fFirstTimeStarted. */
-        void setMachineFirstTimeStarted(bool fFirstTimeStarted, const QUuid &uID);
-
         /** Except Mac OS X: Returns redefined machine-window icon names. */
         QStringList machineWindowIconNames(const QUuid &uID);
 #ifndef VBOX_WS_MAC
@@ -495,7 +617,7 @@ public:
         /** Defines requested Runtime UI visual-state as @a visualState. */
         void setRequestedVisualState(UIVisualStateType visualState, const QUuid &uID);
 
-#ifdef VBOX_WS_X11
+#ifdef VBOX_WS_NIX
         /** Returns whether legacy full-screen mode is requested. */
         bool legacyFullscreenModeRequested();
 
@@ -503,7 +625,7 @@ public:
         bool distinguishMachineWindowGroups(const QUuid &uID);
         /** Defines whether internal machine-window name should be unique. */
         void setDistinguishMachineWindowGroups(const QUuid &uID, bool fEnabled);
-#endif /* VBOX_WS_X11 */
+#endif /* VBOX_WS_NIX */
 
         /** Returns whether guest-screen auto-resize according machine-window size is enabled. */
         bool guestScreenAutoResizeEnabled(const QUuid &uID);
@@ -528,19 +650,6 @@ public:
         /** Returns whether automatic mounting/unmounting of guest-screens enabled. */
         bool autoMountGuestScreensEnabled(const QUuid &uID);
 
-#ifdef VBOX_WITH_VIDEOHWACCEL
-        /** Returns whether 2D acceleration should use linear sretch. */
-        bool useLinearStretch(const QUuid &uID);
-        /** Returns whether 2D acceleration should use YV12 pixel format. */
-        bool usePixelFormatYV12(const QUuid &uID);
-        /** Returns whether 2D acceleration should use UYVY pixel format. */
-        bool usePixelFormatUYVY(const QUuid &uID);
-        /** Returns whether 2D acceleration should use YUY2 pixel format. */
-        bool usePixelFormatYUY2(const QUuid &uID);
-        /** Returns whether 2D acceleration should use AYUV pixel format. */
-        bool usePixelFormatAYUV(const QUuid &uID);
-#endif /* VBOX_WITH_VIDEOHWACCEL */
-
 #ifndef VBOX_WS_MAC
         /** Returns whether mini-toolbar is enabled for full and seamless screens. */
         bool miniToolbarEnabled(const QUuid &uID);
@@ -621,35 +730,45 @@ public:
         ScalingOptimizationType scalingOptimizationType(const QUuid &uID);
     /** @} */
 
-    /** @name Virtual Machine: Information dialog
+    /** @name Virtual Machine: Session Information dialog
       * @{ */
-        /** Returns information-window geometry using @a pWidget and @a pParentWidget as hints. */
-        QRect informationWindowGeometry(QWidget *pWidget, QWidget *pParentWidget, const QUuid &uID);
+        /** Returns session information dialog geometry using @a pWidget and @a pParentWidget as hints. */
+        QRect sessionInformationDialogGeometry(QWidget *pWidget, QWidget *pParentWidget);
         /** Returns whether information-window should be maximized or not. */
-        bool informationWindowShouldBeMaximized(const QUuid &uID);
+        bool sessionInformationDialogShouldBeMaximized();
         /** Defines information-window @a geometry and @a fMaximized state. */
-        void setInformationWindowGeometry(const QRect &geometry, bool fMaximized, const QUuid &uID);
-
-        /** Returns information-window elements. */
-        QMap<InformationElementType, bool> informationWindowElements();
-        /** Defines information-window @a elements. */
-        void setInformationWindowElements(const QMap<InformationElementType, bool> &elements);
+        void setSessionInformationDialogGeometry(const QRect &geometry, bool fMaximized);
     /** @} */
 
     /** @name Guest Control related dialogs
       * @{ */
         void setGuestControlProcessControlSplitterHints(const QList<int> &hints);
         QList<int> guestControlProcessControlSplitterHints();
-        QRect fileManagerDialogGeometry(QWidget *pWidget, const QRect &defaultGeometry);
+        QRect fileManagerDialogGeometry(QWidget *pWidget, QWidget *pParentWidget);
         bool fileManagerDialogShouldBeMaximized();
         void setFileManagerDialogGeometry(const QRect &geometry, bool fMaximized);
-        QRect guestProcessControlDialogGeometry(QWidget *pWidget, const QRect &defaultGeometry);
+        QRect guestProcessControlDialogGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
         bool guestProcessControlDialogShouldBeMaximized();
         void setGuestProcessControlDialogGeometry(const QRect &geometry, bool fMaximized);
         void setFileManagerVisiblePanels(const QStringList &panelNameList);
         QStringList fileManagerVisiblePanels();
     /** @} */
 
+    /** @name Soft Keyboard
+      * @{ */
+        QRect softKeyboardDialogGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
+        void setSoftKeyboardDialogGeometry(const QRect &geometry, bool fMaximized);
+        bool softKeyboardDialogShouldBeMaximized();
+        void setSoftKeyboardOptions(bool fShowNumPad, bool fHideOSMenuKeys, bool fMultimediaKeys);
+        void softKeyboardOptions(bool &fOutShowNumPad, bool &fOutHideOSMenuKeys, bool &fOutHideMultimediaKeys);
+        void setSoftKeyboardColorTheme(const QStringList &colorStringList);
+        QStringList softKeyboardColorTheme();
+        void setSoftKeyboardSelectedColorTheme(const QString &strColorThemeName);
+        QString softKeyboardSelectedColorTheme();
+        void setSoftKeyboardSelectedLayout(const QUuid &uLayoutUid);
+        QUuid softKeyboardSelectedLayout();
+    /** @} */
+
     /** @name File Manager options
       * @{ */
         void setFileManagerOptions(bool fListDirectoriesFirst,
@@ -676,6 +795,9 @@ public:
 
         /** Returns machine close hook script name as simple string. */
         QString machineCloseHookScript(const QUuid &uID);
+
+        /** Returns whether machine should discard state on power off. */
+        bool discardStateOnPowerOff(const QUuid &uID);
     /** @} */
 
 #ifdef VBOX_WITH_DEBUGGER_GUI
@@ -689,8 +811,8 @@ public:
 #ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
     /** @name VirtualBox: Extra-data Manager window
       * @{ */
-        /** Returns Extra-data Manager geometry using @a pWidget as hint. */
-        QRect extraDataManagerGeometry(QWidget *pWidget);
+        /** Returns Extra-data Manager geometry using @a pWidget and @a pParentWidget as hint. */
+        QRect extraDataManagerGeometry(QWidget *pWidget, QWidget *pParentWidget);
         /** Returns whether Extra-data Manager should be maximized or not. */
         bool extraDataManagerShouldBeMaximized();
         /** Defines Extra-data Manager @a geometry and @a fMaximized state. */
@@ -705,8 +827,8 @@ public:
 
     /** @name Virtual Machine: Log Viewer dialog
       * @{ */
-        /** Returns log-window geometry using @a pWidget and @a defaultGeometry as hints. */
-        QRect logWindowGeometry(QWidget *pWidget, const QRect &defaultGeometry);
+        /** Returns log-window geometry using @a pWidget, @a pParentWidget and @a defaultGeometry as hints. */
+        QRect logWindowGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
         /** Returns whether log-window should be maximized or not. */
         bool logWindowShouldBeMaximized();
         /** Defines log-window @a geometry and @a fMaximized state. */
@@ -722,6 +844,40 @@ public:
         bool logViewerShowLineNumbers();
         /** Tries to find system font by searching by family and style strings within the font database. */
         QFont logViewerFont();
+        void setLogViewerVisiblePanels(const QStringList &panelNameList);
+        QStringList logViewerVisiblePanels();
+    /** @} */
+
+    /** @name Help Browser
+      * @{ */
+        void setHelpBrowserLastUrlList(const QStringList &urlList);
+        QStringList helpBrowserLastUrlList();
+        void setHelpBrowserZoomPercentage(int iZoomPercentage);
+        int helpBrowserZoomPercentage();
+        QRect helpBrowserDialogGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
+        void setHelpBrowserDialogGeometry(const QRect &geometry, bool fMaximized);
+        bool helpBrowserDialogShouldBeMaximized();
+        void setHelpBrowserBookmarks(const QStringList &bookmarks);
+        QStringList helpBrowserBookmarks();
+    /** @} */
+
+    /** @name Manager UI: VM Activity Monitor and Overview
+      * @{ */
+        void setVMActivityOverviewHiddenColumnList(const QStringList &hiddenColumnList);
+        QStringList VMActivityOverviewHiddenColumnList();
+        bool VMActivityOverviewShowAllMachines();
+        void setVMActivityOverviewShowAllMachines(bool fShow);
+        void setVMActivityMonitorDataSeriesColors(const QStringList &colorList);
+        QStringList VMActivityMonitorDataSeriesColors();
+        bool VMActivityMonitorShowVMExits();
+        void setVMActivityMonitorShowVMExits(bool fShow);
+    /** @} */
+
+    /** @name Medium Selector
+      * @{ */
+        QRect mediumSelectorDialogGeometry(QWidget *pWidget, QWidget *pParentWidget, const QRect &defaultGeometry);
+        void setMediumSelectorDialogGeometry(const QRect &geometry, bool fMaximized);
+        bool mediumSelectorDialogShouldBeMaximized();
     /** @} */
 
 private slots:
@@ -737,6 +893,8 @@ private:
     void prepareGlobalExtraDataMap();
     /** Prepare extra-data event-handler. */
     void prepareExtraDataEventHandler();
+    /** Prepare extra-data settings. */
+    void prepareExtraDataSettings();
 #ifdef VBOX_GUI_WITH_EXTRADATA_MANAGER_UI
     // /** Prepare window. */
     // void prepareWindow();
@@ -782,8 +940,21 @@ private:
     /** Translates bool flag into 'restricted' value. */
     QString toFeatureRestricted(bool fRestricted);
 
-    QRect dialogGeometry(const QString &strKey, QWidget *pWidget, const QRect &defaultGeometry);
+    /** Defines saved dialog geometry according to specified attributes.
+      * @param  strKey      Brings geometry extra-data key of particular dialog.
+      * @param  geometry    Brings the dialog geometry to save.
+      * @param  fMaximized  Brings whether saved dialog geometry should be marked as maximized. */
     void setDialogGeometry(const QString &strKey, const QRect &geometry, bool fMaximized);
+    /** Returns saved dialog geometry according to specified attributes.
+      * @param  strKey           Brings geometry extra-data key of particular dialog.
+      * @param  pWidget          Brings the widget to limit geometry bounds according to.
+      * @param  pParentWidget    Brings the widget to center geometry rectangle according to.
+      * @param  defaultGeometry  Brings the default geometry which should be used to
+      *                          calculate resulting geometry if saved was not found. */
+    QRect dialogGeometry(const QString &strKey, QWidget *pWidget, QWidget *pParentWidget = 0, const QRect &defaultGeometry = QRect());
+    /** Returns true if the dialog should be maximized.
+      * @param  strKey           Brings geometry extra-data key of particular dialog. */
+    bool dialogShouldBeMaximized(const QString &strKey);
 
     /** Returns string consisting of @a strBase appended with @a uScreenIndex for the *non-primary* screen-index.
       * If @a fSameRuleForPrimary is 'true' same rule will be used for *primary* screen-index. Used for storing per-screen extra-data. */
