@@ -374,7 +374,7 @@ void UIVisoCreatorWidget::setVisoFilePath(const QString& strPath)
 {
     if (m_strVisoFilePath == strPath)
         return;
-    m_strVisoFilePath = strPath;
+    m_strVisoFilePath = QDir::fromNativeSeparators(QDir::cleanPath(strPath));
     emit sigVisoFilePathChanged(m_strVisoFilePath);
 }
 
@@ -439,7 +439,11 @@ void UIVisoCreatorWidget::sltOpenAction()
     QString strFileName =  QIFileDialog::getOpenFileName(UIMediumTools::defaultFolderPathForType(UIMediumDeviceType_DVD),
                                                          "VISO files (*.viso)", pActive, UIVisoCreatorWidget::tr("Select a VISO file to load"));
     if (!strFileName.isEmpty() && m_pVISOContentBrowser)
+    {
+        setVisoFilePath(QFileInfo(strFileName).absolutePath());
+        setVisoName(QFileInfo(strFileName).baseName());
         m_pVISOContentBrowser->parseVisoFileContent(strFileName);
+    }
 }
 
 void UIVisoCreatorWidget::sltSaveAsAction()
@@ -485,6 +489,11 @@ void UIVisoCreatorWidget::sltISOContentImportedOrRemoved(bool fImported)
         m_pRemoveISOAction->setEnabled(fImported);
 }
 
+void UIVisoCreatorWidget::sltVisoVolumeIdParsed(const QString &strVolumeId)
+{
+    setVisoName(strVolumeId);
+}
+
 void UIVisoCreatorWidget::sltSettingsChanged()
 {
     AssertReturnVoid(m_pSettingsWidget);
@@ -591,6 +600,8 @@ void UIVisoCreatorWidget::prepareConnections()
                 this, &UIVisoCreatorWidget::sltContentBrowserTableSelectionChanged);
         connect(m_pVISOContentBrowser, &UIVisoContentBrowser::sigISOContentImportedOrRemoved,
                 this, &UIVisoCreatorWidget::sltISOContentImportedOrRemoved);
+        connect(m_pVISOContentBrowser, &UIVisoContentBrowser::sigVolumeIdParsed,
+                this, &UIVisoCreatorWidget::sltVisoVolumeIdParsed);
     }
 
     if (m_pActionPreferences)
@@ -830,16 +841,14 @@ void UIVisoCreatorDialog::prepareWidgets(const QString& strVisoFilePath, const Q
     m_pButtonBox->button(QDialogButtonBox::Cancel)->setShortcut(QKeySequence(Qt::Key_Escape));
     pMainLayout->addWidget(m_pButtonBox);
 
-    connect(m_pButtonBox->button(QIDialogButtonBox::Help), &QPushButton::pressed,
+    connect(m_pButtonBox->button(QDialogButtonBox::Help), &QPushButton::pressed,
             m_pButtonBox, &QIDialogButtonBox::sltHandleHelpRequest);
 
-    m_pButtonBox->button(QDialogButtonBox::Help)->setShortcut(UIShortcutPool::standardSequence(QKeySequence::HelpContents));
-
     connect(m_pButtonBox, &QIDialogButtonBox::rejected, this, &UIVisoCreatorDialog::close);
     connect(m_pButtonBox, &QIDialogButtonBox::accepted, this, &UIVisoCreatorDialog::accept);
 
 
-    uiCommon().setHelpKeyword(m_pButtonBox->button(QIDialogButtonBox::Help), "create-optical-disk-image");
+    uiCommon().setHelpKeyword(m_pButtonBox->button(QDialogButtonBox::Help), "create-optical-disk-image");
 
     sltRetranslateUI();
     connect(&translationEventListener(), &UITranslationEventListener::sigRetranslateUI,
@@ -855,7 +864,10 @@ void UIVisoCreatorDialog::sltRetranslateUI()
         m_pButtonBox->button(QDialogButtonBox::Ok)->setToolTip(UIVisoCreatorWidget::tr("Creates VISO file with the selected content"));
     }
     if (m_pButtonBox && m_pButtonBox->button(QDialogButtonBox::Help))
+    {
         m_pButtonBox->button(QDialogButtonBox::Help)->setToolTip(UIVisoCreatorWidget::tr("Opens the help browser and navigates to the related section"));
+        m_pButtonBox->button(QDialogButtonBox::Help)->setShortcut(UIShortcutPool::standardSequence(QKeySequence::HelpContents));
+    }
     updateWindowTitle();
 }
 
@@ -984,9 +996,9 @@ bool UIVisoCreatorDialog::saveVISOFile()
 
         QTextStream stream(&file);
         stream << QString("%1 %2").arg("--iprt-iso-maker-file-marker-bourne-sh").arg(QUuid::createUuid().toString()) << "\n";
-        stream<< "--volume-id=" << strVisoName << "\n";
+        stream<< "--volume-id=" << UIVisoContentBrowser::sanitizePath(strVisoName) << "\n";
         if (!strImportedISOPath.isEmpty())
-            stream << "--import-iso=" << strImportedISOPath << "\n";
+            stream << "--import-iso " << strImportedISOPath << "\n";
         stream << VisoEntryList.join("\n");
         if (!customOptions().isEmpty())
         {
