@@ -4,32 +4,48 @@
  */
 
 /*
- * Copyright (C) 2010-2011 Oracle Corporation
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ___VBoxUtils_darwin_h
-#define ___VBoxUtils_darwin_h
+#ifndef FEQT_INCLUDED_SRC_platform_darwin_VBoxUtils_darwin_h
+#define FEQT_INCLUDED_SRC_platform_darwin_VBoxUtils_darwin_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
-#include <VBox/VBoxCocoa.h>
-#include <iprt/cdefs.h> /* for RT_C_DECLS_BEGIN/RT_C_DECLS_END & stuff */
-
-#include <ApplicationServices/ApplicationServices.h>
+/* Qt includes: */
 #include <QRect>
 
-ADD_COCOA_NATIVE_REF(NSEvent);
-ADD_COCOA_NATIVE_REF(NSImage);
-ADD_COCOA_NATIVE_REF(NSView);
-ADD_COCOA_NATIVE_REF(NSWindow);
-ADD_COCOA_NATIVE_REF(NSString);
+/* GUI includes: */
+#include "UILibraryDefs.h"
 
+/* Other VBox includes: */
+#include <VBox/VBoxCocoa.h>
+#include <ApplicationServices/ApplicationServices.h>
+#undef PVM // Stupid, stupid apple headers (sys/param.h)!!
+
+/* External includes: */
+#include <ApplicationServices/ApplicationServices.h>
+
+/* Forward declarations: */
 class QImage;
 class QMainWindow;
 class QMenu;
@@ -37,6 +53,28 @@ class QPixmap;
 class QToolBar;
 class QWidget;
 
+/* Cocoa declarations: */
+ADD_COCOA_NATIVE_REF(NSButton);
+ADD_COCOA_NATIVE_REF(NSEvent);
+ADD_COCOA_NATIVE_REF(NSImage);
+ADD_COCOA_NATIVE_REF(NSString);
+ADD_COCOA_NATIVE_REF(NSView);
+ADD_COCOA_NATIVE_REF(NSWindow);
+
+
+/** Mac OS X: Standard window button types. */
+enum StandardWindowButtonType
+{
+    StandardWindowButtonType_Close,            // Since OS X 10.2
+    StandardWindowButtonType_Miniaturize,      // Since OS X 10.2
+    StandardWindowButtonType_Zoom,             // Since OS X 10.2
+    StandardWindowButtonType_Toolbar,          // Since OS X 10.2
+    StandardWindowButtonType_DocumentIcon,     // Since OS X 10.2
+    StandardWindowButtonType_DocumentVersions, // Since OS X 10.7
+    StandardWindowButtonType_FullScreen        // Since OS X 10.7
+};
+
+
 RT_C_DECLS_BEGIN
 
 /********************************************************************************
@@ -46,7 +84,8 @@ RT_C_DECLS_BEGIN
  ********************************************************************************/
 NativeNSWindowRef darwinToNativeWindowImpl(NativeNSViewRef pView);
 NativeNSViewRef darwinToNativeViewImpl(NativeNSWindowRef pWindow);
-NativeNSStringRef darwinToNativeString(const char* pcszString);
+NativeNSButtonRef darwinNativeButtonOfWindowImpl(NativeNSWindowRef pWindow, StandardWindowButtonType enmButtonType);
+SHARED_LIBRARY_STUFF NativeNSStringRef darwinToNativeString(const char* pcszString);
 QString darwinFromNativeString(NativeNSStringRef pString);
 
 /********************************************************************************
@@ -57,10 +96,10 @@ QString darwinFromNativeString(NativeNSStringRef pString);
 void darwinSetShowsToolbarButtonImpl(NativeNSWindowRef pWindow, bool fEnabled);
 void darwinSetShowsResizeIndicatorImpl(NativeNSWindowRef pWindow, bool fEnabled);
 void darwinSetHidesAllTitleButtonsImpl(NativeNSWindowRef pWindow);
-void darwinLabelWindow(NativeNSWindowRef pWindow, NativeNSImageRef pImage, bool fCenter);
+SHARED_LIBRARY_STUFF void darwinLabelWindow(NativeNSWindowRef pWindow, NativeNSImageRef pImage, double dDpr);
 void darwinSetShowsWindowTransparentImpl(NativeNSWindowRef pWindow, bool fEnabled);
-void darwinSetWindowHasShadow(NativeNSWindowRef pWindow, bool fEnabled);
-void darwinSetMouseCoalescingEnabled(bool fEnabled);
+SHARED_LIBRARY_STUFF void darwinSetWindowHasShadow(NativeNSWindowRef pWindow, bool fEnabled);
+SHARED_LIBRARY_STUFF void darwinSetMouseCoalescingEnabled(bool fEnabled);
 
 void darwintest(NativeNSWindowRef pWindow);
 /********************************************************************************
@@ -74,23 +113,24 @@ void darwinTest(NativeNSViewRef pView, NativeNSViewRef pView1, int h);
 void darwinWindowInvalidateShapeImpl(NativeNSWindowRef pWindow);
 void darwinWindowInvalidateShadowImpl(NativeNSWindowRef pWindow);
 int  darwinWindowToolBarHeight(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF int darwinWindowTitleHeight(NativeNSWindowRef pWindow);
 bool darwinIsToolbarVisible(NativeNSWindowRef pWindow);
-bool darwinIsWindowMaximized(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF bool darwinIsWindowMaximized(NativeNSWindowRef pWindow);
 void darwinMinaturizeWindow(NativeNSWindowRef pWindow);
-void darwinEnableFullscreenSupport(NativeNSWindowRef pWindow);
-void darwinEnableTransienceSupport(NativeNSWindowRef pWindow);
-void darwinToggleFullscreenMode(NativeNSWindowRef pWindow);
-bool darwinIsInFullscreenMode(NativeNSWindowRef pWindow);
-bool darwinIsOnActiveSpace(NativeNSWindowRef pWindow);
-bool darwinScreensHaveSeparateSpaces();
+SHARED_LIBRARY_STUFF void darwinEnableFullscreenSupport(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF void darwinEnableTransienceSupport(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF void darwinToggleFullscreenMode(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF void darwinToggleWindowZoom(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF bool darwinIsInFullscreenMode(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF bool darwinIsOnActiveSpace(NativeNSWindowRef pWindow);
+SHARED_LIBRARY_STUFF bool darwinScreensHaveSeparateSpaces();
+SHARED_LIBRARY_STUFF bool darwinIsScrollerStyleOverlay();
 
 bool darwinOpenFile(NativeNSStringRef pstrFile);
 
-double darwinBackingScaleFactor(NativeNSWindowRef pWindow);
-
-float darwinSmallFontSize();
-bool darwinSetFrontMostProcess();
-uint64_t darwinGetCurrentProcessId();
+SHARED_LIBRARY_STUFF float darwinSmallFontSize();
+SHARED_LIBRARY_STUFF bool darwinSetFrontMostProcess();
+SHARED_LIBRARY_STUFF uint64_t darwinGetCurrentProcessId();
 
 void darwinInstallResizeDelegate(NativeNSWindowRef pWindow);
 void darwinUninstallResizeDelegate(NativeNSWindowRef pWindow);
@@ -99,13 +139,13 @@ bool darwinUnifiedToolbarEvents(const void *pvCocoaEvent, const void *pvCarbonEv
 bool darwinMouseGrabEvents(const void *pvCocoaEvent, const void *pvCarbonEvent, void *pvUser);
 void darwinCreateContextMenuEvent(void *pvWin, int x, int y);
 
-bool darwinIsApplicationCommand(ConstNativeNSEventRef pEvent);
+SHARED_LIBRARY_STUFF bool darwinIsApplicationCommand(ConstNativeNSEventRef pEvent);
 
 void darwinRetranslateAppMenu();
 
 void darwinSendMouseGrabEvents(QWidget *pWidget, int type, int button, int buttons, int x, int y);
 
-QString darwinResolveAlias(const QString &strFile);
+SHARED_LIBRARY_STUFF QString darwinResolveAlias(const QString &strFile);
 
 RT_C_DECLS_END
 
@@ -160,6 +200,15 @@ NativeNSWindowRef darwinToNativeWindow(NativeNSViewRef pView);
  */
 NativeNSViewRef darwinToNativeView(NativeNSWindowRef pWindow);
 
+/**
+ * Returns a reference to the native button of QWidget.
+ *
+ * @returns corresponding NSButton* of the QWidget.
+ * @param   pWidget       Brings the pointer to the QWidget.
+ * @param   enmButtonType Brings the type of the native button required.
+ */
+NativeNSButtonRef darwinNativeButtonOfWindow(QWidget *pWidget, StandardWindowButtonType enmButtonType);
+
 /********************************************************************************
  *
  * Graphics stuff (Qt Wrapper)
@@ -171,17 +220,14 @@ NativeNSViewRef darwinToNativeView(NativeNSWindowRef pWindow);
  * @returns CGContextRef of the QWidget.
  * @param   pWidget      Pointer to the QWidget
  */
-CGContextRef darwinToCGContextRef(QWidget *pWidget);
-CGImageRef darwinToCGImageRef(const QImage *pImage);
-CGImageRef darwinToCGImageRef(const QPixmap *pPixmap);
-CGImageRef darwinToCGImageRef(const char *pczSource);
+SHARED_LIBRARY_STUFF CGImageRef darwinToCGImageRef(const QImage *pImage);
+SHARED_LIBRARY_STUFF CGImageRef darwinToCGImageRef(const QPixmap *pPixmap);
+SHARED_LIBRARY_STUFF CGImageRef darwinToCGImageRef(const char *pczSource);
 
-NativeNSImageRef darwinToNSImageRef(const CGImageRef pImage);
-NativeNSImageRef darwinToNSImageRef(const QImage *pImage);
-NativeNSImageRef darwinToNSImageRef(const QPixmap *pPixmap);
-NativeNSImageRef darwinToNSImageRef(const char *pczSource);
-
-#ifndef __OBJC__
+SHARED_LIBRARY_STUFF NativeNSImageRef darwinToNSImageRef(const CGImageRef pImage);
+SHARED_LIBRARY_STUFF NativeNSImageRef darwinToNSImageRef(const QImage *pImage);
+SHARED_LIBRARY_STUFF NativeNSImageRef darwinToNSImageRef(const QPixmap *pPixmap);
+SHARED_LIBRARY_STUFF NativeNSImageRef darwinToNSImageRef(const char *pczSource);
 
 #include <QEvent>
 class UIGrabMouseEvent: public QEvent
@@ -224,13 +270,12 @@ private:
  *
  ********************************************************************************/
 void darwinSetShowsToolbarButton(QToolBar *aToolBar, bool fEnabled);
-void darwinLabelWindow(QWidget *pWidget, QPixmap *pPixmap, bool fCenter);
+SHARED_LIBRARY_STUFF void darwinLabelWindow(QWidget *pWidget, QPixmap *pPixmap);
 void darwinSetShowsResizeIndicator(QWidget *pWidget, bool fEnabled);
-void darwinSetHidesAllTitleButtons(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinSetHidesAllTitleButtons(QWidget *pWidget);
 void darwinSetShowsWindowTransparent(QWidget *pWidget, bool fEnabled);
-void darwinSetWindowHasShadow(QWidget *pWidget, bool fEnabled);
-void darwinSetDockIconMenu(QMenu *pMenu);
-void darwinDisableIconsInMenus(void);
+SHARED_LIBRARY_STUFF void darwinSetWindowHasShadow(QWidget *pWidget, bool fEnabled);
+SHARED_LIBRARY_STUFF void darwinDisableIconsInMenus(void);
 
 void darwinTest(QWidget *pWidget1, QWidget *pWidget2, int h);
 
@@ -239,35 +284,35 @@ void darwinTest(QWidget *pWidget1, QWidget *pWidget2, int h);
  * Simple helper methods (Qt Wrapper)
  *
  ********************************************************************************/
-void darwinWindowAnimateResize(QWidget *pWidget, const QRect &aTarget);
+SHARED_LIBRARY_STUFF void darwinWindowAnimateResize(QWidget *pWidget, const QRect &aTarget);
 void darwinWindowAnimateResizeNew(QWidget *pWidget, int h, bool fAnimate);
 void darwinWindowInvalidateShape(QWidget *pWidget);
 void darwinWindowInvalidateShadow(QWidget *pWidget);
 int  darwinWindowToolBarHeight(QWidget *pWidget);
+SHARED_LIBRARY_STUFF int darwinWindowTitleHeight(QWidget *pWidget);
 bool darwinIsToolbarVisible(QToolBar *pToolBar);
-bool darwinIsWindowMaximized(QWidget *pWidget);
+SHARED_LIBRARY_STUFF bool darwinIsWindowMaximized(QWidget *pWidget);
 void darwinMinaturizeWindow(QWidget *pWidget);
-void darwinEnableFullscreenSupport(QWidget *pWidget);
-void darwinEnableTransienceSupport(QWidget *pWidget);
-void darwinToggleFullscreenMode(QWidget *pWidget);
-bool darwinIsInFullscreenMode(QWidget *pWidget);
-bool darwinIsOnActiveSpace(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinEnableFullscreenSupport(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinEnableTransienceSupport(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinToggleFullscreenMode(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinToggleWindowZoom(QWidget *pWidget);
+SHARED_LIBRARY_STUFF bool darwinIsInFullscreenMode(QWidget *pWidget);
+SHARED_LIBRARY_STUFF bool darwinIsOnActiveSpace(QWidget *pWidget);
 bool darwinOpenFile(const QString &strFile);
 
-double darwinBackingScaleFactor(QWidget *pWidget);
-
 QString darwinSystemLanguage(void);
 QPixmap darwinCreateDragPixmap(const QPixmap& aPixmap, const QString &aText);
 
 void darwinInstallResizeDelegate(QWidget *pWidget);
 void darwinUninstallResizeDelegate(QWidget *pWidget);
 
-void darwinRegisterForUnifiedToolbarContextMenuEvents(QMainWindow *pWindow);
-void darwinUnregisterForUnifiedToolbarContextMenuEvents(QMainWindow *pWindow);
+SHARED_LIBRARY_STUFF void darwinRegisterForUnifiedToolbarContextMenuEvents(QMainWindow *pWindow);
+SHARED_LIBRARY_STUFF void darwinUnregisterForUnifiedToolbarContextMenuEvents(QMainWindow *pWindow);
 
-void darwinMouseGrab(QWidget *pWidget);
-void darwinMouseRelease(QWidget *pWidget);
-#endif /* !__OBJC__ */
+SHARED_LIBRARY_STUFF void darwinMouseGrab(QWidget *pWidget);
+SHARED_LIBRARY_STUFF void darwinMouseRelease(QWidget *pWidget);
 
-#endif /* !___VBoxUtils_darwin_h */
+SHARED_LIBRARY_STUFF void *darwinCocoaToCarbonEvent(void *pvCocoaEvent);
 
+#endif /* !FEQT_INCLUDED_SRC_platform_darwin_VBoxUtils_darwin_h */
