@@ -4,30 +4,40 @@
  */
 
 /*
- * Copyright (C) 2012-2013 Oracle Corporation
+ * Copyright (C) 2012-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_SUP_DRV
 #define SUPDRV_AGNOSTIC
 #include "SUPDrvInternal.h"
@@ -45,10 +55,19 @@
 #include <iprt/param.h>
 #include <iprt/uuid.h>
 
+#if defined(RT_OS_LINUX)
+# if RTLNX_VER_MIN(4,15,10)
+#  include <asm/nospec-branch.h>
+# endif /* < 4.15.10 */
+# if RTLNX_VER_MIN(5,17,0)
+#  include <asm/linkage.h>
+# endif /* < 5.17.0 */
+#endif /* !RT_OS_LINUX */
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /** Pointer to a user tracer module registration record. */
 typedef struct SUPDRVTRACERUMOD *PSUPDRVTRACERUMOD;
 
@@ -86,7 +105,8 @@ typedef struct SUPDRVTPPROVIDER
      *  tracer. */
     bool                    fRegistered;
     /** The provider name (for logging purposes). */
-    char                    szName[1];
+    RT_FLEXIBLE_ARRAY_EXTENSION
+    char                    szName[RT_FLEXIBLE_ARRAY];
 } SUPDRVTPPROVIDER;
 /** Pointer to the data for a tracepoint provider. */
 typedef SUPDRVTPPROVIDER *PSUPDRVTPPROVIDER;
@@ -151,15 +171,16 @@ typedef struct SUPDRVTRACERUMOD
     /** The number of probe locations. */
     uint32_t                cProbeLocs;
     /** Ring-0 probe location info. */
-    SUPDRVPROBELOC          aProbeLocs[1];
+    RT_FLEXIBLE_ARRAY_EXTENSION
+    SUPDRVPROBELOC          aProbeLocs[RT_FLEXIBLE_ARRAY];
 } SUPDRVTRACERUMOD;
 /** Magic value for SUPDRVVTGCOPY. */
 #define SUPDRVTRACERUMOD_MAGIC UINT32_C(0x00080486)
 
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** Simple SUPR0Printf-style logging.  */
 #ifdef DEBUG_bird
 # define LOG_TRACER(a_Args)  SUPR0Printf a_Args
@@ -168,16 +189,16 @@ typedef struct SUPDRVTRACERUMOD
 #endif
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** The address of the current probe fire routine for kernel mode. */
 PFNRT       g_pfnSupdrvProbeFireKernel = supdrvTracerProbeFireStub;
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Internal Functions                                                                                                           *
+*********************************************************************************************************************************/
 static void supdrvVtgReleaseObjectCopy(PSUPDRVDEVEXT pDevExt, PSUPDRVVTGCOPY pThis);
 
 
@@ -248,7 +269,6 @@ static int supdrvVtgValidateString(const char *psz)
  * @param   pVtgHdr             The header.
  * @param   uVtgHdrAddr         The address where the header is actually
  *                              loaded.
- * @param   cbVtgObj            The alleged size of the header.
  * @param   pbImage             The image base, if available.
  * @param   cbImage             The image size, if available.
  * @param   fUmod               Whether this is a user module.
@@ -339,13 +359,16 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
                         u64Tmp, pVtgHdr->uProbeLocs.u64, pVtgHdr->uProbeLocsEnd.u64);
             return VERR_SUPDRV_VTG_BAD_HDR_TOO_MUCH;
         }
+        /*SUPR0Printf("supdrvVtgValidateHdr: cbProbeLocs %#x -> %#x\n", pVtgHdr->cbProbeLocs, (uint32_t)u64Tmp);*/
         pVtgHdr->cbProbeLocs  = (uint32_t)u64Tmp;
 
         u64Tmp = pVtgHdr->uProbeLocs.u64 - uVtgHdrAddr;
 #ifdef RT_OS_DARWIN
         /* The loader and/or ld64-97.17 seems not to generate fixups for our
            __VTGObj section. Detect this by comparing them with the
-           u64VtgObjSectionStart member and assume max image size of 4MB. */
+           u64VtgObjSectionStart member and assume max image size of 4MB.
+           Seems to be worked around by the __VTGPrLc.End and __VTGPrLc.Begin
+           padding fudge, meaning that the linker misplaced the relocations. */
         if (   (int64_t)u64Tmp != (int32_t)u64Tmp
             && pVtgHdr->u64VtgObjSectionStart != uVtgHdrAddr
             && pVtgHdr->u64VtgObjSectionStart < _4M
@@ -353,6 +376,7 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
             && !fUmod)
         {
             uint64_t offDelta = uVtgHdrAddr - pVtgHdr->u64VtgObjSectionStart;
+            /*SUPR0Printf("supdrvVtgValidateHdr: offDelta=%#llx\n", offDelta);*/
             pVtgHdr->uProbeLocs.u64        += offDelta;
             pVtgHdr->uProbeLocsEnd.u64     += offDelta;
             u64Tmp += offDelta;
@@ -364,6 +388,7 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
                         u64Tmp, pVtgHdr->uProbeLocs.u64, uVtgHdrAddr);
             return VERR_SUPDRV_VTG_BAD_HDR_PTR;
         }
+        /*SUPR0Printf("supdrvVtgValidateHdr: offProbeLocs %#x -> %#x\n", pVtgHdr->offProbeLocs, (int32_t)u64Tmp);*/
         pVtgHdr->offProbeLocs = (int32_t)u64Tmp;
     }
 
@@ -371,7 +396,10 @@ static int supdrvVtgValidateHdr(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const
      * The non-area description fields.
      */
     if (memcmp(pVtgHdr->szMagic, VTGOBJHDR_MAGIC, sizeof(pVtgHdr->szMagic)))
+    {
+        SUPR0Printf("supdrvVtgValidateHdr: %p: %.16Rhxs\n", pVtgHdr, pVtgHdr->szMagic);
         return VERR_SUPDRV_VTG_MAGIC;
+    }
     if (   pVtgHdr->cBits != ARCH_BITS
         && (   !fUmod
             || (   pVtgHdr->cBits != 32
@@ -530,6 +558,8 @@ static int supdrvVtgValidate(PVTGOBJHDR pVtgHdr, RTUINTPTR uVtgHdrAddr, const ui
         MY_VALIDATE_ATTR(pProvider->AttrNames);
         MY_VALIDATE_ATTR(pProvider->AttrArguments);
         MY_CHECK_RET(pProvider->bReserved == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
+        MY_CHECK_RET(pProvider->cProbesEnabled == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
+        MY_CHECK_RET(pProvider->uSettingsSerialNo == 0, VERR_SUPDRV_VTG_BAD_PROVIDER);
     }
 
     /*
@@ -859,7 +889,7 @@ static void supdrvTracerRemoveAllProviders(PSUPDRVDEVEXT pDevExt)
  * Registers the VTG tracepoint providers of a driver.
  *
  * @returns VBox status code.
- * @param   pszName             The driver name.
+ * @param   pDevExt             The device instance data.
  * @param   pVtgHdr             The VTG object header.
  * @param   pImage              The image if applicable.
  * @param   pSession            The session if applicable.
@@ -944,7 +974,7 @@ static int supdrvTracerRegisterVtgObj(PSUPDRVDEVEXT pDevExt, PVTGOBJHDR pVtgHdr,
         const char      *pszName = supdrvVtgGetString(pVtgHdr, pDesc->offName);
         size_t const     cchName = strlen(pszName) + (pUmod ? 16 : 0);
 
-        pProv = (PSUPDRVTPPROVIDER)RTMemAllocZ(RT_OFFSETOF(SUPDRVTPPROVIDER, szName[cchName + 1 + cchModName + 1]));
+        pProv = (PSUPDRVTPPROVIDER)RTMemAllocZ(RT_UOFFSETOF_DYN(SUPDRVTPPROVIDER, szName[cchName + 1 + cchModName + 1]));
         if (pProv)
         {
             pProv->Core.pszName         = &pProv->szName[0];
@@ -979,10 +1009,10 @@ static int supdrvTracerRegisterVtgObj(PSUPDRVDEVEXT pDevExt, PVTGOBJHDR pVtgHdr,
             pProv->fRegistered          = true;
 
             if (!pUmod)
-                memcpy(pProv->szName, pszName, cchName + 1);
+                RT_BCOPY_UNFORTIFIED(pProv->szName, pszName, cchName + 1);
             else
                 RTStrPrintf(pProv->szName, cchName + 1, "%s%u", pszName, (uint32_t)pSession->Process);
-            memcpy((void *)pProv->Core.pszModName, pszModName, cchModName + 1);
+            RT_BCOPY_UNFORTIFIED((void *)pProv->Core.pszModName, pszModName, cchModName + 1);
 
             /*
              * Do the actual registration and list manipulations while holding
@@ -1088,13 +1118,13 @@ SUPR0DECL(int) SUPR0TracerRegisterDrv(PSUPDRVSESSION pSession, PVTGOBJHDR pVtgHd
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterDrv);
 
 
 /**
  * Deregister the VTG tracepoint providers of a driver.
  *
  * @param   pSession            The support driver session handle.
- * @param   pVtgHdr             The VTG header.
  */
 SUPR0DECL(void) SUPR0TracerDeregisterDrv(PSUPDRVSESSION pSession)
 {
@@ -1121,6 +1151,7 @@ SUPR0DECL(void) SUPR0TracerDeregisterDrv(PSUPDRVSESSION pSession)
      */
     supdrvTracerProcessZombies(pDevExt);
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerDeregisterDrv);
 
 
 /**
@@ -1167,6 +1198,7 @@ SUPR0DECL(int) SUPR0TracerRegisterModule(void *hMod, PVTGOBJHDR pVtgHdr)
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterModule);
 
 
 /**
@@ -1266,6 +1298,7 @@ SUPR0DECL(int) SUPR0TracerRegisterImpl(void *hMod, PSUPDRVSESSION pSession, PCSU
     return rc;
 
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerRegisterImpl);
 
 
 /**
@@ -1451,6 +1484,7 @@ SUPR0DECL(int) SUPR0TracerDeregisterImpl(void *hMod, PSUPDRVSESSION pSession)
 
     return rc;
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerDeregisterImpl);
 
 
 /*
@@ -1464,29 +1498,58 @@ SUPR0DECL(int) SUPR0TracerDeregisterImpl(void *hMod, PSUPDRVSESSION pSession)
 __asm__("\
         .section .text                                                  \n\
                                                                         \n\
-        .p2align 2,,3                                                   \n\
+        .p2align 4                                                      \n\
         .global SUPR0TracerFireProbe                                    \n\
+        .type   SUPR0TracerFireProbe, @function                         \n\
 SUPR0TracerFireProbe:                                                   \n\
 ");
 # if   defined(RT_ARCH_AMD64)
-__asm__(" \
-	    movq    g_pfnSupdrvProbeFireKernel(%rip), %rax                  \n\
-	    jmp	    *%rax \n\
+__asm__("\
+            movq    g_pfnSupdrvProbeFireKernel(%rip), %rax              \n\
+            "
+#  if defined(RT_OS_LINUX)
+#   if RTLNX_VER_MIN(4,15,10)
+            ANNOTATE_RETPOLINE_SAFE
+#   endif
+#  endif
+            " \n\
+            jmp     *%rax \n\
 ");
 # elif defined(RT_ARCH_X86)
 __asm__("\
-	    movl    g_pfnSupdrvProbeFireKernel, %eax                        \n\
-	    jmp	    *%eax \n\
+            movl    g_pfnSupdrvProbeFireKernel, %eax                    \n\
+            "
+#  if defined(RT_OS_LINUX)
+#   if RTLNX_VER_MIN(4,15,10)
+            ANNOTATE_RETPOLINE_SAFE
+#   endif
+#  endif
+            " \n\
+            jmp     *%eax \n\
 ");
 # else
 #  error "Which arch is this?"
 # endif
 __asm__("\
+        .size SUPR0TracerFireProbe, . - SUPR0TracerFireProbe            \n\
                                                                         \n\
         .type supdrvTracerProbeFireStub,@function                       \n\
         .global supdrvTracerProbeFireStub                               \n\
 supdrvTracerProbeFireStub:                                              \n\
-        ret                                                             \n\
+        "
+# if defined(RT_OS_LINUX)
+#  if RTLNX_VER_MIN(5,17,0)
+        ASM_RET "\n\
+        "
+#  else /* < 5.17.0 */
+        "ret \n\
+        "
+#  endif /* < 5.17.0 */
+# else /* !RT_OS_LINUX */
+        "ret \n\
+        "
+# endif /* !RT_OS_LINUX */
+        ".size supdrvTracerProbeFireStub, . - supdrvTracerProbeFireStub  \n\
                                                                         \n\
         .previous                                                       \n\
 ");
@@ -1494,6 +1557,7 @@ supdrvTracerProbeFireStub:                                              \n\
  )
 # endif
 #endif
+SUPR0_EXPORT_SYMBOL(SUPR0TracerFireProbe);
 
 
 /**
@@ -1714,11 +1778,11 @@ static int supdrvVtgCreateObjectCopy(PSUPDRVDEVEXT pDevExt, PCVTGOBJHDR pVtgHdr,
      * Calculate the space required, allocate and copy in the data.
      */
     int             rc;
-    size_t const    cProbeLocs   = pVtgHdr->cbProbeLocs / (pVtgHdr->cBits == 32 ? sizeof(VTGPROBELOC32) : sizeof(VTGPROBELOC64));
-    size_t const    cbProbeLocs  = cProbeLocs * sizeof(VTGPROBELOC);
-    size_t const    offProbeLocs = RT_ALIGN(pVtgHdr->cbObj, 8);
+    uint32_t const  cProbeLocs   = pVtgHdr->cbProbeLocs / (pVtgHdr->cBits == 32 ? sizeof(VTGPROBELOC32) : sizeof(VTGPROBELOC64));
+    uint32_t const  cbProbeLocs  = cProbeLocs * sizeof(VTGPROBELOC);
+    uint32_t const  offProbeLocs = RT_ALIGN(pVtgHdr->cbObj, 8);
     size_t const    cb           = offProbeLocs + cbProbeLocs + cbStrTab + 1;
-    PSUPDRVVTGCOPY  pThis = (PSUPDRVVTGCOPY)RTMemAlloc(RT_OFFSETOF(SUPDRVVTGCOPY, Hdr) + cb);
+    PSUPDRVVTGCOPY  pThis = (PSUPDRVVTGCOPY)RTMemAlloc(RT_UOFFSETOF(SUPDRVVTGCOPY, Hdr) + cb);
     if (!pThis)
         return VERR_NO_MEMORY;
 
@@ -2010,7 +2074,7 @@ int  VBOXCALL   supdrvIOCtl_TracerUmodRegister(PSUPDRVDEVEXT pDevExt, PSUPDRVSES
     /*
      * Allocate the tracker data we keep in the session.
      */
-    pUmod = (PSUPDRVTRACERUMOD)RTMemAllocZ(  RT_OFFSETOF(SUPDRVTRACERUMOD, aProbeLocs[cProbeLocs])
+    pUmod = (PSUPDRVTRACERUMOD)RTMemAllocZ(  RT_UOFFSETOF_DYN(SUPDRVTRACERUMOD, aProbeLocs[cProbeLocs])
                                            + (Hdr.cbProbeEnabled / sizeof(uint32_t) * sizeof(SUPDRVPROBEINFO)) );
     if (!pUmod)
         return VERR_NO_MEMORY;
@@ -2136,6 +2200,7 @@ int  VBOXCALL   supdrvIOCtl_TracerUmodDeregister(PSUPDRVDEVEXT pDevExt, PSUPDRVS
         RTR0MemObjFree(pUmod->hMemObjMap, false /*fFreeMappings*/);
         RTR0MemObjFree(pUmod->hMemObjLock, false /*fFreeMappings*/);
         RTMemFree(pUmod);
+        rc = VINF_SUCCESS;
     }
     else
         rc = VERR_NOT_FOUND;
@@ -2217,6 +2282,7 @@ SUPR0DECL(void) SUPR0TracerUmodProbeFire(PSUPDRVSESSION pSession, PSUPDRVTRACERU
 
     supdrvTracerUmodProbeFire(pSession->pDevExt, pSession, pCtx);
 }
+SUPR0_EXPORT_SYMBOL(SUPR0TracerUmodProbeFire);
 
 
 void  VBOXCALL  supdrvIOCtl_TracerUmodProbeFire(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPDRVTRACERUSRCTX pCtx)
@@ -2448,7 +2514,6 @@ int VBOXCALL supdrvTracerInit(PSUPDRVDEVEXT pDevExt)
 /**
  * Late module termination hook.
  *
- * @returns VBox status code.
  * @param   pDevExt             The device extension structure.
  */
 void VBOXCALL supdrvTracerTerm(PSUPDRVDEVEXT pDevExt)
@@ -2456,7 +2521,9 @@ void VBOXCALL supdrvTracerTerm(PSUPDRVDEVEXT pDevExt)
     LOG_TRACER(("supdrvTracerTerm\n"));
 
     supdrvTracerRemoveAllProviders(pDevExt);
-
+#ifdef VBOX_WITH_NATIVE_DTRACE
+    supdrvDTraceFini();
+#endif
     RTSemFastMutexDestroy(pDevExt->mtxTracer);
     pDevExt->mtxTracer = NIL_RTSEMFASTMUTEX;
     LOG_TRACER(("supdrvTracerTerm: Done\n"));
