@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2015-2016 Oracle Corporation
+ * Copyright (C) 2015-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -23,9 +33,12 @@
 
 #include <algorithm>
 #include <map>
-#include <string>
+#include <iprt/sanitized/string>
 #include <vector>
 
+#include <VBox/version.h>
+
+#include <iprt/err.h>
 #include <iprt/initterm.h>
 #include <iprt/message.h>
 #include <iprt/string.h>
@@ -132,17 +145,17 @@ static int ParseAlias(char *pszLine, size_t& id, std::string& desc)
     /* First there's a hexadeciman number. */
     uint32_t uVal;
     char *pszNext;
-    int rc = RTStrToUInt32Ex(pszLine, &pszNext, 16, &uVal);
-    if (   rc == VWRN_TRAILING_CHARS
-        || rc == VWRN_TRAILING_SPACES
-        || rc == VINF_SUCCESS)
+    int vrc = RTStrToUInt32Ex(pszLine, &pszNext, 16, &uVal);
+    if (   vrc == VWRN_TRAILING_CHARS
+        || vrc == VWRN_TRAILING_SPACES
+        || vrc == VINF_SUCCESS)
     {
         /* Skip the whipespace following it and at the end of the line. */
         pszNext = RTStrStripL(pszNext);
         if (*pszNext != '\0')
         {
-            rc = RTStrValidateEncoding(pszNext);
-            if (RT_SUCCESS(rc))
+            vrc = RTStrValidateEncoding(pszNext);
+            if (RT_SUCCESS(vrc))
             {
                 size_t cchDesc = strlen(pszNext);
                 if (cchDesc <= USB_ID_DATABASE_MAX_STRING)
@@ -155,13 +168,13 @@ static int ParseAlias(char *pszLine, size_t& id, std::string& desc)
                 RTMsgError("String to long: %zu", cchDesc);
             }
             else
-                RTMsgError("Invalid encoding: '%s' (rc=%Rrc)", pszNext, rc);
+                RTMsgError("Invalid encoding: '%s' (vrc=%Rrc)", pszNext, vrc);
         }
         else
             RTMsgError("Error parsing '%s'", pszLine);
     }
     else
-        RTMsgError("Error converting number at the start of '%s': %Rrc", pszLine, rc);
+        RTMsgError("Error converting number at the start of '%s': %Rrc", pszLine, vrc);
     return ERROR_IN_PARSE_LINE;
 }
 
@@ -191,8 +204,8 @@ static int ParseUsbIds(PRTSTREAM pInStrm, const char *pszFile)
     for (;;)
     {
         char szLine[_4K];
-        int rc = RTStrmGetLine(pInStrm, szLine, sizeof(szLine));
-        if (RT_SUCCESS(rc))
+        int vrc = RTStrmGetLine(pInStrm, szLine, sizeof(szLine));
+        if (RT_SUCCESS(vrc))
         {
             iLine++;
 
@@ -231,10 +244,10 @@ static int ParseUsbIds(PRTSTREAM pInStrm, const char *pszFile)
                      && *RTStrStripL(szLine) != '\0')
                 vendor.vendorID = 0;
         }
-        else if (rc == VERR_EOF)
+        else if (vrc == VERR_EOF)
             return RTEXITCODE_SUCCESS;
         else
-            return RTMsgErrorExit(RTEXITCODE_FAILURE, "RTStrmGetLine failed: %Rrc", rc);
+            return RTMsgErrorExit(RTEXITCODE_FAILURE, "RTStrmGetLine failed: %Rrc", vrc);
     }
 }
 
@@ -246,15 +259,25 @@ static void WriteSourceFile(FILE *pOut, const char *argv0, PBLDPROGSTRTAB pStrTa
             " */\n"
             "\n"
             "/*\n"
-            " * Copyright (C) 2015-2016 Oracle Corporation\n"
+            " * Copyright (C) 2015-" VBOX_C_YEAR " Oracle and/or its affiliates.\n"
             " *\n"
-            " * This file is part of VirtualBox Open Source Edition(OSE), as\n"
-            " * available from http ://www.virtualbox.org. This file is free software;\n"
-            " * you can redistribute it and / or modify it under the terms of the GNU\n"
-            " * General Public License(GPL) as published by the Free Software\n"
-            " * Foundation, in version 2 as it comes in the \"COPYING\" file of the\n"
-            " * VirtualBox OSE distribution.VirtualBox OSE is distributed in the\n"
-            " * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.\n"
+            " * This file is part of VirtualBox base platform packages, as\n"
+            " * available from https://www.virtualbox.org.\n"
+            " *\n"
+            " * This program is free software; you can redistribute it and/or\n"
+            " * modify it under the terms of the GNU General Public License\n"
+            " * as published by the Free Software Foundation, in version 3 of the\n"
+            " * License.\n"
+            " *\n"
+            " * This program is distributed in the hope that it will be useful, but\n"
+            " * WITHOUT ANY WARRANTY; without even the implied warranty of\n"
+            " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU\n"
+            " * General Public License for more details.\n"
+            " *\n"
+            " * You should have received a copy of the GNU General Public License\n"
+            " * along with this program; if not, see <https://www.gnu.org/licenses>.\n"
+            " *\n"
+            " * SPDX-License-Identifier: GPL-3.0-only\n"
             " */"
             "\n"
             "\n"
@@ -314,9 +337,9 @@ int main(int argc, char *argv[])
     /*
      * Initialize IPRT and convert argv to UTF-8.
      */
-    int rc = RTR3InitExe(argc, &argv, 0);
-    if (RT_FAILURE(rc))
-        return RTMsgInitFailure(rc);
+    int vrc = RTR3InitExe(argc, &argv, 0);
+    if (RT_FAILURE(vrc))
+        return RTMsgInitFailure(vrc);
 
     /*
      * Parse arguments and read input files.
@@ -346,17 +369,17 @@ int main(int argc, char *argv[])
         }
 
         PRTSTREAM pInStrm;
-        rc = RTStrmOpen(argv[i], "r", &pInStrm);
-        if (RT_FAILURE(rc))
+        vrc = RTStrmOpen(argv[i], "r", &pInStrm);
+        if (RT_FAILURE(vrc))
             return RTMsgErrorExit((RTEXITCODE)ERROR_OPEN_FILE,
-                                  "Failed to open file '%s' for reading: %Rrc", argv[i], rc);
+                                  "Failed to open file '%s' for reading: %Rrc", argv[i], vrc);
 
-        rc = ParseUsbIds(pInStrm, argv[i]);
+        vrc = ParseUsbIds(pInStrm, argv[i]);
         RTStrmClose(pInStrm);
-        if (rc != 0)
+        if (vrc != 0)
         {
             RTMsgError("Failed parsing USB devices file '%s'", argv[i]);
-            return rc;
+            return vrc;
         }
     }
 
