@@ -12,15 +12,25 @@
  */
 
 /*
- * Copyright (C) 2006-2014 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /*
@@ -52,6 +62,7 @@
 
 
 #include <stdio.h>
+#include <iprt/win/windows.h>  /* Avoid -Wall warnings. */
 #include "VirtualBox.h"
 
 #define SAFE_RELEASE(x) \
@@ -115,7 +126,7 @@ int testErrorInfo(IVirtualBox *virtualBox)
         rc = GetErrorInfo(0, &errorInfo);
 
         if (FAILED(rc))
-            printf("Error getting error info! rc = 0x%x\n", rc);
+            printf("Error getting error info! rc=%#lx\n", rc);
         else
         {
             BSTR errorDescription = NULL;
@@ -123,7 +134,7 @@ int testErrorInfo(IVirtualBox *virtualBox)
             rc = errorInfo->GetDescription(&errorDescription);
 
             if (FAILED(rc) || !errorDescription)
-                printf("Error getting error description! rc = 0x%x\n", rc);
+                printf("Error getting error description! rc=%#lx\n", rc);
             else
             {
                 printf("Successfully retrieved error description: %S\n", errorDescription);
@@ -159,7 +170,7 @@ int testStartVM(IVirtualBox *virtualBox)
         rc = GetErrorInfo(0, &errorInfo);
 
         if (FAILED(rc))
-            printf("Error getting error info! rc = 0x%x\n", rc);
+            printf("Error getting error info! rc=%#lx\n", rc);
         else
         {
             BSTR errorDescription = NULL;
@@ -167,7 +178,7 @@ int testStartVM(IVirtualBox *virtualBox)
             rc = errorInfo->GetDescription(&errorDescription);
 
             if (FAILED(rc) || !errorDescription)
-                printf("Error getting error description! rc = 0x%x\n", rc);
+                printf("Error getting error description! rc=%#lx\n", rc);
             else
             {
                 printf("Successfully retrieved error description: %S\n", errorDescription);
@@ -191,19 +202,19 @@ int testStartVM(IVirtualBox *virtualBox)
             rc = machine->get_Id(&guid); /* Get the GUID of the machine. */
             if (!SUCCEEDED(rc))
             {
-                printf("Error retrieving machine ID! rc = 0x%x\n", rc);
+                printf("Error retrieving machine ID! rc=%#lx\n", rc);
                 break;
             }
 
             /* Create the session object. */
             rc = CoCreateInstance(CLSID_Session,        /* the VirtualBox base object */
                                   NULL,                 /* no aggregation */
-                                  CLSCTX_INPROC_SERVER, /* the object lives in a server process on this machine */
+                                  CLSCTX_INPROC_SERVER, /* the object lives in the current process */
                                   IID_ISession,         /* IID of the interface */
                                   (void**)&session);
             if (!SUCCEEDED(rc))
             {
-                printf("Error creating Session instance! rc = 0x%x\n", rc);
+                printf("Error creating Session instance! rc=%#lx\n", rc);
                 break;
             }
 
@@ -212,7 +223,7 @@ int testStartVM(IVirtualBox *virtualBox)
                                           NULL, &progress);
             if (!SUCCEEDED(rc))
             {
-                printf("Could not open remote session! rc = 0x%x\n", rc);
+                printf("Could not open remote session! rc=%#lx\n", rc);
                 break;
             }
 
@@ -255,40 +266,38 @@ int testStartVM(IVirtualBox *virtualBox)
 }
 
 
-int main(int argc, char *argv[])
+int main()
 {
-    HRESULT rc;
-    IVirtualBox *virtualBox;
+    /* Initialize the COM subsystem. */
+    CoInitialize(NULL);
 
-    do
+    /* Instantiate the VirtualBox root object. */
+    IVirtualBoxClient *virtualBoxClient;
+    HRESULT rc = CoCreateInstance(CLSID_VirtualBoxClient, /* the VirtualBoxClient object */
+                                  NULL,                   /* no aggregation */
+                                  CLSCTX_INPROC_SERVER,   /* the object lives in the current process */
+                                  IID_IVirtualBoxClient,  /* IID of the interface */
+                                  (void**)&virtualBoxClient);
+    if (SUCCEEDED(rc))
     {
-        /* Initialize the COM subsystem. */
-        CoInitialize(NULL);
-
-        /* Instantiate the VirtualBox root object. */
-        rc = CoCreateInstance(CLSID_VirtualBox,       /* the VirtualBox base object */
-                              NULL,                   /* no aggregation */
-                              CLSCTX_LOCAL_SERVER,    /* the object lives in a server process on this machine */
-                              IID_IVirtualBox,        /* IID of the interface */
-                              (void**)&virtualBox);
-
-        if (!SUCCEEDED(rc))
+        IVirtualBox *virtualBox;
+        rc = virtualBoxClient->get_VirtualBox(&virtualBox);
+        if (SUCCEEDED(rc))
         {
-            printf("Error creating VirtualBox instance! rc = 0x%x\n", rc);
-            break;
+            listVMs(virtualBox);
+
+            testErrorInfo(virtualBox);
+
+            /* Enable the following line to get a VM started. */
+            //testStartVM(virtualBox);
+
+            /* Release the VirtualBox object. */
+            virtualBox->Release();
+            virtualBoxClient->Release();
         }
-
-        listVMs(virtualBox);
-
-        testErrorInfo(virtualBox);
-
-        /* Enable the following line to get a VM started. */
-        //testStartVM(virtualBox);
-
-        /* Release the VirtualBox object. */
-        virtualBox->Release();
-
-    } while (0);
+        else
+            printf("Error creating VirtualBox instance! rc=%#lx\n", rc);
+    }
 
     CoUninitialize();
     return 0;
