@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2009-2014 Oracle Corporation
+ * Copyright (C) 2009-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 /*
@@ -42,10 +52,13 @@
 
 #include "proxy.h"
 #include "proxy_pollmgr.h"
+#include "pxtcp.h"
 
 #include "lwip/sys.h"
 #include "lwip/tcpip.h"
+#include "lwip/ip_addr.h"
 #include "lwip/udp.h"
+#include "lwip/tcp.h"
 
 #ifndef RT_OS_WINDOWS
 #include <sys/poll.h>
@@ -83,6 +96,8 @@ struct pxdns {
     struct udp_pcb *pcb4;
     struct udp_pcb *pcb6;
 
+    struct tcp_pcb *ltcp;
+
     size_t generation;
     size_t nresolvers;
     union sockaddr_inet *resolvers;
@@ -177,6 +192,8 @@ struct request {
 static void pxdns_create_resolver_sockaddrs(struct pxdns *pxdns,
                                             const char **nameservers);
 
+static err_t pxdns_accept_syn(void *arg, struct tcp_pcb *newpcb, struct pbuf *syn);
+
 static void pxdns_recv4(void *arg, struct udp_pcb *pcb, struct pbuf *p,
                         ip_addr_t *addr, u16_t port);
 static void pxdns_recv6(void *arg, struct udp_pcb *pcb, struct pbuf *p,
@@ -210,6 +227,16 @@ pxdns_init(struct netif *proxy_netif)
 
     LWIP_UNUSED_ARG(proxy_netif);
 
+    pxdns->ltcp = tcp_new();
+    if (pxdns->ltcp != NULL) {
+        tcp_bind_ip6(pxdns->ltcp, IP6_ADDR_ANY, 53);
+        pxdns->ltcp = tcp_listen_dual(pxdns->ltcp);
+        if (pxdns->ltcp != NULL) {
+            tcp_arg(pxdns->ltcp, pxdns);
+            tcp_accept_syn(pxdns->ltcp, pxdns_accept_syn);
+        }
+    }
+
     pxdns->pmhdl4.callback = pxdns_pmgr_pump;
     pxdns->pmhdl4.data = (void *)pxdns;
     pxdns->pmhdl4.slot = -1;
@@ -295,7 +322,7 @@ pxdns_set_nameservers(void *arg)
     const char **nameservers = (const char **)arg;
 
     if (g_proxy_options->nameservers != NULL) {
-        RTMemFree(g_proxy_options->nameservers);
+        RTMemFree((void *)g_proxy_options->nameservers);
     }
     g_proxy_options->nameservers = nameservers;
 
@@ -334,7 +361,7 @@ pxdns_create_resolver_sockaddrs(struct pxdns *pxdns, const char **nameservers)
         goto update_resolvers;
     }
 
-    resolvers = (union sockaddr_inet *)calloc(sizeof(resolvers[0]), nnames);
+    resolvers = (union sockaddr_inet *)calloc(nnames, sizeof(resolvers[0]));
     if (resolvers == NULL) {
         nresolvers = 0;
         goto update_resolvers;
@@ -413,6 +440,8 @@ pxdns_hash_add(struct pxdns *pxdns, struct request *req)
     struct request **chain;
 
     LWIP_ASSERT1(req->pprev_hash == NULL);
+    ++pxdns->active_queries;
+
     chain = &pxdns->request_hash[HASH(req->id)];
     if ((req->next_hash = *chain) != NULL) {
         (*chain)->pprev_hash = &req->next_hash;
@@ -442,6 +471,7 @@ pxdns_timeout_add(struct pxdns *pxdns, struct request *req)
     omask = pxdns->timeout_mask;
     pxdns->timeout_mask |= 1U << req->timeout_slot;
     if (omask == 0) {
+        sys_untimeout(pxdns_timer, pxdns);
         sys_timeout(1 * 1000, pxdns_timer, pxdns);
     }
 }
@@ -493,7 +523,6 @@ pxdns_request_register(struct pxdns *pxdns, struct request *req)
 
     pxdns_hash_add(pxdns, req);
     pxdns_timeout_add(pxdns, req);
-    ++pxdns->active_queries;
 
     sys_mutex_unlock(&pxdns->lock);
 }
@@ -506,7 +535,6 @@ pxdns_request_deregister(struct pxdns *pxdns, struct request *req)
 
     pxdns_hash_del(pxdns, req);
     pxdns_timeout_del(pxdns, req);
-    --pxdns->active_queries;
 
     sys_mutex_unlock(&pxdns->lock);
 }
@@ -534,7 +562,6 @@ pxdns_request_find(struct pxdns *pxdns, u16_t id)
     if (req != NULL) {
         pxdns_hash_del(pxdns, req);
         pxdns_timeout_del(pxdns, req);
-        --pxdns->active_queries;
     }
 
     sys_mutex_unlock(&pxdns->lock);
@@ -685,6 +712,14 @@ pxdns_forward_outbound(struct pxdns *pxdns, struct request *req)
 {
     union sockaddr_inet *resolver;
     ssize_t nsent;
+#ifdef RT_OS_WINDOWS
+    const char *pSendData = (const char *)&req->data[0];
+    int         cbSendData = (int)req->size;
+    Assert((size_t)cbSendData == req->size);
+#else
+    const void *pSendData = &req->data[0];
+    size_t      cbSendData = req->size;
+#endif
 
     DPRINTF2(("%s: req %p: sending to resolver #%lu\n",
               __func__, (void *)req, (unsigned long)req->residx));
@@ -694,13 +729,13 @@ pxdns_forward_outbound(struct pxdns *pxdns, struct request *req)
     resolver = &pxdns->resolvers[req->residx];
 
     if (resolver->sa.sa_family == AF_INET) {
-        nsent = sendto(pxdns->sock4, req->data, req->size, 0,
+        nsent = sendto(pxdns->sock4, pSendData, cbSendData, 0,
                        &resolver->sa, sizeof(resolver->sin));
-        
+
     }
     else if (resolver->sa.sa_family == AF_INET6) {
         if (pxdns->sock6 != INVALID_SOCKET) {
-            nsent = sendto(pxdns->sock6, req->data, req->size, 0,
+            nsent = sendto(pxdns->sock6, pSendData, cbSendData, 0,
                            &resolver->sa, sizeof(resolver->sin6));
         }
         else {
@@ -798,7 +833,11 @@ pxdns_pmgr_pump(struct pollmgr_handler *handler, SOCKET fd, int revents)
     }
 
 
+#ifdef RT_OS_WINDOWS
+    nread = recv(fd, (char *)pollmgr_udpbuf, sizeof(pollmgr_udpbuf), 0);
+#else
     nread = recv(fd, pollmgr_udpbuf, sizeof(pollmgr_udpbuf), 0);
+#endif
     if (nread < 0) {
         DPRINTF(("%s: %R[sockerr]\n", __func__, SOCKERRNO()));
         return POLLIN;
@@ -862,3 +901,42 @@ pxdns_pcb_reply(void *ctx)
 
     pxdns_request_free(req);
 }
+
+
+/**
+ * TCP DNS proxy.  This kicks in for large replies that don't fit into
+ * 512 bytes of UDP payload.  Client will retry with TCP to get
+ * complete reply.
+ */
+static err_t
+pxdns_accept_syn(void *arg, struct tcp_pcb *newpcb, struct pbuf *syn)
+{
+    struct pxdns *pxdns = (struct pxdns *)arg;
+    union sockaddr_inet *si;
+    ipX_addr_t *dst;
+    u16_t dst_port;
+
+    tcp_accepted(pxdns->ltcp);
+
+    if (pxdns->nresolvers == 0) {
+        return ERR_CONN;
+    }
+
+    si = &pxdns->resolvers[0];
+
+    if (si->sa.sa_family == AF_INET6) {
+        dst = (ipX_addr_t *)&si->sin6.sin6_addr;
+        dst_port = ntohs(si->sin6.sin6_port);
+    }
+    else {
+        dst = (ipX_addr_t *)&si->sin.sin_addr;
+        dst_port = ntohs(si->sin.sin_port);
+    }
+
+    /*
+     * XXX: TODO: need to implement protocol hooks.  E.g. here if
+     * connect fails, we should try connecting to a different server.
+     */
+    return pxtcp_pcb_accept_outbound(newpcb, syn,
+               si->sa.sa_family == AF_INET6, dst, dst_port);
+}
