@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2006-2015 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -206,6 +216,7 @@ static const RTISO2022MAP g_IsoIr102Map =
 };
 
 
+#if 0 /* unused */
 /** GL mappings for ISO-IR-87 (Japanese),  with space and delete. */
 static const RTISO2022MAP g_IsoIr87Map =
 {
@@ -216,6 +227,7 @@ static const RTISO2022MAP g_IsoIr87Map =
     { 0x24, 0x2a, 0x42, 0xff, 0xff, 0xff } /* Esc into G2 */,
     { 0x24, 0x2b, 0x42, 0xff, 0xff, 0xff } /* Esc into G3 */,
 };
+#endif
 
 
 /**
@@ -517,7 +529,7 @@ static int rtIso2022Decoder_FindEscAndSet(PRTISO2022DECODERSTATE pThis,
     uint32_t i = cMaps;
     while (i-- > 0)
     {
-        uint32_t cchMatch;
+        uint32_t cchMatch = 0; /* (MSC maybe used uninitialized) */
         PCRTISO2022MAP pMap = papMaps[i];
         /** @todo skip non-Teletex codesets if we ever add more than we need for it. */
         if (   pMap->abEscLoadXX[0] == b0
@@ -529,7 +541,8 @@ static int rtIso2022Decoder_FindEscAndSet(PRTISO2022DECODERSTATE pThis,
                 pThis->apMapGn[0] = pMap;
             return cchMatch + 1;
         }
-        else if (!ppMapRet) /* ppMapRet is NULL if Gn. */
+
+        if (!ppMapRet) /* ppMapRet is NULL if Gn. */
         {
             uint32_t iGn;
             if (   pMap->abEscLoadG1[0] == b0
@@ -1032,7 +1045,6 @@ static int rtIso2022RecodeAsUtf8(uint32_t uProfile, const char *pchSrc, uint32_t
     int rc = rtIso2022Decoder_Init(&Decoder, pchSrc, cchSrc, 102, 106, 107, 102, NULL /*pErrInfo*/);
     if (RT_SUCCESS(rc))
     {
-        size_t cchUtf8 = 0;
         for (;;)
         {
             RTUNICP uc;
@@ -1237,8 +1249,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
             rc = RTStrValidateEncodingEx(pch, cch, 0);
             if (RT_SUCCESS(rc))
                 break;
-            return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_UTF8_STRING_ENCODING,
-                                 "%s: Bad UTF-8 encoding (%Rrc)", pszErrorTag, rc);
+            return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_UTF8_STRING_ENCODING, "%s: Bad UTF-8 encoding (%Rrc, %.*Rhxs)",
+                                 pszErrorTag, rc, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
 
         case ASN1_TAG_NUMERIC_STRING:
             while (cch-- > 0)
@@ -1247,7 +1259,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                 if (   !RT_C_IS_DIGIT(ch)
                     && ch != ' ')
                     return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_NUMERIC_STRING_ENCODING,
-                                         "%s: Bad numberic string: ch=%#x", pszErrorTag, ch);
+                                         "%s: Bad numeric string: ch=%#x (pos %u in %.*Rhxs)", pszErrorTag, ch,
+                                         pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
             }
             break;
 
@@ -1270,7 +1283,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                     && ch != '?'
                    )
                     return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_PRINTABLE_STRING_ENCODING,
-                                         "%s: Bad printable string: ch=%#x", pszErrorTag, ch);
+                                         "%s: Bad printable string: ch=%#x (pos %u in %.*Rhxs)", pszErrorTag, ch,
+                                         pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
             }
             break;
 
@@ -1279,8 +1293,16 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
             {
                 unsigned char ch = *pch++;
                 if (ch == 0 || ch >= 0x80)
-                    return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_IA5_STRING_ENCODING,
-                                         "%s: Bad IA5 string: ch=%#x", pszErrorTag, ch);
+                {
+                    /* Ignore C-style zero terminator as the "Microsoft ECC Product Root Certificate Authority 2018"
+                       for instance, has a policy qualifier string "http://www.microsoft.com/pkiops/Docs/Repository.htm\0" */
+                    /** @todo should '\0' really be excluded above? */
+                    if (ch != 0 || cch != 0)
+                        return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_IA5_STRING_ENCODING,
+                                             "%s: Bad IA5 string: ch=%#x (pos %u in %.*Rhxs)", pszErrorTag, ch,
+                                             pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
+                    break;
+                }
             }
             break;
 
@@ -1310,7 +1332,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                 unsigned char ch = *pch++;
                 if (ch < 0x20 || ch >= 0x7f)
                     return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_VISIBLE_STRING_ENCODING,
-                                         "%s: Bad visible string: ch=%#x", pszErrorTag, ch);
+                                         "%s: Bad visible string: ch=%#x (pos %u in %.*Rhxs)", pszErrorTag, ch,
+                                         pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
             }
             break;
 
@@ -1327,7 +1350,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                     RTUNICP uc = RT_MAKE_U32_FROM_U8(pb[3], pb[2], pb[1], pb[0]); /* big endian */
                     if (!RTUniCpIsValid(uc))
                         return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_UNIVERSAL_STRING_ENCODING,
-                                             "%s: Bad universal string: uc=%#x", pszErrorTag, uc);
+                                             "%s: Bad universal string: uc=%#x (pos %u in %.*Rhxs)", pszErrorTag, uc,
+                                             pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
                     cchUtf8 += RTUniCpCalcUtf8Len(uc);
 
                     /* next */
@@ -1337,7 +1361,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                 break;
             }
             return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_UNIVERSAL_STRING_ENCODING,
-                                 "%s: Bad universal string: size not a multiple of 4: cch=%#x", pszErrorTag, cch);
+                                 "%s: Bad universal string: size not a multiple of 4: cch=%#x (%.*Rhxs)",
+                                 pszErrorTag, cch, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
 
         case ASN1_TAG_BMP_STRING:
             if (!(cch & 1))
@@ -1349,7 +1374,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                     RTUNICP uc = RT_MAKE_U32_FROM_U8(pb[1], pb[0], 0, 0); /* big endian */
                     if (!RTUniCpIsValid(uc))
                         return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_BMP_STRING_ENCODING,
-                                             "%s: Bad BMP string: uc=%#x", pszErrorTag, uc);
+                                             "%s: Bad BMP string: uc=%#x (pos %u in %.*Rhxs)", pszErrorTag, uc,
+                                             pThis->Asn1Core.cb - cch + 1, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
                     cchUtf8 += RTUniCpCalcUtf8Len(uc);
 
                     /* next */
@@ -1359,7 +1385,8 @@ static int rtAsn1String_CheckSanity(PCRTASN1STRING pThis, PRTERRINFO pErrInfo, c
                 break;
             }
             return RTErrInfoSetF(pErrInfo, VERR_ASN1_INVALID_BMP_STRING_ENCODING,
-                                 "%s: Bad BMP string: odd number of bytes cch=%#x", pszErrorTag, cch);
+                                 "%s: Bad BMP string: odd number of bytes cch=%#x (%.*Rhxs)",
+                                 pszErrorTag, cch, pThis->Asn1Core.cb, pThis->Asn1Core.uData.pch);
 
         default:
             AssertMsgFailedReturn(("uTag=%#x\n", uTag), VERR_INTERNAL_ERROR_3);
@@ -1735,6 +1762,7 @@ RTDECL(void) RTAsn1String_Delete(PRTASN1STRING pThis)
 
 RTDECL(int) RTAsn1String_Enum(PRTASN1STRING pThis, PFNRTASN1ENUMCALLBACK pfnCallback, uint32_t uDepth, void *pvUser)
 {
+    RT_NOREF_PV(pThis); RT_NOREF_PV(pfnCallback); RT_NOREF_PV(uDepth); RT_NOREF_PV(pvUser);
     Assert(pThis && (!RTAsn1String_IsPresent(pThis) || pThis->Asn1Core.pOps == &g_RTAsn1String_Vtable));
 
     /* No children to enumerate. */
@@ -1751,6 +1779,7 @@ RTDECL(int) RTAsn1String_Compare(PCRTASN1STRING pLeft, PCRTASN1STRING pRight)
 
 RTDECL(int) RTAsn1String_CheckSanity(PCRTASN1STRING pThis, uint32_t fFlags, PRTERRINFO pErrInfo, const char *pszErrorTag)
 {
+    RT_NOREF_PV(fFlags);
     if (RT_UNLIKELY(!RTAsn1String_IsPresent(pThis)))
         return RTErrInfoSetF(pErrInfo, VERR_ASN1_NOT_PRESENT, "%s: Missing (STRING).", pszErrorTag);
     return rtAsn1String_CheckSanity(pThis, pErrInfo, pszErrorTag, NULL /*pcchUtf8*/);
@@ -1786,6 +1815,7 @@ RTDECL(int) RTAsn1String_CheckSanity(PCRTASN1STRING pThis, uint32_t fFlags, PRTE
     \
     RTDECL(int) RT_CONCAT(a_Api,_Enum)(PRTASN1STRING pThis, PFNRTASN1ENUMCALLBACK pfnCallback, uint32_t uDepth, void *pvUser) \
     { \
+        RT_NOREF_PV(pThis); RT_NOREF_PV(pfnCallback); RT_NOREF_PV(uDepth); RT_NOREF_PV(pvUser); \
         Assert(   pThis \
                && (   !RTAsn1String_IsPresent(pThis) \
                    || (   pThis->Asn1Core.pOps == &g_RTAsn1String_Vtable \
