@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2006-2016 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -34,6 +44,7 @@
 #include "internal/iprt.h"
 #include <iprt/bignum.h>
 
+#include <iprt/asm-mem.h>
 #include <iprt/asm.h>
 #include <iprt/asm-math.h>
 #include <iprt/err.h>
@@ -676,15 +687,15 @@ RTDECL(int) RTBigNumInit(PRTBIGNUM pBigNum, uint32_t fFlags, void const *pvRaw,
                 {
                     default: AssertFailed();
 #if RTBIGNUM_ELEMENT_SIZE == 8
-                                                          /* fall thru */
-                    case 7: uLast = (uLast << 8) | pb[6]; /* fall thru */
-                    case 6: uLast = (uLast << 8) | pb[5]; /* fall thru */
-                    case 5: uLast = (uLast << 8) | pb[4]; /* fall thru */
+                                                          RT_FALL_THRU();
+                    case 7: uLast = (uLast << 8) | pb[6]; RT_FALL_THRU();
+                    case 6: uLast = (uLast << 8) | pb[5]; RT_FALL_THRU();
+                    case 5: uLast = (uLast << 8) | pb[4]; RT_FALL_THRU();
                     case 4: uLast = (uLast << 8) | pb[3];
 #endif
-                                                          /* fall thru */
-                    case 3: uLast = (uLast << 8) | pb[2]; /* fall thru */
-                    case 2: uLast = (uLast << 8) | pb[1]; /* fall thru */
+                                                          RT_FALL_THRU();
+                    case 3: uLast = (uLast << 8) | pb[2]; RT_FALL_THRU();
+                    case 2: uLast = (uLast << 8) | pb[1]; RT_FALL_THRU();
                     case 1: uLast = (uLast << 8) | pb[0];
                 }
                 pBigNum->pauElements[i] = uLast;
@@ -715,15 +726,15 @@ RTDECL(int) RTBigNumInit(PRTBIGNUM pBigNum, uint32_t fFlags, void const *pvRaw,
                 {
                     default: AssertFailed();
 #if RTBIGNUM_ELEMENT_SIZE == 8
-                                                          /* fall thru */
-                    case 7: uLast = (uLast << 8) | *pb++; /* fall thru */
-                    case 6: uLast = (uLast << 8) | *pb++; /* fall thru */
-                    case 5: uLast = (uLast << 8) | *pb++; /* fall thru */
+                                                          RT_FALL_THRU();
+                    case 7: uLast = (uLast << 8) | *pb++; RT_FALL_THRU();
+                    case 6: uLast = (uLast << 8) | *pb++; RT_FALL_THRU();
+                    case 5: uLast = (uLast << 8) | *pb++; RT_FALL_THRU();
                     case 4: uLast = (uLast << 8) | *pb++;
 #endif
-                                                          /* fall thru */
-                    case 3: uLast = (uLast << 8) | *pb++; /* fall thru */
-                    case 2: uLast = (uLast << 8) | *pb++; /* fall thru */
+                                                          RT_FALL_THRU();
+                    case 3: uLast = (uLast << 8) | *pb++; RT_FALL_THRU();
+                    case 2: uLast = (uLast << 8) | *pb++; RT_FALL_THRU();
                     case 1: uLast = (uLast << 8) | *pb++;
                 }
                 pBigNum->pauElements[i] = uLast;
@@ -749,9 +760,9 @@ RTDECL(int) RTBigNumInit(PRTBIGNUM pBigNum, uint32_t fFlags, void const *pvRaw,
             AssertCompile(RTBIGNUM_ALIGNMENT <= 4);
             switch (pBigNum->cAllocated - pBigNum->cUsed)
             {
-                default: AssertFailed(); /* fall thru */
-                case 3: *puUnused++ = 0; /* fall thru */
-                case 2: *puUnused++ = 0; /* fall thru */
+                default: AssertFailed(); RT_FALL_THRU();
+                case 3: *puUnused++ = 0; RT_FALL_THRU();
+                case 2: *puUnused++ = 0; RT_FALL_THRU();
                 case 1: *puUnused++ = 0;
             }
         }
@@ -839,12 +850,13 @@ RTDECL(int) RTBigNumDestroy(PRTBIGNUM pBigNum)
         if (pBigNum->pauElements)
         {
             Assert(pBigNum->cAllocated > 0);
-            if (pBigNum->fSensitive)
+            if (!pBigNum->fSensitive)
+                RTMemFree(pBigNum->pauElements);
+            else
             {
                 RTMemSaferFree(pBigNum->pauElements, pBigNum->cAllocated * RTBIGNUM_ELEMENT_SIZE);
                 RT_ZERO(*pBigNum);
             }
-            RTMemFree(pBigNum->pauElements);
             pBigNum->pauElements = NULL;
         }
     }
@@ -2230,7 +2242,7 @@ static int rtBigNumMagnitudeDivideKnuth(PRTBIGNUM pQuotient, PRTBIGNUM pRemainde
      * Delete temporary variables.
      */
     RTBigNumDestroy(&NormDividend);
-    if (pDivisor == &NormDivisor)
+    if (pNormDivisor == &NormDivisor)
         RTBigNumDestroy(&NormDivisor);
     return rc;
 }
@@ -2663,6 +2675,8 @@ static int rtBigNumMagnitudeExponentiate(PRTBIGNUM pResult, PCRTBIGNUM pBase, PC
                     if (RT_FAILURE(rc))
                         break;
                 }
+
+                RTBigNumDestroy(&TmpMultiplicand);
             }
         }
         RTBigNumDestroy(&Pow2);
