@@ -4,41 +4,53 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #ifdef RT_OS_DARWIN
 /* pick the correct prototype for unsetenv. */
 # define _POSIX_C_SOURCE 1
 #endif
 #include <iprt/env.h>
-#include <iprt/string.h>
+
 #include <iprt/alloca.h>
 #include <iprt/assert.h>
 #if defined(DEBUG) && defined(RT_OS_LINUX)
 # include <iprt/asm.h>
 #endif
+#include <iprt/err.h>
+#include <iprt/string.h>
 
 #include <stdlib.h>
 #include <errno.h>
@@ -46,14 +58,23 @@
 #include "internal/alignmentchecks.h"
 
 
-RTDECL(bool) RTEnvExist(const char *pszVar)
+RTDECL(bool) RTEnvExistsBad(const char *pszVar)
 {
-    return RTEnvGet(pszVar) != NULL;
+    AssertReturn(strchr(pszVar, '=') == NULL, false);
+    return RTEnvGetBad(pszVar) != NULL;
 }
 
 
-RTDECL(const char *) RTEnvGet(const char *pszVar)
+RTDECL(bool) RTEnvExist(const char *pszVar)
 {
+    return RTEnvExistsBad(pszVar);
+}
+
+
+RTDECL(const char *) RTEnvGetBad(const char *pszVar)
+{
+    AssertReturn(strchr(pszVar, '=') == NULL, NULL);
+
     IPRT_ALIGNMENT_CHECKS_DISABLE(); /* glibc causes trouble */
     const char *pszValue = getenv(pszVar);
     IPRT_ALIGNMENT_CHECKS_ENABLE();
@@ -61,7 +82,13 @@ RTDECL(const char *) RTEnvGet(const char *pszVar)
 }
 
 
-RTDECL(int) RTEnvPut(const char *pszVarEqualValue)
+RTDECL(const char *) RTEnvGet(const char *pszVar)
+{
+    return RTEnvGetBad(pszVar);
+}
+
+
+RTDECL(int) RTEnvPutBad(const char *pszVarEqualValue)
 {
     /** @todo putenv is a source memory leaks. deal with this on a per system basis. */
     if (!putenv((char *)pszVarEqualValue))
@@ -69,8 +96,17 @@ RTDECL(int) RTEnvPut(const char *pszVarEqualValue)
     return RTErrConvertFromErrno(errno);
 }
 
-RTDECL(int) RTEnvSet(const char *pszVar, const char *pszValue)
+
+RTDECL(int) RTEnvPut(const char *pszVarEqualValue)
 {
+    return RTEnvPutBad(pszVarEqualValue);
+}
+
+
+RTDECL(int) RTEnvSetBad(const char *pszVar, const char *pszValue)
+{
+    AssertMsgReturn(strchr(pszVar, '=') == NULL, ("'%s'\n", pszVar), VERR_ENV_INVALID_VAR_NAME);
+
 #if defined(_MSC_VER)
     /* make a local copy and feed it to putenv. */
     const size_t cchVar = strlen(pszVar);
@@ -98,9 +134,14 @@ RTDECL(int) RTEnvSet(const char *pszVar, const char *pszValue)
 }
 
 
-RTDECL(int) RTEnvUnset(const char *pszVar)
+RTDECL(int) RTEnvSet(const char *pszVar, const char *pszValue)
 {
-    AssertReturn(!strchr(pszVar, '='), VERR_INVALID_PARAMETER);
+    return RTEnvSetBad(pszVar, pszValue);
+}
+
+RTDECL(int) RTEnvUnsetBad(const char *pszVar)
+{
+    AssertReturn(strchr(pszVar, '=') == NULL, VERR_ENV_INVALID_VAR_NAME);
 
     /*
      * Check that it exists first.
@@ -131,3 +172,8 @@ RTDECL(int) RTEnvUnset(const char *pszVar)
     return RTErrConvertFromErrno(errno);
 }
 
+RTDECL(int) RTEnvUnset(const char *pszVar)
+{
+    return RTEnvUnsetBad(pszVar);
+}
+
