@@ -4,38 +4,76 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/path.h>
 #include <iprt/initterm.h>
 #include <iprt/stream.h>
-#include <iprt/err.h>
+#include <iprt/test.h>
+#include <iprt/errcore.h>
 
 
+struct TSTCTX
+{
+    int var;
+    int rc;
+} TestCtx = { 42, VINF_SUCCESS };
+
+static DECLCALLBACK(int) mountpointsEnumCallback(const char *pszMountpoint, void *pvUser)
+{
+    TSTCTX *pCtx = (TSTCTX *)pvUser;
+    RTTESTI_CHECK(pCtx->var == 42);
+
+    RTTestIPrintf(RTTESTLVL_ALWAYS, "Mountpoint: %s\n", pszMountpoint);
+
+    return pCtx->rc;
+}
+
+static void tstMountpointsEnum(RTTEST hTest)
+{
+    RTTEST_CHECK_RC_OK(hTest, RTFsMountpointsEnum(mountpointsEnumCallback, &TestCtx));
+}
+
+/** @todo r=andy Overhaul this code to make use of the RTTEST APIs. */
 int main(int argc, char **argv)
 {
-    RTR3InitExe(argc, &argv, 0);
+    RTTEST hTest;
+    RTEXITCODE rcExit = RTTestInitExAndCreate(argc, &argv, 0 /* fFlags */, "tstRTFsQueries", &hTest);
+    if (rcExit != RTEXITCODE_SUCCESS)
+        return rcExit;
+    RTTestBanner(hTest);
 
     /*
      * Process all arguments (including the executable).
@@ -104,9 +142,13 @@ int main(int argc, char **argv)
         }
     }
 
-    if (!cErrors)
-        RTPrintf("tstRTFsQueries: SUCCESS\n");
-    else
-        RTPrintf("tstRTFsQueries: FAIlURE - %u errors\n", cErrors);
-    return !!cErrors;
+    tstMountpointsEnum(hTest);
+
+    if (cErrors)
+        RTTestFailed(hTest, "tstRTFsQueries: FAIlURE - %u errors\n", cErrors);
+
+    /*
+     * Done.
+     */
+    return RTTestSummaryAndDestroy(hTest);
 }
