@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2008-2019 Oracle Corporation
+ * Copyright (C) 2008-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -310,15 +320,44 @@ int main()
     GOODCIDR("1.2.3.4/32",      0x01020304, 32);
     GOODCIDR("1.2.3.4/24",      0x01020304, 24); /* address is not truncated to prefix */
 
+    GOODCIDR("1.2.3.0/0xffffff00",      0x01020300, 24);
+    GOODCIDR("1.2.3.4/0xffffff00",      0x01020304, 24);
+    GOODCIDR("1.2.3.4/0xffffffff",      0x01020304, 32);
+
+    GOODCIDR("1.2.3.0/255.255.255.0",   0x01020300, 24);
+    GOODCIDR("1.2.3.4/255.255.255.0",   0x01020304, 24);
+    GOODCIDR("1.2.3.4/255.255.255.255", 0x01020304, 32);
+
+    GOODCIDR("0.0.0.0/0",       0x00000000,  0);
+    GOODCIDR("0.0.0.0/0x0",     0x00000000,  0);
+    GOODCIDR("0.0.0.0/0.0.0.0", 0x00000000,  0);
+
+    /*
+     * we allow zero prefix mostly for the sake of the above
+     * "everything"/default case, but allow it on everything - a
+     * conscientious caller should be doing more checks on the result
+     * anyway.
+     */
+    GOODCIDR("1.2.3.4/0",       0x01020304,  0);        /* prefix can be zero */
+
     GOODCIDR("\t " "1.2.3.4/24",       0x01020304, 24); /* leading spaces ok */
     GOODCIDR(      "1.2.3.4/24" " \t", 0x01020304, 24); /* trailing spaces ok */
     GOODCIDR("\t " "1.2.3.4/24" " \t", 0x01020304, 24); /* both are ok */
 
-    BADCIDR("1.2.3.4/0");       /* prefix can't be zero */
+    /* trailing space with netmask notation */
+    GOODCIDR("1.2.3.0/0xffffff00" " ",    0x01020300, 24);
+    GOODCIDR("1.2.3.0/255.255.255.0" " ", 0x01020300, 24);
+
+    BADCIDR("1.2.3.4/24.");
+    BADCIDR("1.2.3.4/24 .");
+    BADCIDR("1.2.3.4/240.");
+    BADCIDR("1.2.3.4/240.");
+
     BADCIDR("1.2.3.4/33");      /* prefix is too big */
+    BADCIDR("1.2.3.4/256");     /* prefix is too big */
+    BADCIDR("1.2.3.4/257");     /* prefix is too big */
     BADCIDR("1.2.3.4/-1");      /* prefix is negative */
     BADCIDR("1.2.3.4/");        /* prefix is missing */
-    BADCIDR("1.2.3.4/");        /* prefix is missing */
     BADCIDR("1.2.3.4/a");       /* prefix is not a number */
     BADCIDR("1.2.3.4/0xa");     /* prefix is not decimal */
 //  BADCIDR("1.2.3.0/024");     /* XXX: prefix is not decimal */
@@ -329,7 +368,8 @@ int main()
     BADCIDR("1.2.3.0/24" "x");  /* trailing chars */
     BADCIDR("1.2.3.0/24" " x"); /* trailing chars */
 
-
+    BADCIDR("1.2.3.0/0xffffff01");    /* non-contiguous mask */
+    BADCIDR("1.2.3.0/255.255.255.1"); /* non-contiguous mask */
 
     /* NB: RTNetIsIPv4AddrStr does NOT allow leading/trailing whitespace */
     IS_ADDR("1.2.3.4");
