@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2010-2020 Oracle Corporation
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -38,12 +48,16 @@
 #include <iprt/param.h>
 #include <iprt/process.h>
 #include <iprt/string.h>
+#ifdef RT_OS_WINDOWS
+# include <iprt/system.h>
+#endif
 #include <iprt/test.h>
 
 
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
+static RTTEST     g_hTest            = NIL_RTTEST;
 static const char g_szTest4Message[] = "This is test #4, everything is working fine.\n\r";
 static const char g_szTest5Message[] = "This is test #5, everything is working fine.\n\r";
 
@@ -203,46 +217,73 @@ static void tstRTPipe4(void)
 static void tstRTPipe3(void)
 {
     RTTestISub("Full write buffer");
-
-    RTPIPE  hPipeR = (RTPIPE)999999;
-    RTPIPE  hPipeW = (RTPIPE)999999;
-    RTTESTI_CHECK_RC_RETV(RTPipeCreate(&hPipeR, &hPipeW, 0), VINF_SUCCESS);
-
-    static char s_abBuf[_256K];
-    int         rc      = VINF_SUCCESS;
-    size_t      cbTotal = 0;
-    memset(s_abBuf, 0xff, sizeof(s_abBuf));
-    for (;;)
+#ifdef RT_OS_WINDOWS
+    /* Kludge! The RTPipeWrite+RTPipeWriteBlocking code is able to write twice the size
+               of the pipe buffer, so the testcase won't behave right if we go lower
+               than 50% on the readback. */
+    for (unsigned uPct = 50; uPct < 100; uPct += 12)
+#else
+    for (unsigned uPct = 25; uPct < 100; uPct += 12)
+#endif
     {
-        RTTESTI_CHECK(cbTotal < _1G);
-        if (cbTotal > _1G)
-            break;
+        RTPIPE  hPipeR = (RTPIPE)999999;
+        RTPIPE  hPipeW = (RTPIPE)999999;
+        RTTESTI_CHECK_RC_RETV(RTPipeCreate(&hPipeR, &hPipeW, 0), VINF_SUCCESS);
+        RTTestIErrContext("uPct=%d", uPct);
 
-        size_t cbWritten = _1G;
-        rc = RTPipeWrite(hPipeW, s_abBuf, sizeof(s_abBuf), &cbWritten);
-        RTTESTI_CHECK_MSG(rc == VINF_SUCCESS || rc == VINF_TRY_AGAIN, ("rc=%Rrc\n", rc));
-        if (rc != VINF_SUCCESS)
-            break;
-        cbTotal += cbWritten;
+        static char s_abBuf[_256K];
+        int         rc      = VINF_SUCCESS;
+        size_t      cbTotal = 0;
+        memset(s_abBuf, 0xff, sizeof(s_abBuf));
+        for (;;)
+        {
+            RTTESTI_CHECK(cbTotal < _1G);
+            if (cbTotal > _1G)
+                break;
+
+            size_t cbWritten = _1G;
+            rc = RTPipeWrite(hPipeW, s_abBuf, sizeof(s_abBuf), &cbWritten);
+            RTTESTI_CHECK_MSG(rc == VINF_SUCCESS || rc == VINF_TRY_AGAIN, ("rc=%Rrc\n", rc));
+            if (rc != VINF_SUCCESS)
+                break;
+            cbTotal += cbWritten;
+        }
+
+        if (rc == VINF_TRY_AGAIN)
+        {
+            size_t const cbToRead = RT_MIN(sizeof(s_abBuf), cbTotal * uPct / 100);
+            RTTestIPrintf(RTTESTLVL_ALWAYS, "cbTotal=%zu (%#zx)  cbToRead=%zu (%#zx)\n", cbTotal, cbTotal, cbToRead, cbToRead);
+            RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 0), VERR_TIMEOUT);
+            RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 1), VERR_TIMEOUT);
+            size_t cbRead;
+            RTTESTI_CHECK_RC(RTPipeReadBlocking(hPipeR, s_abBuf, cbToRead, &cbRead), VINF_SUCCESS);
+
+#ifdef RT_OS_WINDOWS
+            /* Kludge! Older Windows NT versions does not complete a pending write till
+                       all the pending data has been read and the pipe buffer is
+                       completely empty. Applies to NT4, W2K, and XP. Though on XP the
+                       test works at 50% for some reason and we can write double the
+                       amount, so possibly a different issue there, so we'l
+
+                       This does not seem to be a bug in pipe-win.cpp. */
+            if (RTSystemGetNtVersion() < (uPct == 50 ? RTSYSTEM_MAKE_NT_VERSION(5,2,0) : RTSYSTEM_MAKE_NT_VERSION(6,0,0)))
+                RTTestIPrintf(RTTESTLVL_ALWAYS, "old buggy windows - skipping 2nd part (0: %Rrc, 1: %Rrc)\n",
+                              RTPipeSelectOne(hPipeW, 0), RTPipeSelectOne(hPipeW, 1));
+            else
+#endif
+            {
+                RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 0), VINF_SUCCESS);
+                RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 1), VINF_SUCCESS);
+
+                size_t cbWritten = _1G;
+                rc = RTPipeWrite(hPipeW, s_abBuf, sizeof(s_abBuf), &cbWritten);
+                RTTESTI_CHECK(rc == VINF_SUCCESS);
+            }
+        }
+
+        RTTESTI_CHECK_RC(RTPipeClose(hPipeR), VINF_SUCCESS);
+        RTTESTI_CHECK_RC(RTPipeClose(hPipeW), VINF_SUCCESS);
     }
-
-    if (rc == VINF_TRY_AGAIN)
-    {
-        RTTestIPrintf(RTTESTLVL_ALWAYS, "cbTotal=%zu (%#zx)\n", cbTotal, cbTotal);
-        RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 0), VERR_TIMEOUT);
-        RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 1), VERR_TIMEOUT);
-        size_t cbRead;
-        RTTESTI_CHECK_RC(RTPipeRead(hPipeR, s_abBuf, RT_MIN(sizeof(s_abBuf), cbTotal) / 2, &cbRead), VINF_SUCCESS);
-        RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 0), VINF_SUCCESS);
-        RTTESTI_CHECK_RC(RTPipeSelectOne(hPipeW, 1), VINF_SUCCESS);
-
-        size_t cbWritten = _1G;
-        rc = RTPipeWrite(hPipeW, s_abBuf, sizeof(s_abBuf), &cbWritten);
-        RTTESTI_CHECK(rc == VINF_SUCCESS);
-    }
-
-    RTTESTI_CHECK_RC(RTPipeClose(hPipeR), VINF_SUCCESS);
-    RTTESTI_CHECK_RC(RTPipeClose(hPipeW), VINF_SUCCESS);
 }
 
 static void tstRTPipe2(void)
@@ -485,17 +526,16 @@ int main(int argc, char **argv)
     if (argc == 3 && !strcmp(argv[1], "--child-5"))
         return tstRTPipe5Child(argv[2]);
 
-    RTTEST hTest;
-    int rc = RTTestInitAndCreate("tstRTPipe", &hTest);
+    int rc = RTTestInitAndCreate("tstRTPipe", &g_hTest);
     if (rc)
         return rc;
-    RTTestBanner(hTest);
+    RTTestBanner(g_hTest);
 
     /*
      * The tests.
      */
     tstRTPipe1();
-    if (RTTestErrorCount(hTest) == 0)
+    if (RTTestErrorCount(g_hTest) == 0)
     {
         bool fMayPanic = RTAssertMayPanic();
         bool fQuiet    = RTAssertAreQuiet();
@@ -513,6 +553,6 @@ int main(int argc, char **argv)
     /*
      * Summary.
      */
-    return RTTestSummaryAndDestroy(hTest);
+    return RTTestSummaryAndDestroy(g_hTest);
 }
 
