@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2010-2016 Oracle Corporation
+ * Copyright (C) 2010-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -37,6 +47,7 @@
 #include <iprt/mem.h>
 #include <iprt/message.h>
 #include <iprt/param.h>
+#include <iprt/path.h> /* For CWD testing. */
 #include <iprt/pipe.h>
 #include <iprt/string.h>
 #include <iprt/stream.h>
@@ -82,6 +93,161 @@ static const char * const g_apszArgs4[] =
 };
 
 
+static int tstRTCreateProcExCwdChild(int argc, char **argv)
+{
+    int rc = RTR3InitExeNoArguments(0);
+    if (RT_FAILURE(rc))
+        return RTMsgInitFailure(rc);
+
+    int cErrors = 0;
+
+    if (argc < 3)
+        return RTEXITCODE_FAILURE;
+
+    const char *pszCWD = argv[2];
+
+    RTStrmPrintf(g_pStdOut, "childcwd: Called with CWD '%s'\n", pszCWD);
+
+    if (!RTStrICmp(pszCWD, "<cwd-not-specified>")) /* Bail out early if no CWD has been explicitly specified. */
+        return RTEXITCODE_SUCCESS;
+
+    /* Validate if we really are in the CWD the parent told us. */
+    char szCWD[RTPATH_MAX];
+    rc = RTPathGetCurrent(szCWD, sizeof(szCWD));
+    if (RT_FAILURE(rc))
+    {
+        RTStrmPrintf(g_pStdErr, "childcwd: Unable to retrieve CWD, rc=%Rrc\n", rc);
+        cErrors++;
+    }
+    else
+    {
+        if (RTStrCmp(szCWD, pszCWD))
+        {
+            RTStrmPrintf(g_pStdErr, "childcwd: CWD is '%s', but expected '%s'\n", szCWD, pszCWD);
+            cErrors++;
+        }
+        else
+        {
+            /* Check if we can query information of the current CWD. */
+            char *pszUser = NULL;
+            if (   argc >= 4
+                && argv[3][0] != '\0')
+            {
+                pszUser = argv[3];
+            }
+
+            RTFSOBJINFO objInfo;
+            rc = RTPathQueryInfo(szCWD, &objInfo, RTFSOBJATTRADD_NOTHING);
+            if (pszUser)
+                RTStrmPrintf(g_pStdOut, "childcwd: Accessing CWD '%s' via user '%s' -> %Rrc\n", szCWD, pszUser, rc);
+            else
+                RTStrmPrintf(g_pStdOut, "childcwd: Accesing CWD '%s' -> %Rrc\n", szCWD, rc);
+            if (RT_FAILURE(rc))
+                cErrors++;
+        }
+    }
+
+    RTStrmPrintf(g_pStdOut, "childcwd: Exiting (%d errors)\n", cErrors);
+    return cErrors == 0 ? RTEXITCODE_SUCCESS : RTEXITCODE_FAILURE;
+}
+
+static void tstRTCreateProcExCwd(const char *pszAsUser, const char *pszPassword)
+{
+    RTTestISub("Current working directory (CWD)");
+
+    const char *apszArgs[5] =
+    {
+        g_szExecName,
+        "--testcase-child-cwd",
+        "<cwd-not-specified>",
+        pszAsUser,
+        NULL
+    };
+
+    bool const fMayPanic = RTAssertSetMayPanic(false);
+    bool const fQuiet    = RTAssertSetQuiet(true);
+
+    RTPROCESS hProc;
+    uint32_t  fFlags = 0;
+    /* Missing RTPROC_FLAGS_CWD. */
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         (void *)apszArgs[2] /* pvExtraData (CWD) */, &hProc),
+                          VERR_INVALID_PARAMETER);
+
+    /* Invalid flag combinations. Windows flags ignored elsewhere. */
+#ifdef RT_OS_WINDOWS
+    int rc = VERR_INVALID_PARAMETER;
+#else
+    int rc = VERR_INVALID_POINTER; /* Due to missing pvExtraData. */
+#endif
+    fFlags = RTPROC_FLAGS_CWD | RTPROC_FLAGS_DESIRED_SESSION_ID | RTPROC_FLAGS_SERVICE;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         NULL, &hProc), rc);
+
+    /* Windows-only flags. Ignored elsewhere. */
+#ifdef RT_OS_WINDOWS
+    rc = VERR_INVALID_POINTER;
+#else
+    rc = VINF_SUCCESS;
+#endif
+    fFlags = RTPROC_FLAGS_DESIRED_SESSION_ID | RTPROC_FLAGS_SERVICE;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         NULL, &hProc), rc);
+
+    /* RTPROC_FLAGS_CWD set, but CWD missing as pvExtradata. */
+    fFlags = RTPROC_FLAGS_CWD;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         NULL, &hProc),
+                          VERR_INVALID_POINTER);
+
+    char szCWD[RTPATH_MAX];
+    char szResolved[RTPATH_MAX];
+
+    /* Try current CWD, whatever that is. */
+    RTTESTI_CHECK_RC_OK_RETV(RTPathGetCurrent(szCWD, sizeof(szCWD)));
+    RTTESTI_CHECK_RC_OK_RETV(RTPathReal(szCWD, szResolved, sizeof(szResolved)));
+    apszArgs[2] = szResolved;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         (void *)szCWD /* pvExtraData (CWD) */, &hProc),
+                          VINF_SUCCESS);
+    RTPROCSTATUS ProcStatus = { -1, RTPROCEXITREASON_ABEND };
+    RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
+    if (ProcStatus.enmReason != RTPROCEXITREASON_NORMAL || ProcStatus.iStatus != 0)
+        RTTestIFailed("enmReason=%d iStatus=%d", ProcStatus.enmReason, ProcStatus.iStatus);
+
+    /* Try temporary directory. */
+    RTTESTI_CHECK_RC_OK_RETV(RTPathTemp(szCWD, sizeof(szCWD)));
+    RTTESTI_CHECK_RC_OK_RETV(RTPathReal(szCWD, szResolved, sizeof(szResolved)));
+    apszArgs[2] = szResolved;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         (void *)szCWD /* pvExtraData (CWD) */, &hProc),
+                          VINF_SUCCESS);
+    RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
+    if (ProcStatus.enmReason != RTPROCEXITREASON_NORMAL || ProcStatus.iStatus != 0)
+        RTTestIFailed("enmReason=%d iStatus=%d", ProcStatus.enmReason, ProcStatus.iStatus);
+
+    /* Try user home. */
+    RTTESTI_CHECK_RC_OK_RETV(RTPathUserHome(szCWD, sizeof(szCWD)));
+    RTTESTI_CHECK_RC_OK_RETV(RTPathReal(szCWD, szResolved, sizeof(szResolved)));
+    apszArgs[2] = szResolved;
+    RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, fFlags,
+                                         NULL, NULL, NULL, pszAsUser, pszPassword,
+                                         (void *)szCWD /* pvExtraData (CWD) */, &hProc),
+                          VINF_SUCCESS);
+    RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
+    if (ProcStatus.enmReason != RTPROCEXITREASON_NORMAL || ProcStatus.iStatus != 0)
+        RTTestIFailed("enmReason=%d iStatus=%d", ProcStatus.enmReason, ProcStatus.iStatus);
+
+    RTAssertSetMayPanic(fMayPanic);
+    RTAssertSetQuiet(fQuiet);
+}
+
 static int tstRTCreateProcEx6Child(int argc, char **argv)
 {
     int rc = RTR3InitExeNoArguments(0);
@@ -218,7 +384,7 @@ static void tstRTCreateProcEx6(const char *pszAsUser, const char *pszPassword)
     /* Use the process environment first. */
     RTPROCESS hProc;
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/,
-                                         NULL, NULL, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTPROCSTATUS ProcStatus = { -1, RTPROCEXITREASON_ABEND };
     RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
 
@@ -232,7 +398,7 @@ static void tstRTCreateProcEx6(const char *pszAsUser, const char *pszPassword)
     RTTESTI_CHECK_RC_RETV(RTEnvSetEx(hEnvChange, "testcase-child-6", "changed"), VINF_SUCCESS);
     int rc;
     RTTESTI_CHECK_RC(rc = RTProcCreateEx(g_szExecName, apszArgs, hEnvChange, RTPROC_FLAGS_ENV_CHANGE_RECORD,
-                                         NULL, NULL, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     if (RT_SUCCESS(rc))
     {
         ProcStatus.enmReason = RTPROCEXITREASON_ABEND;
@@ -247,7 +413,7 @@ static void tstRTCreateProcEx6(const char *pszAsUser, const char *pszPassword)
     /* Use profile environment this time. */
     apszArgs[2] = "noinherit";
     RTTESTI_CHECK_RC(rc = RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, RTPROC_FLAGS_PROFILE,
-                                         NULL, NULL, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     if (RT_SUCCESS(rc))
     {
         ProcStatus.enmReason = RTPROCEXITREASON_ABEND;
@@ -262,7 +428,7 @@ static void tstRTCreateProcEx6(const char *pszAsUser, const char *pszPassword)
     apszArgs[2] = "noinherit-change-record";
     RTTESTI_CHECK_RC(rc = RTProcCreateEx(g_szExecName, apszArgs, hEnvChange,
                                          RTPROC_FLAGS_PROFILE | RTPROC_FLAGS_ENV_CHANGE_RECORD,
-                                         NULL, NULL, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     if (RT_SUCCESS(rc))
     {
         ProcStatus.enmReason = RTPROCEXITREASON_ABEND;
@@ -333,6 +499,8 @@ static void tstRTCreateProcEx6(const char *pszAsUser, const char *pszPassword)
         else
             RTTestIFailed("RTEnvGetByIndexEx(%s,%u,,,,) failed: %Rrc", pszEnv1, i, rc);
     }
+
+    RTEnvDestroy(hEnvCur);
 }
 
 
@@ -385,17 +553,17 @@ static void tstRTCreateProcEx5(const char *pszUser, const char *pszPassword)
     /* Test for invalid logons. */
     RTPROCESS hProc;
     int rc = RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL, NULL, NULL,
-                            "non-existing-user", "wrong-password", &hProc);
+                            "non-existing-user", "wrong-password", NULL, &hProc);
     if (rc != VERR_AUTHENTICATION_FAILURE && rc != VERR_PRIVILEGE_NOT_HELD && rc != VERR_PROC_TCB_PRIV_NOT_HELD)
         RTTestIFailed("rc=%Rrc", rc);
 
     /* Test for invalid application. */
     RTTESTI_CHECK_RC(RTProcCreateEx("non-existing-app", apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                    NULL, NULL, NULL, NULL, &hProc), VERR_FILE_NOT_FOUND);
+                                    NULL, NULL, NULL, NULL, NULL, &hProc), VERR_FILE_NOT_FOUND);
 
     /* Test a (hopefully) valid user/password logon (given by parameters of this function). */
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                         NULL, NULL, pszUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, pszUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTPROCSTATUS ProcStatus = { -1, RTPROCEXITREASON_ABEND };
     RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
 
@@ -411,7 +579,15 @@ static int tstRTCreateProcEx4Child(int argc, char **argv)
         return RTMsgInitFailure(rc);
 
     int cErrors = 0;
-    for (int i = 0; i < argc; i++)
+    if (argc >= (int)RT_ELEMENTS(g_apszArgs4))
+    {
+        RTStrmPrintf(g_pStdErr,
+                     "Expected argument count to be <= %u, got %u\n",
+                     argc, RT_ELEMENTS(g_apszArgs4));
+        cErrors++;
+    }
+
+    for (int i = 0; i < RT_MIN(argc, (int)RT_ELEMENTS(g_apszArgs4) - 1); i++)
         if (strcmp(argv[i], g_apszArgs4[i]))
         {
             RTStrmPrintf(g_pStdErr,
@@ -430,7 +606,7 @@ static void tstRTCreateProcEx4(const char *pszAsUser, const char *pszPassword)
 
     RTPROCESS hProc;
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, g_apszArgs4, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                         NULL, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTPROCSTATUS ProcStatus = { -1, RTPROCEXITREASON_ABEND };
     RTTESTI_CHECK_RC(RTProcWait(hProc, RTPROCWAIT_FLAGS_BLOCK, &ProcStatus), VINF_SUCCESS);
 
@@ -471,7 +647,7 @@ static void tstRTCreateProcEx3(const char *pszAsUser, const char *pszPassword)
     Handle.u.hPipe = hPipeW;
     RTPROCESS hProc;
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                         &Handle, &Handle, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         &Handle, &Handle, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTTESTI_CHECK_RC(RTPipeClose(hPipeW), VINF_SUCCESS);
 
     char    szOutput[_4K];
@@ -536,7 +712,7 @@ static void tstRTCreateProcEx2(const char *pszAsUser, const char *pszPassword)
     Handle.u.hPipe = hPipeW;
     RTPROCESS hProc;
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                         NULL, &Handle, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         NULL, &Handle, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTTESTI_CHECK_RC(RTPipeClose(hPipeW), VINF_SUCCESS);
 
     char    szOutput[_4K];
@@ -602,7 +778,7 @@ static void tstRTCreateProcEx1(const char *pszAsUser, const char *pszPassword)
     Handle.u.hPipe = hPipeW;
     RTPROCESS hProc;
     RTTESTI_CHECK_RC_RETV(RTProcCreateEx(g_szExecName, apszArgs, RTENV_DEFAULT, 0 /*fFlags*/, NULL,
-                                         &Handle, NULL, pszAsUser, pszPassword, &hProc), VINF_SUCCESS);
+                                         &Handle, NULL, pszAsUser, pszPassword, NULL, &hProc), VINF_SUCCESS);
     RTTESTI_CHECK_RC(RTPipeClose(hPipeW), VINF_SUCCESS);
 
     char    szOutput[_4K];
@@ -654,6 +830,8 @@ int main(int argc, char **argv)
         return tstRTCreateProcEx5Child(argc, argv);
     if (argc >= 2 && !strcmp(argv[1], "--testcase-child-6"))
         return tstRTCreateProcEx6Child(argc, argv);
+    if (argc >= 2 && !strcmp(argv[1], "--testcase-child-cwd"))
+        return tstRTCreateProcExCwdChild(argc, argv);
 
     /*
      * Main process.
@@ -691,6 +869,7 @@ int main(int argc, char **argv)
     if (pszAsUser)
         tstRTCreateProcEx5(pszAsUser, pszPassword);
     tstRTCreateProcEx6(pszAsUser, pszPassword);
+    tstRTCreateProcExCwd(pszAsUser, pszPassword);
 
     /** @todo Cover files, ++ */
 
