@@ -4,18 +4,32 @@
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
-#ifndef ___VDICore_h___
+#ifndef VBOX_INCLUDED_SRC_Storage_VDICore_h
+#define VBOX_INCLUDED_SRC_Storage_VDICore_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 
 /*******************************************************************************
@@ -39,7 +53,7 @@
 /** Image info, not handled anyhow.
  *  Must be less than 64 bytes in length, including the trailing 0.
  */
-#define VDI_IMAGE_FILE_INFO   "<<< Oracle VM VirtualBox Disk Image >>>\n"
+#define VDI_IMAGE_FILE_INFO   "<<< Oracle VirtualBox Disk Image >>>\n"
 
 /** The Sector size.
  * Currently we support only 512 bytes sectors.
@@ -236,9 +250,9 @@ typedef struct VDIHEADER
 /**
  * File alignment boundary for both the block array and data area. Should be
  * at least the size of a physical sector on disk for performance reasons.
- * With the growing market share of disks with 4K sectors this needs to be
- * bumped, and maybe again later. */
-#define VDI_DATA_ALIGN _4K
+ * Bumped to 1MB because SSDs tend to have 8kb per page so we don't have to worry
+ * about proper alignment in the near future again. */
+#define VDI_DATA_ALIGN _1M
 
 /** Block 'pointer'. */
 typedef uint32_t    VDIIMAGEBLOCKPOINTER;
@@ -467,6 +481,10 @@ DECLINLINE(void) setImageBlocksAllocated(PVDIHEADER ph, unsigned cBlocks)
     AssertFailed();
 }
 
+#ifdef _MSC_VER
+# pragma warning(disable:4366) /* (harmless "misalignment") */
+#endif
+
 DECLINLINE(PRTUUID) getImageCreationUUID(PVDIHEADER ph)
 {
     switch (GET_MAJOR_HEADER_VERSION(ph))
@@ -510,6 +528,10 @@ DECLINLINE(PRTUUID) getImageParentModificationUUID(PVDIHEADER ph)
     return NULL;
 }
 
+#ifdef _MSC_VER
+# pragma warning(default:4366)
+#endif
+
 /**
  * Image structure
  */
@@ -541,6 +563,8 @@ typedef struct VDIIMAGEDESC
     unsigned                offStartBlockData;
     /** Total size of image block (including the extra data). */
     unsigned                cbTotalBlockData;
+    /** Allocation Block Size */
+    unsigned                cbAllocationBlock;
     /** Container filename. (UTF-8) */
     const char             *pszFilename;
     /** Physical geometry of this image (never actually stored). */
@@ -555,6 +579,8 @@ typedef struct VDIIMAGEDESC
     PVDINTERFACEIOINT       pIfIo;
     /** Current size of the image (used for range validation when reading). */
     uint64_t                cbImage;
+    /** The static region list. */
+    VDREGIONLIST            RegionList;
 } VDIIMAGEDESC, *PVDIIMAGEDESC;
 
 /**
@@ -615,5 +641,5 @@ typedef enum VDIECONV
     VDIECONV_F2H
 } VDIECONV;
 
-#endif
+#endif /* !VBOX_INCLUDED_SRC_Storage_VDICore_h */
 
