@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2017 Oracle Corporation
+ * Copyright (C) 2017-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -37,7 +47,6 @@
 /*********************************************************************************************************************************
 *   Structures and Typedefs                                                                                                      *
 *********************************************************************************************************************************/
-
 /**
  * Plugin structure.
  */
@@ -54,25 +63,29 @@ typedef struct VDPLUGIN
 typedef VDPLUGIN *PVDPLUGIN;
 
 
-/*********************************************************************************************************************************
-*   Defined Constants And Macros                                                                                                 *
-*********************************************************************************************************************************/
-
-
 /*********************************************************************************************************************************
 *   Global Variables                                                                                                             *
 *********************************************************************************************************************************/
-
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
 /** Head of loaded plugin list. */
 static RTLISTANCHOR g_ListPluginsLoaded;
+#endif
 
 /** Number of image backends supported. */
 static unsigned g_cBackends = 0;
 /** Array of pointers to the image backends. */
 static PCVDIMAGEBACKEND *g_apBackends = NULL;
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
 /** Array of handles to the corresponding plugin. */
 static RTLDRMOD *g_ahBackendPlugins = NULL;
-/** Builtin image backends. */
+#endif
+/**
+ * Builtin image backends.
+ *
+ * @note As long as the pfnProb() calls aren't scored, the ordering influences
+ *       which backend take precedence.  In particular, the RAW backend should
+ *       be thowards the end of the list.
+ */
 static PCVDIMAGEBACKEND aStaticBackends[] =
 {
     &g_VmdkBackend,
@@ -83,9 +96,9 @@ static PCVDIMAGEBACKEND aStaticBackends[] =
     &g_QedBackend,
     &g_QCowBackend,
     &g_VhdxBackend,
-    &g_RawBackend,
     &g_CueBackend,
     &g_VBoxIsoMakerBackend,
+    &g_RawBackend,
     &g_ISCSIBackend
 };
 
@@ -93,8 +106,13 @@ static PCVDIMAGEBACKEND aStaticBackends[] =
 static unsigned g_cCacheBackends = 0;
 /** Array of pointers to the cache backends. */
 static PCVDCACHEBACKEND *g_apCacheBackends = NULL;
-/** Array of handles to the corresponding plugin. */
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+/** Array of handles to the corresponding plugin.
+ *
+ * @todo r=bird: This looks rather pointless.
+ */
 static RTLDRMOD *g_ahCacheBackendPlugins = NULL;
+#endif
 /** Builtin cache backends. */
 static PCVDCACHEBACKEND aStaticCacheBackends[] =
 {
@@ -132,15 +150,20 @@ static int vdAddBackends(RTLDRMOD hPlugin, PCVDIMAGEBACKEND *ppBackends, unsigne
     if (RT_UNLIKELY(!pTmp))
         return VERR_NO_MEMORY;
     g_apBackends = pTmp;
+    memcpy(&g_apBackends[g_cBackends], ppBackends, cBackends * sizeof(PCVDIMAGEBACKEND));
 
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
     RTLDRMOD *pTmpPlugins = (RTLDRMOD*)RTMemRealloc(g_ahBackendPlugins,
            (g_cBackends + cBackends) * sizeof(RTLDRMOD));
     if (RT_UNLIKELY(!pTmpPlugins))
         return VERR_NO_MEMORY;
     g_ahBackendPlugins = pTmpPlugins;
-    memcpy(&g_apBackends[g_cBackends], ppBackends, cBackends * sizeof(PCVDIMAGEBACKEND));
     for (unsigned i = g_cBackends; i < g_cBackends + cBackends; i++)
         g_ahBackendPlugins[i] = hPlugin;
+#else
+    RT_NOREF(hPlugin);
+#endif
+
     g_cBackends += cBackends;
     return VINF_SUCCESS;
 }
@@ -158,20 +181,27 @@ static int vdAddBackends(RTLDRMOD hPlugin, PCVDIMAGEBACKEND *ppBackends, unsigne
  */
 static int vdAddCacheBackends(RTLDRMOD hPlugin, PCVDCACHEBACKEND *ppBackends, unsigned cBackends)
 {
-    PCVDCACHEBACKEND *pTmp = (PCVDCACHEBACKEND*)RTMemRealloc(g_apCacheBackends,
-           (g_cCacheBackends + cBackends) * sizeof(PCVDCACHEBACKEND));
+    PCVDCACHEBACKEND *pTmp = (PCVDCACHEBACKEND*)RTMemReallocTag(g_apCacheBackends,
+                                                                (g_cCacheBackends + cBackends) * sizeof(PCVDCACHEBACKEND),
+                                                                "may-leak:vdAddCacheBackend");
     if (RT_UNLIKELY(!pTmp))
         return VERR_NO_MEMORY;
     g_apCacheBackends = pTmp;
+    memcpy(&g_apCacheBackends[g_cCacheBackends], ppBackends, cBackends * sizeof(PCVDCACHEBACKEND));
 
-    RTLDRMOD *pTmpPlugins = (RTLDRMOD*)RTMemRealloc(g_ahCacheBackendPlugins,
-           (g_cCacheBackends + cBackends) * sizeof(RTLDRMOD));
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+    RTLDRMOD *pTmpPlugins = (RTLDRMOD*)RTMemReallocTag(g_ahCacheBackendPlugins,
+                                                       (g_cCacheBackends + cBackends) * sizeof(RTLDRMOD),
+                                                       "may-leak:vdAddCacheBackend");
     if (RT_UNLIKELY(!pTmpPlugins))
         return VERR_NO_MEMORY;
     g_ahCacheBackendPlugins = pTmpPlugins;
-    memcpy(&g_apCacheBackends[g_cCacheBackends], ppBackends, cBackends * sizeof(PCVDCACHEBACKEND));
     for (unsigned i = g_cCacheBackends; i < g_cCacheBackends + cBackends; i++)
         g_ahCacheBackendPlugins[i] = hPlugin;
+#else
+    RT_NOREF(hPlugin);
+#endif
+
     g_cCacheBackends += cBackends;
     return VINF_SUCCESS;
 }
@@ -221,6 +251,7 @@ static int vdAddFilterBackends(RTLDRMOD hPlugin, PCVDFILTERBACKEND *ppBackends,
         return VERR_NO_MEMORY;
     g_apFilterBackends = pTmp;
 
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
     PRTLDRMOD pTmpPlugins = (PRTLDRMOD)RTMemRealloc(g_pahFilterBackendPlugins,
                                                     (g_cFilterBackends + cBackends) * sizeof(RTLDRMOD));
     if (RT_UNLIKELY(!pTmpPlugins))
@@ -230,6 +261,10 @@ static int vdAddFilterBackends(RTLDRMOD hPlugin, PCVDFILTERBACKEND *ppBackends,
     memcpy(&g_apFilterBackends[g_cFilterBackends], ppBackends, cBackends * sizeof(PCVDFILTERBACKEND));
     for (unsigned i = g_cFilterBackends; i < g_cFilterBackends + cBackends; i++)
         g_pahFilterBackendPlugins[i] = hPlugin;
+#else
+    RT_NOREF(hPlugin);
+#endif
+
     g_cFilterBackends += cBackends;
     return VINF_SUCCESS;
 }
@@ -369,8 +404,8 @@ static int vdRemovePlugin(const char *pszFilename)
     {
         while (i < g_cBackends && g_ahBackendPlugins[i] == pIt->hPlugin)
         {
-            memcpy(&g_apBackends[i], &g_apBackends[i + 1], (g_cBackends - i - 1) * sizeof(PCVDIMAGEBACKEND));
-            memcpy(&g_ahBackendPlugins[i], &g_ahBackendPlugins[i + 1], (g_cBackends - i - 1) * sizeof(RTLDRMOD));
+            memmove(&g_apBackends[i], &g_apBackends[i + 1], (g_cBackends - i - 1) * sizeof(PCVDIMAGEBACKEND));
+            memmove(&g_ahBackendPlugins[i], &g_ahBackendPlugins[i + 1], (g_cBackends - i - 1) * sizeof(RTLDRMOD));
             /** @todo for now skip reallocating, doesn't save much */
             g_cBackends--;
         }
@@ -379,8 +414,8 @@ static int vdRemovePlugin(const char *pszFilename)
     {
         while (i < g_cCacheBackends && g_ahCacheBackendPlugins[i] == pIt->hPlugin)
         {
-            memcpy(&g_apCacheBackends[i], &g_apCacheBackends[i + 1], (g_cCacheBackends - i - 1) * sizeof(PCVDCACHEBACKEND));
-            memcpy(&g_ahCacheBackendPlugins[i], &g_ahCacheBackendPlugins[i + 1], (g_cCacheBackends - i - 1) * sizeof(RTLDRMOD));
+            memmove(&g_apCacheBackends[i], &g_apCacheBackends[i + 1], (g_cCacheBackends - i - 1) * sizeof(PCVDCACHEBACKEND));
+            memmove(&g_ahCacheBackendPlugins[i], &g_ahCacheBackendPlugins[i + 1], (g_cCacheBackends - i - 1) * sizeof(RTLDRMOD));
             /** @todo for now skip reallocating, doesn't save much */
             g_cCacheBackends--;
         }
@@ -389,8 +424,8 @@ static int vdRemovePlugin(const char *pszFilename)
     {
         while (i < g_cFilterBackends && g_pahFilterBackendPlugins[i] == pIt->hPlugin)
         {
-            memcpy(&g_apFilterBackends[i], &g_apFilterBackends[i + 1], (g_cFilterBackends - i - 1) * sizeof(PCVDFILTERBACKEND));
-            memcpy(&g_pahFilterBackendPlugins[i], &g_pahFilterBackendPlugins[i + 1], (g_cFilterBackends - i - 1) * sizeof(RTLDRMOD));
+            memmove(&g_apFilterBackends[i], &g_apFilterBackends[i + 1], (g_cFilterBackends - i - 1) * sizeof(PCVDFILTERBACKEND));
+            memmove(&g_pahFilterBackendPlugins[i], &g_pahFilterBackendPlugins[i + 1], (g_cFilterBackends - i - 1) * sizeof(RTLDRMOD));
             /** @todo for now skip reallocating, doesn't save much */
             g_cFilterBackends--;
         }
@@ -585,10 +620,14 @@ DECLHIDDEN(int) vdPluginLoadFromFilename(const char *pszFilename)
 #ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
     /* Plugin loaded? Nothing to do. */
     if (vdPluginFind(pszFilename))
+    {
+        LogFlowFunc(("Plugin '%s' already loaded\n", pszFilename));
         return VINF_SUCCESS;
+    }
 
     RTLDRMOD hPlugin = NIL_RTLDRMOD;
     int rc = SUPR3HardenedLdrLoadPlugIn(pszFilename, &hPlugin, NULL);
+    LogFlowFunc(("SUPR3HardenedLdrLoadPlugIn('%s') -> %Rrc\n", pszFilename, rc));
     if (RT_SUCCESS(rc))
     {
         VDBACKENDREGISTER BackendRegister;
@@ -688,7 +727,7 @@ DECLHIDDEN(int) vdPluginLoadFromPath(const char *pszPath)
                     break;
                 }
 
-                rc = vdPluginLoadFromFilename(pszPluginPath);
+                vdPluginLoadFromFilename(pszPluginPath);
                 RTStrFree(pszPluginPath);
             }
 
@@ -716,12 +755,12 @@ DECLHIDDEN(int) vdPluginLoadFromPath(const char *pszPath)
 #endif
 }
 
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
 /**
  * internal: scans plugin directory and loads found plugins.
  */
 static int vdLoadDynamicBackends(void)
 {
-#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
     /*
      * Enumerate plugin backends from the application directory where the other
      * shared libraries are.
@@ -732,10 +771,8 @@ static int vdLoadDynamicBackends(void)
         return rc;
 
     return vdPluginLoadFromPath(szPath);
-#else
-    return VINF_SUCCESS;
-#endif
 }
+#endif
 
 /**
  * Worker for VDPluginUnloadFromFilename() and vdPluginUnloadFromPath().
@@ -810,7 +847,7 @@ DECLHIDDEN(int) vdPluginUnloadFromPath(const char *pszPath)
                     break;
                 }
 
-                rc = vdPluginUnloadFromFilename(pszPluginPath);
+                vdPluginUnloadFromFilename(pszPluginPath);
                 RTStrFree(pszPluginPath);
             }
 
@@ -851,11 +888,13 @@ DECLHIDDEN(int) vdPluginInit(void)
     if (RT_SUCCESS(rc))
     {
         rc = vdAddCacheBackends(NIL_RTLDRMOD, aStaticCacheBackends, RT_ELEMENTS(aStaticCacheBackends));
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
         if (RT_SUCCESS(rc))
         {
             RTListInit(&g_ListPluginsLoaded);
             rc = vdLoadDynamicBackends();
         }
+#endif
     }
 
     return rc;
@@ -872,6 +911,16 @@ DECLHIDDEN(int) vdPluginTerm(void)
     if (!g_apBackends)
         return VERR_INTERNAL_ERROR;
 
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+    if (g_pahFilterBackendPlugins)
+        RTMemFree(g_pahFilterBackendPlugins);
+#endif
+    if (g_apFilterBackends)
+        RTMemFree(g_apFilterBackends);
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+    if (g_ahCacheBackendPlugins)
+        RTMemFree(g_ahCacheBackendPlugins);
+#endif
     if (g_apCacheBackends)
         RTMemFree(g_apCacheBackends);
     RTMemFree(g_apBackends);
@@ -882,6 +931,16 @@ DECLHIDDEN(int) vdPluginTerm(void)
     /* Clear the supported cache backends. */
     g_cCacheBackends = 0;
     g_apCacheBackends = NULL;
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+    g_ahCacheBackendPlugins = NULL;
+#endif
+
+    /* Clear the supported filter backends. */
+    g_cFilterBackends = 0;
+    g_apFilterBackends = NULL;
+#ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
+    g_pahFilterBackendPlugins = NULL;
+#endif
 
 #ifndef VBOX_HDD_NO_DYNAMIC_BACKENDS
     PVDPLUGIN pPlugin, pPluginNext;
