@@ -4,15 +4,25 @@
  */
 
 /*
- * Copyright (C) 2006-2019 Oracle Corporation
+ * Copyright (C) 2006-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
@@ -20,24 +30,27 @@
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
 #define LOG_GROUP LOG_GROUP_PDM_DEVICE
+#define PDMPCIDEV_INCLUDE_PRIVATE  /* Hack to get pdmpcidevint.h included at the right point. */
 #include "PDMInternal.h"
 #include <VBox/vmm/pdm.h>
-#include <VBox/vmm/mm.h>
-#include <VBox/vmm/pgm.h>
-#include <VBox/vmm/iom.h>
-#include <VBox/vmm/hm.h>
-#include <VBox/vmm/cfgm.h>
-#include <VBox/vmm/apic.h>
-#ifdef VBOX_WITH_REM
-# include <VBox/vmm/rem.h>
+#if defined(VBOX_VMM_TARGET_ARMV8)
+# include <VBox/vmm/gic.h>
+#else
+# include <VBox/vmm/apic.h>
 #endif
+#include <VBox/vmm/cfgm.h>
 #include <VBox/vmm/dbgf.h>
+#include <VBox/vmm/hm.h>
+#include <VBox/vmm/mm.h>
+#include <VBox/vmm/iom.h>
+#include <VBox/vmm/pgm.h>
 #include <VBox/vmm/vm.h>
 #include <VBox/vmm/uvm.h>
 #include <VBox/vmm/vmm.h>
 
 #include <VBox/version.h>
 #include <VBox/log.h>
+#include <VBox/msi.h>
 #include <VBox/err.h>
 #include <iprt/alloc.h>
 #include <iprt/alloca.h>
@@ -131,23 +144,9 @@ int pdmR3DevInit(PVM pVM)
     /*
      * Get the RC & R0 devhlps and create the devhlp R3 task queue.
      */
-    PCPDMDEVHLPRC pHlpRC = NIL_RTRCPTR;
-    if (VM_IS_RAW_MODE_ENABLED(pVM))
-    {
-        rc = PDMR3LdrGetSymbolRC(pVM, NULL, "g_pdmRCDevHlp", &pHlpRC);
-        AssertReleaseRCReturn(rc, rc);
-    }
-
-    PCPDMDEVHLPR0 pHlpR0;
-    rc = PDMR3LdrGetSymbolR0(pVM, NULL, "g_pdmR0DevHlp", &pHlpR0);
-    AssertReleaseRCReturn(rc, rc);
-
-    rc = PDMR3QueueCreateInternal(pVM, sizeof(PDMDEVHLPTASK), 8, 0, pdmR3DevHlpQueueConsumer, true, "DevHlp",
-                                  &pVM->pdm.s.pDevHlpQueueR3);
+    rc = PDMR3QueueCreateInternal(pVM, sizeof(PDMDEVHLPTASK), pVM->cCpus * 8, 0, pdmR3DevHlpQueueConsumer, true, "DevHlp",
+                                  &pVM->pdm.s.hDevHlpQueue);
     AssertRCReturn(rc, rc);
-    pVM->pdm.s.pDevHlpQueueR0 = PDMQueueR0Ptr(pVM->pdm.s.pDevHlpQueueR3);
-    pVM->pdm.s.pDevHlpQueueRC = PDMQueueRCPtr(pVM->pdm.s.pDevHlpQueueR3);
-
 
     /*
      *
@@ -272,6 +271,8 @@ int pdmR3DevInit(PVM pVM)
      */
     for (i = 0; i < cDevs; i++)
     {
+        PDMDEVREGR3 const * const pReg = paDevs[i].pDev->pReg;
+
         /*
          * Gather a bit of config.
          */
@@ -285,6 +286,66 @@ int pdmR3DevInit(PVM pVM)
             AssertMsgFailed(("configuration error: failed to query boolean \"Trusted\", rc=%Rrc\n", rc));
             return rc;
         }
+
+        /* RZEnabled, R0Enabled, RCEnabled*/
+        bool fR0Enabled = false;
+        bool fRCEnabled = false;
+        if (   (pReg->fFlags & (PDM_DEVREG_FLAGS_R0 | PDM_DEVREG_FLAGS_RC))
+#ifdef VBOX_WITH_PGM_NEM_MODE
+            && !PGMR3IsNemModeEnabled(pVM) /* No ring-0 in simplified memory mode. */
+#endif
+            && !SUPR3IsDriverless())
+        {
+            if (pReg->fFlags & PDM_DEVREG_FLAGS_R0)
+            {
+                if (pReg->fFlags & PDM_DEVREG_FLAGS_REQUIRE_R0)
+                    fR0Enabled = true;
+                else
+                {
+                    rc = CFGMR3QueryBoolDef(paDevs[i].pNode, "R0Enabled", &fR0Enabled,
+                                            !(pReg->fFlags & PDM_DEVREG_FLAGS_OPT_IN_R0));
+                    AssertLogRelRCReturn(rc, rc);
+                }
+            }
+
+            if (pReg->fFlags & PDM_DEVREG_FLAGS_RC)
+            {
+                if (pReg->fFlags & PDM_DEVREG_FLAGS_REQUIRE_RC)
+                    fRCEnabled = true;
+                else
+                {
+                    rc = CFGMR3QueryBoolDef(paDevs[i].pNode, "RCEnabled", &fRCEnabled,
+                                            !(pReg->fFlags & PDM_DEVREG_FLAGS_OPT_IN_RC));
+                    AssertLogRelRCReturn(rc, rc);
+                }
+                fRCEnabled = false;
+            }
+        }
+
+#ifdef VBOX_WITH_DBGF_TRACING
+        DBGFTRACEREVTSRC hDbgfTraceEvtSrc = NIL_DBGFTRACEREVTSRC;
+        bool fTracingEnabled = false;
+        bool fGCPhysRwAll = false;
+        rc = CFGMR3QueryBoolDef(paDevs[i].pNode, "TracingEnabled", &fTracingEnabled,
+                                false);
+        AssertLogRelRCReturn(rc, rc);
+        if (fTracingEnabled)
+        {
+            rc = CFGMR3QueryBoolDef(paDevs[i].pNode, "TraceAllGstMemRw", &fGCPhysRwAll,
+                                    false);
+            AssertLogRelRCReturn(rc, rc);
+
+            /* Traced devices need to be trusted for now. */
+            if (fTrusted)
+            {
+                rc = DBGFR3TracerRegisterEvtSrc(pVM, pReg->szName, &hDbgfTraceEvtSrc);
+                AssertLogRelRCReturn(rc, rc);
+            }
+            else
+                AssertMsgFailedReturn(("configuration error: Device tracing needs a trusted device\n"), VERR_INCOMPATIBLE_CONFIG);
+        }
+#endif
+
         /* config node */
         PCFGMNODE pConfigNode = CFGMR3GetChild(paDevs[i].pNode, "Config");
         if (!pConfigNode)
@@ -301,62 +362,136 @@ int pdmR3DevInit(PVM pVM)
         /*
          * Allocate the device instance and critical section.
          */
-        AssertReturn(paDevs[i].pDev->cInstances < paDevs[i].pDev->pReg->cMaxInstances, VERR_PDM_TOO_MANY_DEVICE_INSTANCES);
-        size_t cb = RT_UOFFSETOF_DYN(PDMDEVINS, achInstanceData[paDevs[i].pDev->pReg->cbInstance]);
-        cb = RT_ALIGN_Z(cb, 16);
-        PPDMDEVINS pDevIns;
-        if (paDevs[i].pDev->pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0))
-            rc = MMR3HyperAllocOnceNoRel(pVM, cb, 0, MM_TAG_PDM_DEVICE, (void **)&pDevIns);
-        else
-            rc = MMR3HeapAllocZEx(pVM, MM_TAG_PDM_DEVICE, cb, (void **)&pDevIns);
-        AssertLogRelMsgRCReturn(rc,
-                                ("Failed to allocate %d bytes of instance data for device '%s'. rc=%Rrc\n",
-                                cb, paDevs[i].pDev->pReg->szName, rc),
-                                rc);
+        AssertLogRelReturn(paDevs[i].pDev->cInstances < pReg->cMaxInstances,
+                           VERR_PDM_TOO_MANY_DEVICE_INSTANCES);
+        PPDMDEVINS   pDevIns;
         PPDMCRITSECT pCritSect;
-        if (paDevs[i].pDev->pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0))
-            rc = MMHyperAlloc(pVM, sizeof(*pCritSect), 0, MM_TAG_PDM_DEVICE, (void **)&pCritSect);
+        if (fR0Enabled || fRCEnabled)
+        {
+            AssertLogRel(fR0Enabled /* not possible to just enabled raw-mode atm. */);
+
+            rc = PDMR3LdrLoadR0(pVM->pUVM, pReg->pszR0Mod, paDevs[i].pDev->pszR0SearchPath);
+            if (RT_FAILURE(rc))
+                return VMR3SetError(pVM->pUVM, rc, RT_SRC_POS, "Failed to load ring-0 module '%s' for device '%s'",
+                                    pReg->pszR0Mod, pReg->szName);
+
+            PDMDEVICECREATEREQ Req;
+            Req.Hdr.u32Magic      = SUPVMMR0REQHDR_MAGIC;
+            Req.Hdr.cbReq         = sizeof(Req);
+            Req.pDevInsR3         = NULL;
+            /** @todo Add tracer id in request so R0 can set up DEVINSR0 properly. */
+            Req.fFlags            = pReg->fFlags;
+            Req.fClass            = pReg->fClass;
+            Req.cMaxInstances     = pReg->cMaxInstances;
+            Req.uSharedVersion    = pReg->uSharedVersion;
+            Req.cbInstanceShared  = pReg->cbInstanceShared;
+            Req.cbInstanceR3      = pReg->cbInstanceCC;
+            Req.cbInstanceRC      = pReg->cbInstanceRC;
+            Req.cMaxPciDevices    = pReg->cMaxPciDevices;
+            Req.cMaxMsixVectors   = pReg->cMaxMsixVectors;
+            Req.iInstance         = paDevs[i].iInstance;
+            Req.fRCEnabled        = fRCEnabled;
+            Req.afReserved[0]     = false;
+            Req.afReserved[1]     = false;
+            Req.afReserved[2]     = false;
+#ifdef VBOX_WITH_DBGF_TRACING
+            Req.hDbgfTracerEvtSrc = hDbgfTraceEvtSrc;
+#else
+            Req.hDbgfTracerEvtSrc = NIL_DBGFTRACEREVTSRC;
+#endif
+            rc = RTStrCopy(Req.szDevName, sizeof(Req.szDevName), pReg->szName);
+            AssertLogRelRCReturn(rc, rc);
+            rc = RTStrCopy(Req.szModName, sizeof(Req.szModName), pReg->pszR0Mod);
+            AssertLogRelRCReturn(rc, rc);
+
+            rc = VMMR3CallR0Emt(pVM, pVM->apCpusR3[0], VMMR0_DO_PDM_DEVICE_CREATE, 0, &Req.Hdr);
+            AssertLogRelMsgRCReturn(rc, ("VMMR0_DO_PDM_DEVICE_CREATE for %s failed: %Rrc\n", pReg->szName, rc), rc);
+
+            pDevIns = Req.pDevInsR3;
+            pCritSect = pDevIns->pCritSectRoR3;
+
+            Assert(pDevIns->Internal.s.fIntFlags & PDMDEVINSINT_FLAGS_R0_ENABLED);
+            AssertLogRelReturn(pDevIns->Internal.s.idxR0Device < PDM_MAX_RING0_DEVICE_INSTANCES, VERR_PDM_DEV_IPE_1);
+            AssertLogRelReturn(pVM->pdm.s.apDevRing0Instances[pDevIns->Internal.s.idxR0Device] == pDevIns, VERR_PDM_DEV_IPE_1);
+        }
         else
-            rc = MMR3HeapAllocZEx(pVM, MM_TAG_PDM_DEVICE, sizeof(*pCritSect), (void **)&pCritSect);
-        AssertLogRelMsgRCReturn(rc, ("Failed to allocate a critical section for the device (%Rrc)\n",  rc), rc);
+        {
+            /* The code in this else branch works by the same rules as the PDMR0Device.cpp
+               code, except there is only the ring-3 components of the device instance.
+               Changes here may need to be reflected in PDMR0DEvice.cpp and vice versa! */
+            uint32_t cb = RT_UOFFSETOF_DYN(PDMDEVINS, achInstanceData[pReg->cbInstanceCC]);
+            cb  = RT_ALIGN_32(cb, 64);
+            uint32_t const offShared   = cb;
+            cb += RT_ALIGN_32(pReg->cbInstanceShared, 64);
+            uint32_t const cbCritSect  = RT_ALIGN_32(sizeof(*pCritSect), 64);
+            cb += cbCritSect;
+            uint32_t const cbMsixState = RT_ALIGN_32(pReg->cMaxMsixVectors * 16 + (pReg->cMaxMsixVectors + 7) / 8, _4K);
+            uint32_t const cbPciDev    = RT_ALIGN_32(RT_UOFFSETOF_DYN(PDMPCIDEV, abMsixState[cbMsixState]), 64);
+            uint32_t const cPciDevs    = RT_MIN(pReg->cMaxPciDevices, 1024);
+            uint32_t const cbPciDevs   = cbPciDev * cPciDevs;
+            cb += cbPciDevs;
+            AssertLogRelMsgReturn(cb <= PDM_MAX_DEVICE_INSTANCE_SIZE_R3,
+                                  ("Device %s total instance size is to big: %u, max %u\n",
+                                   pReg->szName, cb, PDM_MAX_DEVICE_INSTANCE_SIZE_R3),
+                                  VERR_ALLOCATION_TOO_BIG);
+
+#if 0  /* Several devices demands cacheline aligned data, if not page aligned. Real problem in NEM mode. */
+            rc = MMR3HeapAllocZEx(pVM, MM_TAG_PDM_DEVICE, cb, (void **)&pDevIns);
+            AssertLogRelMsgRCReturn(rc, ("Failed to allocate %zu bytes of instance data for device '%s'. rc=%Rrc\n",
+                                         cb, pReg->szName, rc), rc);
+#else
+            pDevIns = (PPDMDEVINS)RTMemPageAllocZ(cb);
+            AssertLogRelMsgReturn(pDevIns, ("Failed to allocate %zu bytes of instance data for device '%s'\n", cb, pReg->szName),
+                                  VERR_NO_PAGE_MEMORY);
+#endif
+
+            /* Initialize it: */
+            pDevIns->u32Version             = PDM_DEVINSR3_VERSION;
+            pDevIns->iInstance              = paDevs[i].iInstance;
+            pDevIns->cbRing3                = cb;
+            //pDevIns->fR0Enabled           = false;
+            //pDevIns->fRCEnabled           = false;
+            pDevIns->pvInstanceDataR3       = (uint8_t *)pDevIns + offShared;
+            pDevIns->pvInstanceDataForR3    = &pDevIns->achInstanceData[0];
+            pCritSect = (PPDMCRITSECT)((uint8_t *)pDevIns + offShared + RT_ALIGN_32(pReg->cbInstanceShared, 64));
+            pDevIns->pCritSectRoR3          = pCritSect;
+            pDevIns->cbPciDev               = cbPciDev;
+            pDevIns->cPciDevs               = cPciDevs;
+            for (uint32_t iPciDev = 0; iPciDev < cPciDevs; iPciDev++)
+            {
+                PPDMPCIDEV pPciDev = (PPDMPCIDEV)((uint8_t *)pDevIns->pCritSectRoR3 + cbCritSect + cbPciDev * iPciDev);
+                if (iPciDev < RT_ELEMENTS(pDevIns->apPciDevs))
+                    pDevIns->apPciDevs[iPciDev] = pPciDev;
+                pPciDev->cbConfig           = _4K;
+                pPciDev->cbMsixState        = cbMsixState;
+                pPciDev->idxSubDev          = (uint16_t)iPciDev;
+                pPciDev->Int.s.idxSubDev    = (uint16_t)iPciDev;
+                pPciDev->u32Magic           = PDMPCIDEV_MAGIC;
+            }
+        }
 
-        /*
-         * Initialize it.
-         */
-        pDevIns->u32Version                     = PDM_DEVINS_VERSION;
-        pDevIns->iInstance                      = paDevs[i].iInstance;
-        //pDevIns->Internal.s.pNextR3             = NULL;
-        //pDevIns->Internal.s.pPerDeviceNextR3    = NULL;
-        pDevIns->Internal.s.pDevR3              = paDevs[i].pDev;
-        pDevIns->Internal.s.pVMR3               = pVM;
-        pDevIns->Internal.s.pVMR0               = pVM->pVMR0;
-        pDevIns->Internal.s.pVMRC               = pVM->pVMRC;
-        //pDevIns->Internal.s.pLunsR3             = NULL;
-        pDevIns->Internal.s.pCfgHandle          = paDevs[i].pNode;
-        //pDevIns->Internal.s.pHeadPciDevR3       = NULL;
-        //pDevIns->Internal.s.pHeadPciDevR0       = 0;
-        //pDevIns->Internal.s.pHeadPciDevRC       = 0;
-        pDevIns->Internal.s.fIntFlags           = PDMDEVINSINT_FLAGS_SUSPENDED;
-        //pDevIns->Internal.s.uLastIrqTag         = 0;
         pDevIns->pHlpR3                         = fTrusted ? &g_pdmR3DevHlpTrusted : &g_pdmR3DevHlpUnTrusted;
-        pDevIns->pHlpRC                         = pHlpRC;
-        pDevIns->pHlpR0                         = pHlpR0;
-        pDevIns->pReg                           = paDevs[i].pDev->pReg;
+        pDevIns->pReg                           = pReg;
         pDevIns->pCfg                           = pConfigNode;
         //pDevIns->IBase.pfnQueryInterface        = NULL;
         //pDevIns->fTracing                       = 0;
         pDevIns->idTracing                      = ++pVM->pdm.s.idTracingDev;
-        pDevIns->pvInstanceDataR3               = &pDevIns->achInstanceData[0];
-        pDevIns->pvInstanceDataRC               = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_RC
-                                                ? MMHyperR3ToRC(pVM, pDevIns->pvInstanceDataR3) : NIL_RTRCPTR;
-        pDevIns->pvInstanceDataR0               = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_R0
-                                                ? MMHyperR3ToR0(pVM, pDevIns->pvInstanceDataR3) : NIL_RTR0PTR;
 
-        pDevIns->pCritSectRoR3                  = pCritSect;
-        pDevIns->pCritSectRoRC                  = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_RC
-                                                ? MMHyperR3ToRC(pVM, pCritSect) : NIL_RTRCPTR;
-        pDevIns->pCritSectRoR0                  = pDevIns->pReg->fFlags & PDM_DEVREG_FLAGS_R0
-                                                ? MMHyperR3ToR0(pVM, pCritSect) : NIL_RTR0PTR;
+        //pDevIns->Internal.s.pNextR3             = NULL;
+        //pDevIns->Internal.s.pPerDeviceNextR3    = NULL;
+        pDevIns->Internal.s.pDevR3              = paDevs[i].pDev;
+        //pDevIns->Internal.s.pLunsR3             = NULL;
+        //pDevIns->Internal.s.pfnAsyncNotify      = NULL;
+        pDevIns->Internal.s.pCfgHandle          = paDevs[i].pNode;
+        pDevIns->Internal.s.pVMR3               = pVM;
+#ifdef VBOX_WITH_DBGF_TRACING
+        pDevIns->Internal.s.hDbgfTraceEvtSrc    = hDbgfTraceEvtSrc;
+#else
+        pDevIns->Internal.s.hDbgfTraceEvtSrc    = NIL_DBGFTRACEREVTSRC;
+#endif
+        //pDevIns->Internal.s.pHeadPciDevR3       = NULL;
+        pDevIns->Internal.s.fIntFlags          |= PDMDEVINSINT_FLAGS_SUSPENDED;
+        //pDevIns->Internal.s.uLastIrqTag         = 0;
 
         rc = pdmR3CritSectInitDeviceAuto(pVM, pDevIns, pCritSect, RT_SRC_POS,
                                          "%s#%uAuto", pDevIns->pReg->szName, pDevIns->iInstance);
@@ -365,6 +500,8 @@ int pdmR3DevInit(PVM pVM)
         /*
          * Link it into all the lists.
          */
+        RTCritSectRwEnterExcl(&pVM->pdm.s.CoreListCritSectRw);
+
         /* The global instance FIFO. */
         PPDMDEVINS pPrev1 = pVM->pdm.s.pDevInstances;
         if (!pPrev1)
@@ -387,20 +524,75 @@ int pdmR3DevInit(PVM pVM)
             pPrev2->Internal.s.pPerDeviceNextR3 = pDevIns;
         }
 
+#ifdef VBOX_WITH_DBGF_TRACING
+        /*
+         * Allocate memory for the MMIO/IO port registration tracking if DBGF tracing is enabled.
+         */
+        if (hDbgfTraceEvtSrc != NIL_DBGFTRACEREVTSRC)
+        {
+            pDevIns->Internal.s.paDbgfTraceTrack = (PPDMDEVINSDBGFTRACK)RTMemAllocZ(PDM_MAX_DEVICE_DBGF_TRACING_TRACK);
+            if (!pDevIns->Internal.s.paDbgfTraceTrack)
+            {
+                LogRel(("PDM: Failed to construct '%s'/%d! %Rra\n", pDevIns->pReg->szName, pDevIns->iInstance, VERR_NO_MEMORY));
+                if (VMR3GetErrorCount(pVM->pUVM) == 0)
+                    VMSetError(pVM, rc, RT_SRC_POS, "Failed to construct device '%s' instance #%u",
+                               pDevIns->pReg->szName, pDevIns->iInstance);
+                RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw);
+                return VERR_NO_MEMORY;
+            }
+
+            pDevIns->Internal.s.idxDbgfTraceTrackNext = 0;
+            pDevIns->Internal.s.cDbgfTraceTrackMax = PDM_MAX_DEVICE_DBGF_TRACING_TRACK / sizeof(PDMDEVINSDBGFTRACK);
+            pDevIns->pHlpR3 = &g_pdmR3DevHlpTracing;
+        }
+#endif
+
         /*
          * Call the constructor.
          */
         paDevs[i].pDev->cInstances++;
+        RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw);
+
         Log(("PDM: Constructing device '%s' instance %d...\n", pDevIns->pReg->szName, pDevIns->iInstance));
         rc = pDevIns->pReg->pfnConstruct(pDevIns, pDevIns->iInstance, pDevIns->pCfg);
         if (RT_FAILURE(rc))
         {
             LogRel(("PDM: Failed to construct '%s'/%d! %Rra\n", pDevIns->pReg->szName, pDevIns->iInstance, rc));
-            paDevs[i].pDev->cInstances--;
+            if (VMR3GetErrorCount(pVM->pUVM) == 0)
+                VMSetError(pVM, rc, RT_SRC_POS, "Failed to construct device '%s' instance #%u",
+                           pDevIns->pReg->szName, pDevIns->iInstance);
             /* Because we're damn lazy, the destructor will be called even if
                the constructor fails.  So, no unlinking. */
+            //paDevs[i].pDev->cInstances--;
             return rc == VERR_VERSION_MISMATCH ? VERR_PDM_DEVICE_VERSION_MISMATCH : rc;
         }
+
+        /*
+         * Call the ring-0 constructor if applicable.
+         */
+        if (fR0Enabled)
+        {
+            PDMDEVICEGENCALLREQ Req;
+            RT_ZERO(Req.Params);
+            Req.Hdr.u32Magic = SUPVMMR0REQHDR_MAGIC;
+            Req.Hdr.cbReq    = sizeof(Req);
+            Req.enmCall      = PDMDEVICEGENCALL_CONSTRUCT;
+            Req.idxR0Device  = pDevIns->Internal.s.idxR0Device;
+            Req.pDevInsR3    = pDevIns;
+            rc = VMMR3CallR0Emt(pVM, pVM->apCpusR3[0], VMMR0_DO_PDM_DEVICE_GEN_CALL, 0, &Req.Hdr);
+            pDevIns->Internal.s.fIntFlags |= PDMDEVINSINT_FLAGS_R0_CONTRUCT;
+            if (RT_FAILURE(rc))
+            {
+                LogRel(("PDM: Failed to construct (ring-0) '%s'/%d! %Rra\n", pDevIns->pReg->szName, pDevIns->iInstance, rc));
+                if (VMR3GetErrorCount(pVM->pUVM) == 0)
+                    VMSetError(pVM, rc, RT_SRC_POS, "The ring-0 constructor of device '%s' instance #%u failed",
+                               pDevIns->pReg->szName, pDevIns->iInstance);
+                /* No unlinking, see above. */
+                //paDevs[i].pDev->cInstances--;
+                return rc == VERR_VERSION_MISMATCH ? VERR_PDM_DEVICE_VERSION_MISMATCH : rc;
+            }
+        }
+
     } /* for device instances */
 
 #ifdef VBOX_WITH_USB
@@ -429,21 +621,25 @@ int pdmR3DevInitComplete(PVM pVM)
     /*
      * Iterate thru the device instances and work the callback.
      */
+    RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
     for (PPDMDEVINS pDevIns = pVM->pdm.s.pDevInstances; pDevIns; pDevIns = pDevIns->Internal.s.pNextR3)
     {
         if (pDevIns->pReg->pfnInitComplete)
         {
-            PDMCritSectEnter(pDevIns->pCritSectRoR3, VERR_IGNORED);
+            RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
+            PDMCritSectEnter(pVM, pDevIns->pCritSectRoR3, VERR_IGNORED);
             rc = pDevIns->pReg->pfnInitComplete(pDevIns);
-            PDMCritSectLeave(pDevIns->pCritSectRoR3);
+            PDMCritSectLeave(pVM, pDevIns->pCritSectRoR3);
             if (RT_FAILURE(rc))
             {
                 AssertMsgFailed(("InitComplete on device '%s'/%d failed with rc=%Rrc\n",
                                  pDevIns->pReg->szName, pDevIns->iInstance, rc));
                 return rc;
             }
+            RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
         }
     }
+    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
 
 #ifdef VBOX_WITH_USB
     rc = pdmR3UsbVMInitComplete(pVM);
@@ -465,11 +661,16 @@ int pdmR3DevInitComplete(PVM pVM)
  */
 PPDMDEV pdmR3DevLookup(PVM pVM, const char *pszName)
 {
-    size_t cchName = strlen(pszName);
+    size_t const cchName = strlen(pszName);
+    RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
     for (PPDMDEV pDev = pVM->pdm.s.pDevs; pDev; pDev = pDev->pNext)
-        if (    pDev->cchName == cchName
-            &&  !strcmp(pDev->pReg->szName, pszName))
+        if (   pDev->cchName == cchName
+            && !strcmp(pDev->pReg->szName, pszName))
+        {
+            RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
             return pDev;
+        }
+    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
     return NULL;
 }
 
@@ -491,12 +692,26 @@ static int pdmR3DevLoadModules(PVM pVM)
     RegCB.pVM              = pVM;
     RegCB.pCfgNode         = NULL;
 
+#if defined(VBOX_VMM_TARGET_ARMV8)
     /*
-     * Load the internal VMM APIC device.
+     * Register the internal VMM GIC device.
      */
-    int rc = APICR3RegisterDevice(&RegCB.Core);
+    int rc = pdmR3DevReg_Register(&RegCB.Core, &g_DeviceGIC);
     AssertRCReturn(rc, rc);
 
+    /*
+     * Register the internal VMM GIC device, NEM variant.
+     */
+    rc = pdmR3DevReg_Register(&RegCB.Core, &g_DeviceGICNem);
+    AssertRCReturn(rc, rc);
+#else
+    /*
+     * Register the internal VMM APIC device.
+     */
+    int rc = pdmR3DevReg_Register(&RegCB.Core, &g_DeviceAPIC);
+    AssertRCReturn(rc, rc);
+#endif
+
     /*
      * Load the builtin module.
      */
@@ -623,13 +838,19 @@ static int pdmR3DevLoad(PVM pVM, PPDMDEVREGCBINT pRegCB, const char *pszFilename
             if (RT_SUCCESS(rc))
                 Log(("PDM: Successfully loaded device module %s (%s).\n", pszName, pszFilename));
             else
+            {
+                VMR3SetError(pVM->pUVM, rc, RT_SRC_POS, "VBoxDevicesRegister failed with rc=%Rrc for module %s (%s)",
+                             rc, pszName, pszFilename);
                 AssertMsgFailed(("VBoxDevicesRegister failed with rc=%Rrc for module %s (%s)\n", rc, pszName, pszFilename));
+            }
         }
         else
         {
             AssertMsgFailed(("Failed to locate 'VBoxDevicesRegister' in %s (%s) rc=%Rrc\n", pszName, pszFilename, rc));
             if (rc == VERR_SYMBOL_NOT_FOUND)
                 rc = VERR_PDM_NO_REGISTRATION_EXPORT;
+            VMR3SetError(pVM->pUVM, rc, RT_SRC_POS, "Failed to locate 'VBoxDevicesRegister' in %s (%s) rc=%Rrc",
+                         pszName, pszFilename, rc);
         }
     }
     else
@@ -657,14 +878,14 @@ static DECLCALLBACK(int) pdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCPDMDEVR
                     ("Invalid name '%.*s'\n", sizeof(pReg->szName), pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn(   !(pReg->fFlags & PDM_DEVREG_FLAGS_RC)
-                    || (   pReg->szRCMod[0]
-                        && strlen(pReg->szRCMod) < sizeof(pReg->szRCMod)),
-                    ("Invalid GC module name '%s' - (Device %s)\n", pReg->szRCMod, pReg->szName),
+                    || (   pReg->pszRCMod[0]
+                        && strlen(pReg->pszRCMod) < RT_SIZEOFMEMB(PDMDEVICECREATEREQ, szModName)),
+                    ("Invalid GC module name '%s' - (Device %s)\n", pReg->pszRCMod, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn(   !(pReg->fFlags & PDM_DEVREG_FLAGS_R0)
-                    || (   pReg->szR0Mod[0]
-                        && strlen(pReg->szR0Mod) < sizeof(pReg->szR0Mod)),
-                    ("Invalid R0 module name '%s' - (Device %s)\n", pReg->szR0Mod, pReg->szName),
+                    || (   pReg->pszR0Mod[0]
+                        && strlen(pReg->pszR0Mod) < RT_SIZEOFMEMB(PDMDEVICECREATEREQ, szModName)),
+                    ("Invalid R0 module name '%s' - (Device %s)\n", pReg->pszR0Mod, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn((pReg->fFlags & PDM_DEVREG_FLAGS_HOST_BITS_MASK) == PDM_DEVREG_FLAGS_HOST_BITS_DEFAULT,
                     ("Invalid host bits flags! fFlags=%#x (Device %s)\n", pReg->fFlags, pReg->szName),
@@ -678,8 +899,13 @@ static DECLCALLBACK(int) pdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCPDMDEVR
     AssertMsgReturn(pReg->cMaxInstances > 0,
                     ("Max instances %u! (Device %s)\n", pReg->cMaxInstances, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
-    AssertMsgReturn(pReg->cbInstance <= (uint32_t)(pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0)  ? 96 * _1K : _1M),
-                    ("Instance size %d bytes! (Device %s)\n", pReg->cbInstance, pReg->szName),
+    uint32_t const cbMaxInstance = pReg->fFlags & (PDM_DEVREG_FLAGS_RC | PDM_DEVREG_FLAGS_R0)
+                                 ? PDM_MAX_DEVICE_INSTANCE_SIZE : PDM_MAX_DEVICE_INSTANCE_SIZE_R3;
+    AssertMsgReturn(pReg->cbInstanceShared <= cbMaxInstance,
+                    ("Instance size %u bytes! (Max %u; Device %s)\n", pReg->cbInstanceShared, cbMaxInstance, pReg->szName),
+                    VERR_PDM_INVALID_DEVICE_REGISTRATION);
+    AssertMsgReturn(pReg->cbInstanceCC <= cbMaxInstance,
+                    ("Instance size %d bytes! (Max %u; Device %s)\n", pReg->cbInstanceCC, cbMaxInstance, pReg->szName),
                     VERR_PDM_INVALID_DEVICE_REGISTRATION);
     AssertMsgReturn(pReg->pfnConstruct,
                     ("No constructor! (Device %s)\n", pReg->szName),
@@ -690,17 +916,29 @@ static DECLCALLBACK(int) pdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCPDMDEVR
     AssertLogRelMsg(pReg->u32VersionEnd == PDM_DEVREG_VERSION,
                     ("u32VersionEnd=%#x, expected %#x. (szName=%s)\n",
                      pReg->u32VersionEnd, PDM_DEVREG_VERSION, pReg->szName));
+    AssertLogRelMsgReturn(pReg->cMaxPciDevices <= 8, ("%#x (szName=%s)\n", pReg->cMaxPciDevices, pReg->szName),
+                          VERR_PDM_INVALID_DEVICE_REGISTRATION);
+    AssertLogRelMsgReturn(pReg->cMaxMsixVectors <= VBOX_MSIX_MAX_ENTRIES,
+                          ("%#x (szName=%s)\n", pReg->cMaxMsixVectors, pReg->szName),
+                          VERR_PDM_INVALID_DEVICE_REGISTRATION);
+    AssertLogRelMsgReturn(pReg->fFlags & PDM_DEVREG_FLAGS_NEW_STYLE /* the flag is required now */,
+                          ("PDM_DEVREG_FLAGS_NEW_STYLE not set for szName=%s!\n", pReg->szName),
+                          VERR_PDM_INVALID_DEVICE_REGISTRATION);
 
     /*
      * Check for duplicate and find FIFO entry at the same time.
      */
     PCPDMDEVREGCBINT pRegCB = (PCPDMDEVREGCBINT)pCallbacks;
+    PVM const        pVM    = pRegCB->pVM;
+    RTCritSectRwEnterExcl(&pVM->pdm.s.CoreListCritSectRw);
+
     PPDMDEV pDevPrev = NULL;
-    PPDMDEV pDev = pRegCB->pVM->pdm.s.pDevs;
+    PPDMDEV pDev     = pVM->pdm.s.pDevs;
     for (; pDev; pDevPrev = pDev, pDev = pDev->pNext)
-        AssertMsgReturn(strcmp(pDev->pReg->szName, pReg->szName),
-                        ("Device '%s' already exists\n", pReg->szName),
-                        VERR_PDM_DEVICE_NAME_CLASH);
+        AssertMsgReturnStmt(strcmp(pDev->pReg->szName, pReg->szName),
+                            ("Device '%s' already exists\n", pReg->szName),
+                            RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw),
+                            VERR_PDM_DEVICE_NAME_CLASH);
 
     /*
      * Allocate new device structure, initialize and insert it into the list.
@@ -723,6 +961,8 @@ static DECLCALLBACK(int) pdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCPDMDEVR
                 pDevPrev->pNext = pDev;
             else
                 pRegCB->pVM->pdm.s.pDevs = pDev;
+
+            RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw);
             Log(("PDM: Registered device '%s'\n", pReg->szName));
             return VINF_SUCCESS;
         }
@@ -731,6 +971,8 @@ static DECLCALLBACK(int) pdmR3DevReg_Register(PPDMDEVREGCB pCallbacks, PCPDMDEVR
     }
     else
         rc = VERR_NO_MEMORY;
+
+    RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw);
     return rc;
 }
 
@@ -751,7 +993,8 @@ int pdmR3DevFindLun(PVM pVM, const char *pszDevice, unsigned iInstance, unsigned
     /*
      * Iterate registered devices looking for the device.
      */
-    size_t cchDevice = strlen(pszDevice);
+    size_t const cchDevice = strlen(pszDevice);
+    RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
     for (PPDMDEV pDev = pVM->pdm.s.pDevs; pDev; pDev = pDev->pNext)
     {
         if (    pDev->cchName == cchDevice
@@ -771,16 +1014,20 @@ int pdmR3DevFindLun(PVM pVM, const char *pszDevice, unsigned iInstance, unsigned
                     {
                         if (pLun->iLun == iLun)
                         {
+                            RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
                             *ppLun = pLun;
                             return VINF_SUCCESS;
                         }
                     }
+                    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
                     return VERR_PDM_LUN_NOT_FOUND;
                 }
             }
+            RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
             return VERR_PDM_DEVICE_INSTANCE_NOT_FOUND;
         }
     }
+    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
     return VERR_PDM_DEVICE_NOT_FOUND;
 }
 
@@ -799,8 +1046,11 @@ int pdmR3DevFindLun(PVM pVM, const char *pszDevice, unsigned iInstance, unsigned
  * @param   ppBase          Where to store the base interface pointer. Optional.
  * @thread  EMT
  */
-VMMR3DECL(int) PDMR3DeviceAttach(PUVM pUVM, const char *pszDevice, unsigned iInstance, unsigned iLun, uint32_t fFlags, PPPDMIBASE ppBase)
+VMMR3DECL(int) PDMR3DeviceAttach(PUVM pUVM, const char *pszDevice, unsigned iInstance, unsigned iLun,
+                                 uint32_t fFlags, PPPDMIBASE ppBase)
 {
+    if (ppBase)
+        *ppBase = NULL;
     UVM_ASSERT_VALID_EXT_RETURN(pUVM, VERR_INVALID_VM_HANDLE);
     PVM pVM = pUVM->pVM;
     VM_ASSERT_VALID_EXT_RETURN(pVM, VERR_INVALID_VM_HANDLE);
@@ -811,6 +1061,7 @@ VMMR3DECL(int) PDMR3DeviceAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
     /*
      * Find the LUN in question.
      */
+    RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
     PPDMLUN pLun;
     int rc = pdmR3DevFindLun(pVM, pszDevice, iInstance, iLun, &pLun);
     if (RT_SUCCESS(rc))
@@ -823,9 +1074,11 @@ VMMR3DECL(int) PDMR3DeviceAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
         {
             if (!pLun->pTop)
             {
-                PDMCritSectEnter(pDevIns->pCritSectRoR3, VERR_IGNORED);
+                RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
+                PDMCritSectEnter(pVM, pDevIns->pCritSectRoR3, VERR_IGNORED);
                 rc = pDevIns->pReg->pfnAttach(pDevIns, iLun, fFlags);
-                PDMCritSectLeave(pDevIns->pCritSectRoR3);
+                PDMCritSectLeave(pVM, pDevIns->pCritSectRoR3);
+                RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
             }
             else
                 rc = VERR_PDM_DRIVER_ALREADY_ATTACHED;
@@ -836,8 +1089,7 @@ VMMR3DECL(int) PDMR3DeviceAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
         if (ppBase)
             *ppBase = pLun->pTop ? &pLun->pTop->IBase : NULL;
     }
-    else if (ppBase)
-        *ppBase = NULL;
+    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
 
     if (ppBase)
         LogFlow(("PDMR3DeviceAttach: returns %Rrc *ppBase=%p\n", rc, *ppBase));
@@ -907,21 +1159,22 @@ VMMR3_INT_DECL(PPDMCRITSECT) PDMR3DevGetCritSect(PVM pVM, PPDMDEVINS pDevIns)
  *
  * @thread  EMT
  */
-VMMR3DECL(int) PDMR3DriverAttach(PUVM pUVM, const char *pszDevice, unsigned iInstance, unsigned iLun, uint32_t fFlags, PPPDMIBASE ppBase)
+VMMR3DECL(int) PDMR3DriverAttach(PUVM pUVM, const char *pszDevice, unsigned iInstance, unsigned iLun,
+                                 uint32_t fFlags, PPPDMIBASE ppBase)
 {
     LogFlow(("PDMR3DriverAttach: pszDevice=%p:{%s} iInstance=%d iLun=%d fFlags=%#x ppBase=%p\n",
              pszDevice, pszDevice, iInstance, iLun, fFlags, ppBase));
+    if (ppBase)
+        *ppBase = NULL;
     UVM_ASSERT_VALID_EXT_RETURN(pUVM, VERR_INVALID_VM_HANDLE);
     PVM pVM = pUVM->pVM;
     VM_ASSERT_VALID_EXT_RETURN(pVM, VERR_INVALID_VM_HANDLE);
     VM_ASSERT_EMT(pVM);
 
-    if (ppBase)
-        *ppBase = NULL;
-
     /*
      * Find the LUN in question.
      */
+    RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
     PPDMLUN pLun;
     int rc = pdmR3DevFindLun(pVM, pszDevice, iInstance, iLun, &pLun);
     if (RT_SUCCESS(rc))
@@ -936,11 +1189,13 @@ VMMR3DECL(int) PDMR3DriverAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
             PPDMDEVINS pDevIns = pLun->pDevIns;
             if (pDevIns->pReg->pfnAttach)
             {
-                PDMCritSectEnter(pDevIns->pCritSectRoR3, VERR_IGNORED);
+                RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
+                PDMCritSectEnter(pVM, pDevIns->pCritSectRoR3, VERR_IGNORED);
                 rc = pDevIns->pReg->pfnAttach(pDevIns, iLun, fFlags);
                 if (RT_SUCCESS(rc) && ppBase)
                     *ppBase = pLun->pTop ? &pLun->pTop->IBase : NULL;
-                PDMCritSectLeave(pDevIns->pCritSectRoR3);
+                PDMCritSectLeave(pVM, pDevIns->pCritSectRoR3);
+                RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
             }
             else
                 rc = VERR_PDM_DEVICE_NO_RT_ATTACH;
@@ -952,7 +1207,9 @@ VMMR3DECL(int) PDMR3DriverAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
                 pDrvIns = pDrvIns->Internal.s.pDown;
             if (pDrvIns->pReg->pfnAttach)
             {
+                RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
                 rc = pDrvIns->pReg->pfnAttach(pDrvIns, fFlags);
+                RTCritSectRwEnterShared(&pVM->pdm.s.CoreListCritSectRw);
                 if (RT_SUCCESS(rc) && ppBase)
                     *ppBase = pDrvIns->Internal.s.pDown
                             ? &pDrvIns->Internal.s.pDown->IBase
@@ -962,6 +1219,7 @@ VMMR3DECL(int) PDMR3DriverAttach(PUVM pUVM, const char *pszDevice, unsigned iIns
                 rc = VERR_PDM_DRIVER_NO_RT_ATTACH;
         }
     }
+    RTCritSectRwLeaveShared(&pVM->pdm.s.CoreListCritSectRw);
 
     if (ppBase)
         LogFlow(("PDMR3DriverAttach: returns %Rrc *ppBase=%p\n", rc, *ppBase));
@@ -1000,7 +1258,7 @@ VMMR3DECL(int) PDMR3DriverDetach(PUVM pUVM, const char *pszDevice, unsigned iDev
     LogFlow(("PDMR3DriverDetach: pszDevice=%p:{%s} iDevIns=%u iLun=%u pszDriver=%p:{%s} iOccurrence=%u fFlags=%#x\n",
              pszDevice, pszDevice, iDevIns, iLun, pszDriver, pszDriver, iOccurrence, fFlags));
     UVM_ASSERT_VALID_EXT_RETURN(pUVM, VERR_INVALID_VM_HANDLE);
-    PVM pVM = pUVM->pVM;
+    PVM const pVM = pUVM->pVM;
     VM_ASSERT_VALID_EXT_RETURN(pVM, VERR_INVALID_VM_HANDLE);
     VM_ASSERT_EMT(pVM);
     AssertPtr(pszDevice);
@@ -1011,6 +1269,7 @@ VMMR3DECL(int) PDMR3DriverDetach(PUVM pUVM, const char *pszDevice, unsigned iDev
     /*
      * Find the LUN in question.
      */
+    RTCritSectRwEnterExcl(&pVM->pdm.s.CoreListCritSectRw);
     PPDMLUN pLun;
     int rc = pdmR3DevFindLun(pVM, pszDevice, iDevIns, iLun, &pLun);
     if (RT_SUCCESS(rc))
@@ -1035,13 +1294,14 @@ VMMR3DECL(int) PDMR3DriverDetach(PUVM pUVM, const char *pszDevice, unsigned iDev
                 }
             }
             if (pDrvIns)
-                rc = pdmR3DrvDetach(pDrvIns, fFlags);
+                rc = pdmR3DrvDetach(pVM, pDrvIns, fFlags);
             else
                 rc = VERR_PDM_DRIVER_INSTANCE_NOT_FOUND;
         }
         else
             rc = VINF_PDM_NO_DRIVER_ATTACHED_TO_LUN;
     }
+    RTCritSectRwLeaveExcl(&pVM->pdm.s.CoreListCritSectRw);
 
     LogFlow(("PDMR3DriverDetach: returns %Rrc\n", rc));
     return rc;
