@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2019-2023 Oracle and/or its affiliates.
+ * Copyright (C) 2019-2024 Oracle and/or its affiliates.
  *
  * This file is part of VirtualBox base platform packages, as
  * available from https://www.virtualbox.org.
@@ -607,13 +607,13 @@ VMMR3_INT_DECL(int) PDMR3TaskDestroySpecific(PVM pVM, PDMTASKTYPE enmType, void
     AssertReturn(iTaskSet < RT_ELEMENTS(pVM->pdm.s.apTaskSets), VERR_INVALID_HANDLE);
     PPDMTASKSET const pTaskSet = pVM->pdm.s.apTaskSets[iTaskSet];
     AssertPtrReturn(pTaskSet, VERR_INVALID_HANDLE);
-    AssertPtrReturn(pTaskSet->u32Magic == PDMTASKSET_MAGIC, VERR_INVALID_MAGIC);
+    AssertReturn(pTaskSet->u32Magic == PDMTASKSET_MAGIC, VERR_INVALID_MAGIC);
     PPDMTASK const    pTask    = &pTaskSet->aTasks[iTask];
 
     VM_ASSERT_EMT0_RETURN(pVM, VERR_VM_THREAD_NOT_EMT); /* implicit serialization by requiring EMT(0) */
 
-    AssertPtrReturn(pTask->pvOwner == pvOwner, VERR_NOT_OWNER);
-    AssertPtrReturn(pTask->enmType == enmType, VERR_NOT_OWNER);
+    AssertReturn(pTask->pvOwner == pvOwner, VERR_NOT_OWNER);
+    AssertReturn(pTask->enmType == enmType, VERR_NOT_OWNER);
 
     /*
      * Do the job.
