@@ -4,22 +4,32 @@
  */
 
 /*
- * Copyright (C) 2007-2015 Oracle Corporation
+ * Copyright (C) 2007-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only
  */
 
 
 /*********************************************************************************************************************************
 *   Header Files                                                                                                                 *
 *********************************************************************************************************************************/
-///@todo #define LOG_GROUP LOG_GROUP_PDM_THREAD
+/// @todo \#define LOG_GROUP LOG_GROUP_PDM_THREAD
 #include "PDMInternal.h"
 #include <VBox/vmm/pdm.h>
 #include <VBox/vmm/mm.h>
@@ -98,7 +108,7 @@ static DECLCALLBACK(int) pdmR3ThreadWakeUp(PPDMTHREAD pThread)
  * Allocates new thread instance.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the pointer to the instance.
  */
 static int pdmR3ThreadNew(PVM pVM, PPPDMTHREAD ppThread)
@@ -123,7 +133,7 @@ static int pdmR3ThreadNew(PVM pVM, PPPDMTHREAD ppThread)
  * Initialize a new thread, this actually creates the thread.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where the thread instance data handle is.
  * @param   cbStack     The stack size, see RTThreadCreate().
  * @param   enmType     The thread type, see RTThreadCreate().
@@ -185,7 +195,7 @@ static int pdmR3ThreadInit(PVM pVM, PPPDMTHREAD ppThread, size_t cbStack, RTTHRE
         RTSemEventMultiDestroy(pThread->Internal.s.BlockEvent);
         pThread->Internal.s.BlockEvent = NIL_RTSEMEVENTMULTI;
     }
-    MMHyperFree(pVM, pThread);
+    MMR3HeapFree(pThread);
     *ppThread = NULL;
 
     return rc;
@@ -196,7 +206,7 @@ static int pdmR3ThreadInit(PVM pVM, PPPDMTHREAD ppThread, size_t cbStack, RTTHRE
  * Device Helper for creating a thread associated with a device.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDevIns     The device instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -229,7 +239,7 @@ int pdmR3ThreadCreateDevice(PVM pVM, PPDMDEVINS pDevIns, PPPDMTHREAD ppThread, v
  * USB Device Helper for creating a thread associated with an USB device.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pUsbIns     The USB device instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -262,7 +272,7 @@ int pdmR3ThreadCreateUsb(PVM pVM, PPDMUSBINS pUsbIns, PPPDMTHREAD ppThread, void
  * Driver Helper for creating a thread associated with a driver.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDrvIns     The driver instance.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
@@ -295,7 +305,7 @@ int pdmR3ThreadCreateDriver(PVM pVM, PPDMDRVINS pDrvIns, PPPDMTHREAD ppThread, v
  * Creates a PDM thread for internal use in the VM.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
  * @param   pfnThread   The thread function.
@@ -326,7 +336,7 @@ VMMR3DECL(int) PDMR3ThreadCreate(PVM pVM, PPPDMTHREAD ppThread, void *pvUser, PF
  * Creates a PDM thread for VM use by some external party.
  *
  * @returns VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   ppThread    Where to store the thread 'handle'.
  * @param   pvUser      The user argument to the thread function.
  * @param   pfnThread   The thread function.
@@ -474,7 +484,7 @@ VMMR3DECL(int) PDMR3ThreadDestroy(PPDMTHREAD pThread, int *pRcThread)
  * destroyed (not currently implemented).
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDevIns     the device instance.
  */
 int pdmR3ThreadDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
@@ -509,7 +519,7 @@ int pdmR3ThreadDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
  * This function is called by PDMUsb when a device is destroyed.
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pUsbIns     The USB device instance.
  */
 int pdmR3ThreadDestroyUsb(PVM pVM, PPDMUSBINS pUsbIns)
@@ -544,7 +554,7 @@ int pdmR3ThreadDestroyUsb(PVM pVM, PPDMUSBINS pUsbIns)
  * This function is called by PDMDriver when a driver is destroyed.
  *
  * @returns VBox status code of the first failure.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  * @param   pDrvIns     The driver instance.
  */
 int pdmR3ThreadDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
@@ -576,7 +586,7 @@ int pdmR3ThreadDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
 /**
  * Called For VM power off.
  *
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 void pdmR3ThreadDestroyAll(PVM pVM)
 {
@@ -672,7 +682,7 @@ VMMR3DECL(int) PDMR3ThreadIAmSuspending(PPDMTHREAD pThread)
         }
     }
 
-    AssertMsgFailed(("rc=%d enmState=%d\n", rc, pThread->enmState));
+    AssertLogRelMsgFailed(("rc=%d enmState=%d thread=%s\n", rc, pThread->enmState, RTThreadGetName(pThread->Thread)));
     pdmR3ThreadBailMeOut(pThread);
     return rc;
 }
@@ -708,7 +718,7 @@ VMMR3DECL(int) PDMR3ThreadIAmRunning(PPDMTHREAD pThread)
             return rc;
     }
 
-    AssertMsgFailed(("rc=%d enmState=%d\n", rc, pThread->enmState));
+    AssertLogRelMsgFailed(("rc=%d enmState=%d thread=%s\n", rc, pThread->enmState, RTThreadGetName(pThread->Thread)));
     pdmR3ThreadBailMeOut(pThread);
     return rc;
 }
@@ -829,7 +839,11 @@ static DECLCALLBACK(int) pdmR3ThreadMain(RTTHREAD Thread, void *pvUser)
     }
 
     if (RT_FAILURE(rc))
-        LogRel(("PDMThread: Thread '%s' (%RTthrd) quit unexpectedly with rc=%Rrc.\n", RTThreadGetName(Thread), Thread, rc));
+        LogRel(("PDMThread: Thread '%s' (%RTthrd) quit unexpectedly with rc=%Rrc in state %d.\n",
+                RTThreadGetName(Thread), Thread, rc, pThread->enmState));
+    else if (pThread->enmState != PDMTHREADSTATE_TERMINATING)
+        LogRel(("PDMThread: Thread '%s' (%RTthrd) is quitting in state %d (expected %d).\n",
+                RTThreadGetName(Thread), Thread, pThread->enmState, PDMTHREADSTATE_TERMINATING));
 
     /*
      * Advance the state to terminating and then on to terminated.
@@ -972,7 +986,7 @@ VMMR3DECL(int) PDMR3ThreadSuspend(PPDMTHREAD pThread)
  * and drivers have been notified about the suspend / power off.
  *
  * @return VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 int pdmR3ThreadSuspendAll(PVM pVM)
 {
@@ -984,7 +998,10 @@ int pdmR3ThreadSuspendAll(PVM pVM)
             case PDMTHREADSTATE_RUNNING:
             {
                 int rc = PDMR3ThreadSuspend(pThread);
-                AssertRCReturn(rc, rc);
+                AssertLogRelMsgReturnStmt(RT_SUCCESS(rc),
+                                          ("PDMR3ThreadSuspend -> %Rrc for '%s'\n", rc, RTThreadGetName(pThread->Thread)),
+                                          RTCritSectLeave(&pUVM->pdm.s.ListCritSect),
+                                          rc);
                 break;
             }
 
@@ -1064,7 +1081,7 @@ VMMR3DECL(int) PDMR3ThreadResume(PPDMTHREAD pThread)
  * and drivers have been notified about the resume / power on .
  *
  * @return VBox status code.
- * @param   pVM         Pointer to the VM.
+ * @param   pVM         The cross context VM structure.
  */
 int pdmR3ThreadResumeAll(PVM pVM)
 {
