@@ -4,24 +4,34 @@
  */
 
 /*
- * Copyright (C) 2007-2016 Oracle Corporation
+ * Copyright (C) 2007-2024 Oracle and/or its affiliates.
  *
- * This file is part of VirtualBox Open Source Edition (OSE), as
- * available from http://www.virtualbox.org. This file is free software;
- * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License (GPL) as published by the Free Software
- * Foundation, in version 2 as it comes in the "COPYING" file of the
- * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
- * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ * This file is part of VirtualBox base platform packages, as
+ * available from https://www.virtualbox.org.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation, in version 3 of the
+ * License.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <https://www.gnu.org/licenses>.
  *
  * The contents of this file may alternatively be used under the terms
  * of the Common Development and Distribution License Version 1.0
- * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
- * VirtualBox OSE distribution, in which case the provisions of the
+ * (CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+ * in the VirtualBox distribution, in which case the provisions of the
  * CDDL are applicable instead of those of the GPL.
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
+ *
+ * SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
  */
 
 
@@ -76,7 +86,7 @@ typedef struct BS3FMTSTATE
     char            szTmp[BS3FMT_TMP_SIZE];
 } BS3FMTSTATE;
 /** Pointer to a BS3Kit string formatter state. */
-typedef BS3FMTSTATE *PBS3FMTSTATE;
+typedef BS3FMTSTATE BS3_FAR *PBS3FMTSTATE;
 
 
 
@@ -97,7 +107,7 @@ static size_t bs3StrFormatU32(PBS3FMTSTATE pState, uint32_t uValue);
  * @param   pszNumber           The formatted number string.
  * @param   cchNumber           The length of the number.
  */
-static size_t bs3StrFormatNumberString(PBS3FMTSTATE pState, char const *pszNumber, size_t cchNumber)
+static size_t bs3StrFormatNumberString(PBS3FMTSTATE pState, char const BS3_FAR *pszNumber, size_t cchNumber)
 {
     /*
      * Calc the length of the core number with prefixes.
@@ -224,8 +234,8 @@ static size_t bs3StrFormatU64(PBS3FMTSTATE pState, uint64_t uValue)
 #endif
 
     {
-        const char *pachDigits = !(pState->fFlags & STR_F_CAPITAL) ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
-        char       *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
+        const char BS3_FAR *pachDigits = !(pState->fFlags & STR_F_CAPITAL) ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
+        char       BS3_FAR *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
 
         *--psz = '\0';
 #if ARCH_BITS == 64
@@ -262,8 +272,8 @@ static size_t bs3StrFormatU64(PBS3FMTSTATE pState, uint64_t uValue)
 static size_t bs3StrFormatU32(PBS3FMTSTATE pState, uint32_t uValue)
 {
 #if ARCH_BITS < 64
-    const char *pachDigits = !(pState->fFlags & STR_F_CAPITAL) ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
-    char       *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
+    const char BS3_FAR *pachDigits = !(pState->fFlags & STR_F_CAPITAL) ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
+    char       BS3_FAR *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
 
     *--psz = '\0';
     if (pState->uBase == 10)
@@ -304,9 +314,9 @@ static size_t bs3StrFormatU16(PBS3FMTSTATE pState, uint16_t uValue)
 {
     if (pState->uBase == 10)
     {
-        const char *pachDigits = !(pState->fFlags & STR_F_CAPITAL)
-                               ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
-        char       *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
+        const char BS3_FAR *pachDigits = !(pState->fFlags & STR_F_CAPITAL)
+                                       ? g_achBs3HexDigits : g_achBs3HexDigitsUpper;
+        char       BS3_FAR *psz = &pState->szTmp[BS3FMT_TMP_SIZE];
 
         *--psz = '\0';
         do
@@ -361,12 +371,15 @@ static size_t bs3StrFormatS16(PBS3FMTSTATE pState, int16_t iValue)
 
 
 #undef Bs3StrFormatV
-BS3_CMN_DEF(size_t, Bs3StrFormatV,(const char BS3_FAR *pszFormat, va_list va,
+BS3_CMN_DEF(size_t, Bs3StrFormatV,(const char BS3_FAR *pszFormat, va_list BS3_FAR va,
                                    PFNBS3STRFORMATOUTPUT pfnOutput, void BS3_FAR *pvUser))
 {
     BS3FMTSTATE State;
     size_t      cchRet = 0;
     char        ch;
+#if ARCH_BITS == 16
+    typedef int SIZE_CHECK_TYPE1[sizeof(va) == 4 && sizeof(va[0]) == 4];
+#endif
 
     State.pfnOutput = pfnOutput;
     State.pvUser    = pvUser;
@@ -537,7 +550,6 @@ BS3_CMN_DEF(size_t, Bs3StrFormatV,(const char BS3_FAR *pszFormat, va_list va,
                     while (--State.cchWidth >= cch)
                         cchRet += State.pfnOutput(' ', State.pvUser);
 
-                cchRet += cch;
                 while (cch-- > 0)
                     cchRet += State.pfnOutput(*psz++, State.pvUser);
 
@@ -737,7 +749,7 @@ BS3_CMN_DEF(size_t, Bs3StrFormatV,(const char BS3_FAR *pszFormat, va_list va,
              */
             case 'P':
                 State.fFlags |= STR_F_CAPITAL;
-                /* fall thru */
+                RT_FALL_THRU();
             case 'p':
             {
                 void BS3_FAR *pv = va_arg(va, void BS3_FAR *);
