@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # $Id: testcase.py $
-# pylint: disable=C0302
+# pylint: disable=too-many-lines
 
 """
 Test Manager - Test Case.
@@ -8,30 +8,41 @@ Test Manager - Test Case.
 
 __copyright__ = \
 """
-Copyright (C) 2012-2016 Oracle Corporation
+Copyright (C) 2012-2024 Oracle and/or its affiliates.
 
-This file is part of VirtualBox Open Source Edition (OSE), as
-available from http://www.virtualbox.org. This file is free software;
-you can redistribute it and/or modify it under the terms of the GNU
-General Public License (GPL) as published by the Free Software
-Foundation, in version 2 as it comes in the "COPYING" file of the
-VirtualBox OSE distribution. VirtualBox OSE is distributed in the
-hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+This file is part of VirtualBox base platform packages, as
+available from https://www.virtualbox.org.
+
+This program is free software; you can redistribute it and/or
+modify it under the terms of the GNU General Public License
+as published by the Free Software Foundation, in version 3 of the
+License.
+
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, see <https://www.gnu.org/licenses>.
 
 The contents of this file may alternatively be used under the terms
 of the Common Development and Distribution License Version 1.0
-(CDDL) only, as it comes in the "COPYING.CDDL" file of the
-VirtualBox OSE distribution, in which case the provisions of the
+(CDDL), a copy of it is provided in the "COPYING.CDDL" file included
+in the VirtualBox distribution, in which case the provisions of the
 CDDL are applicable instead of those of the GPL.
 
 You may elect to license modified versions of this file under the
 terms and conditions of either the GPL or the CDDL or both.
+
+SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
 """
-__version__ = "$Revision: 109040 $"
+__version__ = "$Revision: 164827 $"
 
 
 # Standard python imports.
 import copy;
+import sys;
 import unittest;
 
 # Validation Kit imports.
@@ -41,6 +52,10 @@ from testmanager.core.base              import ModelDataBase, ModelDataBaseTestC
 from testmanager.core.globalresource    import GlobalResourceData;
 from testmanager.core.useraccount       import UserAccountLogic;
 
+# Python 3 hacks:
+if sys.version_info[0] >= 3:
+    long = int;     # pylint: disable=redefined-builtin,invalid-name
+
 
 
 class TestCaseGlobalRsrcDepData(ModelDataBase):
@@ -249,7 +264,7 @@ class TestCaseDependencyData(ModelDataBase):
 
         The dictionary keys are ksParam_*.
         """
-        dErrors = dict()
+        dErrors = {}
 
         self.idTestCase         = self._validateInt(   dErrors, self.ksParam_idTestCase,        self.idTestCase);
         self.idTestCasePreReq   = self._validateInt(   dErrors, self.ksParam_idTestCasePreReq,  self.idTestCasePreReq);
@@ -297,7 +312,7 @@ class TestCaseDependencyData(ModelDataBase):
         Get list of Test Case IDs which current
         Test Case depends on
         """
-        if len(aTestCaseDependencyData) == 0:
+        if not aTestCaseDependencyData:
             return []
 
         aoRet = []
@@ -640,6 +655,7 @@ class TestCaseData(ModelDataBase):
                 'fCpu64BitGuest':       False,
                 'fChipsetIoMmu':        False,
                 'fRawMode':             False,
+                'fNativeApi':           False,
                 'cMbMemory':            985034,
                 'cMbScratch':           1234089,
                 'iTestBoxScriptRev':    1,
@@ -657,6 +673,7 @@ class TestCaseData(ModelDataBase):
                 'fCpu64BitGuest':       True,
                 'fChipsetIoMmu':        True,
                 'fRawMode':             True,
+                'fNativeApi':           True,
                 'cMbMemory':            9999999999,
                 'cMbScratch':           9999999999999,
                 'iTestBoxScriptRev':    9999999,
@@ -677,12 +694,15 @@ class TestCaseData(ModelDataBase):
             'sOsVersion':           oTestBoxData.sOsVersion,
             'sCpuVendor':           oTestBoxData.sCpuVendor,
             'sCpuArch':             oTestBoxData.sCpuArch,
+            'iCpuFamily':           oTestBoxData.getCpuFamily(),
+            'iCpuModel':            oTestBoxData.getCpuModel(),
             'cCpus':                oTestBoxData.cCpus,
             'fCpuHwVirt':           oTestBoxData.fCpuHwVirt,
             'fCpuNestedPaging':     oTestBoxData.fCpuNestedPaging,
             'fCpu64BitGuest':       oTestBoxData.fCpu64BitGuest,
             'fChipsetIoMmu':        oTestBoxData.fChipsetIoMmu,
             'fRawMode':             oTestBoxData.fRawMode,
+            'fNativeApi':           oTestBoxData.fNativeApi,
             'cMbMemory':            oTestBoxData.cMbMemory,
             'cMbScratch':           oTestBoxData.cMbScratch,
             'iTestBoxScriptRev':    oTestBoxData.iTestBoxScriptRev,
@@ -851,7 +871,7 @@ class TestCaseDataEx(TestCaseData):
                 aoNewValues.append(TestCaseArgsData().initFromParams(oDispWrapper, fStrict = False));
         return aoNewValues;
 
-    def _validateAndConvertAttribute(self, sAttr, sParam, oValue, aoNilValues, fAllowNull, oDb): # pylint: disable=R0914
+    def _validateAndConvertAttribute(self, sAttr, sParam, oValue, aoNilValues, fAllowNull, oDb): # pylint: disable=too-many-locals
         """
         Validate special arrays and requirement expressions.
 
@@ -871,7 +891,7 @@ class TestCaseDataEx(TestCaseData):
                     try:
                         oTestCase.idTestCase = int(oTestCase.idTestCase);
                         oTestCase.initFromDbWithId(oDb, oTestCase.idTestCase);
-                    except Exception, oXcpt:
+                    except Exception as oXcpt:
                         asErrors.append('Test case dependency #%s: %s' % (oTestCase.idTestCase, oXcpt));
                 aoNewValues.append(oTestCase);
 
@@ -882,23 +902,23 @@ class TestCaseDataEx(TestCaseData):
                     try:
                         oGlobalRsrc.idTestCase = int(oGlobalRsrc.idGlobalRsrc);
                         oGlobalRsrc.initFromDbWithId(oDb, oGlobalRsrc.idGlobalRsrc);
-                    except Exception, oXcpt:
+                    except Exception as oXcpt:
                         asErrors.append('Resource dependency #%s: %s' % (oGlobalRsrc.idGlobalRsrc, oXcpt));
                 aoNewValues.append(oGlobalRsrc);
 
         else:
             assert sAttr == 'aoTestCaseArgs';
-            if self.aoTestCaseArgs is None or len(self.aoTestCaseArgs) == 0:
+            if not self.aoTestCaseArgs:
                 return (None, 'The testcase requires at least one argument variation to be valid.');
 
             # Note! We'll be returning an error dictionary instead of an string here.
             dErrors = {};
 
-            for iVar in range(len(self.aoTestCaseArgs)):
-                oVar = copy.copy(self.aoTestCaseArgs[iVar]);
+            for iVar, oVar in enumerate(self.aoTestCaseArgs):
+                oVar = copy.copy(oVar);
                 oVar.idTestCase = self.idTestCase;
                 dCurErrors = oVar.validateAndConvert(oDb, ModelDataBase.ksValidateFor_Other);
-                if len(dCurErrors) == 0:
+                if not dCurErrors:
                     pass; ## @todo figure out the ID?
                 else:
                     asErrors = [];
@@ -907,8 +927,8 @@ class TestCaseDataEx(TestCaseData):
                     dErrors[iVar] = '<br>\n'.join(asErrors)
                 aoNewValues.append(oVar);
 
-            for iVar in range(len(self.aoTestCaseArgs)):
-                sArgs = self.aoTestCaseArgs[iVar].sArgs;
+            for iVar, oVar in enumerate(self.aoTestCaseArgs):
+                sArgs = oVar.sArgs;
                 for iVar2 in range(iVar + 1, len(self.aoTestCaseArgs)):
                     if self.aoTestCaseArgs[iVar2].sArgs == sArgs:
                         sMsg = 'Duplicate argument variation "%s".' % (sArgs);
@@ -918,16 +938,16 @@ class TestCaseDataEx(TestCaseData):
                         else:                   dErrors[iVar2]  = sMsg;
                         break;
 
-            return (aoNewValues, dErrors if len(dErrors) > 0 else None);
+            return (aoNewValues, dErrors if dErrors else None);
 
-        return (aoNewValues, None if len(asErrors) == 0 else ' <br>'.join(asErrors));
+        return (aoNewValues, None if not asErrors else ' <br>'.join(asErrors));
 
     def _validateAndConvertWorker(self, asAllowNullAttributes, oDb, enmValidateFor = ModelDataBase.ksValidateFor_Other):
         dErrors = TestCaseData._validateAndConvertWorker(self, asAllowNullAttributes, oDb, enmValidateFor);
 
         # Validate dependencies a wee bit for paranoid reasons. The scheduler
         # queue generation code does the real validation here!
-        if len(dErrors) == 0 and self.idTestCase is not None:
+        if not dErrors and self.idTestCase is not None:
             for oDep in self.aoDepTestCases:
                 if oDep.idTestCase == self.idTestCase:
                     if self.ksParam_aoDepTestCases in dErrors:
@@ -964,13 +984,14 @@ class TestCaseLogic(ModelLogicBase):
             aoRet.append(TestCaseData().initFromDbRow(aoRow))
         return aoRet
 
-    def fetchForListing(self, iStart, cMaxRows, tsNow):
+    def fetchForListing(self, iStart, cMaxRows, tsNow, aiSortColumns = None):
         """
         Fetches test cases.
 
         Returns an array (list) of TestCaseDataEx items, empty list if none.
         Raises exception on error.
         """
+        _ = aiSortColumns;
         if tsNow is None:
             self._oDb.execute('SELECT   *\n'
                               'FROM     TestCases\n'
@@ -992,7 +1013,7 @@ class TestCaseLogic(ModelLogicBase):
             aoRows.append(TestCaseDataEx().initFromDbRowEx(aoRow, self._oDb, tsNow));
         return aoRows;
 
-    def fetchForChangeLog(self, idTestCase, iStart, cMaxRows, tsNow): # pylint: disable=R0914
+    def fetchForChangeLog(self, idTestCase, iStart, cMaxRows, tsNow): # pylint: disable=too-many-locals
         """
         Fetches change log entries for a testbox.
 
@@ -1124,7 +1145,7 @@ class TestCaseLogic(ModelLogicBase):
             aoEntries.append(ChangeLogEntry(uidAuthor, None, tsEffective, tsExpire, oNew, oOld, aoChanges));
 
         # If we're at the end of the log, add the initial entry.
-        if len(aoRows) <= cMaxRows and len(aoRows) > 0:
+        if len(aoRows) <= cMaxRows and aoRows:
             oNew = aoRows[-1];
             aoEntries.append(ChangeLogEntry(oNew.uidAuthor, None,
                                             aaoChanges[-1][0], aaoChanges[-2][0] if len(aaoChanges) > 1 else oNew.tsExpire,
@@ -1143,7 +1164,7 @@ class TestCaseLogic(ModelLogicBase):
         #
         assert isinstance(oData, TestCaseDataEx);
         dErrors = oData.validateAndConvert(self._oDb, oData.ksValidateFor_Add);
-        if len(dErrors) > 0:
+        if dErrors:
             raise TMInvalidData('Invalid input data: %s' % (dErrors,));
 
         #
@@ -1177,7 +1198,7 @@ class TestCaseLogic(ModelLogicBase):
         self._oDb.maybeCommit(fCommit);
         return True;
 
-    def editEntry(self, oData, uidAuthor, fCommit = False):  # pylint: disable=R0914
+    def editEntry(self, oData, uidAuthor, fCommit = False):  # pylint: disable=too-many-locals
         """
         Edit a testcase entry (extended).
         Caller is expected to rollback the database transactions on exception.
@@ -1188,7 +1209,7 @@ class TestCaseLogic(ModelLogicBase):
         #
         assert isinstance(oData, TestCaseDataEx);
         dErrors = oData.validateAndConvert(self._oDb, oData.ksValidateFor_Edit);
-        if len(dErrors) > 0:
+        if dErrors:
             raise TMInvalidData('Invalid input data: %s' % (dErrors,));
 
         #
@@ -1226,7 +1247,7 @@ class TestCaseLogic(ModelLogicBase):
         for idDep in aidOldDeps:
             if idDep in aidNewDeps:
                 asKeepers.append(str(idDep));
-        if len(asKeepers) > 0:
+        if asKeepers:
             sQuery += '     AND idTestCasePreReq NOT IN (' + ', '.join(asKeepers) + ')\n';
         self._oDb.execute(sQuery);
 
@@ -1251,7 +1272,7 @@ class TestCaseLogic(ModelLogicBase):
         for idDep in aidOldDeps:
             if idDep in aidNewDeps:
                 asKeepers.append(str(idDep));
-        if len(asKeepers) > 0:
+        if asKeepers:
             sQuery = '     AND idGlobalRsrc NOT IN (' + ', '.join(asKeepers) + ')\n';
         self._oDb.execute(sQuery);
 
@@ -1274,7 +1295,7 @@ class TestCaseLogic(ModelLogicBase):
                                           , (oData.idTestCase, ));
         for oNewVar in oData.aoTestCaseArgs:
             asKeepers.append(self._oDb.formatBindArgs('%s', (oNewVar.sArgs,)));
-        if len(asKeepers) > 0:
+        if asKeepers:
             sQuery += '    AND  sArgs NOT IN (' + ', '.join(asKeepers) + ')\n';
         self._oDb.execute(sQuery);
 
@@ -1409,7 +1430,7 @@ class TestCaseLogic(ModelLogicBase):
 # Unit testing.
 #
 
-# pylint: disable=C0111
+# pylint: disable=missing-docstring
 class TestCaseGlobalRsrcDepDataTestCase(ModelDataBaseTestCase):
     def setUp(self):
         self.aoSamples = [TestCaseGlobalRsrcDepData(),];
