@@ -7,7 +7,7 @@ Test Manager Web-UI - Base Classes.
 
 __copyright__ = \
 """
-Copyright (C) 2012-2022 Oracle and/or its affiliates.
+Copyright (C) 2012-2024 Oracle and/or its affiliates.
 
 This file is part of VirtualBox base platform packages, as
 available from https://www.virtualbox.org.
@@ -36,7 +36,7 @@ terms and conditions of either the GPL or the CDDL or both.
 
 SPDX-License-Identifier: GPL-3.0-only OR CDDL-1.0
 """
-__version__ = "$Revision: 153224 $"
+__version__ = "$Revision: 164827 $"
 
 
 # Standard python imports.
@@ -144,7 +144,7 @@ class WuiDispatcherBase(object):
         # Debugger bits.
         self._fDbgSqlTrace      = False;
         self._fDbgSqlExplain    = False;
-        self._dDbgParams        = dict();
+        self._dDbgParams        = {};
         for sKey, sValue in oSrvGlue.getParameters().items():
             if sKey in self.kasDbgParams:
                 self._dDbgParams[sKey] = sValue;
@@ -321,7 +321,7 @@ class WuiDispatcherBase(object):
         #
         # Load the template.
         #
-        with open(os.path.join(self._oSrvGlue.pathTmWebUI(), self._sTemplate)) as oFile:
+        with open(os.path.join(self._oSrvGlue.pathTmWebUI(), self._sTemplate)) as oFile: # pylint: disable=unspecified-encoding
             sTmpl = oFile.read();
 
         #
@@ -591,7 +591,7 @@ class WuiDispatcherBase(object):
 
             aoListOfTestCases.append(oListEntryTestCase)
 
-        if aoListOfTestCases == []:
+        if not aoListOfTestCases:
             if asDefaults is None:
                 raise WuiException('%s is missing parameters: "%s"' % (self._sAction, sName))
             aoListOfTestCases = asDefaults
@@ -948,7 +948,7 @@ class WuiDispatcherBase(object):
 
         sRedirAction is what action to redirect to on success.
         """
-        import cgitb;
+        import cgitb;   # pylint: disable=deprecated-module ## @todo these will be retired in python 3.13!
 
         idEntry = self.getIntParam(sParamId, iMin = 1, iMax = 0x7ffffffe)
         fCascade = self.getBoolParam('fCascadeDelete', False);
