@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-2023 The OpenSSL Project Authors. All Rights Reserved.
+ * Copyright 2020-2024 The OpenSSL Project Authors. All Rights Reserved.
  *
  * Licensed under the Apache License 2.0 (the "License").  You may not use
  * this file except in compliance with the License.  You can obtain a copy
@@ -110,6 +110,8 @@ ossl_ed25519_verify(const uint8_t *tbs, size_t tbs_len,
                     const uint8_t *context, size_t context_len,
                     OSSL_LIB_CTX *libctx, const char *propq);
 int
+ossl_ed25519_pubkey_verify(const uint8_t *pub, size_t pub_len);
+int
 ossl_ed448_public_from_private(OSSL_LIB_CTX *ctx, uint8_t out_public_key[57],
                                const uint8_t private_key[57], const char *propq);
 int
@@ -126,6 +128,9 @@ ossl_ed448_verify(OSSL_LIB_CTX *ctx,
                   const uint8_t *context, size_t context_len,
                   const uint8_t phflag, const char *propq);
 
+int
+ossl_ed448_pubkey_verify(const uint8_t *pub, size_t pub_len);
+
 int
 ossl_x448(uint8_t out_shared_key[56], const uint8_t private_key[56],
           const uint8_t peer_public_value[56]);
